/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor;

import java.lang.reflect.InvocationTargetException;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.xquark.xml.xdbc.*;

public class ExtractorDriver implements XMLDriver {
    private static final String RCSRevision = "$Revision: 1.12 $";
    private static final String RCSName     = "$Name:  $";

	public static final String EXTRACTOR_URL_PREFIX = "xdbc:xquark:extractor:";

    static private ExtractorDriver instance = new ExtractorDriver();
    
    static private Map datasources = Collections.synchronizedMap(new HashMap());

    private ExtractorDriver() {
		XMLDriverManager.registerDriver(this);
    }

	public boolean acceptsURI(String uri) throws XMLDBCException {
		return uri != null && uri.startsWith(EXTRACTOR_URL_PREFIX);
	}

	public String getSpecificPart(String uri) throws XMLDBCException {
		if (acceptsURI(uri)) return uri.substring(EXTRACTOR_URL_PREFIX.length());
		else throw new XMLDBCException("Unrecognized URI "+uri);
	}
    
    public XMLDataSource getDataSource(String uri) throws XMLDBCException {
    	if (!acceptsURI(uri)) return null;

		String uriContents = getSpecificPart(uri);
		if (uriContents.startsWith("file:") || uriContents.startsWith("http:") || uriContents.startsWith("jndi:")) {
			XMLDataSource source = (XMLDataSource) datasources.get(uriContents);
			if (source != null) return source;
			try {
				Class dsImpl = Class.forName("org.xquark.extractor.Extractor");
				Class[] paramTypes = new Class[] { String.class };
				Object[] params = new Object[] { uriContents };
				source = (XMLDataSource)dsImpl.getConstructor(paramTypes).newInstance(params);
				datasources.put(uriContents, source);
				return source;
			} catch (ClassNotFoundException e) {
				return null;
			} catch (IllegalArgumentException e) {
				return null;
			} catch (SecurityException e) {
				return null;
			} catch (InstantiationException e) {
				return null;
			} catch (IllegalAccessException e) {
				return null;
			} catch (InvocationTargetException e) {
				return null;
			} catch (NoSuchMethodException e) {
				return null;
			}
		} else {
			return null;
		}
    }

	public XMLDataSource getDataSource(String uri, String user, String password) throws XMLDBCException {
		if (!acceptsURI(uri)) return null;

		String uriContents = getSpecificPart(uri);
		if (uriContents.startsWith("jdbc:")) {
			String sourceKey = user+"@"+uriContents;
			XMLDataSource source = (XMLDataSource) datasources.get(sourceKey);
			if (source != null) return source;
			try {
				Class dsImpl = Class.forName("org.xquark.extractor.Extractor");
				Class[] paramTypes = new Class[] { String.class, String.class, String.class };
				Object[] params = new Object[] { uriContents, user, password };
				source = (XMLDataSource)dsImpl.getConstructor(paramTypes).newInstance(params);
				datasources.put(sourceKey, source);
				return source;
			} catch (ClassNotFoundException e) {
				return null;
			} catch (IllegalArgumentException e) {
				return null;
			} catch (SecurityException e) {
				return null;
			} catch (InstantiationException e) {
				return null;
			} catch (IllegalAccessException e) {
				return null;
			} catch (InvocationTargetException e) {
				return null;
			} catch (NoSuchMethodException e) {
				return null;
			}
		} else {
			return null;
		}
	}

}
