/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.algebra;

import java.util.*;

import org.xquark.extractor.common.Debug;
import org.xquark.extractor.common.MessageLibrary;
import org.xquark.extractor.common.SqlWrapperException;

public final class AnalyzePredicateHandUpVisitor extends DefaultAlgebraVisitor {
	private static final String RCSRevision = "$Revision: 1.4 $";
	private static final String RCSName = "$Name:  $";

	List _predicateList = null;
	Set _referredTI = null;
	Expression _blockingNode = null;

	public AnalyzePredicateHandUpVisitor() {
	}

	public void reinit() {
		_predicateList = null;
		_referredTI = null;
		_blockingNode = null;
	}

	public void setPredicateList(List predicateList) {
		_predicateList = predicateList;
	}

	public boolean isBlocked() {
		return null != _blockingNode;
	}

	public Expression getBlockingNode() {
		return _blockingNode;
	}

	public List getUnresolvedPredicateList() {
		return _predicateList;
	}

	public void visit(Expression arg) {
		visitFather(arg);
	}

	public void visit(Join arg) throws SqlWrapperException {
		//Trace.enter(this, "visit(Join arg)");

		Set visibleTI = arg.visibleTableInstances();

		Set referedTI = null;
		Expression predicate = null;
		for (int i = _predicateList.size() - 1; i >= 0; i--) {
			predicate = (Expression) _predicateList.get(i);
			referedTI = predicate.getReferredTableInstances();
			if (visibleTI.containsAll(referedTI)) {
				_predicateList.remove(predicate);
				arg.addPredicate(predicate);
			}
		}

		if (!_predicateList.isEmpty()) {
			visitFather(arg);
		}

		//Trace.exit(this, "visit(Join arg)");
	}

	public void visit(RenameRelation arg) throws SqlWrapperException {
		//Trace.enter(this, "visit(RenameRelation arg)", arg);

		Set visibleTI = arg.visibleTableInstances();
		Debug.assertTrue(1 == visibleTI.size(), "1 == visibleTI.size()");
		Map replaceMap = new HashMap();

		Expression ti = null;
		for (Iterator i = visibleTI.iterator(); i.hasNext();) {
			ti = (Expression) i.next();
			if (ti instanceof RenameRelation) {
				replaceMap.put(ti, arg);
			}
		}

		TopDownReplacer replacer = new TopDownReplacer(replaceMap);
		Expression predicate = null;
		for (int i = 0; i < _predicateList.size(); i++) {
			predicate = (Expression) _predicateList.get(i);
			predicate.accept(replacer);
		}

		visitFather(arg);

		//Trace.exit(this, "visit(RenameRelation arg)", arg);
	}

	public void visit(UnOpProject arg) {
		//Trace.enter(this, "visit(UnOpProject arg)", arg);
		/* default implementation for UnOpSort, UnOpGroup, UnOpRestrict */
		if (arg.getDistinct()) {
			visitProjectDistinct(arg);
		} else {
			visitProject(arg);
		}

		//Trace.exit(this, "visit(UnOpProject arg)", arg);
	}

	protected void visitFather(Expression arg) {
		//		Trace.enter(this, "visitFather(Expression arg)");
		Expression father = arg.getFather();
		if (father != null) {
			father.accept(this);
		} else {
			throw new SqlWrapperException(MessageLibrary.getMessage("IE_ERR", "Can not arrange predicate " /** @todo   + _predicate.pprint() */
			), null);
		}

		//		//Trace.exit(this, "visitFather(Expression arg)");
	}
	private void visitProjectDistinct(UnOpProject arg) {
		//Trace.enter(this, "visitProjectDistinct(UnOpProject arg)", arg);
		_blockingNode = arg;
		//Trace.exit(this, "visitProjectDistinct(UnOpProject arg)", arg);
	}

	private void visitProject(UnOpProject arg) {
		Debug.nyi("visitProject(UnOpProject arg)");
	}

}
