/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.algebra;

import java.util.List;
import java.util.Map;

import org.xquark.extractor.common.SqlWrapperException;
import org.xquark.extractor.sql.SqlExpression;

public class AttributeExpression extends Expression
{

    private static final String RCSRevision = "$Revision: 1.5 $";
    private static final String RCSName     = "$Name:  $";

    protected String _attribute;
    protected RenameRelation _tableInstance;
    protected Expression _underlyingExpr;

    protected AttributeExpression()
    {
    }
    /**
    @param tableInstance
    @param column
    @param attribute
    @roseuid 3B2787BD02B1
     */

    public AttributeExpression(RenameRelation tableInstance, String attribute)
    {
        setTableInstance (tableInstance);
        setAttribute(attribute);
    }

    synchronized Object clone(Map clonedExprs) throws CloneNotSupportedException
    {
        //Trace.enter(this, "clone(Map clonedExprs)");
        AttributeExpression retVal = (AttributeExpression)super.clone(clonedExprs);
        Expression clonedExpr = null;

        retVal.setAttribute(getAttribute());

        if (_tableInstance != null) {
            clonedExpr = (Expression)clonedExprs.get(_tableInstance);
            if (null == clonedExpr) {
                clonedExpr = (Expression)_tableInstance.clone(clonedExprs);
            }
            retVal.setTableInstance((RenameRelation)clonedExpr);
        }

        if (_underlyingExpr != null) {
            clonedExpr = (Expression)clonedExprs.get(_underlyingExpr);
            if (null == clonedExpr) {
                clonedExpr = (Expression)_underlyingExpr.clone(clonedExprs);
            }
            retVal.setUnderlyingExpr(clonedExpr);
        }

        clonedExprs.put(this, retVal);
        //Trace.exit(this, "clone(Map clonedExprs)");
        return retVal;
    }

    public boolean equals (Object o)
    {
        if (o == this)
            return true;
        if (!(o instanceof AttributeExpression))
            return false;
        AttributeExpression p = (AttributeExpression) o;

        RenameRelation pTi = p.getTableInstance();

        if (null == getTableInstance() && null == pTi) {
        }
        else  if (null == getTableInstance() || null == pTi) {
            return false;
        }
        else if  ( ! getTableInstance().equals(p.getTableInstance()) ) {
            return false;
        }

        if ( ! getName().equalsIgnoreCase(p.getName()) )
            return false;
        return true;
    }

    public int hashCode()
    {
        if (null != _tableInstance) {
            String str = _tableInstance.getName()+ "." + _attribute;
            return str.hashCode();
        }
        else {
            return _attribute.hashCode();
        }
    }

    /**
    Access method for the _attribute property.

    @return   the current value of the _attribute property
     */
    public String getAttribute()
    {
        return _attribute;
    }

    /**
    Sets the value of the _attribute property.

    @param aAttribute the new value of the _attribute property
     */
    public void setAttribute(String aAttribute)
    {
        _attribute = aAttribute;
    }

    /**
    Access method for the _tableInstance property.

    @return   the current value of the _tableInstance property
     */
    public RenameRelation getTableInstance()
    {
        return _tableInstance;
    }

    public List getOperands()
    {
        return null;
    }

    /**
    Sets the value of the _tableInstance property.

    @param aTableInstance the new value of the _tableInstance property
     */
    public void setTableInstance(RenameRelation aTableInstance)
    {
        _tableInstance = aTableInstance;
    }

    public String getName()
    {
        return _attribute;
    }
    /**
    Access method for the _underlyinExpr property.

    @return   the current value of the _underlyinExpr property
     */
    public Expression getUnderlyinExpr()
    {
        return _underlyingExpr;
    }

    /**
    Sets the value of the _underlyinExpr property.

    @param aUnderlyinExpr the new value of the _underlyinExpr property
     */
    public void setUnderlyingExpr(Expression aUnderlyinExpr)
    {
        _underlyingExpr = aUnderlyinExpr;
        while (_underlyingExpr instanceof AttributeExpression
                	|| _underlyingExpr instanceof RenameItem) {
            if (_underlyingExpr instanceof AttributeExpression)
                _underlyingExpr = ((AttributeExpression)_underlyingExpr).getUnderlyinExpr();
            else
                _underlyingExpr = ((RenameItem)_underlyingExpr).getOperand();
        }
    }

    public String pprint ()
    {
        StringBuffer retVal = new StringBuffer();
        if (null != _tableInstance) {
            retVal.append(getTableInstance().getName());
            retVal.append(".");
        }
        retVal.append(_attribute);
        return retVal.toString();
    }

    public SqlExpression accept (GenSqlVisitor visitor) throws SqlWrapperException
    {
        return visitor.visit(this);
    }

    public void accept (AlgebraVisitor visitor) throws SqlWrapperException
    {
        visitor.visit(this);
    }
    
	/**
	 * The semantics chosen for equals of algebra nodes is "the same algebra operator
	 * hierarchy referring to the same database objects".
	 * @param o the expression to be compared with this.
	 */
	public boolean deepEquals(Object o)
	{
		if (o instanceof AttributeExpression)
		{
			AttributeExpression cast = (AttributeExpression)o;
			return	 _attribute.equalsIgnoreCase(cast.getAttribute())
						&& (_tableInstance.getName().equalsIgnoreCase(cast.getTableInstance().getName())
							|| _tableInstance.deepEquals(cast.getTableInstance()));	// if alias name is <> check real relation
						 // NOTE: assuming underlying expression is browsed somewhere else.
		}
		return false;
	}
}
