/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.algebra;

import java.util.List;

import org.xquark.extractor.common.SqlWrapperException;

/**
 * Browse an algebra tree from leaf to root stopping at "end node" applying a
 * subvisitor to {@link org.xquark.extractor.algebra.Relation} nodes.
 */
public class BottomUpTractor extends DefaultAlgebraVisitor {

	private static final String RCSRevision = "$Revision: 1.4 $";
	private static final String RCSName = "$Name:  $";

	/* process of GoUp  end up at "_endNode"*/
	private Expression _endNode = null;

	private AlgebraVisitor _visitor = null;

	public BottomUpTractor() {
	}

	public BottomUpTractor(AlgebraVisitor visitor) {
		setVisitor(visitor);
	}

	public BottomUpTractor(AlgebraVisitor visitor, Expression endNode) {
		setEndNode(endNode);
		setVisitor(visitor);
	}

	public void setEndNode(Expression endNode) {
		_endNode = endNode;
	}

	public Expression getEndNode() {
		return _endNode;
	}

	public void setVisitor(AlgebraVisitor visitor) {
		_visitor = visitor;
	}

	public AlgebraVisitor getVisitor() {
		return _visitor;
	}

	public void visit(Expression arg) throws SqlWrapperException {
		//Trace.enter(this, "visit(Expression arg)", arg.pprint());

		Expression father = arg.getFather();
		if (father != null && getEndNode() != arg) {
			father.accept(this);
		}

		//Trace.exit(this,"visit(Expression arg)",arg.pprint());
	}

	public void visit(UnaryAlgebra arg) throws SqlWrapperException {
		//Trace.enter(this,"UnaryAlgebra(Join arg)",arg.pprint());

		List list = arg.getParameterList();

		if (null != list) {
			treat(list);
		}

		visitFather(arg);

		//Trace.exit(this, "UnaryAlgebra(Join arg)", arg.pprint());
	}

	public void visit(Join arg) throws SqlWrapperException {
		//Trace.enter(this, "visit(Join arg)", arg.pprint());

		List list = arg.getPredicateList();
		treat(list);
		visitFather(arg);

		//Trace.exit(this, "visit(Join arg)", arg.pprint());
	}

	public void visit(BinOpOuterJoin arg) throws SqlWrapperException {
		//Trace.enter(this, "visit(BinOpOuterJoin arg)", arg.pprint());

		List list = arg.getPredicateList();
		treat(list);
		visitFather(arg);

		//Trace.exit(this, "visit(BinOpOuterJoin arg)", arg.pprint());
	}

	protected void treat(List list) {
		//Trace.enter(this, "treat(List list)");
		Expression expr = null;
		if (null != list && !list.isEmpty()) {
			for (int i = 0; i < list.size(); i++) {
				expr = (Expression) list.get(i);
				expr.accept(_visitor);
			}
		}
		//Trace.exit(this, "treat(List list)");
	}

	protected void visitFather(Expression arg) {
		//		Trace.enter(this, "visitFather(Expression arg)");
		Expression father = arg.getFather();
		if (father != null && getEndNode() != arg) {
			father.accept(this);
		}
		//		Trace.exit(this, "visitFather(Expression arg)");
	}
}
