/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.algebra;

import org.xquark.extractor.common.SqlWrapperException;
import org.xquark.extractor.sql.SqlExpression;
import org.xquark.extractor.xfunctions.*;

public abstract class DefaultGenSqlVisitor implements GenSqlVisitor {

    private static final String RCSRevision = "$Revision: 1.4 $";
    private static final String RCSName     = "$Name:  $";



    public SqlExpression visit(Expression arg) throws SqlWrapperException {
        return visit(arg);
    }

    public SqlExpression visit(Attribute arg) throws SqlWrapperException {
        return visit((Expression)arg);
    }

    public SqlExpression visit(AttributeExpression arg) throws SqlWrapperException {
        return visit((Expression)arg);
    }

    public SqlExpression visit(BinaryOperator arg) throws SqlWrapperException {
        return visit((Expression)arg);
    }

    public SqlExpression visit(BinaryAtomicOp arg) throws SqlWrapperException {
        return visit((BinaryOperator)arg);
    }

    public SqlExpression visit(BinaryAlgebra arg) throws SqlWrapperException {
        return visit((BinaryOperator)arg);
    }

    public SqlExpression visit(BinOpArithmetic arg) throws SqlWrapperException {
        return visit((BinaryAtomicOp)arg);
    }

    public SqlExpression visit(BinOpBoolean arg) throws SqlWrapperException {
        return visit((BinaryAtomicOp)arg);
    }

    public SqlExpression visit(BinOpCompare arg) throws SqlWrapperException {
        return visit((BinaryAtomicOp)arg);
    }

    public SqlExpression visit(BinOpCompareAny arg) throws SqlWrapperException {
        return visit((BinaryAtomicOp)arg);
    }

    public SqlExpression visit(BinOpDifference arg) throws SqlWrapperException {
        return visit((BinaryAlgebra)arg);
    }

    public SqlExpression visit(BinOpIntersect arg) throws SqlWrapperException {
        return visit((BinaryAlgebra)arg);
    }

	public SqlExpression visit(BinOpMerge arg) throws SqlWrapperException {
		return visit((BinaryAlgebra)arg);
	}

    public SqlExpression visit(BinOpOuterJoin arg) throws SqlWrapperException {
        return visit((BinaryOperator)arg);
    }

    public SqlExpression visit(BinOpUnion arg) throws SqlWrapperException {
        return visit((BinaryAlgebra)arg);
    }

    public SqlExpression visit(DummyTable arg) throws SqlWrapperException {
        return visit((Expression)arg);
    }

    public SqlExpression visit(ExternalVariable arg) throws SqlWrapperException {
        return visit((Literal)arg);
    }

    public SqlExpression visit(FunAggregate arg) throws SqlWrapperException {
        return visit((Expression)arg);
    }

    public SqlExpression visit(IfThenElse arg) throws SqlWrapperException {
        return visit((Expression)arg);
    }

    public SqlExpression visit(Join arg) throws SqlWrapperException {
        return visit((Expression)arg);
    }

    public SqlExpression visit(Literal arg) throws SqlWrapperException {
        return visit((Expression)arg);
    }

    public SqlExpression visit(LitBoolean arg) throws SqlWrapperException {
        return visit((Literal)arg);
    }

    public SqlExpression visit(LitDate arg) throws SqlWrapperException {
        return visit((Literal)arg);
    }

    public SqlExpression visit(LitDecimal arg) throws SqlWrapperException{
        return visit((Literal)arg);
    }

    public SqlExpression visit(LitDouble arg) throws SqlWrapperException {
        return visit((Literal)arg);
    }

    public SqlExpression visit(LitFloat arg) throws SqlWrapperException {
        return visit((Literal)arg);
    }

    public SqlExpression visit(LitInteger arg) throws SqlWrapperException {
        return visit((Literal)arg);
    }

    public SqlExpression visit(LitNull arg) throws SqlWrapperException{
        return visit((Literal)arg);
    }

    public SqlExpression visit(LitString arg) throws SqlWrapperException {
        return visit((Literal)arg);
    }


    public SqlExpression visit(OuterJoinPredicate arg) throws SqlWrapperException {
        return visit((Expression)arg);
    }

    public SqlExpression visit(RenameItem arg) throws SqlWrapperException {
        return visit((UnaryAtomicOp)arg);
    }

    public SqlExpression visit(RenameRelation arg) throws SqlWrapperException {
        return visit((UnaryAlgebra)arg);
    }

    public SqlExpression visit(SortSpecification arg) throws SqlWrapperException {
        return visit((Expression)arg);
    }

    public SqlExpression visit(Table arg) throws SqlWrapperException {
        return visit((Expression)arg);
    }

    public SqlExpression visit(TupleExpression arg) throws SqlWrapperException {
        return visit((Expression)arg);
    }

    public SqlExpression visit(TempValue arg) throws SqlWrapperException {
        return visit((UnaryOperator)arg);
    }

    public SqlExpression visit(ConstantTempTable arg) throws SqlWrapperException {
        return visit((Table)arg);
    }

    public SqlExpression visit(UnaryOperator arg) throws SqlWrapperException {
        return visit((Expression)arg);
    }

    public SqlExpression visit(UnaryAlgebra arg) throws SqlWrapperException {
        return visit((UnaryOperator)arg);
    }

    public SqlExpression visit(UnaryAtomicOp arg) throws SqlWrapperException {
        return visit((UnaryOperator)arg);
    }

    public SqlExpression visit(UnOpAggregate arg) throws SqlWrapperException {
        return visit((UnOpProject)arg);
    }

    public SqlExpression visit(UnOpExists arg) throws SqlWrapperException {
        return visit((UnaryAtomicOp)arg);
    }

    public SqlExpression visit(UnOpGroup arg) throws SqlWrapperException {
        return visit((UnaryAlgebra)arg);
    }

    public SqlExpression visit(UnOpIsNull arg) throws SqlWrapperException {
        return visit((UnaryAtomicOp)arg);
    }

    public SqlExpression visit(UnOpMinus arg) throws SqlWrapperException {
        return visit((UnaryAtomicOp)arg);
    }

    public SqlExpression visit(UnOpNot arg) throws SqlWrapperException {
        return visit((UnaryAtomicOp)arg);
    }

    public SqlExpression visit(UnOpProject arg) throws SqlWrapperException {
        return visit((UnaryAlgebra)arg);
    }

	public SqlExpression visit(TempTable arg) throws SqlWrapperException {
		return visit((UnaryAlgebra)arg);
	}

    public SqlExpression visit(UnOpRestrict arg) throws SqlWrapperException {
        return visit((UnaryAlgebra)arg);
    }

    public SqlExpression visit(UnOpSort arg) throws SqlWrapperException {
        return visit((UnaryAlgebra)arg);
    }

    public SqlExpression visit(XFunction arg) throws SqlWrapperException {
        return visit((Expression)arg);
    }

    public SqlExpression visit(Xfunction_Boolean arg) throws SqlWrapperException {
        return visit((XFunction)arg);
    }

    public SqlExpression visit(Xfunction_Date arg) throws SqlWrapperException {
        return visit((XFunction)arg);
    }

    public SqlExpression visit(Xfunction_DateTime arg) throws SqlWrapperException {
        return visit((XFunction)arg);
    }

    public SqlExpression visit(Xfunction_Integer arg) throws SqlWrapperException {
        return visit((XFunction)arg);
    }

    public SqlExpression visit(Xfunction_String arg) throws SqlWrapperException {
        return visit((XFunction)arg);
    }

    public SqlExpression visit(AfTrim arg) throws SqlWrapperException {
        return visit((Xfunction_String)arg);
    }

    public SqlExpression visit(AfCast arg) throws SqlWrapperException {
        return visit((Expression)arg);
    }

    public SqlExpression visit(XfCast arg) throws SqlWrapperException {
        return visit((Expression)arg);
    }

    public SqlExpression visit(XfAbs arg) throws SqlWrapperException{
        return visit((XFunction)arg);
    }

    public SqlExpression visit(XfCeiling arg) throws SqlWrapperException {
        return visit((Xfunction_Integer)arg);
    }

    public SqlExpression visit(XfConcat arg) throws SqlWrapperException {
        return visit((Xfunction_String)arg);
    }

    public SqlExpression visit(XfContains arg) throws SqlWrapperException {
        return visit((Xfunction_Boolean)arg);
    }

    public SqlExpression visit(XfCurrentDateTime arg) throws SqlWrapperException {
        return visit((XFunction)arg);
    }

    public SqlExpression visit(XfGetCenturyFromDate arg) throws SqlWrapperException {
        return visit((Xfunction_Integer)arg);
    }

    public SqlExpression visit(XfGetHourFromTime arg) throws SqlWrapperException {
        return visit((Xfunction_Integer)arg);
    }

    public SqlExpression visit(XfGetMinutesFromtime arg) throws SqlWrapperException {
        return visit((Xfunction_Integer)arg);
    }

    public SqlExpression visit(XfGetSecondsFromtime arg) throws SqlWrapperException {
        return visit((Xfunction_Integer)arg);
    }

    public SqlExpression visit(XfDate arg) throws SqlWrapperException{
        return visit((XFunction)arg);
    }

    public SqlExpression visit(XfDateTime arg) throws SqlWrapperException{
        return visit((XFunction)arg);
    }

    public SqlExpression visit(XfEndsWith arg) throws SqlWrapperException {
        return visit((Xfunction_Boolean)arg);
    }

    public SqlExpression visit(XfFloor arg) throws SqlWrapperException {
        return visit((Xfunction_Integer)arg);
    }

    public SqlExpression visit(XfLowerCase arg) throws SqlWrapperException {
        return visit((Xfunction_String)arg);
    }

    public SqlExpression visit(XfRound arg) throws SqlWrapperException {
        return visit((Xfunction_Integer)arg);
    }

    public SqlExpression visit(XfStartsWith arg) throws SqlWrapperException {
        return visit((Xfunction_Boolean)arg);
    }

    public SqlExpression visit(XfStringLength arg) throws SqlWrapperException {
        return visit((Xfunction_Integer)arg);
    }

    public SqlExpression visit(XfSubString arg) throws SqlWrapperException {
        return visit((Xfunction_String)arg);
    }

    public SqlExpression visit(XfTime arg) throws SqlWrapperException{
        return visit((XFunction)arg);
    }

    public SqlExpression visit(XfUpperCase arg) throws SqlWrapperException {
        return visit((Xfunction_String)arg);
    }

}
