/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.algebra;

import java.util.List;
import java.util.Map;

import org.xquark.extractor.common.SqlWrapperException;
import org.xquark.extractor.sql.SqlExpression;

public final class LitList extends Expression
{
	private static final String RCSRevision = "$Revision: 1.4 $";
	private static final String RCSName = "$Name:  $";

	private List _litList = null;


	public LitList(List litList)  {
		setLitList(litList);
	}

	synchronized Object clone(Map clonedExprs) throws CloneNotSupportedException
	{
		//Trace.enter(this, "clone(Map clonedExprs)");

		LitList retVal = (LitList)super.clone(clonedExprs);

		retVal.setLitList( AlgebraTools.clone(getLitList(),clonedExprs) );

		clonedExprs.put(this, retVal);
		//Trace.exit(this, "clone(Map clonedExprs)");
		return retVal;
	}


	public List getLitList()  {
		return _litList;
	}

	public void setLitList(List litList) {
		_litList = litList;
	}

	public String pprint() {
		StringBuffer retVal = new StringBuffer();
		retVal.append('(');

		if (null != _litList && !_litList.isEmpty()){
			for (int i = 0; i < _litList.size()-1; i++) {
				retVal.append(((Expression)_litList.get(i)).pprint());
				retVal.append(", ");
			}
			retVal.append(((Expression)_litList.get( _litList.size()-1)).pprint());
		}
		retVal.append(')');
		return retVal.toString();
	}

	public List getOperands() {
		return null ;
	}

	public SqlExpression accept (GenSqlVisitor visitor) throws SqlWrapperException
	{
		return visitor.visit(this);
	}

	public void accept (AlgebraVisitor visitor) throws SqlWrapperException
	{
		visitor.visit(this);
	}
	
	/**
	 * @see Expression#deepEquals(Object)
	 */
	public boolean deepEquals(Object o)
	{
		if (o instanceof LitList)
			return AlgebraTools.areExprListEquivalent(_litList, ((LitList)o).getLitList());
		return false;
	}
}
