/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.algebra;

import java.io.DataOutputStream;
import java.util.List;

import org.xquark.extractor.metadata.ExtractorMappingInfo;
import org.xquark.jdbc.typing.TypeMap;
import org.xquark.schema.SimpleType;
import org.xquark.xquery.parser.QName;
import org.xquark.xquery.parser.XQueryException;
import org.xquark.xquery.parser.XQueryExpression;

public final class  MapItem implements Cloneable {

    private static final String RCSRevision = "$Revision: 1.20 $";
    private static final String RCSName     = "$Name:  $";


    private String       _path              = null;
    private QName        _qName             = null;
    private String       _itemName          = null;

    /* corresponding Expression in (most likely) UnOpProject*/
    private Expression   _relatedExpression = null;
    
    private ExtractorMappingInfo  _mappingInfo = null;
    
    /** Path list of children */
    private List         _children          = null;

    /** Result of the {@link Mapper#optimize()} method are stored here by
     * {@link org.xquark.extractor.runtime.SynchronizedResultSet}. These are the map
     * items corresponding to the paths contained in @link #_children 
     */
    private List        _childChildItems  = null;
    
	/**
	 * in the case of sorted outer union, this BitMap indicate which parts
     * this MapItem participates
     */
    private long         _partOf              = 0;


    public MapItem()
    {}

    public Object clone () throws CloneNotSupportedException
    {
        MapItem retVal = (MapItem)super.clone();
        retVal.setPath((getPath() == null) ? null : new String(getPath()) );
        // change LARS 02/03/04
        //retVal.setQName((getQName() == null) ? null : (QName)getQName().clone(typeVisitor));
		try  {
			retVal.setQName((getQName() == null) ? null : new QName(getQName().getNameSpace(),getQName().getPrefixName(),getQName().getLocalName(),null, null));
		} catch (XQueryException xqe) {
			throw new CloneNotSupportedException("could not clone : " + retVal.getQName());
		}
        retVal.setItemName((getItemName() == null) ? null : new String(getItemName()) );
        retVal.setRelatedExpression((getRelatedExpression() == null) ? null : (Expression)getRelatedExpression().clone());
        retVal.setChildren(AlgebraTools.clone(getChildren()));
        return  retVal;
    }

    public boolean equals(Object o) {
        if (o == this)
            return true;
        if (!(o instanceof MapItem))
            return false;

        MapItem p = (MapItem) o;
        if ( ! getPath().equals(p.getPath()))
            return false;
        return true;
    }

    public MapItem (String path, QName qName,String itemName,List children)
    {
        _path           = path;
        _qName    =     qName;
        _itemName       = itemName;
        _children       = children;

    }

    public String getPath()
    {
        return _path;
    }

    public void setPath (String path)
    {
        _path = path;
    }

    public void setQName (QName qName)
    {
        _qName = qName;
    }

    public QName getQName() {
        return _qName;
    }

    public String getElementName()  {
		if (null!=_qName) {
	        return _qName.getLocalName();
		}
		else {
			return null;
		}
    }

    public String getNamespace() {
		if (null!=_qName) {
	        return _qName.getNameSpace();
		}
		else {
			return null;
		}
    }


    public String getItemName()
    {
        return _itemName;
    }

    public void setItemName (String itemName)
    {
        _itemName = itemName;
    }

    public Expression getRelatedExpression()
    {
        return _relatedExpression;
    }

    public void setRelatedExpression (Expression relatedExpression)
    {
        _relatedExpression = relatedExpression;
    }

    public boolean isLeafPath()
    {
        return null == _children;
    }

    public List getChildren()
    {
        return _children;
    }

    public void setChildren(List children)
    {
        _children = children;
    }

    public List getChildItemList()
    {
        return _childChildItems;
    }

    public void setChildItemLis(List childChildItemLis)
    {
        _childChildItems = childChildItemLis;
    }

    public String pprint ()
    {
        char[] spaces =
            (new String("                                            ")).toCharArray();
        StringBuffer retVal = new StringBuffer ();

        retVal.append( _path);
        if (_path.length()< 40) {
            retVal.append(spaces, 0, 40 - _path.length());
        }
        retVal.append("\t") ;

        if (null != _qName) {
            String qname = _qName.toString();
            retVal.append( qname);
            if (qname.length()< 40) {
               retVal.append(spaces, 0,40- qname.length());
            }
        }
        else {
            retVal.append( spaces,0,40);
        }
        retVal.append("\t") ;

        if (null != _itemName) {
            retVal.append( _itemName);
            if (_itemName.length()< 40) {
               retVal.append(spaces, 0, 40 - _itemName.length());
            }
        }
        else {
            retVal.append( spaces,0,40);
        }
        retVal.append("\t") ;

        retVal.append(Long.toBinaryString(_partOf));
        retVal.append("\t") ;

        if (null!=_children) {
            retVal.append ("[");
            for (int i = 0; i < _children.size(); i++) {
                retVal.append(_children.get(i));
                retVal.append(",\t");
            }
            retVal.append ("]");
        }
        else {
            retVal.append("[null]");
        }

        return retVal.toString();
    };

    public void write(DataOutputStream dos)  throws java.io.IOException
    {
        //Trace.enter(this,"write(DataOutputStream dos)");

        String newLine = System.getProperty("line.separator");

        dos.writeBytes("<MapItem>");
        dos.writeBytes(newLine);
            if (null != _path) {
                dos.writeBytes("<Path>");
                dos.writeBytes(_path);
                dos.writeBytes("</Path>");
                dos.writeBytes(newLine);
            }

            if (null != _qName) {
                dos.writeBytes("<ElementName>");
                dos.writeBytes(_qName.toString());
                dos.writeBytes("</ElementName>");
                dos.writeBytes(newLine);
            }

            if (null != _itemName) {
                dos.writeBytes("<ItemName>");
                dos.writeBytes(_itemName);
                dos.writeBytes("</ItemName>");
                dos.writeBytes(newLine);
            }

            if (null != _children) {
                dos.writeBytes("<Children>");
                dos.writeBytes(newLine);
                for (int i = 0; i < _children.size(); i++) {
                    dos.writeBytes("<Child>");
                    dos.writeBytes((String)_children.get(i));
                    dos.writeBytes("</Child>");
                    dos.writeBytes(newLine);
                }
                dos.writeBytes("</Children>");
                dos.writeBytes(newLine);
            }

        dos.writeBytes("</MapItem>");
        dos.writeBytes(newLine);

        //Trace.exit(this,"write(DataOutputStream dos)");
    }



    private String fillUp (String str, int length)
    {
        return null;
    }

    public boolean isPartOf(long parts)
    {
        return  (_partOf & parts) > 0 ;
    }
    public void  participate (long parts)
    {
        _partOf = _partOf | parts;
    }
    public long getPart()
    {
        return _partOf;
    }
    public void  setPart(long parts)
    {
        _partOf = parts;
    }

    /**
     * Returns and generate if necessary a MappingInfo object.
     * NOTE: Could retrieve the ExtractorMappingInfo generated for metadata.
     */
	public ExtractorMappingInfo getMappingInfo() {
		return _mappingInfo;
	}
    public ExtractorMappingInfo getMappingInfo(TypeMap typeMap) throws XQueryException {
        if (_mappingInfo == null) {
            Expression algebraExpr = _relatedExpression;
            // First try to find column mapping info (with column metadata) in
            // order to find precise info to get data from JDBC
            if (algebraExpr instanceof RenameItem)
                algebraExpr = ((RenameItem)algebraExpr).getOperand();
            
            if (algebraExpr instanceof AttributeExpression)
                algebraExpr = ((AttributeExpression)algebraExpr).getUnderlyinExpr();
            
            if (algebraExpr instanceof Attribute) {
                _mappingInfo = ((Attribute)algebraExpr).getAttribute().getMappingInfo();
            }
            else {	// Normally computed attributes : create dummy mapping info (could do better ?)
	            XQueryExpression xqExpr = _relatedExpression.getOrginalXExpr();
	            if (xqExpr != null) {	// for pseudo-id
		            SimpleType sType = xqExpr.getQType().getSimpleType();
		            if (sType != null)
	                    _mappingInfo = new ExtractorMappingInfo(sType, typeMap);
	            }
            }
        }
        if (_mappingInfo != null)
        	_mappingInfo.checkForExtraction();
        return _mappingInfo;
    }

}

