/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.algebra;

import java.io.DataOutputStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.xquark.extractor.common.Debug;
import org.xquark.xquery.parser.QName;
import org.xquark.xquery.parser.XQueryExpression;
import org.xquark.xquery.typing.QType;
import org.xquark.xquery.typing.QTypeElement;
import org.xquark.xquery.typing.QTypeSequence;

public final class Mapper implements Cloneable {
	private static final String RCSRevision = "$Revision: 1.6 $";
	private static final String RCSName = "$Name:  $";

	List _mapItemList = new ArrayList(); /* a list of mapitem*/

	List _keyNameList = null; /* dduit de _keyList, contient les noms des keys, lazy build*/
	List _sortAttributeList; /* other attributes to be sorted */

	Expression _partAttribute = null;
	//    List        _numberAttributeList = null;
	long _commonPartBitMap = -1; /* */

	List _idList = null;
	List _mapItemAddedForId = null;

	Numbering _numbering;

	boolean _optimized = false;

	public Mapper() {

	}

	public Object clone() throws CloneNotSupportedException {
		Mapper retVal = (Mapper) super.clone();

		/* note : here a shallow clone is of arrays is neccessory*/
		if (null != _mapItemList) {
			retVal._mapItemList = new ArrayList();
			retVal._mapItemList.addAll(_mapItemList);
		}
		if (null != _idList) {
			retVal._idList = new ArrayList();
			retVal._idList.addAll(_idList);
		}
		if (null != _mapItemAddedForId) {
			retVal._mapItemAddedForId = new ArrayList();
			retVal._mapItemAddedForId.addAll(_mapItemAddedForId);
		}
		//        if (null !=_sortAttributeList) {
		//            retVal._sortAttributeList = new ArrayList();
		//            retVal._sortAttributeList.addAll(_sortAttributeList);
		//        }

		return retVal;
	}

	/**
	 * Used for all kind of data to retrieve.
	 * @param itemList
	 */
	public void createMap(List itemList) {
		//Trace.enter(this, "createMap(List itemList)");

		if (itemList != null) {
			Iterator iter = itemList.iterator();
			Expression item;
			while (iter.hasNext()) {
				item = (Expression) iter.next();
				createMapItem(item);
			}
		}

		//Trace.message(this, "createMap(List itemList)", this);

		//Trace.exit(this, "createMap(List itemList)");
	}

	protected MapItem createMapItem(Expression expression) {
		//Trace.enter(this, "createMapItem(Expression expression)", expression);

		MapItem retVal = null;
		if (expression instanceof TupleExpression)
			retVal = createMapItem((TupleExpression) expression);
		else if (expression instanceof AttributeExpression)
			retVal = createMapItem((AttributeExpression) expression);
		else if (expression instanceof RenameItem)
			retVal = createMapItem((RenameItem) expression);
		else {
			MapItem mapItem = new MapItem();

			XQueryExpression xExpr = expression.getOrginalXExpr();
			mapItem.setPath(xExpr.toString());

			mapItem.setQName(null);

			mapItem.setItemName(expression.getName());
			mapItem.setRelatedExpression(expression);

			addMapItem(mapItem);

			retVal = mapItem;
		}
		//Trace.exit(this, "createMapItem(Expression expression)");
		return retVal;
	}

	protected MapItem createMapItem(TupleExpression tuple) {
		//Trace.enter(this, "createMapForItem(TupleExpression)", tuple.pprint());

		MapItem mapItem = new MapItem();

		XQueryExpression xExpr = tuple.getOrginalXExpr();
		mapItem.setPath(xExpr.toString());

		/* find out elementName (tag)*/
		QType qtype = xExpr.getQType();
		if (qtype instanceof QTypeElement) {
			QTypeElement qte = (QTypeElement) qtype;
			mapItem.setQName(((QName) qte.getName()));
		} else if (qtype instanceof QTypeSequence) {
			mapItem.setQName(null);
		} else {
			Debug.assertTrue(false, "NYI!!");
		}

		mapItem.setRelatedExpression(tuple);

		List children = new ArrayList();
		List subExprs = tuple.getItemList();
		for (int i = 0; i < subExprs.size(); i++) {
			xExpr = ((Expression) subExprs.get(i)).getOrginalXExpr();
			children.add(xExpr.toString());
		}
		mapItem.setChildren(children);
		addMapItem(mapItem);

		//Trace.exit(this, "createMapForItem(TupleExpression)", tuple.pprint());
		return mapItem;

	}

	//    public List getKeyList()
	//    {
	//        Trace.enter(this, "getKeyList()");
	//        Trace.exit(this, "getKeyList()");
	//        return _keyList;
	//    }

	public List getIdentifierList() {
		//Trace.enter(this, "getIdentifierList()");
		//Trace.exit(this, "getIdentifierList()");
		return _idList;
	}

	public List getMapItemsForId() {
		//Trace.enter(this, "getMapItemsForId()");
		//Trace.exit(this, "getMapItemsForId()");
		return _mapItemAddedForId;
	}

	public void setPartAttribute(Expression part) {
		_partAttribute = part;
	}

	public String getPartAttributeName() {
		return _partAttribute.getName();
	}

	public Expression getPartAttribute() {
		return _partAttribute;
	}

	public int[] getEliminationGuide() {
		return _numbering.getEliminationGuide();
	}

	public boolean isMultiPart() {
		return _partAttribute != null;
	}

	public List getCommonIdentifierAttributeNames() {
		//Trace.enter(this, "getCommonIdentifierAttributeNamese()");
		List retVal = null;
		if (null != _partAttribute) {
			List commonIdentifierMapItems = getCommonIdentifierMapItems();
			Debug.assertTrue(null != commonIdentifierMapItems, "null!=commonIdentifierMapItems");
			retVal = new ArrayList();
			for (int i = 0; i < commonIdentifierMapItems.size(); i++) {
				MapItem mapItem = (MapItem) commonIdentifierMapItems.get(i);
				if (mapItem.isLeafPath()) {
					retVal.add(mapItem.getItemName());
				} else {
					List chidren = getChildren(mapItem);
					for (int j = 0; j < chidren.size(); j++) {
						MapItem child = (MapItem) chidren.get(j);
						retVal.add(child.getItemName());
					}

				}
			}
		}
		//Trace.exit(this, "getCommonIdentifierAttributeNames()");
		return retVal;
	}

	public List getCommonIdentifierAttributes() {
		//Trace.enter(this, "getCommonIdentifierAttributes()");
		List retVal = null;
		if (null != _partAttribute) {
			List commonIdentifierMapItems = getCommonIdentifierMapItems();
			Debug.assertTrue(null != commonIdentifierMapItems, "null!=commonIdentifierMapItems");
			retVal = new ArrayList();
			for (int i = 0; i < commonIdentifierMapItems.size(); i++) {
				MapItem mapItem = (MapItem) commonIdentifierMapItems.get(i);
				if (mapItem.isLeafPath()) {
					retVal.add(mapItem.getRelatedExpression());
				} else {
					List chidren = getChildren(mapItem);
					for (int j = 0; j < chidren.size(); j++) {
						MapItem child = (MapItem) chidren.get(j);
						retVal.add(child.getRelatedExpression());
					}

				}
			}
		}
		//Trace.exit(this, "getCommonIdentifierAttributes()");
		return retVal;
	}

	private List getCommonIdentifierMapItems() {
		//Trace.enter(this, "getCommonIdentifierMapItems()");
		List retVal = null;
		if (null != _partAttribute) {
			retVal = new ArrayList();
			for (int i = 0; i < _idList.size(); i++) {
				MapItem mapItem = (MapItem) _idList.get(i);
				long part = mapItem.getPart();
				if (isCommon(part)) {
					retVal.add(mapItem);
				}
			}
		}

		//Trace.exit(this, "getCommonIdentifierMapItems()");
		return retVal;
	}

	private boolean isCommon(long part) {
		//Trace.enter(this, "isCommon(long part)");

		boolean retVal = false;
		int count = 0;
		long ruler = 1;
		long tmp = 0;
		for (int i = 0; i < 64; i++) {
			tmp = (part >> i);
			tmp = tmp & ruler;
			count += tmp;
		}
		retVal = count > 1;

		//Trace.exit(this, "isCommon(long part)");
		return retVal;
	}

	//    private boolean isCommon(long part)
	//    {
	//        Trace.enter(this,"isCommon(long part)");
	//
	//        boolean retVal = false;
	//        if (-1 == _commonPartBitMap) {
	//            _commonPartBitMap = getCommonPartNumber();
	//        }
	//        retVal = _commonPartBitMap == part;
	//
	//        Trace.exit(this,"isCommon(long part)");
	//        return retVal;
	//    }

	public long getCommonPartNumber() {
		//Trace.enter(this, "getCommonPartNumber()");
		long retVal = -1;
		if (null != _partAttribute) {
			long maxPart = 0;
			for (int i = 0; i < _idList.size(); i++) {
				long part = ((MapItem) _idList.get(i)).getPart();
				if (maxPart < part) {
					maxPart = part;
				}
			}
			retVal = maxPart;
		}
		//Trace.exit(this, "getCommonPartNumber()");
		return retVal;
	}

	public List getNonCommonIdentifierAttributeNames() {
		//Trace.enter(this, "getNonCommonIdentifierAttributeNames()");
		List retVal = null;
		if (null != _partAttribute) {
			List commonNonIdentifierMapItems = getNonCommonIdentifierMapItems();
			Debug.assertTrue(null != commonNonIdentifierMapItems, "null!=commonNonIdentifierMapItems");
			retVal = new ArrayList();
			for (int i = 0; i < commonNonIdentifierMapItems.size(); i++) {
				MapItem mapItem = (MapItem) commonNonIdentifierMapItems.get(i);
				if (mapItem.isLeafPath()) {
					retVal.add(mapItem.getItemName());
				} else {
					List chidren = getChildren(mapItem);
					for (int j = 0; j < chidren.size(); j++) {
						MapItem child = (MapItem) chidren.get(j);
						retVal.add(child.getItemName());
					}

				}
			}
		}

		//Trace.exit(this, "getNonCommonIdentifierAttributeNames()");
		return retVal;
	}

	public List getNonCommonIdentifierAttributes() {
		//Trace.enter(this, "getNonCommonIdentifierAttributes()");
		List retVal = null;
		if (null != _partAttribute) {
			List commonNonIdentifierMapItems = getNonCommonIdentifierMapItems();
			Debug.assertTrue(null != commonNonIdentifierMapItems, "null!=commonNonIdentifierMapItems");
			retVal = new ArrayList();
			for (int i = 0; i < commonNonIdentifierMapItems.size(); i++) {
				MapItem mapItem = (MapItem) commonNonIdentifierMapItems.get(i);
				if (mapItem.isLeafPath()) {
					retVal.add(mapItem.getRelatedExpression());
				} else {
					List chidren = getChildren(mapItem);
					for (int j = 0; j < chidren.size(); j++) {
						MapItem child = (MapItem) chidren.get(j);
						retVal.add(child.getRelatedExpression());
					}

				}
			}
		}

		//Trace.exit(this, "getNonCommonIdentifierAttributes()");
		return retVal;
	}

	private List getNonCommonIdentifierMapItems() {
		//Trace.enter(this, "getNonCommonIdentifierMapItems()");
		List retVal = null;
		if (null != _partAttribute) {
			retVal = new ArrayList();
			for (int i = 0; i < _idList.size(); i++) {
				MapItem mapItem = (MapItem) _idList.get(i);
				long part = mapItem.getPart();
				if (!isCommon(part)) {
					retVal.add(mapItem);
				}
			}
		}

		//Trace.exit(this, "getNonCommonIdentifierMapItems()");
		return retVal;
	}

	public List getAllIdentifierAttributeNames() {
		//Trace.enter(this, "getNonCommonIdentifierAttributeNames()");
		List retVal = null;

		if (null != _idList) {
			retVal = new ArrayList();
			for (int i = 0; i < _idList.size(); i++) {
				MapItem mapItem = (MapItem) _idList.get(i);
				if (mapItem.isLeafPath()) {
					retVal.add(mapItem.getItemName());
				} else {
					List chidren = getChildren(mapItem);
					for (int j = 0; j < chidren.size(); j++) {
						MapItem child = (MapItem) chidren.get(j);
						retVal.add(child.getItemName());
					}

				}
			}
		}

		//Trace.exit(this, "getNonCommonIdentifierAttributeNames()");
		return retVal;
	}

	public List getAllIdentifierAttributes() {
		//Trace.enter(this, "getAllIdentifierAttributes()");
		List retVal = null;

		if (null != _idList) {
			retVal = new ArrayList();
			for (int i = 0; i < _idList.size(); i++) {
				MapItem mapItem = (MapItem) _idList.get(i);
				if (mapItem.isLeafPath()) {
					retVal.add(mapItem.getRelatedExpression());
				} else {
					List chidren = getChildren(mapItem);
					for (int j = 0; j < chidren.size(); j++) {
						MapItem child = (MapItem) chidren.get(j);
						retVal.add(child.getRelatedExpression());
					}

				}
			}
		}

		//Trace.exit(this, "getAllIdentifierAttributes()");
		return retVal;
	}

	/**
	 * Used for primary keys.
	 * @param path
	 * @param attributeList
	 */
	public void addIdentifier(String path, List attributeList) {
		//Trace.enter(this, "addKey(String path, List attributeList)");

		if (null == _idList) {
			_idList = new ArrayList();
		}

		if (null == _mapItemAddedForId) {
			_mapItemAddedForId = new ArrayList();
		}

		if (1 < attributeList.size()) {

			List children = new ArrayList();
			for (int i = 0; i < attributeList.size(); i++) {
				Expression attrbute = (Expression) attributeList.get(i);
				MapItem mapItem = findMapItem(attrbute.getName());
				if (null == mapItem) {
					mapItem = new MapItem(path + "/" + Integer.toString(i), null, attrbute.getName(), null);
					mapItem.setRelatedExpression(attrbute);
					_mapItemList.add(mapItem);
					_mapItemAddedForId.add(mapItem);
				}
				children.add(mapItem.getPath());
			}
			MapItem mapItem = new MapItem(path, null, null, children);
			_idList.add(mapItem);
			_mapItemAddedForId.add(mapItem);
			_mapItemList.add(mapItem);
		} else {
			Expression attrbute = (Expression) attributeList.get(0);
			addIdentifier(path, attrbute);
		}

		//Trace.exit(this, "addIdentifier(String path, List attributeList)");
		return;
	}

	/**
	 * Used for pseudo-id (rowid).
	 * @param path
	 * @param attrbute
	 * @return
	 */
	public MapItem addIdentifier(String path, Expression attrbute) {
		//Trace.enter(this, "addIdentifier(String path, AttributeExpression attrbute)");
		MapItem retVal = null;

		if (null == _idList) {
			_idList = new ArrayList();
		}

		if (null == _mapItemAddedForId) {
			_mapItemAddedForId = new ArrayList();
		}

		MapItem mapItem = findMapItem(attrbute.getName());
		if (null == mapItem) {
			mapItem = new MapItem(path, null, attrbute.getName(), null);
			mapItem.setRelatedExpression(attrbute);
			_mapItemList.add(mapItem);
			_mapItemAddedForId.add(mapItem);
		}
		_idList.add(mapItem);
		retVal = mapItem;

		//Trace.exit(this, "addIdentifier(String path, AttributeExpression attrbute)");
		return retVal;
	}

	//    public void addSortAttribute(MapItem sortAttrMapItem)
	//    {
	//        Trace.enter(this,"addSortAttribute(MapItem sortAttrMapItem)");
	//
	//        if (null == _sortAttributeList) {
	//            _sortAttributeList = new ArrayList ();
	//        }
	//
	//        MapItem mapItem = map(sortAttrMapItem.getPath());
	//        if (null == mapItem){
	//            addMapItem (sortAttrMapItem);
	//            _sortAttributeList.add(sortAttrMapItem);
	//        }
	//        else{
	//            _sortAttributeList.add(sortAttrMapItem);
	//        }
	//
	//        Trace.exit(this,"addSortAttribute(MapItem sortAttrMapItem)");
	//        return ;
	//    }

	//    public List getSortAttributeList()
	//    {
	//        Trace.enter(this,"getSortAttributeList()");
	//        Trace.exit(this,"getSortAttributeList()");
	//        return _sortAttributeList;
	//    }

	public List getMapItemList() {
		//Trace.enter(this, "getMapItemList()");
		//Trace.exit(this, "getMapItemList()");
		return _mapItemList;
	}

	public void setMapItemList(List mapItemList) {
		//Trace.enter(this, "setMapItemList(List mapItemList)");
		//Trace.exit(this, "setMapItemList(List mapItemList)");
		_mapItemList = mapItemList;
	}

	public void setNumbering(Numbering numbering) {
		_numbering = numbering;
	}

	public List getSortList() {
		Debug.assertTrue(null != _numbering, "null != _numbering");
		return _numbering.getSortList();
	}

	public MapItem map(String path) {
		//Trace.enter(this, "MapItem map(String path)");

		MapItem retVal = null;
		for (int i = 0; i < _mapItemList.size(); i++) {
			MapItem mapItem = (MapItem) _mapItemList.get(i);
			if (path.equals(mapItem.getPath())) {
				retVal = mapItem;
				break;
			}
		}

		//Trace.exit(this, "MapItem map(String path)");
		return retVal;
	}

	public MapItem findMapItem(String itemName) {
		//Trace.enter(this, "findItem(String itemName)");

		MapItem retVal = null;
		for (int i = 0; i < _mapItemList.size(); i++) {
			MapItem mapItem = (MapItem) _mapItemList.get(i);
			if (itemName.equals(mapItem.getItemName())) {
				retVal = mapItem;
				break;
			}
		}

		//Trace.exit(this, "findItem(String itemName)");
		return retVal;
	}

	public List getChildren(MapItem mapItem) {
		//Trace.enter(this, "getChildren(MapItem mapItem)");
		List retVal = null;

		optimize();
		retVal = mapItem.getChildItemList();

		//Trace.exit(this, "getChildren(MapItem mapItem)");
		return retVal;
	}

	protected MapItem createMapItem(AttributeExpression attribute) {
		//Trace.enter(this, "createMapForItem(AttributeExpression)", attribute.pprint());

		MapItem mapItem = new MapItem();

		XQueryExpression xExpr = attribute.getOrginalXExpr();
		mapItem.setPath(xExpr.getStringValue());

		QType qtype = xExpr.getQType();
		if (qtype instanceof QTypeElement) {
			QTypeElement qte = (QTypeElement) xExpr.getQType();
			mapItem.setQName((QName) qte.getName());
		}

		//        mapItem._relatedExpression = attribute;

		mapItem.setItemName(attribute.getName());
		mapItem.setRelatedExpression(attribute);

		addMapItem(mapItem);

		//Trace.exit(this, "createMapForItem(AttributeExpression)", attribute.pprint());
		return mapItem;
	}

	protected MapItem createMapItem(RenameItem rename) {
		//Trace.enter(this, "createMapForItem(RenameItem)", rename.pprint());

		MapItem mapItem = new MapItem();

		XQueryExpression xExpr = rename.getOrginalXExpr();
		mapItem.setPath(xExpr.toString());

		QType qtype = xExpr.getQType();
		if (qtype instanceof QTypeElement) {
			QTypeElement qte = (QTypeElement) xExpr.getQType();
			mapItem.setQName((QName) qte.getName());
		}

		mapItem.setItemName(rename.getName());
		mapItem.setRelatedExpression(rename);

		addMapItem(mapItem);

		//Trace.exit(this, "createMapItem(RenameItem)", rename.pprint());
		return mapItem;
	}

	public void write(DataOutputStream dos) throws java.io.IOException {
		//Trace.enter(this, "write(DataOutputStream dos)");

		String newLine = System.getProperty("line.separator");

		MapItem mapItem = null;

		dos.writeBytes("<Mapper>");
		dos.writeBytes(newLine);
		dos.writeBytes("<MapItemList>");
		dos.writeBytes(newLine);
		for (int i = 0; i < _mapItemList.size(); i++) {
			mapItem = (MapItem) _mapItemList.get(i);
			mapItem.write(dos);
		}
		dos.writeBytes("</MapItemList>");
		dos.writeBytes(newLine);

		if (null != _idList) {
			dos.writeBytes("<IdList>");
			dos.writeBytes(newLine);
			for (int i = 0; i < _idList.size(); i++) {
				mapItem = (MapItem) _idList.get(i);
				mapItem.write(dos);
			}
			dos.writeBytes("</IdList>");
			dos.writeBytes(newLine);
		}

		if (null != _mapItemAddedForId) {
			dos.writeBytes("<MapItemAddedForId>");
			dos.writeBytes(newLine);
			for (int i = 0; i < _mapItemAddedForId.size(); i++) {
				mapItem = (MapItem) _mapItemAddedForId.get(i);
				mapItem.write(dos);
			}
			dos.writeBytes("</MapItemAddedForId>");
			dos.writeBytes(newLine);
		}

		if (null != _partAttribute) {
			dos.writeBytes("<PartAttribute>");
			dos.writeChars(_partAttribute.getName());
			dos.writeBytes("</PartAttribute>");
			dos.writeBytes(newLine);
		}

		dos.writeBytes("</Mapper>");
		dos.writeBytes(newLine);

		//Trace.exit(this, "write(DataOutputStream dos)");
	}

	public String pprint() {
		//Trace.enter(this, "pprint()");
		StringBuffer retVal = new StringBuffer();

		retVal.append("\n=====================      mapper      ==========================================================\n");
		retVal.append("No.  path,                   tag,                    item name,              part             children,\n");
		retVal.append("\n-------------------------------------------------------------------------------------------------\n");
		for (int i = 0; i < _mapItemList.size(); i++) {
			MapItem mapItem = (MapItem) _mapItemList.get(i);
			retVal.append(Integer.toString(i + 1));
			retVal.append("\t");
			retVal.append(mapItem.pprint());
			retVal.append("\n");

		}
		retVal.append("\n_mapItemAddedForId-----------------------------------------------------------------------------------\n");
		if (null != _mapItemAddedForId) {
			for (int i = 0; i < _mapItemAddedForId.size(); i++) {
				MapItem mapItem = (MapItem) _mapItemAddedForId.get(i);
				retVal.append(Integer.toString(i + 1));
				retVal.append("\t");
				retVal.append(mapItem.pprint());
				retVal.append("\n");
			}
		}
		retVal.append("\n_idList-------------------------------------------------------------------------------------------\n");
		if (null != _mapItemAddedForId) {
			for (int i = 0; i < _idList.size(); i++) {
				MapItem mapItem = (MapItem) _idList.get(i);
				retVal.append(Integer.toString(i + 1));
				retVal.append("\t");
				retVal.append(mapItem.pprint());
				retVal.append("\n");
			}
		}
		retVal.append("\n=================================================================================================\n");
		//Trace.exit(this, "pprint()");
		return retVal.toString();
	}

	protected void addMapItem(MapItem mapItem) {
		//Trace.enter(this, "addMapItem(MapItem mapItem)");

		String path = mapItem.getPath();
		MapItem mapItemB = map(path);
		if (null != mapItemB) {
		} else {
			_mapItemList.add(mapItem);
		}

		//Trace.exit(this, "addMapItem(MapItem mapItem)");
	}

	public void setPart(long part) {
		//Trace.enter(this, "setPart(long part)");

		for (int i = 0; i < _mapItemList.size(); i++) {
			MapItem mapItem = (MapItem) _mapItemList.get(i);
			mapItem.setPart(part);
		}

		//Trace.exit(this, "setPart(long part)");
	}

	public void merge(Mapper mapper) {
		//Trace.enter(this, "merge(Mapper mapper)");
		if (null != mapper) {
			/* merg mapItemList,duplicate path not allowed in the list */
			List mapItemList = mapper.getMapItemList();
			for (int i = 0; i < mapItemList.size(); i++) {
				MapItem mapItemToBeMerged = (MapItem) mapItemList.get(i);
				MapItem mapItem = map(mapItemToBeMerged.getPath());
				if (null != mapItem) {
					/*the mapItemToBeMerged already exits in this mapper*/
					long mapItemPart = mapItem.getPart();
					long mapItemToBeMergedPart = mapItemToBeMerged.getPart();
					mapItem.setPart(mapItemToBeMergedPart | mapItemPart);
				} else {
					_mapItemList.add(mapItemToBeMerged);
				}
			}

			/* merge idList. */
			List idList = mapper.getIdentifierList();
			if (null != idList) {
				for (int i = 0; i < idList.size(); i++) {
					MapItem mapItemToBeMerged = (MapItem) idList.get(i);
					int index = _idList.indexOf(mapItemToBeMerged);
					if (-1 == index) {

						_idList.add(mapItemToBeMerged);

						/* merge _mapItemAddedForId */

						index = mapper._mapItemAddedForId.indexOf(mapItemToBeMerged);
						if (-1 != index) {
							/* mapItemToBeMerged is added exclusively for adding Id */
							_mapItemAddedForId.add(mapItemToBeMerged);

						}
						if (!mapItemToBeMerged.isLeafPath()) {
							List childrenItem = mapper.getChildren(mapItemToBeMerged);
							for (int j = 0; j < childrenItem.size(); j++) {
								MapItem childItem = (MapItem) childrenItem.get(j);
								index = mapper._mapItemAddedForId.indexOf(childItem);
								if (-1 != index) {
									/* childItem is added exclusively for adding Id */
									_mapItemAddedForId.add(childItem);
								}
							}
						}
					} else {
						/*the mapItemToBeMerged already exits in this mapper*/
						MapItem mapItem = (MapItem) _idList.get(index);
						long mapItemPart = mapItem.getPart();
						long mapItemToBeMergedPart = mapItemToBeMerged.getPart();
						long combinedParts = mapItemToBeMergedPart | mapItemPart;
						mapItem.setPart(combinedParts);

						/* update _mapItemAddedForId */

						if (!mapItem.isLeafPath()) {
							List chidrenItem = getChildren(mapItem);
							for (int k = 0; k < chidrenItem.size(); k++) {
								MapItem child = (MapItem) chidrenItem.get(k);
								child.setPart(combinedParts);
							}
						}

					}
				}
			} else {
				//                Debug.assertTrue(false, "internal logic error");
			}

			/* move the keys to the end of the _mapItemList */
			_mapItemList.removeAll(_mapItemAddedForId);
			_mapItemList.addAll(_mapItemAddedForId);
		}
		//Trace.exit(this, "merge(Mapper mapper)");
	}

	/**
	 * Converts the child path list with the corresponding map items for every
	 * map item. 
	 */
	public void optimize() {
		if (!_optimized) {
			MapItem mapItem = null;
			for (int i = 0; i < _mapItemList.size(); i++) {
				mapItem = (MapItem) _mapItemList.get(i);
				if (!mapItem.isLeafPath()) {
					List childItemList = new ArrayList();
					MapItem childItem = null;
					List childPathList = mapItem.getChildren();
					String path = null;
					for (int j = 0; j < childPathList.size(); j++) {
						path = (String) childPathList.get(j);
						childItem = (MapItem) map(path);
						Debug.assertTrue(null != childItem, "null!=child");
						childItemList.add(childItem);
					}
					mapItem.setChildItemLis(childItemList);
				}
			}
			_optimized = true;
		}
	}
}
