/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.algebra;

import java.util.ArrayList;
import java.util.List;

import org.xquark.extractor.common.Debug;
/**

query structure
---------------------
A
	B
		C
			D
			E
	S
		T
	X

SOU table sturcture
1   A
2   A   B
3   A   B   C1  C2
4   A   B   C1  C2   D
6   A   B   C1  C2      E1  E2
7   A                           S
8   A                           S   T
9   A                                   X

initialized matrix:
IdNo:       1   2   3   4   5   6   7   8   9   10
QUERY       A   B   C1  C2  D   E1  E2  S   T   X
1           1
2           1   1
3           1   1   1   1
4           1   1   1   1   1
5           1   1   1   1       1   1
6           1                           1
7           1                           1   1
8           1                                   1


simplified matrix:
IdNo:       1   2   4   5   7   8   9   10
QUERY       A   B   C2  D   E2  S   T   X
1           1
2           1   1
3           1   1   1
4           1   1   1   1
5           1   1   1       1
6           1                   1
7           1                   1   1
8           1                           1



simplified matrix:
IdNo:       1   2   4   5   7   8   9   10
QUERY       A   B   C2  D   E2  S   T   X
1             1
2             2   1
3             2   2   1
4             2   2   2   1
5             2   2   3       1
6             3                   1
7             3                   2   1
8             4                           1


*/

public final class Numbering implements Constants{
	private static final String RCSRevision = "$Revision: 1.4 $";
	private static final String RCSName     = "$Name:  $";


	/* a list of mapItems for id */
	List _idList = null;

	/* a list of mapItems for id */
	List _sortSpecListList = null;
//    List        _numberAttributeList = null;
	int _subBueryNumber;
	int _matrix[][] ;
	int _maxRow;
	int _maxCol;


	/** @todo  */
	Mapper _mapper = null;

	public Numbering(List idList, int subBueryNumber) {
		_idList = idList;
		_subBueryNumber = subBueryNumber;
	}

//    public void setNumberAttributeList(List numberAttributeList)
//    {
//        _numberAttributeList = numberAttributeList;
//    }

	private void initializeMatrix()
	{
		//Trace.enter(this,"initializeMatrix()");
		/**/

		long bitMap ;
		_maxCol = _idList.size() ;
		_maxRow = _subBueryNumber ;
		_matrix = new int[_maxRow+1][_maxCol+2];    /* a column added for guide */


		long ruler ;
		for (int col = 1; col <= _maxCol ; col++) {
			MapItem mapItem = (MapItem)_idList.get(col-1);
			bitMap = mapItem.getPart();
			ruler = 1;
			for (int row = 1; row <= _maxRow; row++) {

				_matrix[row ][col] = ((bitMap >> (row-1)) & ruler) > 0 ? 1 : 0;
			}
		}

		for (int i = 1; i <=_maxCol; i++) {
			_matrix[0][i] = i;
		}
		for (int i = 1; i <=_maxRow; i++) {
			_matrix[i][0] = i;
		}
		//Trace.message(this,"initializeMatrix()", "Initialized matrix : \n" + printMatrix());
		//Trace.exit(this,"initializeMatrix()");
	}
	private void simplifyMatrix()
	{
		//Trace.enter(this,"simplifyMatrix()");
		/* remove duplecated*/
		for (int col = 1; col < _maxCol ; col++) {
			if ( colEquals(col, col + 1)) {
				removeCol(col);
			}
		}
		//Trace.message(this,"simplifyMatrix()", "Simplified matrix : \n" + printMatrix());

		//Trace.exit(this,"simplifyMatrix()");
	}

	private boolean colEquals(int col1 , int col2)
	{
		//Trace.enter(this, "colEquals(int col1 , int col2)", ""/*Integer.toString(col1)+ "\t"+ Integer.toString(col1)*/);
		boolean retVal = true;
		for (int row = 1; row <= _maxRow; row++) {
			if (_matrix[row][col1] != _matrix[row][col2]) {
				retVal = false;
				break;
			}
		}
		//Trace.exit(this, "colEquals(int col1 , int col2)", retVal ? "true":"false" );
		return retVal;
	}

	private void removeCol(int col)
	{
		//Trace.enter(this, "removeCol(int col)", ""/*Integer.toString(col)*/);
		for (int row = 0; row <= _maxRow; row++) {
			_matrix[row][col] = 0;
		}
		//Trace.exit(this, "removeCol(int col)", ""/*Integer.toString(col)*/);
	}

	private int slave(int query, int variable)
	{
		//Trace.enter(this, "slave(int query, int variable)", ""/*Integer.toString(query)+ "\t"+ Integer.toString(variable)*/);
		int retVal = 0;
		for (int col = variable + 1; col <= _maxCol; col++) {
			if (1 == _matrix[query][col] ) {
				retVal = col;
				break;
			}
		}
		//Trace.exit(this, "slave(int query, int variable)", Integer.toString(retVal));
		return retVal;
	}

	public void number()
	{
		//Trace.enter(this, "number()");

		initializeMatrix();

		eliminationGuide();

		simplifyMatrix();

		int curNum = 0;
		int oldSlave;
		int slave;

		/* numbering */
		for (int col = 1; col <= _maxCol; col++) {
			oldSlave = 0;
			curNum = 1;
			for (int row = 1; row <= _maxRow; row++) {
				if (0 != _matrix[row][col]) {
					slave = slave(row, col);
					if (oldSlave == slave) {
						_matrix[row][col] = curNum;
					}
					else {
						oldSlave = slave;
						curNum ++;
						_matrix[row][col] = curNum;
					}
				}
			}
		}
		//Trace.message(this,"number()", "numerodation : \n" + printMatrix());
		removeEmptyColumn();
		//Trace.message(this,"number()", "numerodation : \n" + printMatrix());
		//Trace.exit(this, "number()");
	}

	private void removeEmptyColumn()
	{
		//Trace.enter(this, "removeEmptyColumn()");
		for (int col = 1; col <= _maxCol; col++) {
			if (isEmpty(col)) {
				_matrix[0][col] = 0;
			}
		}
		//Trace.exit(this, "removeEmptyColumn()");
	}

	private boolean isEmpty(int col)
	{
		//Trace.enter(this, "isEmpty(int col)", ""/*Integer.toString(col)*/);
		boolean retVal = true;
		int counter = 0;
		for (int row = 1; row <= _maxRow; row++) {
			if (0 != _matrix[row][col]) {
				counter ++;
			}
		}
		retVal= counter < 2;
		//Trace.exit(this, "isEmpty(int col)", retVal ? "true":"false");
		return retVal;
	}

	public int[] getNumbersForQuery(int query)
	{
		//Trace.enter(this, "getNumbersForQuery(int query)", ""/*Integer.toBinaryString(query)*/);
		int[] retVal = new int[_maxCol+1];
		for (int col = 0; col < _maxCol; col++) {
			retVal[col] = _matrix[query][col];
		}
		//Trace.exit(this, "getNumbersForQuery(int query)", Integer.toBinaryString(query));
		return retVal;
	}

	void setSortSpecListList(List sortSpecListList)
	{
		_sortSpecListList = sortSpecListList;
	}

	List getSortList()
	{
		//Trace.enter(this, "getSortList()");

		List retVal = new ArrayList();

		List sortSpecList = null;
		MapItem mapItem;
		List subIdList;
		AttributeExpression numberColumn;

		LitInteger pseudoNumberColomn = new LitInteger(0);

		AttributeExpression sortExpr = null;
		SortSpecification   sortSpec = null;

		int idCursor = 0;   // a cursor on matrix column
		int lastIdOfCurrentQuery = 0;
		for (int query = 1; query <= _subBueryNumber; query++) {

			/* Sort instruments indicated by users */
			if (null != _sortSpecListList) {
				sortSpecList = (List)_sortSpecListList.get(query-1);
				if (null != sortSpecList) {
					retVal.addAll(sortSpecList);
				}
			}

			/* ids collumns of current query */
			lastIdOfCurrentQuery = lastIdColumnIn(query);
			for (int j = idCursor; j < lastIdOfCurrentQuery; j++) {
//                if (0 == _matrix[0][idCursor+1]) {
//                    continue;
//                }
				mapItem = (MapItem)_idList.get(j);
				if (mapItem.isLeafPath()) {
					sortExpr = new AttributeExpression(null, mapItem.getItemName());
					sortExpr.setUnderlyingExpr(mapItem.getRelatedExpression());
					sortSpec = new SortSpecification(sortExpr, ASCENDING_ORDER);
					retVal.add(sortSpec);
				}
				else {
					String childPath = null;
					MapItem subItem = null;
					subIdList = mapItem.getChildren();
					for (int k = 0; k < subIdList.size(); k++) {

						childPath = (String)subIdList.get(k);
						subItem = _mapper.map(childPath);
						sortExpr = new AttributeExpression(null, subItem.getItemName());
						sortExpr.setUnderlyingExpr(subItem.getRelatedExpression());
						sortSpec = new SortSpecification(sortExpr,ASCENDING_ORDER);
						retVal.add(sortSpec);
					}
				}
			}

			idCursor = lastIdOfCurrentQuery;

			/* number collumns */
			if (0 < _matrix[0][idCursor]) {
				numberColumn = new AttributeExpression(null,
					GenAlgebraVisitor.NUMBER_COLUMN_PREFIX+Integer.toString(_matrix[0][idCursor]));
				/* for feeding typevisitor*/
				numberColumn.setUnderlyingExpr(pseudoNumberColomn);
				sortSpec = new SortSpecification(numberColumn, ASCENDING_ORDER);
				retVal.add(sortSpec);
			}
		}

		//Trace.exit(this, "getSortList()");
		return retVal;
	}

	private void eliminationGuide ()
	{
		int lastId;
		int lastPresence;
		for (int query = 1; query <= _maxRow; query++) {
			lastId =  lastIdColumnIn(query);
			lastPresence = lastPresenceOf(query,lastId);
			_matrix[query][_maxCol+1] = lastPresence;
		}
	}

	public int[] getEliminationGuide()
	{
		int[] retVal = new int[_maxRow];
		for (int query = 1; query <= _maxRow; query++) {
			retVal[query-1] = _matrix[query][_maxCol+1];
		}
		return retVal;
	}

	private int lastIdColumnIn(int query)
	{
		int retVal = 0;
		for (int i = 1; i <= _maxCol; i++) {
			if (0 < _matrix[query][i]) {
				retVal = i;
			}
		}
		Debug.assertTrue(0 != retVal,"0 != retVal");
		return retVal;
	}

	private int lastPresenceOf(int query, int id)
	{
		int retVal = query;
		for (int i = query; i <=_maxRow; i++) {
			if (0 < _matrix[i][id]) {
				retVal = i;
			}
			else {
				break;
			}
		}
		Debug.assertTrue(0 != retVal,"0 != retVal");
		return retVal;
	}


	String printMatrix()
	{
		StringBuffer retVal = new StringBuffer();

		String newLine = System.getProperty("line.separator");
		for (int row = 0; row <= _maxRow; row++) {
			for (int col = 0; col <= _maxCol +1 ; col++) {
				retVal.append(_matrix[row][col]);
				retVal.append("\t");
			}
			retVal.append(newLine);
		}
		return retVal.toString();
	}
}
