/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.algebra;

import java.util.*;

import org.xquark.extractor.common.SqlWrapperException;
import org.xquark.extractor.runtime.IDProvider;
import org.xquark.extractor.sql.SqlExpression;

public final class RenameRelation extends UnaryAlgebra {
	private static final String RCSRevision = "$Revision: 1.8 $";
	private static final String RCSName = "$Name:  $";

	private static final char DEFAULT_PREFIX = 'r';

	private String _name = null;
	private Set _providedTableInstances = Collections.singleton(this);

	public RenameRelation(IDProvider provider) {
		setName(provider);
	}

	public RenameRelation(Expression relation, IDProvider provider) {
		super(relation);
		_name = relation.getName();
		setName(provider);
	}

	public RenameRelation(Expression relation, String name) {
		super(relation);
		setName(name);
	}

	synchronized Object clone(Map clonedExprs) throws CloneNotSupportedException {
		RenameRelation retVal = (RenameRelation) super.clone(clonedExprs);
		retVal.setName(_name);
		retVal._providedTableInstances = Collections.singleton(retVal);

		clonedExprs.put(this, retVal);
		return retVal;
	}

	public boolean equals(Object o) {
		if (o == this)
			return true;
		if (!(o instanceof RenameRelation))
			return false;

		RenameRelation p = (RenameRelation) o;
		if (!getName().equals(p.getName()))
			return false;
		return true;
	}

	public int hashCode() {
		return _name.hashCode();
	}

	/**
	 * Access method for the _name property.
	 * 
	 * @return the current value of the _name property
	 */
	public String getName() {
		return _name;
	}

	public String getUniqueName() {
		return _name;
	}

	/**
	 * Sets the value of the _name property.
	 * 
	 * @param aName the new value of the _name property
	 */
	public void setName(String aName) {
		_name = aName;
	}

	public void setName(IDProvider provider) {
		// Note: no need to synchronize since private per thread alias
		if (_name == null)
			_name = DEFAULT_PREFIX + provider.getID();
		else
			_name += provider.getID();
	}

	public List getOperands() {
		return null;
	}

	public boolean replaceChild(Expression oldChild, Expression newChild) {
		boolean retVal = false;
		if (getOperand() == oldChild) {
			setOperand(newChild);
			retVal = true;
		}
		return retVal;
	}

	public Set providedTableInstances() {
		return _providedTableInstances;
	}

	public Set visibleTableInstances() {
		return _providedTableInstances;
	}

	public AttributeExpression findNonNullAttribute() {
		AttributeExpression ret = ((Relation) _operand).findNonNullAttribute();
		if (ret != null)
			ret.setTableInstance(this);
		return ret;
	}

	public List getKeys() {
		return _keys;
	}
	public String pprint() {
		return "renameRelation(" + getOperand().pprint() + " AS " + getName() + ")";
	}

	public SqlExpression accept(GenSqlVisitor visitor) throws SqlWrapperException {
		return visitor.visit(this);
	}

	public void accept(AlgebraVisitor visitor) throws SqlWrapperException {
		visitor.visit(this);
	}

	// NOTE; Expression#deepEquals(Object) not overridden cause alias names should
	// not be considered for expression equality
}
