/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.algebra;

import java.util.ArrayList;
import java.util.List;

import org.xquark.extractor.common.Debug;

public final class SqlTypeCombinedStructure extends SqlType
{
	private static final String RCSRevision = "$Revision: 1.5 $";
	private static final String RCSName     = "$Name:  $";

	protected List _structures = null;;

	public SqlTypeCombinedStructure()
	{
	}
	/**
	@param List structureList
	@param structureList
	@roseuid 3BD1968302B7
	 */
	public SqlTypeCombinedStructure(List structureList)
	{
		setStructures(structureList);
	}

	public Object clone() {
		//Trace.enter(this, "clone()");
		try {
			SqlTypeCombinedStructure retVal = (SqlTypeCombinedStructure)super.clone();
			if (null != _structures) {
				retVal.setStructures( AlgebraTools.clone(_structures));
			}
			//Trace.exit(this, "clone()");
			return retVal;
		}
		catch (CloneNotSupportedException ex) {
			Debug.assertTrue(false, "error in implementation of clone");
			//Trace.exit(this, "clone()");
			return null;
		}
	}

	/**
	Access method for the _structures property.

	@return   the current value of the _structures property
	 */
	public List getStructures()
	{
		return _structures;
	}

	/**
	Sets the value of the _structures property.

	@param aStructures the new value of the _structures property
	 */
	public void setStructures(List aStructures)
	{
		_structures = aStructures;
	}

	public void addStructure(SqlType sqlType)
	{
		if (null == _structures) {
			_structures = new ArrayList();
		}
		_structures.add(sqlType);

	}

	public List getAttributes() {
		List retVal = new ArrayList();

		for (int i = 0; i < _structures.size(); i++) {
			SqlType type = (SqlType)_structures.get(i);
			if (type instanceof SqlTypeAtom) {
				retVal.add(type);
			}
			else if (type instanceof SqlTypeStructure) {
			  retVal.addAll(  ((SqlTypeStructure)type).getAttributes() );
			}
		}
		return retVal;
	}

	/**
	@return boolean
	@roseuid 3BD196AA0000
	 */
	public boolean isTuple()
	{
		return SqlType.ONE == _multiplicity ;
	}

	/**
	@param type
	@return boolean
	@roseuid 3BD196AA0050
	 */
	public boolean isCompatibleTo(SqlType type)
	{
	 return true;
	}

	/**
	@return boolean
	@roseuid 3BD196AA0096
	 */
	public boolean isRelation()
	{
		return SqlType.MANY == _multiplicity ;
	}

	/**
	@return boolean
	@roseuid 3BD196AA00B4
	 */
	public boolean isAtom()
	{
	 return false;
	}

	/**
	@return boolean
	@roseuid 3BD196AA00C8
	 */
	public boolean isBoolean()
	{
		boolean retVal = true;
		for (int i = 0; i < _structures.size(); i++) {
			if (! ((SqlType)_structures.get(i)).isBoolean() ) {
				retVal = false;
				break;
			}
		}
		return retVal;
	}

	/**
	@return boolean
	@roseuid 3BD196AA00E6
	 */
	public boolean isNumeric()
	{
		boolean retVal = true;
		for (int i = 0; i < _structures.size(); i++) {
			if (! ((SqlType)_structures.get(i)).isNumeric() ) {
				retVal = false;
				break;
			}
		}
		return retVal;
	}

	/**
	@return boolean
	@roseuid 3BD196AA010E
	 */
	public boolean isInteger()
	{
		boolean retVal = true;
		for (int i = 0; i < _structures.size(); i++) {
			if (! ((SqlType)_structures.get(i)).isInteger() ) {
				retVal = false;
				break;
			}
		}
		return retVal;
	}

	/**
	@return boolean
	@roseuid 3BD196AA0136
	 */
	public boolean isString()
	{
		boolean retVal = true;
		for (int i = 0; i < _structures.size(); i++) {
			if (! ((SqlType)_structures.get(i)).isString() ) {
				retVal = false;
				break;
			}
		}
		return retVal;
	}

	/**
	@return boolean
	@roseuid 3BD196AA0154
	 */
	public boolean isDataTime()
	{
		boolean retVal = true;
		for (int i = 0; i < _structures.size(); i++) {
			if (! ((SqlType)_structures.get(i)).isDataTime() ) {
				retVal = false;
				break;
			}
		}
		return retVal;
	}

	/**
	@return String
	@roseuid 3BD196AA017C
	 */
	public String pprint()
	{
		StringBuffer retVal = new StringBuffer();
		retVal.append("CombinedStructure [");
		for (int i = 0; i < _structures.size(); i++) {
			retVal.append(((SqlType)_structures.get(i)).pprint());
			retVal.append("\t");
		}
		retVal.append("]");
		return retVal.toString();
	}

}
