/*
 * This file belongs to the XQuark distribution. Copyright (C) 2003 Universite
 * de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307. You can also get it at
 * http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.algebra;

import java.util.*;

import org.xquark.extractor.common.Debug;
import org.xquark.extractor.common.SqlWrapperException;
import org.xquark.extractor.sql.SqlExpression;

public class Table extends Expression implements Relation {

	private static final String RCSRevision = "$Revision: 1.5 $";

	private static final String RCSName = "$Name:  $";

	protected org.xquark.extractor.metadata.Table _table;

	protected List _attributes;

	protected List _keys;

	private Set _providedTableInstances = Collections.singleton(this);

	/*
	 * when a key is once used(asked by AddKeys), it is is recorded in
	 * _usedKeysAttributeList
	 */
	protected List _usedKeysAttributeList = null;

	public Table(org.xquark.extractor.metadata.Table table) {
		setTable(table);
	}

	synchronized Object clone(Map clonedExprs)
			throws CloneNotSupportedException {
		Table retVal = (Table) super.clone(clonedExprs);
		clonedExprs.put(this, retVal);
		return retVal;

	}

	/**
	 * Access method for the _table property.
	 * 
	 * @return the current value of the _table property
	 */
	public org.xquark.extractor.metadata.Table getTable() {
		return _table;
	}

	/**
	 * Sets the value of the _table property.
	 * 
	 * @param aTable
	 *            the new value of the _table property
	 */
	public void setTable(org.xquark.extractor.metadata.Table aTable) {
		_table = aTable;
		Debug.assertTrue(null != _table, "null!=_table");

		List mAttributes = _table.getChildren();

		/* create _attributes */
		_attributes = new ArrayList();

		org.xquark.extractor.metadata.Attribute mAttr;
		Attribute attribute;
		Iterator iter = mAttributes.iterator();
		while (iter.hasNext()) {
			mAttr = (org.xquark.extractor.metadata.Attribute) iter.next();
			attribute = new Attribute(mAttr);
			_attributes.add(attribute);
		}

		/* create _keys */
		List keysInMetadata = null;

		keysInMetadata = _table.getKeys();
		if (null != keysInMetadata) {
			_keys = new ArrayList();
			List key = null;
			Attribute keyAttribute = null;

			List keyInMetadata = null;
			org.xquark.extractor.metadata.Attribute keyAttributeInMetadata = null;
			for (int i = 0; i < keysInMetadata.size(); i++) {
				keyInMetadata = (List) keysInMetadata.get(i);
				key = new ArrayList();
				for (int j = 0; j < keyInMetadata.size(); j++) {
					keyAttributeInMetadata = (org.xquark.extractor.metadata.Attribute) keyInMetadata
							.get(j);

					if (keyAttributeInMetadata instanceof org.xquark.extractor.metadata.PseudoAttribute) {
						if (keyAttributeInMetadata.getName().equalsIgnoreCase(
								"rowid")) {
							keyAttribute = new PseudoAttribute(
									(org.xquark.extractor.metadata.PseudoAttribute) keyAttributeInMetadata);
						} else {
							Debug
									.nyi("PseudoAttribute other than oracle rowid");
						}
					} else if (keyAttributeInMetadata instanceof org.xquark.extractor.metadata.Attribute) {
						keyAttribute = findAttribute(keyAttributeInMetadata);
					} else {
						Debug.nyi("unknow key attribute");
					}

					Debug.assertTrue(null != keyAttribute,
							"null != keyAttribute");
					key.add(keyAttribute);
				}
				_keys.add(key);
			}
		}
	}

	protected Attribute findAttribute(
			org.xquark.extractor.metadata.Attribute attributeInMetadata) {
		Attribute retVal = null;

		org.xquark.extractor.metadata.Attribute cursor = null;
		for (int i = 0; i < _attributes.size(); i++) {
			cursor = ((Attribute) _attributes.get(i)).getAttribute();
			if (cursor.equals(attributeInMetadata)) {
				retVal = (Attribute) _attributes.get(i);
				break;
			}
		}
		return retVal;
	}

	/**
	 * Access method for the _attributes property.
	 * 
	 * @return the current value of the _attributes property
	 */
	public List getAttributes() {
		return _attributes;
	}

	public List getOperands() {
		return null;
	}

	/**
	 * Sets the value of the _attributes property.
	 * 
	 * @param aAttributes
	 *            the new value of the _attributes property
	 */
	public void setAttributes(List aAttributes) {
		_attributes = aAttributes;
	}

	public String getName() {
		return _table.getName();
	}

	public String getAlias() {
		return _table.getAlias();
	}

	public String getSchemaName() {
		return _table.getFather().getName();
	}

	public String getCatalogName() {
		return _table.getFather().getFather().getName();
	}

	public Set providedTableInstances() {
		return _providedTableInstances;
	}

	public Set visibleTableInstances() {
		return providedTableInstances();
	}

	public AttributeExpression findNonNullAttribute() {
		Iterator it = _attributes.iterator();
		Attribute att = null;
		AttributeExpression ret = null;

		while (it.hasNext() && ret == null) {
			att = (Attribute) it.next();
			if (!att.getAttribute().getNullable()) {
				ret = new AttributeExpression(null, att.getName());
				ret.setUnderlyingExpr(att);
			}
		}
		return ret;
	}

	public List getKeys() {
		return _keys;
	}

	public List nameTest(String name) {
		List retVal = null;

		if (name.equalsIgnoreCase("*")) {
			retVal = _attributes;
		} else if (null == name) {
			retVal = null;
		} else {
			retVal = new ArrayList();

			Iterator iter = _attributes.iterator();
			Attribute item;
			while (iter.hasNext()) {
				item = (Attribute) iter.next();
				if (item.getAlias().equalsIgnoreCase(name)) {
					retVal.add(item);
				}
			}
			if (0 == retVal.size()) {
				retVal = null;
			}
		}

		return retVal;
	}

	public String pprintStructure() {
		StringBuffer retVal = new StringBuffer();

		String newLine = System.getProperty("line.separator");

		retVal.append("Table : ");
		retVal.append(_table.getName());
		retVal.append(newLine);

		for (int i = 0; i < _attributes.size(); i++) {
			retVal.append("\t");
			retVal.append(((Attribute) _attributes.get(i)).pprint());
			retVal.append(newLine);
		}
		return retVal.toString();
	}

	public void setUsedKeyAttributeList(List list) {
		_usedKeysAttributeList = list;
	}

	public List getUsedKeyAttributeList() {
		return _usedKeysAttributeList;
	}

	public SqlExpression accept(GenSqlVisitor visitor)
			throws SqlWrapperException {
		return visitor.visit(this);
	}

	public void accept(AlgebraVisitor visitor) throws SqlWrapperException {
		visitor.visit(this);
	}

	/**
	 * @see Expression#deepEquals(Object)
	 */
	public boolean deepEquals(Object o) {
		if (o instanceof Table) {
			Table cast = (Table) o;
			return _table.equals(cast.getTable()); // NOTE: assuming metadata
			// object are not
			// duplicated...
		}
		return false;
	}
}