/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.algebra;

import java.util.*;

import org.xquark.extractor.common.SqlWrapperException;
import org.xquark.extractor.sql.SqlExpression;

public final class UnOpRestrict extends UnaryAlgebra implements PredicateHolder
{
    private static final String RCSRevision = "$Revision: 1.6 $";
    private static final String RCSName     = "$Name:  $";


    protected List _predicateList;
    
    /**
     * True if the restriction is a having clause. This distinction is made
     * necessary by the fact that having clauses may exist without group or where
     * clauses, to allow SQL generation to make the difference
     */
    protected boolean _postResult = false;

	public UnOpRestrict(Expression operand)	{ super(operand);}

	public UnOpRestrict(Expression operand, Expression predicate)
	{
		super ( operand ) ;
		addPredicate ( predicate ) ;
	}

    public UnOpRestrict(Expression operand, List predicateList)
    {
        super ( operand ) ;
        setPredicateList ( predicateList ) ;
    }

	public UnOpRestrict(Expression operand, List predicateList, boolean postResult)
	{
		this(operand, predicateList) ;
		setPostResult(postResult) ;
	}

    synchronized Object clone(Map clonedExprs) throws CloneNotSupportedException
    {
        UnOpRestrict retVal = (UnOpRestrict)super.clone(clonedExprs);
        retVal.setPredicateList(AlgebraTools.clone(_predicateList, clonedExprs));

        clonedExprs.put(this, retVal);
        return retVal;
    }

    public List getPredicateList()
    {
        return _predicateList;
    }

    public List getParameterList()
    {
        return getPredicateList();
    }

    public void setPredicateList(List predicateList)
    {
        _predicateList = predicateList;

        if ( null != _predicateList )
        {
            Iterator    itr ;
            itr = _predicateList.iterator() ;
            while ( itr.hasNext () ) {
                ( (Expression)itr.next() ).setFather(this) ;
            }
        }
    }


    public void addPredicate(Expression predicate)
    {
        if ( null == predicate )
        {
           return ;
        }

        if (null == _predicateList) {
            _predicateList = new ArrayList ();
        }

        _predicateList.add(predicate);
        predicate.setFather(this);
    }


    public void removePredicate(Expression predicate)
    {
        _predicateList.remove(predicate);
    }


    public List getKeys() {
		return ((Relation)getOperand()).getKeys();
    }

    private void splitPredicate(Expression predicate, List predicateList)
    {
        Expression p1 , p2 ;
        BinOpBoolean    binOpBoolean ;

        p1 = predicate ;

        if ( p1 instanceof BinOpBoolean )
        {
            binOpBoolean = (BinOpBoolean) p1 ;
            if ( AND == binOpBoolean.getOperator() )
            {
                splitPredicate (binOpBoolean.getLeftOperand() , predicateList) ;
                splitPredicate (binOpBoolean.getRightOperand() , predicateList) ;
            }
            else
            {
                predicateList .add( predicate ) ;
            }
        }
        else
        {
            predicateList .add( predicate ) ;
        }
    }

    public boolean replaceChild(Expression oldChild, Expression newChild)
    {
        boolean retVal = false;
        List list = getPredicateList();

        Expression expr = null ;
        for (int i = 0; i < list.size(); i++) {
            expr = (Expression)list.get(i);
            if (expr.equals(oldChild)) {
                list.set(i, newChild);
				newChild.setFather(this);
                retVal = true;
                break;
            }
        }

        if (getOperand().equals(oldChild)) {
            setOperand(newChild);
            retVal = true;
        }

        return retVal;
    }


    public SqlExpression accept (GenSqlVisitor visitor) throws SqlWrapperException
    {
        return visitor.visit(this);
    }

    public void accept (AlgebraVisitor visitor) throws SqlWrapperException
    {
        visitor.visit(this);
    }

	/**
	 * @see Expression#deepEquals(Object)
	 */
	public boolean deepEquals(Object o)
	{
		if (o instanceof UnOpRestrict)
		{
			UnOpRestrict cast = (UnOpRestrict)o;
			return  super.deepEquals(o)
					&& AlgebraTools.areExprListEquivalent(_predicateList, cast.getPredicateList()); 
		}
		return false;
	}

    public boolean isPostResult() {
        return _postResult;
    }

    public void setPostResult(boolean b) {
        _postResult = b;
    }

}
