/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.algebra;

import java.util.*;

import org.xquark.extractor.common.SqlWrapperException;
import org.xquark.extractor.sql.SqlExpression;

public final class UnOpSort extends UnaryAlgebra
{
    private static final String RCSRevision = "$Revision: 1.5 $";
    private static final String RCSName     = "$Name:  $";


    private List _sortSpecificationList;

    public UnOpSort() {}

    public UnOpSort(Expression operand, List sortSpecificationList)
    {
        super ( operand ) ;
        setSortSpecificationList( sortSpecificationList)  ;
    }

    synchronized Object clone(Map clonedExprs) throws CloneNotSupportedException
    {
        UnOpSort retVal = (UnOpSort)super.clone(clonedExprs);
        retVal.setSortSpecificationList(AlgebraTools.clone(_sortSpecificationList, clonedExprs));

        clonedExprs.put(this, retVal);
        return retVal;
    }

    /**
	 * Access method for the _sortSpecificationList property.
	 * 
	 * @return the current value of the _sortSpecificationList property
	 */
    public List getSortSpecificationList()
    {
        return _sortSpecificationList ;
    }

    public List getParameterList()
    {
        return getSortSpecificationList();
    }

    /**
	 * Sets the value of the _sortSpecificationList property.
	 * 
	 * @param aSortSpecificationList
	 *            the new value of the _sortSpecificationList property
	 */
    public void setSortSpecificationList(List sortSpecificationList) {
		_sortSpecificationList = sortSpecificationList;
		if (null != _sortSpecificationList) {
			Iterator itr;
			itr = _sortSpecificationList.iterator();
			while (itr.hasNext()) {
				((Expression) itr.next()).setFather(this);
			}
		}
	}

    public void appendSortSpecification (SortSpecification sortSpec)
    {
        if (null != sortSpec) {
            if (null == _sortSpecificationList) {
                _sortSpecificationList = new ArrayList();
            }
            _sortSpecificationList.add(sortSpec);
            sortSpec.setFather(this);
        }
    }

    public void insertSortSpecification (SortSpecification sortSpec)
    {
        if (null != sortSpec) {
            if (null == _sortSpecificationList) {
                _sortSpecificationList = new ArrayList();
            }
            _sortSpecificationList.add(0,sortSpec);
            sortSpec.setFather(this);
        }
    }

    public boolean replaceChild(Expression oldChild, Expression newChild)
    {
        boolean retVal = false;

        List list = getSortSpecificationList();

        Expression expr = null ;
        for (int i = 0; i < list.size(); i++) {
            expr = (Expression)list.get(i);
            if (expr.equals(oldChild)) {
                list.set(i, newChild);
				newChild.setFather(this);
                retVal = true;
                break;
            }
        }

        if (getOperand().equals(oldChild)) {
            setOperand(newChild);
            retVal = true;
        }

        return retVal;
    }

    public List getKeys() {
		return ((Relation)getOperand()).getKeys();
    }

    public SqlExpression accept (GenSqlVisitor visitor) throws SqlWrapperException
    {
        return visitor.visit(this);
    }

    public void accept (AlgebraVisitor visitor) throws SqlWrapperException
    {
        visitor.visit(this);
    }

	/**
	 * @see Expression#deepEquals(Object)
	 */
	public boolean deepEquals(Object o)
	{
		if (o instanceof UnOpSort)
		{
			UnOpSort cast = (UnOpSort)o;
			boolean ret = super.deepEquals(o)
					&& AlgebraTools.areExprListEquivalent(_sortSpecificationList, cast.getSortSpecificationList()); 
		}
		return false;
	}
}
