/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.microsoft;

import org.xquark.extractor.algebra.*;
import org.xquark.extractor.runtime.IDProvider;
import org.xquark.extractor.sql.*;
import org.xquark.jdbc.typing.DbType;

public class MicrosoftGenSqlVisitor extends AbstractGenSqlVisitor {

	private static final String RCSRevision = "$Revision: 1.8 $";
	private static final String RCSName     = "$Name:  $";


    public MicrosoftGenSqlVisitor(IDProvider relIDProvider) {
        super((SqlFactory)MicrosoftFactory.getInstance(), relIDProvider);
    }

    protected SqlSelect selectAddFromClause (SqlSelect select, DummyTable operand)
    {
        //Trace.enter(this,"selectAddFromClause (SqlSelect select, DummyTable operand)");

        //Trace.exit(this,"selectAddFromClause (SqlSelect select, DummyTable operand)");
        return select;
    }
    public SqlExpression visit(BinOpArithmetic arg)
    {
        //Trace.enter(this,"visit(BinOpArithmetic arg)");

        SqlExpression  retVal = null;

        Expression      aLOprnd = arg.getLeftOperand();
        SqlExpression   sLOprnd = aLOprnd.accept(this);

        Expression      aROprnd = arg.getRightOperand();
        SqlExpression   sROprnd = aROprnd.accept(this);

        int oprtr = arg.getOperator();

		if (MODULO_ARITHMETICS == oprtr ) {
			retVal = visitModulo(sLOprnd, (SqlTypeAtom)aLOprnd.getType(), sROprnd, (SqlTypeAtom)aROprnd.getType());
		}
		else {
			if ( DIVIDE_ARITHMETICS == oprtr &&
				aLOprnd.getType().isInteger() &&
				aROprnd.getType().isInteger() ) {
				/* SQL Server will execute integer division while XQuery expect decimal result.
				   So we cast left operand to decimal type before the division */
				sLOprnd = _factory.createConvert(sLOprnd,
		    			    new DbType(java.sql.Types.INTEGER),
						    new DbType(java.sql.Types.DECIMAL));
			}
			retVal = _factory.createBinOpArithmetic(oprtr,sLOprnd, sROprnd);
			retVal = _factory.createBinOpArithmetic(oprtr,sLOprnd, sROprnd);
		}

        //Trace.exit(this,"visit(BinOpArithmetic arg)");
        return retVal;
    }

	protected SqlExpression	visitModulo(SqlExpression lOprnd, SqlTypeAtom lOprndType,
										SqlExpression rOprnd, SqlTypeAtom rOprndType) {
		SqlBinOpArithmetic  retVal = null;
		if (lOprndType.isInteger() && rOprndType.isInteger()) {
			retVal = _factory.createBinOpArithmetic(MODULO_ARITHMETICS,lOprnd,rOprnd);
		}
		else {
			/* sign(lOprnd) *  ( abs(lOprnd)- floor(abs(lOprnd/rOprnd))*abs(rOprnd) */

			SfSign  signL = _factory.createSfSign(lOprnd);

			SfAbs absL = _factory.createSfAbs(lOprnd);

			SqlBinOpArithmetic LdivR = _factory.createBinOpArithmetic(DIVIDE_ARITHMETICS,lOprnd, rOprnd);
			SfAbs absLdivR = _factory.createSfAbs(LdivR);
			SfFloor floorAbs = _factory.createSfFloor(absLdivR);
			SfAbs absR = _factory.createSfAbs(rOprnd);

			SqlBinOpArithmetic floorMultiR = _factory.createBinOpArithmetic(MULTIPLY_ARITHMETICS,floorAbs, absR);

			SqlBinOpArithmetic AbsMinusFloor = _factory.createBinOpArithmetic(MINUS_ARITHMETICS,absL, floorMultiR);

			retVal = _factory.createBinOpArithmetic(MULTIPLY_ARITHMETICS,signL, AbsMinusFloor);
		}
		return retVal;
	}


    public SqlExpression visit(SortSpecification arg)
    {
        //Trace.enter(this,"visit(SortSpecification arg)");
        SqlSortSpecification   retVal = null;
        Expression sortOperand = ((UnOpSort)arg.getFather()).getOperand();
        if (sortOperand instanceof BinaryAlgebra && !(sortOperand instanceof BinOpOuterJoin)) {
            Expression sortExpr = arg.getSortExpression();
    //        if (sortExpr instanceof AttributeExpression) {
    //            sortExpr = ((AttributeExpression)sortExpr).getUnderlyinExpr();
    //        }
            SqlExpression   sqlSortExpr = sortExpr.accept(this);

            retVal = _factory.createSortSpecification(sqlSortExpr,arg.getSortDirection());
        }
        else {
            Expression sortExpr = arg.getSortExpression();
            Expression underlyingExpr = null;
            if (sortExpr instanceof AttributeExpression) {
                underlyingExpr = ((AttributeExpression)sortExpr).getUnderlyinExpr();
                if (underlyingExpr instanceof AttributeExpression) {
                    sortExpr = underlyingExpr;
                }
            }
            SqlExpression   sqlSortExpr = sortExpr.accept(this);

            retVal = _factory.createSortSpecification(sqlSortExpr,arg.getSortDirection());
        }
        //Trace.exit(this,"visit(SortSpecification arg)");
        return retVal;
    }


    protected SqlSelect selectAddFromClause (SqlSelect select, BinaryAlgebra operand)
    {
        //Trace.enter(this,"selectAddFromClause (SqlSelect select, BinAlgebra operand)",operand.toString());

        SqlExpression sRelation = operand.accept(this);
        SqlRenameRelation sRR = _factory.createRenameRelation( sRelation,"BinaryAlgebra");

        select.addFromClause(sRR);

        //Trace.exit(this,"selectAddFromClause (SqlSelect select, BinAlgebra operand)",operand.toString());
        return select;
    }














}
