/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.progress.sql;

import java.sql.Types;

import org.xquark.extractor.common.Debug;
import org.xquark.extractor.sql.Context;
import org.xquark.extractor.sql.SqlExpression;
import org.xquark.extractor.sql.SqlSelect;
import org.xquark.jdbc.typing.DbType;

public class SqlConvert extends org.xquark.extractor.sql.SqlConvert {
	private static final String RCSRevision = "$Revision: 1.6 $";
	private static final String RCSName = "$Name:  $";

	public SqlConvert() {
		super();
	}

	public SqlConvert(SqlExpression expr, DbType originalType, DbType targetType) {
		super(expr, originalType, targetType);
	}

	public String toSql(Context context) {
		//Trace.enter(this, "toSql");
		String retVal = null;

		String expr = getExpression().toSql(context);
		if (getExpression() instanceof SqlSelect) {
			expr = '(' + expr + ')';
		}

		int targetTypeCode = _targetType.getJDBCType();

		if (_targetType.isNumeric()) {
			retVal = convertToNumber(expr);
		} else if (_targetType.isString()) {
			retVal = convertToString(expr);
		} else if (Types.DATE == targetTypeCode) {
			retVal = convertToDate(expr);
		} else if (Types.TIME == targetTypeCode) {
			retVal = convertToTime(expr);
		} else if (Types.TIMESTAMP == targetTypeCode) {
			retVal = convertToDateTime(expr);
		} else if (DbType.ORACLE_ROWID == targetTypeCode) {
			retVal = convertToRowid(expr);
		} else {
			Debug.assertTrue(false, "NYI!!");
		}
		//Trace.exit(this, "castTypeIgnoreName(Expression expr, SqlTypeAtom _originalType, SqlTypeAtom targetType)");
		return retVal;
	}

	private String convertToString(String expr) {
		//Trace.enter(this, "convertToString(String expr)", expr);
		StringBuffer retVal = new StringBuffer();

		int originalTypeCode = _originalType.getJDBCType();
		if (_originalType.isString()) {
			retVal.append(expr);
		} else if (DbType.ORACLE_ROWID == originalTypeCode) {
			retVal.append("ROWIDTOCHAR(");
			retVal.append(expr);
			retVal.append(")");
		} else if (Types.DATE == originalTypeCode) {
			retVal.append("LTRIM(TO_CHAR(");
			retVal.append(expr);
			retVal.append(",'SYYYY-MM-DD'))");
		} else if (Types.TIME == originalTypeCode) {
			retVal.append("TO_CHAR(");
			retVal.append(expr);
			retVal.append(",'HH24:MI:SS')");
		} else if (Types.TIMESTAMP == originalTypeCode) {
			retVal.append("LTRIM(TO_CHAR(");
			retVal.append(expr);
			retVal.append(",'SYYYY-MM-DD\"T\"HH24:MI:SS'))");
		} else if (_originalType.isNumeric()) {
			retVal.append("TO_CHAR(");
			retVal.append(expr);
			retVal.append(")");
		} else if (Types.NULL == originalTypeCode) {
			retVal.append("TO_CHAR(");
			retVal.append(expr);
			retVal.append(")");
		} else {
			Debug.assertTrue(false, "cas imprevu");
		}

		//Trace.exit(this, "convertToString(String expr)", expr);
		return retVal.toString();
	}

	private String convertToDate(String expr) {
		//Trace.enter(this, "convertToDate(String expr)", expr);
		StringBuffer retVal = new StringBuffer();

		int originalTypeCode = _originalType.getJDBCType();
		if (_originalType.isString() || _originalType.isNull()) {
			retVal.append("TO_DATE(");
			retVal.append(expr);
			retVal.append(", 'SYYYY-MM-DD')");
		} else if (Types.DATE == originalTypeCode) {
			retVal.append(expr);
		} else {
			Debug.assertTrue(false, "cas imprevu");
		}

		//Trace.exit(this, "convertToDate(String expr)", expr);
		return retVal.toString();
	}

	private String convertToDateTime(String expr) {
		//Trace.enter(this, "convertToDateTime(String expr)", expr);
		StringBuffer retVal = new StringBuffer();

		int originalTypeCode = _originalType.getJDBCType();
		if (_originalType.isString() || _originalType.isNull()) {
			retVal.append("TO_DATE(");
			retVal.append(expr);
			retVal.append(", 'SYYYY-MM-DD\"T\"HH24:MI:SS')");
		} else if (Types.TIMESTAMP == originalTypeCode) {
			retVal.append(expr);
		} else {
			Debug.assertTrue(false, "cas imprevu");
		}

		//Trace.exit(this, "convertToDateTime(String expr)", expr);
		return retVal.toString();
	}

	private String convertToTime(String expr) {
		//Trace.enter(this, "convertToTime(String expr)", expr);
		StringBuffer retVal = new StringBuffer();

		int originalTypeCode = _originalType.getJDBCType();
		if (_originalType.isString() || _originalType.isNull()) {
			retVal.append("TO_DATE(");
			retVal.append(expr);
			retVal.append(", 'HH24:MI:SS')");
		} else if (Types.TIME == originalTypeCode) {
			retVal.append(expr);
		} else {
			Debug.assertTrue(false, "cas imprevu");
		}

		//Trace.exit(this, "convertToTime(String expr)", expr);
		return retVal.toString();
	}

	private String convertToNumber(String expr) {
		//Trace.enter(this, "convertToNumber(String expr)", expr);
		StringBuffer retVal = new StringBuffer();

		int originalTypeCode = _originalType.getJDBCType();
		if (_originalType.isNumeric()) {
			retVal.append(expr);
		} else if (_originalType.isNull()) {
			retVal.append("TO_NUMBER(");
			retVal.append(expr);
			retVal.append(")");
		} else if (_originalType.isString()) {
			retVal.append("TO_NUMBER(");
			retVal.append(expr);
			retVal.append(")");
		} else {
			Debug.assertTrue(false, "cas imprevu");
		}
		//Trace.exit(this, "convertToNumber(String expr)", expr);
		return retVal.toString();
	}
	private String convertToRowid(String expr) {
		//Trace.enter(this, "convertToRowid(String expr)", expr);
		StringBuffer retVal = new StringBuffer();

		int originalTypeCode = _originalType.getJDBCType();
		if (_originalType.isString() || _originalType.isNull()) {
			retVal.append(" CHARTOROWID(");
			retVal.append(expr);
			retVal.append(")");
		} else {
			Debug.assertTrue(false, "cas imprevu");
		}

		//Trace.exit(this, "convertToRowid(String expr)", expr);
		return retVal.toString();
	}

}
