/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.runtime;


import java.io.Writer;
import java.util.List;

import org.w3c.dom.Document;
import org.xml.sax.ContentHandler;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.ext.LexicalHandler;
import org.xquark.schema.validation.PSVInfoSetProvider;
import org.xquark.xml.xdbc.*;

public class XResultSetUnion implements XMLResultSet {
	private static final String RCSRevision = "$Revision: 1.2 $";
	private static final String RCSName = "$Name:  $";

	protected ContentHandler _contentHandler = null;
	protected LexicalHandler _lexicalhandler = null;
	protected ErrorHandler _errorhandler = null;

	private List _cExprList = null;
	private XMLStatementInternal _statement = null;

	private int			_currentQueryIndex = 0;
	private XMLResultSet _currenResultSet = null;

	private int _positionAccumulater = 0;

	public XResultSetUnion(List compiledExprList, XMLStatementInternal statement) throws XMLDBCException{
		setExpressionList(compiledExprList);
		setStatement(statement);
	}

	public void setExpressionList(List compiledExprList) {
		_cExprList = compiledExprList;
	}

	public void setStatement(XMLStatementInternal statement) {
		_statement = statement;
	}

	public XMLStatement getStatement() {
		return _statement;
	}

	protected XMLResultSet evaluate(int queryIndex) throws XMLDBCException {
		return _statement.runQuery((Query.CompiledExpression)_cExprList.get(queryIndex));
	}
	/**********************************************************************************************/

	/**
	 * @see XMLResultSet#setContentHandler
	 */
	public void setContentHandler(ContentHandler contentHandler) {
		if (contentHandler == null)
			throw new NullPointerException("ContentHandler cannot be null.");
		_contentHandler = contentHandler;
	}

	/**
	 * @see ResultSet#setLexicalHandler
	 */
	public void setLexicalHandler(LexicalHandler lexicalhandler) {
		if (lexicalhandler == null)
			throw new NullPointerException("LexicalHandler cannot be null.");
		_lexicalhandler = lexicalhandler;
	}

	/**
	 * @see ResultSet#setErrorHandler
	 */
	public void setErrorHandler(ErrorHandler errorhandler) {
		//org.xquark.xml.misc.Debug.print(this, "START - XResultSetImpl setErrorHandler");
		if (errorhandler == null)
			throw new NullPointerException("ErrorHandler cannot be null.");
		_errorhandler = errorhandler;
	}

	/**
	 * @see ResultSet#setErrorHandler
	 */
	public ContentHandler getContentHandler() {
		return _contentHandler;
	}

	/**
	 * @see ResultSet#setErrorHandler
	 */
	public LexicalHandler getLexicalHandler() {
		return _lexicalhandler;
	}

	/**
	 * @see ResultSet#setErrorHandler
	 */
	public ErrorHandler getErrorHandler() {
		return _errorhandler;
	}

	private void setHandlers(XMLResultSet resultSet) {
		if (null !=_contentHandler ) resultSet.setContentHandler(_contentHandler);
		if (null !=_lexicalhandler ) resultSet.setLexicalHandler(_lexicalhandler);
		if (null !=_errorhandler ) resultSet.setErrorHandler(_errorhandler);
	}
	/**********************************************************************************************/

	/**
	 * @see XMLResultSet#close
	 */
	public void close() throws XMLDBCException {
		if (_currenResultSet != null) {
			_currenResultSet.close();
			_currenResultSet = null;
		}
	}

	/**********************************************************************************************/

	/**
	 * @see XMLResultSet#isBeforeFirst
	 */
	public boolean isBeforeFirst() throws XMLDBCException {
		return 0 == _positionAccumulater + _currenResultSet.getPosition() ? true : false;
	}

	/**
	 * @see XMLResultSet#nextAsDOM
	 */
	public Document nextAsDocument() throws XMLDBCException {
		return _currenResultSet.nextAsDocument();
	}


	public org.w3c.dom.Node nextAsDOM() throws XMLDBCException {
		return _currenResultSet.nextAsDOM(); 
	}

	public void nextAsDOM(org.w3c.dom.Element parent) throws XMLDBCException {
		_currenResultSet.nextAsDOM(parent);
	}

	/**
	 * @see XMLResultSet#nextAsString
	 */
	public String nextAsString() throws XMLDBCException {
		return _currenResultSet.nextAsString();
	}

	public void nextAsStream(Writer out) throws XMLDBCException {
		_currenResultSet.nextAsStream(out);
	}

	/**
	 * @see XMLResultSet#nextAsSAX
	 */
	public void nextAsSAX() throws XMLDBCException, SAXException {
		_currenResultSet.nextAsSAX();
	}

	/**
	 * @see XMLResultSet#hasNext
	 */
	public boolean hasNext() throws XMLDBCException {
		boolean retVal = false;

		if (null == _currenResultSet) {
			_currenResultSet = evaluate(_currentQueryIndex);
			setHandlers(_currenResultSet);
		}
		retVal = _currenResultSet.hasNext();
		if (!retVal) {
			int position = _currenResultSet.getPosition();
			_currenResultSet.close();
			if (_currentQueryIndex < _cExprList.size() - 1 ) {
				do {
					_currentQueryIndex++;
					_currenResultSet = evaluate(_currentQueryIndex);
				}
				while ((!_currenResultSet.hasNext()) && (_currentQueryIndex < _cExprList.size() - 1));
				retVal = _currenResultSet.hasNext();
				if (retVal)
					setHandlers(_currenResultSet);
				
				_positionAccumulater += position;
			}
		}
		return retVal;
	}

	/**
	 * @see XMLResultSet#getPosition
	 */
	public int getPosition() throws XMLDBCException {
		int retVal = _currenResultSet.getPosition();
		retVal = -1 == retVal ? 0 : retVal;
		retVal = _positionAccumulater + retVal;
		retVal = 0 == retVal ? 1 : retVal;
		return retVal;
	}

	/**
	 * Returns the result set prefix map (as prefix-namespace pairs).
	 * This map is used to produce each result in the set.
	 * @return the result set prefix map.
	 * @throws XMLDBCException if a data source access error occurs.
	 * @deprecated
	 */
	public java.util.Map getPrefixMap() throws XMLDBCException {
		return java.util.Collections.EMPTY_MAP;
	}

	/**
	 * Returns the fragments contained in the result set as a single XML document, with
	 * a document root element having the specified namespace, local name and qualified name.
	 * Note: this convenience method can only be called if the cursor is before the
	 * first element. It will process the complete result set in one step.
	 * @param namespace the namespace of the document root element.
	 * @param localName the local name of the document root element.
	 * @param qName the qualified name of the document root element.
	 * @return an XML document containing the fragments of the result set, as children of
	 * the specified document root element.
	 * @throws XMLDBCException if a data source access error occurs.
	 */
	public XMLDocument getFragmentsAsDocument(String namespace, String localName, String qName) throws XMLDBCException {
		XMLDocument retVal = null;
		return retVal;
	}

	/**
	 * Returns the result set metadata (the XML Schema that models the returned data)
	 * as an XMLDocument.
	 * This schema describes the XML type of each result in the set.
	 * @return the result set metadata (a XML Schema).
	 * @throws XMLDBCException if a data source access error occurs.
	 */
	public XMLDocument getMetaData() throws XMLDBCException {
		return null;
	}
	public boolean hasRootTag() {
		return false;
	}
	
	public boolean isDocument() {
		return false;
	}

	public PSVInfoSetProvider getPSVInfoSetProvider() {
		return null;
	}

}
