/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.sql;

import java.util.List;

import org.xquark.extractor.algebra.Expression;
import org.xquark.extractor.algebra.ExternalVariable;
import org.xquark.jdbc.typing.DbType;

abstract public class DefaultSqlFactory implements SqlFactory {
	private static final String RCSRevision = "$Revision: 1.5 $";
	private static final String RCSName     = "$Name:  $";

	protected DefaultSqlFactory() {
	}

	public SqlAttributeExpression createAttributeExpression() {
		return new SqlAttributeExpression();
	}

	public SqlAttributeExpression createAttributeExpression(String tableInstance, String attribute) {
		return new SqlAttributeExpression(tableInstance, attribute);
	}

	public SqlBinOpArithmetic createBinOpArithmetic() {
		return new SqlBinOpArithmetic();
	}

	public SqlBinOpArithmetic createBinOpArithmetic(int operator, SqlExpression leftOperand, SqlExpression rightOperand) {
		return new SqlBinOpArithmetic(operator, leftOperand, rightOperand);
	}

	public SqlBinOpBoolean createBinOpBoolean() {
		return new SqlBinOpBoolean();
	}

	public SqlBinOpBoolean createBinOpBoolean(int operator, SqlExpression leftOperand, SqlExpression rightOperand) {
		return new SqlBinOpBoolean(operator, leftOperand, rightOperand);
	}

	public SqlBinOpCompare createBinOpCompare() {
		return new SqlBinOpCompare();
	}

	public SqlBinOpCompare createBinOpCompare(int operator, SqlExpression leftOperand, SqlExpression rightOperand) {
		return new SqlBinOpCompare(operator, leftOperand, rightOperand);
	}

	public SqlBinOpCompareAny createBinOpCompareAny() {
		return new SqlBinOpCompareAny();
	}

	public SqlBinOpCompareAny createBinOpCompareAny(int operator, SqlExpression leftOperand, SqlExpression rightOperand) {
		return new SqlBinOpCompareAny(operator, leftOperand, rightOperand);
	}

	public SqlBinOpLike createBinOpLike() {
		return new SqlBinOpLike();
	}

	public SqlBinOpLike createBinOpLike(SqlExpression leftOperand, SqlExpression rightOperand, boolean not) {
		return new SqlBinOpLike(leftOperand, rightOperand, not);
	}

	public SqlBinOpUnion createBinOpUnion() {
		return new SqlBinOpUnion();
	}

	public SqlBinOpUnion createBinOpUnion(int operator, SqlExpression leftOperand, SqlExpression rightOperand) {
		return new SqlBinOpUnion( operator, leftOperand, rightOperand);
	}

	abstract public SqlConvert createConvert();
	
	abstract public SqlConvert createConvert(SqlExpression expr, DbType originalType, DbType targetType);

	public SqlIfThenElse createIfThenElse() {
		return new SqlIfThenElse();
	}

	public SqlIfThenElse createIfThenElse(SqlExpression iif, SqlExpression tthen, SqlExpression eslse) {
		return new SqlIfThenElse(iif, tthen, eslse);
	}

	public SqlJoin createJoin() {
		return new SqlJoin();
	}

	public SqlJoin createJoin(List operandList) {
		return new SqlJoin(operandList);
	}

	public SqlOuterJoinPredicate createOuterJoinPredicate() {
		return new SqlOuterJoinPredicate();
	}

	public SqlOuterJoinPredicate createOuterJoinPredicate(SqlExpression predicate,SqlRenameRelation innerTable) {
		return new SqlOuterJoinPredicate(predicate, innerTable);
	}

	public SqlUnOpExists createUnOpExists() {
		return new SqlUnOpExists();
	}

	public SqlUnOpExists createUnOpExists(SqlExpression query, boolean notExists) {
		return new SqlUnOpExists(query, notExists);
	}

	public SqlUnOpIsNull createUnOpIsNull() {
		return new SqlUnOpIsNull();
	}

	public SqlUnOpIsNull createUnOpIsNull(SqlExpression expr, boolean notIsNull) {
		return new SqlUnOpIsNull(expr, notIsNull);
	}

	public SqlUnOpMinus createUnOpMinus() {
		return new SqlUnOpMinus();
	}

	public SqlUnOpMinus createUnOpMinus(SqlExpression expr) {
		return new SqlUnOpMinus(expr);
	}

	public SqlUnOpNot createUnOpNot() {
		return new SqlUnOpNot();
	}

	public SqlUnOpNot createUnOpNot(SqlExpression expr) {
		return new SqlUnOpNot(expr);
	}

	public SqlFunAggregate createFunAggregate() {
		return new SqlFunAggregate();
	}

	public SqlFunAggregate createFunAggregate(int operator, SqlExpression operand, boolean distinct) {
		return new SqlFunAggregate(operator, operand, distinct);
	}

	public SqlBinOpIn createBinOpIn() {
		return new SqlBinOpIn();
	}

	public SqlBinOpIn createBinOpIn(SqlExpression leftOperand, SqlExpression rightOperand) {
		return new SqlBinOpIn(leftOperand, rightOperand);
	}

	public SqlPlaceHolder createPlaceHolder(ExternalVariable externalVariable) {
	    return new SqlPlaceHolder(externalVariable);
	}
	
	public SqlLitBoolean createLitBoolean() {
		return new SqlLitBoolean();
	}

	public SqlLitBoolean createLitBoolean(Boolean value) {
		return new SqlLitBoolean(value);
	}

	public SqlLitDate createLitDate() {
		return new SqlLitDate();
	}

	public SqlLitDate createLitDate(java.sql.Date value) {
		return new SqlLitDate(value);
	}

	public SqlLitDecimal createLitDecimal() {
		return new SqlLitDecimal();
	}

	public SqlLitDecimal createLitDecimal(java.math.BigDecimal value) {
		return new SqlLitDecimal(value);
	}

	public SqlLitDouble createLitDouble() {
		return new SqlLitDouble();
	}

	public SqlLitDouble createLitDouble(Double value) {
		return new SqlLitDouble(value);
	}

	public SqlLitFloat createLitFloat() {
		return new SqlLitFloat();
	}

	public SqlLitFloat createLitFloat(Float value) {
		return new SqlLitFloat(value);
	}

	public SqlLitInteger createLitInteger() {
		return new SqlLitInteger();
	}

	public SqlLitInteger createLitInteger(Integer value) {
		return new SqlLitInteger(value);
	}

	public SqlLitString createLitString() {
		return new SqlLitString();
	}

	public SqlLitString createLitString(String value) {
		return new SqlLitString(value);
	}

	public SqlLitList createLitList() {
		return new SqlLitList();
	}

	public SqlLitList createLitList(List litList) {
		return new SqlLitList(litList);
	}

	public SqlLitNull createLitNull() {
		return new SqlLitNull();
	}

	public SqlSelect createSelect(Expression expr) {
		return new SqlSelect(expr);
	}

	public SqlSortSpecification createSortSpecification() {
		return new SqlSortSpecification();
	}

	public SqlSortSpecification createSortSpecification(SqlExpression sortExpression, int sortDirection) {
		return new SqlSortSpecification(sortExpression, sortDirection);
	}

	public SqlTable createTable() {
		return new SqlTable();
	}

	public SqlTable createTable(String name) {
		return new SqlTable(name);
	}


	public SqlTable createTable(String catalogName, String schemaName, String tableName) {
		return new SqlTable(catalogName, schemaName, tableName);
	}

	public SqlRenameItem createRenameItem() {
		return new SqlRenameItem();
	}

	public SqlRenameItem createRenameItem(SqlExpression expr, String name) {
		return new SqlRenameItem(expr, name);
	}

	public SqlRenameRelation createRenameRelation() {
		return new SqlRenameRelation();
	}

	public SqlRenameRelation createRenameRelation(SqlExpression relation,String alias) {
		return new SqlRenameRelation(relation, alias);
	}

	/* create SQL Functions*/
	public SfAbs createSfAbs() {
		return new SfAbs();
	}

	public SfAbs createSfAbs(SqlExpression arg0) {
		return new SfAbs(arg0);
	}

	public SfCeiling createSfCeiling() {
		return new SfCeiling();
	}

	public SfCeiling createSfCeiling(SqlExpression arg0) {
		return new SfCeiling(arg0);
	}

	public SfConcat createSfConcat() {
		return new SfConcat();
	}

	public SfConcat createSfConcat(List list) {
		return new SfConcat(list);
	}

	public SfContains createSfContains() {
		return new SfContains();
	}

	public SfContains createSfContains(SqlExpression arg0, SqlExpression arg1) {
		return new SfContains(arg0, arg1);
	}

	public SfCurrentDateTime createSfCurrentDateTime() {
		return new SfCurrentDateTime();
	}


	public SfFloor createSfFloor() {
		return new SfFloor();
	}

	public SfFloor createSfFloor(SqlExpression arg0) {
		return new SfFloor(arg0);
	}

	public SfLowerCase createSfLowerCase() {
		return new SfLowerCase();
	}

	public SfLowerCase createSfLowerCase(SqlExpression arg0) {
		return new SfLowerCase(arg0);
	}

	public SfRound createSfRound() {
		return new SfRound();
	}

	public SfRound createSfRound(SqlExpression arg0) {
		return new SfRound(arg0);
	}

	public SfSign createSfSign() {
		return new SfSign();
	}

	public SfSign createSfSign(SqlExpression arg0) {
		return new SfSign(arg0);
	}

	public SfStringLength createSfStringLength() {
		return new SfStringLength();
	}

	public SfStringLength createSfStringLength(SqlExpression arg0) {
		return new SfStringLength(arg0);
	}

	public SfSubString createSfSubString() {
		return new SfSubString();
	}

	public SfSubString createSfSubString(SqlExpression arg0, SqlExpression arg1) {
		return new SfSubString(arg0, arg1);
	}

	public SfTrim createSfTrim() {
		return new SfTrim();
	}

	public SfTrim createSfTrim(SqlExpression arg0) {
		return new SfTrim(arg0);
	}

	public SfUpperCase createSfUpperCase() {
		return new SfUpperCase();
	}

	public SfUpperCase createSfUpperCase(SqlExpression arg0) {
		return new SfUpperCase(arg0);
	}
}
