/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.sql;


public class SqlAttributeExpression extends SqlExpression
{

	private static final String RCSRevision = "$Revision: 1.5 $";
	private static final String RCSName     = "$Name:  $";

    private String _attribute;
    private String _tableInstance;

    /**
    @param tableInstance
    @param attribute
    @roseuid 3BD177CE00F9
     */
    public SqlAttributeExpression(String tableInstance, String attribute)
    {
        setTableInstance (tableInstance);
        setAttribute(attribute);
    }

    /**
    @roseuid 3BD177CE00C7
     */
    public SqlAttributeExpression()
    {

    }

    /**
    Access method for the _attribute property.
    @return   the current value of the _attribute property
    @roseuid 3BD177CE01FE
     */
    public String getAttribute()
    {
        return _attribute;
    }

    /**
    Sets the value of the _attribute property.
    @param aAttribute the new value of the _attribute property
    @roseuid 3BD177CE0230
     */
    public void setAttribute(String aAttribute)
    {
        _attribute = aAttribute;
    }

    /**
    Access method for the _tableInstance property.
    @return   the current value of the _tableInstance property
    @roseuid 3BD177CE02BC
     */
    public String getTableInstance()
    {
        return _tableInstance;
    }

    /**
    Sets the value of the _tableInstance property.
    @param aTableInstance the new value of the _tableInstance property
    @roseuid 3BD177CE02F8
     */
    public void setTableInstance(String aTableInstance)
    {
        _tableInstance = aTableInstance;
    }

    public String getName()
    {
        return getAttribute();
    }

	public String toSql(Context context) {
		return toSql(context,true);
	}
	
    public String toSql(Context context, boolean putQuotes)
    {
        //Trace.enter(this,"toSql(Context context)");

        StringBuffer retVal = new StringBuffer();

        if (getAttribute().equals("*"))
            retVal.append(_attribute);
        else {
			if (getTableInstance() != null) {
				if (putQuotes)
					retVal.append('"');
				retVal.append(_tableInstance);
				if (putQuotes)
					retVal.append('"');
				retVal.append(".");
			}

            if (putQuotes)
                retVal.append('"');
            retVal.append(_attribute);
            if (putQuotes)
                retVal.append('"');
                
			if (context.outerJoinPredicate) {
				if (getTableInstance()
					.equalsIgnoreCase(context.innerTable.getName())) {
					retVal.append("(+)");
				}
			}
        }

        //Trace.exit(this,"toSql(Context context)", retVal);
        return retVal.toString() ;
    }
}
