/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.sql;

import java.sql.SQLException;
import java.sql.Statement;
import java.util.List;

public class SqlCreateTable extends SqlExpression
{

	private static final String RCSRevision = "$Revision: 1.6 $";
	private static final String RCSName     = "$Name:  $";

    private String _tableName;
    private SqlExpression _query;

    /**
     * @param tableName
     * @param query
     */
    public SqlCreateTable(String tableName, SqlExpression query)
    {
        setTableName(tableName);
        setQuery(query);
    }

    public SqlCreateTable() {}

    /**
     * Access method for the _query property.
     * @return   the current value of the _query property
     */
    public SqlExpression getQuery()
    {
        return _query;
    }

    /**
     * Sets the value of the _query property.
     *
     * @param aQuery the new value of the _query property
     */
    public void setQuery(SqlExpression aQuery)
    {
       _query = aQuery;
    }

    public void setTableName(String tableName)
    {
        _tableName = tableName;
    }


    public String toSql(Context context)
    {
        StringBuffer sb = new StringBuffer();
        sb.append(" CREATE GLOBAL TEMPORARY TABLE ");
        sb.append(_tableName);
        sb.append("AS ");
        sb.append(_query.toSql(context));
        return sb.toString() ;
    }

    public void clean(Statement statement) throws SQLException
    {
        statement.executeQuery ( "DROP TABLE " + _tableName ) ;
    }
    
    protected List getResetStatementList(List stmts)
    {
        stmts.add(new StatementInfo("TRUNCATE TABLE " + _tableName));
        return stmts;
    }
}
