/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.sql;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;


public class SqlExpression implements Constants
{

	private static final String RCSRevision = "$Revision: 1.10 $";
	private static final String RCSName     = "$Name:  $";


    private List _precedents = null;;

    private StatementInfo _sql = null; // TODO: check that this field is not set somewhere before generation (2times)

    private List _requestList = null;	// contains either Strings or StatementInfos (according to the accessor used)
    private List _resetStatementList = null;

    public SqlExpression() {}

    public String getName()
    {
        /* this is a default implemetaion for most of subclasses of SqlExpression
        * SqlTable, SqlAttributeExpression, SqlRenameItem, SqlRenameRelation
        *re-implemente this function */
        return null;
    }

    /**
     * Access method for the _precedents property.
     * 
     * @return the current value of the _precedents property
     */
    public List getPrecedents()
    {
        return _precedents;
    }

    /**
     * Sets the value of the _precedents property.
     * 
     * @param aPrecedents the new value of the _precedents property
     */
    public void setPrecedents(List aPrecedents)
    {
        _precedents = aPrecedents;
    }

    public void appendPrecedent(SqlExpression precedent)
    {
        if (null == _precedents)
            _precedents = new ArrayList();
        
        _precedents.add(precedent);
    }

    public ResultSet Execute(Statement statement) throws SQLException
    {
        if ( null != _precedents )
        {
            Iterator iter = _precedents.iterator();
            while (iter.hasNext()) {
                ((SqlExpression)iter.next()).Execute( statement );
            }
        }
        return  statement.executeQuery(toSql().sStmt);
    }

    public void cleanAll(Statement statement) throws SQLException {
        if ( null != _precedents )
        {
            Iterator iter = _precedents.iterator();
            while (iter.hasNext()) {
                ((SqlExpression)iter.next()).cleanAll(statement);
            }
        }
        clean(statement);
    }

    protected void clean(Statement statement) throws SQLException {}


    public List getResetStatementList() {
        if (_resetStatementList == null)
            generateResetStatementList();
        return _resetStatementList;
    }
    
    public void generateResetStatementList() {
        _resetStatementList = new ArrayList();
        generateResetStatementList(_resetStatementList);
    }
    
    protected List generateResetStatementList(List stmts) {
        if ( null != _precedents )
        {
            Iterator iter = _precedents.iterator();
            while (iter.hasNext()) {
                ((SqlExpression)iter.next()).generateResetStatementList(stmts);
            }
        }
        return getResetStatementList(stmts);
    }

    protected List getResetStatementList(List stmts) { return stmts;}

    public final StatementInfo toSql() {
        if (null == _sql) {
            Context context = new Context();
            context.currentStmt.sStmt = toSql(context);
            _sql = context.currentStmt;
        }
        return _sql;
    }

    public String toSql(Context context) {return toString ();}

    public List getRequestList() {
        if (_requestList == null)
            generateRequestList();
        return _requestList;
    }

    public void generateRequestList() {
        _requestList = new ArrayList();
        generateRequestList(_requestList);
    }

    protected void generateRequestList(List requestList)
    {
        if (null != _precedents && ! _precedents.isEmpty()) {
            SqlExpression precedent = null;
            for (int i = 0; i < _precedents.size(); i++) {
                precedent = (SqlExpression) _precedents.get(i);
                precedent.generateRequestList(requestList);
            }
        }
        requestList.add(toSql());
    }

    public String pprint() {
        StringBuffer retVal = new StringBuffer();
        List requestList = getRequestList();

        String request = null;
        for (int i = 0; i < requestList.size(); i++) {
            request = (String) requestList.get(i);
            retVal.append(request);
            retVal.append('\n');
        }

        return retVal.toString();
    }

	public String toString() { return toSql().sStmt;}
}
