/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.sybase;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.sql.*;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;

import org.xquark.extractor.metadata.QNameEncoder;
import org.xquark.jdbc.typing.DBMSInfo;


public final class Loader extends org.xquark.extractor.metadata.Loader {

	private static final String RCSRevision = "$Revision: 1.6 $";
	private static final String RCSName     = "$Name:  $";

	public Loader(Connection connection, DBMSInfo dbmsInfo, QNameEncoder encoder)
	throws SQLException {
		super(connection, dbmsInfo, encoder);
	}

	protected String getDefaultSchema() throws SQLException {
		Statement statement = null;
		ResultSet rs = null;
		try {
			statement = _connection.createStatement();
			rs = statement.executeQuery("select user");
			rs.next();
			return rs.getString(1);
		} finally {
			if (rs != null)	rs.close();
			if (statement != null) statement.close();
		}
	}

    protected String createQuery(String catalogName, String schemaName, String origTableName){
        StringBuffer retVal = new StringBuffer();

        retVal.append("SELECT * FROM ");
        if (null != catalogName && 0 < catalogName.trim().length()) {
            retVal.append(catalogName.trim());
            retVal.append('.');
        }
        if (null != schemaName && 0 < schemaName.trim().length()) {
            retVal.append(schemaName.trim());
            retVal.append('.');
        }
        if (null != origTableName && 0 < origTableName.trim().length()) {
            retVal.append(origTableName.trim());
        }
        retVal.append(" WHERE 0=1");
        return retVal.toString();
    }

    protected PreparedStatement createPreparedStatement(Connection jdbcConnection,String command) throws SQLException {
    	try {
			Class[] argTypes = { String.class, Boolean.TYPE };
			Method method = jdbcConnection.getClass().getMethod("prepareStatement", argTypes);
			Object[] args = { command, Boolean.TRUE };
			return (PreparedStatement)method.invoke(jdbcConnection, args);
    	} catch (InvocationTargetException ex) {
    		throw (SQLException)ex.getTargetException();
		} catch (IllegalAccessException ex) {
			throw new SQLException("Could not create prepared statement");
    	} catch (NoSuchMethodException ex) {
    		throw new SQLException("Could not create prepared statement");
    	}
    }

}
