/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.sybase;

import java.util.List;

import org.xquark.extractor.sql.*;
import org.xquark.jdbc.typing.DbType;

public class SybaseFactory extends DefaultSqlFactory {
	private static final String RCSRevision = "$Revision: 1.6 $";
	private static final String RCSName     = "$Name:  $";

	static private SybaseFactory _instance = null;

	private SybaseFactory() {
	}

	static public SybaseFactory getInstance(){
		if (null ==_instance) {
			_instance = new SybaseFactory();
		}
		return _instance;
	}

	public SqlBinOpArithmetic createBinOpArithmetic() {
		return new org.xquark.extractor.sybase.sql.SqlBinOpArithmetic();
	}

	public SqlBinOpArithmetic createBinOpArithmetic(int operator, SqlExpression leftOperand, SqlExpression rightOperand) {
		return new org.xquark.extractor.sybase.sql.SqlBinOpArithmetic(operator, leftOperand, rightOperand);
	}

	public SqlConvert createConvert() {
		return new org.xquark.extractor.sybase.sql.SqlConvert();
	}

	public SqlConvert createConvert(SqlExpression expr, DbType originalType, DbType targetType){
		return new org.xquark.extractor.sybase.sql.SqlConvert(expr, originalType, targetType);
	}

	public SqlIfThenElse createIfThenElse() {
		return new org.xquark.extractor.sybase.sql.SqlIfThenElse();
	}

	public SqlIfThenElse createIfThenElse(SqlExpression iif, SqlExpression tthen, SqlExpression eslse) {
		return new org.xquark.extractor.sybase.sql.SqlIfThenElse(iif, tthen, eslse);
	}

	public SqlTable createTable() {
		return new org.xquark.extractor.sybase.sql.SqlTable();
	}

	public SqlTable createTable(String name) {
		return new org.xquark.extractor.sybase.sql.SqlTable(name);
	}

	public SqlTable createTable(String catalogName, String schemaName, String tableName) {
		return new org.xquark.extractor.sybase.sql.SqlTable(catalogName, schemaName, tableName);
	}

	public SqlAttributeExpression createAttributeExpression() {
		return new org.xquark.extractor.sybase.sql.SqlAttributeExpression();
	}

	public SqlAttributeExpression createAttributeExpression(String tableInstance, String attribute) {
		return new org.xquark.extractor.sybase.sql.SqlAttributeExpression(tableInstance, attribute);
	}

	public SfConcat createSfConcat() {
		return new org.xquark.extractor.sybase.sql.SfConcat();
	}

	public SfConcat createSfConcat(List list) {
		return new org.xquark.extractor.sybase.sql.SfConcat(list);
	}

	public SfCurrentDateTime createSfCurrentDateTime() {
		return new org.xquark.extractor.sybase.sql.SfCurrentDateTime();
	}


	public SfStringLength createSfStringLength() {
		return new org.xquark.extractor.sybase.sql.SfStringLength();
	}

	public SfStringLength createSfStringLength(SqlExpression arg0) {
		return new org.xquark.extractor.sybase.sql.SfStringLength(arg0);
	}

	public SfSubString createSfSubString() {
		return new org.xquark.extractor.sybase.sql.SfSubString();
	}

	public SfSubString createSfSubString(SqlExpression arg0, SqlExpression arg1) {
		return new org.xquark.extractor.sybase.sql.SfSubString(arg0, arg1);
	}


	public SfTrim createSfTrim() {
		return new org.xquark.extractor.sybase.sql.SfTrim();
	}

	public SfTrim createSfTrim(SqlExpression arg0) {
		return new org.xquark.extractor.sybase.sql.SfTrim(arg0);
	}

	public SqlRenameItem createRenameItem() {
		return new org.xquark.extractor.sybase.sql.SqlRenameItem();
	}

	public SqlRenameItem createRenameItem(SqlExpression expr, String name) {
		return new org.xquark.extractor.sybase.sql.SqlRenameItem(expr, name);
	}

	public SqlRenameRelation createRenameRelation() {
		return new org.xquark.extractor.sybase.sql.SqlRenameRelation();
	}

	public SqlRenameRelation createRenameRelation(SqlExpression relation,String alias) {
		return new org.xquark.extractor.sybase.sql.SqlRenameRelation(relation, alias);
	}
}
