/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.sybase.sql;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Iterator;
import java.util.List;

import org.xquark.extractor.sql.Context;
import org.xquark.extractor.sql.SqlExpression;


public class SqlCreateView extends SqlExpression {
	private static final String RCSRevision = "$Revision: 1.6 $";
	private static final String RCSName = "$Name:  $";


    protected String _name = null;
    protected List _attributeNameList = null;
    protected SqlExpression _query = null;

    public SqlCreateView() {
    }

    public SqlCreateView(String name, SqlExpression query) {
        setName(name);
        setquery(query);
    }

    public SqlCreateView(String name, List attributeNameList, SqlExpression query) {
        setName(name);
        setAttributeNameList(attributeNameList);
        setquery(query);
    }

    public void setName(String name) {
        _name = name;
    }

    public String getName() {
        return _name;
    }

    public void setAttributeNameList(List attributeNameListList) {
        _attributeNameList = attributeNameListList;
    }

    public List getAttributeNameList() {
        return _attributeNameList;
    }

    public void setquery(SqlExpression query) {
        _query = query;
    }

    public SqlExpression getquery() {
        return _query;
    }

    public ResultSet Execute(Statement statement) throws SQLException
    {
        ResultSet retResultSet = null;
        List prcedents = getPrecedents();
        if ( null != prcedents )
        {
            Iterator iter = prcedents.iterator();
            while (iter.hasNext()) {
                ((SqlExpression)iter.next()).Execute( statement );
            }
        }
        String query = this.toSql(new Context());
        //Trace.message(this,"Execute",query);
        try {
            statement.executeUpdate(query);
        }
        catch (SQLException ex) {
            if (true /** @todo the view  exists alreay */) {
                statement.executeUpdate("DROP VIEW " + _name) ;
                statement.executeUpdate(query) ;
            }
        }

        return  retResultSet;
    }

    public String toSql(Context context)
    {
        StringBuffer    retVal = new StringBuffer("CREATE VIEW ");
        retVal.append(_name);
        attributeNameListToSql(retVal, context);
        retVal.append(" AS ");
        retVal.append(_query.toSql(new Context(context)));

        return retVal.toString() ;
    }

    protected StringBuffer attributeNameListToSql(StringBuffer statementBuffer, Context context) {
        StringBuffer retVal = statementBuffer;
        if (null != _attributeNameList && !_attributeNameList.isEmpty()) {
            statementBuffer.append('(');
            String attrName = null;
            for (int i = 0; i < _attributeNameList.size(); i++) {
                attrName = (String)_attributeNameList.get(i);
                statementBuffer.append(attrName);
                if (i < _attributeNameList.size()-1 ) {
                    statementBuffer.append(", ");
                }
            }
            statementBuffer.append(')');
        }
        return statementBuffer;
    }


    protected void clean(Statement statement) throws SQLException {
        statement.execute("DROP VIEW " + _name);
    }
}
