/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.xfunctions;

import java.util.List;

import org.xquark.extractor.algebra.Expression;
import org.xquark.extractor.sql.SqlExpression;
import org.xquark.jdbc.typing.DbType;

public final class AfCast extends Expression {

	private static final String RCSRevision = "$Revision: 1.6 $";
	private static final String RCSName     = "$Name:  $";


	Expression _expression = null;
	DbType _targetType = null;

	public AfCast()
	{
	}

	public AfCast(Expression arg, DbType targetType)
	{
		setExpression(arg);
		setTargetType(targetType);
	}

	public Expression getExpression()
	{
		return _expression;
	}

	public List getOperands()
	{
		return null;
	}

	public void setExpression(Expression expression)
	{
		_expression = expression;
		_expression.setFather(this);
	}

	public DbType getTargetType()
	{
		return _targetType;
	}

	public void setTargetType(DbType targetType)
	{
		_targetType = targetType;
	}

	public String getName()
	{
		return getExpression().getName();
	}

	public boolean replaceChild(Expression oldChild, Expression newChild)
	{
		//Trace.enter(this, "replaceChild(Expression oldChild, Expression newChild)");
		boolean retVal = false;

		if (getExpression().equals(oldChild)) {
			setExpression(newChild);
			retVal = true;
		}

		//Trace.exit(this, "replaceChild(Expression oldChild, Expression newChild)");
		return retVal;
	}

	public String pprint()
	{
		StringBuffer retVal = new StringBuffer();
		retVal.append("AfCast(");
		retVal.append(getExpression().pprint());
		retVal.append(", ");
		retVal.append(getTargetType().pprint());
		retVal.append(")");
		return retVal.toString();
	}

	public SqlExpression accept(org.xquark.extractor.algebra.GenSqlVisitor visitor)
	{
		return visitor.visit(this);
	}

	public void accept(org.xquark.extractor.algebra.AlgebraVisitor visitor)
	{
		visitor.visit(this);
	}
	
	/**
	 * The semantics chosen for equals of algebra nodes is "the same algebra operator
	 * hierarchy referring to the same database objects".
	 * @param o the expression to be compared with this.
     */
    public boolean deepEquals(Object o)
	{
		if (o instanceof AfCast)
		{
			AfCast cast = (AfCast)o;
			return	 _expression.deepEquals(cast.getExpression())
						&& _targetType.equals(cast.getTargetType()); // NOTE: assuming metadata object are not duplicated...
		}
		return false;
	}
}

