/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.xfunctions;

import java.util.ArrayList;
import java.util.List;

import org.xquark.extractor.algebra.Expression;
import org.xquark.extractor.common.Debug;

public abstract class XFunction extends Expression {
	private String _functionName;
	protected List _arguments;

	/**
	 * @roseuid 3BD1465C02E6
	 */
	public XFunction() {
	}
	public XFunction(String functionName) {
		setFunctionName(functionName);
	}
	/**
	 * @param name
	 * @param arguments
	 * @roseuid 3BD143B8028A
	 */
	public XFunction(String functionName, List arguments) {
		setFunctionName(functionName);
		setArguments(arguments);
	}

	/**
	 * Access method for the _name property.
	 *
	 * @return   the current value of the _name property
	 */
	public String getFunctionName() {
		return _functionName;
	}

	/**
	 * Sets the value of the _functionName property.
	 *
	 * @param aName the new value of the _functionName property
	 */
	public void setFunctionName(String functionName) {
		_functionName = functionName;
	}

	/**
	 * Access method for the _arguments property.
	 *
	 * @return   the current value of the _arguments property
	 */
	public List getArguments() {
		return _arguments;
	}

	public List getOperands()
	{
		return _arguments;
	}
	/**
	 * Sets the value of the _arguments property.
	 *
	 * @param aArguments the new value of the _arguments property
	 */
	public void setArguments(List aArguments) {
		_arguments = aArguments;
		if (null != _arguments) {
			for (int i = 0; i < _arguments.size(); i++) {
				((Expression)_arguments.get(i)).setFather(this);
			}
		}
	}

	public int getArgumentNumber() {
		if (null != _arguments) {
			return _arguments.size();
		}
		else {
			return 0;
		}
	}

	public Expression getArgument(int index ) {
		if (null != _arguments) {
			if (_arguments.size()> index) {
				return (Expression)_arguments.get(index);
			}
			else {
				return null;
			}
		}
		else {
			return null;
		}
	}

	public void setArgument(int index, Expression argument) {
		Debug.assertTrue( null!=argument, "null!=argument");
		if (null == _arguments ) {
			_arguments = new ArrayList();
		}
		while( index >_arguments.size()-1) {
			_arguments.add(null);
		}
		_arguments.set(index,argument);
		argument.setFather(this);
	}

	public boolean replaceChild(Expression oldChild, Expression newChild)
	{
		boolean retVal = false;

		if (null != _arguments) {
			Object child = null;
			for (int i = 0; i < _arguments.size(); i++) {
				child = _arguments.get(i);
				if ( oldChild.equals(child)) {
					_arguments.set(i,newChild);
					retVal = true;
					break;
				}
			}
		}
		else {
			/* return false */
		}

		return retVal;
	}

	public String pprint()
	{
		StringBuffer retVal = new StringBuffer();
		retVal.append(getFunctionName());
		retVal.append("(");
		if ( null != _arguments) {
			Expression argument = null;
			for (int i = 0; i < _arguments.size(); i++) {
				argument = (Expression)_arguments.get(i);
				retVal.append(argument.pprint());
				retVal.append(",");
			}
			retVal.delete(retVal.length(),retVal.length());
		}
		retVal.append(")");
		return retVal.toString();
	}

	/**
	 * @see Expression#deepEquals(Object)
	 */
	public boolean deepEquals(Object o)
	{
		if (o instanceof XFunction)
		{
			XFunction cast = (XFunction)o;
			int len = _arguments.size(), i;
			boolean ret = _functionName.equalsIgnoreCase(cast.getFunctionName())
					&& (len == cast.getArguments().size());
			
			for (i = 0; ret && i < len; i++)
				ret &= ((Expression)_arguments.get(i)).deepEquals(cast.getArguments().get(i)); 
			return	ret;
		}
		return false;
	}
}

