/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.xfunctions;

import org.xquark.extractor.algebra.Expression;
import org.xquark.extractor.sql.SqlExpression;

/**
6.2.3 xf:date
xf:date(string $srcval) => date
If the value of $srcval conforms to the lexical representation of a date as defined in [XML Schema
Part 2: Datatypes], the constructor returns the date corresponding to that representation.
Otherwise, the constructor returns the error value as defined in [XQuery 1.0 and XPath 2.0 Data
Model].

6.2.3.1 Examples
xf:date("2001-5-31") returns a date value corresponding to the 31st of May, 2001.

xf:date("2001-4-31") returns an error.
 */
public final class XfDate extends XFunction
{

	private static final String RCSRevision = "$Revision: 1.3 $";
	private static final String RCSName     = "$Name:  $";


    /**
    @roseuid 3BD1922C02D6
     */
    public XfDate()
    {
        super("XfDate");
    }

    /**
    @param dateString
    @roseuid 3BD1916E0214
     */
    public XfDate(Expression arg0)
    {
        super("XfDate");
        setArgument(0, arg0);
    }

    /**
    @param visitor
    @return org.xquark.extractor.sql.SqlExpression
    @throws org.xquark.extractor.common.SqlWrapperException
    @roseuid 3BD1922C02F4
     */
    public SqlExpression accept(org.xquark.extractor.algebra.GenSqlVisitor visitor)
    {
        return visitor.visit(this);
    }

    public void accept(org.xquark.extractor.algebra.AlgebraVisitor visitor)
    {
        visitor.visit(this);
    }
}
