/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.xfunctions;

import org.xquark.extractor.algebra.Expression;
import org.xquark.extractor.sql.SqlExpression;

/**
3.5.16 xf:lower-case
3.5.16.1 Signature
xf:lower-case(string $srcval) => string

3.5.16.2 Parameters
$srcval  static type is string

3.5.16.3 Semantics
Returns the value of $srcval after translating every upper-case letter to its lower-case
correspondent. Every upper-case letter that does not have a lower-case correspondent, and every
character that is not an upper-case letter, is included in the output in its original form.

An "upper-case letter" is a character whose Unicode General Category class includes "Lu". The
corresponding lower-case letter is determined using [Unicode Case Mappings].

3.5.16.4 Examples
xf:lower-case("ABc!D") returns "abc!d".
 */
public final class XfLowerCase extends Xfunction_String
{

	private static final String RCSRevision = "$Revision: 1.3 $";
	private static final String RCSName     = "$Name:  $";

    public XfLowerCase()
    {
      super("XfLowerCase");
    }

    public XfLowerCase(Expression arg )
    {
      super("XfLowerCase");
      setArgument(0,arg);
    }

    public SqlExpression accept(org.xquark.extractor.algebra.GenSqlVisitor visitor)
    {
        return visitor.visit(this);
    }

    public void accept(org.xquark.extractor.algebra.AlgebraVisitor visitor)
    {
        visitor.visit(this);
    }
}
