/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.xfunctions;

import org.xquark.extractor.algebra.Expression;
import org.xquark.extractor.sql.SqlExpression;

/**
2.6.3 xf:round
2.6.3.1 Signature
xf:round(double $srcval) => integer

2.6.3.2 Parameters
$srcval  static type is double

2.6.3.3 Semantics
Returns the number that is closest to the argument and that is an integer. More formally, round(x)
produces the same result as floor(x+0.5). These semantics are consistent with Java's semantics. If
there are two such numbers, then the one that is closest to positive infinity is returned. If the
argument is NaN, then NaN is returned. If the argument is positive infinity, then positive infinity
is returned. If the argument is negative infinity, then negative infinity is returned. If the
argument is positive zero, then positive zero is returned. If the argument is negative zero, then
negative zero is returned. If the argument is less than zero, but greater than or equal to -0.5,
then negative zero is returned.

2.6.3.4 Examples
xf:round(2.5) returns 3.

xf:round(2.4999) returns 2.

xf:round(-2.5) returns -2 (not the possible alternative, -3).
 */
public final class XfRound extends Xfunction_Integer
{

	private static final String RCSRevision = "$Revision: 1.3 $";
	private static final String RCSName     = "$Name:  $";

    public XfRound()
    {
      super("XfRound");
    }

    public XfRound(Expression arg )
    {
      super("XfRound");
      setArgument(0,arg);
    }

    public SqlExpression accept(org.xquark.extractor.algebra.GenSqlVisitor visitor)
    {
        return visitor.visit(this);
    }

    public void accept(org.xquark.extractor.algebra.AlgebraVisitor visitor)
    {
        visitor.visit(this);
    }
}
