/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.xfunctions;

import org.xquark.extractor.algebra.Expression;
import org.xquark.extractor.sql.SqlExpression;

/**
3.5.3 xf:starts-with
3.5.3.1 Signature
xf:starts-with(string $operand1, string $operand2) => boolean

xf:starts-with(string $operand1, string $operand2, anyURI $collation) => boolean

3.5.3.2 Parameters
$operand1  static type is string
$operand2  static type is string
$collation  static type is anyURI

3.5.3.3 Semantics
Returns a boolean indicating whether or not the value of $operand1 starts with a string that is
equal to the value of $operand2 according to the collation that is used.

If the value of $operand2 is the zero-length string, then the function returns true. If the value
of $operand1 is the zero-length string and the value of $operand2 is not the zero-length string,
then the function returns false.

If $collation is specified, then the value of $collation must identify a collation that is
supported in the environment in which the function is invoked. Either an absolute URI or a relative
URI may be specified. The collation identified by $collation, while effectively based on the
Unicode Collation Algorithm, has implementation-defined semantics.

If no collation is specified, then the default collation is used. The default collation is
determined according to the rules in TO BE DETERMINED.

3.5.3.4 Examples
xf:starts-with("goldenrod", "gold") returns true.

xf:starts-with("goldenrod", "") returns true.
 */
public final class XfStartsWith extends Xfunction_Boolean
{

	private static final String RCSRevision = "$Revision: 1.3 $";
	private static final String RCSName     = "$Name:  $";

    public XfStartsWith()
    {
      super("XfStartsWith");
    }
    public XfStartsWith( Expression arg0, Expression arg1)
    {
        super("XfStartsWith");
        setArgument(0, arg0);
        setArgument(1, arg1);
    }

    public SqlExpression accept(org.xquark.extractor.algebra.GenSqlVisitor visitor)
    {
        return visitor.visit(this);
    }

    public void accept(org.xquark.extractor.algebra.AlgebraVisitor visitor)
    {
        visitor.visit(this);
    }
}
