/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 XQuark Group.
 * Copyright (C) 2004 XQuark Group.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.DOMUtils;

import java.util.ArrayList;

import org.xquark.schema.SchemaManager;
import org.xquark.xquery.parser.*;
import org.xquark.xquery.parser.util.Constants;
import org.xquark.xquery.typing.TypeVisitor;


public class GetUnknownPathsVisitor extends DefaultParserVisitor {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.11 $";
	private static final String RCSName = "$Name:  $";
	// **********************************************************************
	// * CLASS VARIABLES
	// **********************************************************************
	private XQueryExpression resultExpr = null;
	private SchemaManager schemamanager = null;
	private TypeVisitor typevisitor = null;
	private ArrayList newVars = new ArrayList();
	private ArrayList knownVars = new ArrayList();
	// **********************************************************************
	// * INITIALIZATION
	// **********************************************************************
	/**
	 *
	 */
	public GetUnknownPathsVisitor(SchemaManager schemamanager, TypeVisitor typevisitor) {
		this.schemamanager = schemamanager;
		this.typevisitor = typevisitor;
	}

	public ArrayList getNewVariables() { 
		if (newVars.isEmpty())
			return null;
		return newVars; 
	}
	
	// **********************************************************************
	// * VISITATION
	// **********************************************************************
	public void visit(XQueryExpression arg) throws XQueryException {
		resultExpr = null;
	}

	public void visit(FunctionCall arg) throws XQueryException {
		ArrayList arguments = arg.getArguments();
		if (arguments != null) {
			for (int i = 0; i < arguments.size(); i++) {
				XQueryExpression argi = (XQueryExpression) arguments.get(i);
				argi.accept(this);
				if (resultExpr != null)
					arguments.set(i, resultExpr);
			}
		}
		resultExpr = null;
	}

	public void visit(FLWRExpression arg) throws XQueryException {
		// add all vars to those known
		knownVars.addAll(arg.getVariables());
		for (int i = 0; i < arg.getVariables().size(); i++) {
			Variable vari = (Variable) arg.getVariables().get(i);
			vari.getExpression().accept(this);
			if (resultExpr != null)
				vari.setExpression(resultExpr, false);
		}
		// get where clause
		if (arg.getWhereClause() != null) {
			arg.getWhereClause().accept(this);
			if (resultExpr != null)
				arg.setWhereClause(resultExpr);
		}
//		arg.getReturnClause().accept(this);
//		if (resultExpr != null)
//			arg.setReturnClause(resultExpr);
//		resultExpr = null;
		// retrieve all vars to those known
		knownVars.removeAll(arg.getVariables());
	}

	public void visit(QuantifiedExpression arg) throws XQueryException {
		// add varibales to those known
		knownVars.addAll(arg.getVariables());
		for (int i = 0; i < arg.getVariables().size(); i++) {
			Variable vari = (Variable) arg.getVariables().get(i);
			vari.getExpression().accept(this);
			if (resultExpr != null)
				vari.setExpression(resultExpr, false);
		}
		arg.getConstraintExpresson().accept(this);
		if (resultExpr != null)
			arg.setConstraintExpresson(resultExpr);
		// retrieve all vars to those known
		knownVars.removeAll(arg.getVariables());
	}
	
	public void visit(XQueryBinaryOperatorExpression arg) throws XQueryException {
		arg.getExpression1().accept(this);
		if (resultExpr != null)
			arg.setExpression1(resultExpr);
		arg.getExpression2().accept(this);
		if (resultExpr != null)
			arg.setExpression2(resultExpr);
		resultExpr = null;
	}

	public void visit(LocatedExpression arg) throws XQueryException {
		resultExpr = null;
		if (arg.getStepNum(0).getExpression() instanceof Variable) {
			Variable var = (Variable)arg.getStepNum(0).getExpression();
			if (!knownVars.contains(var)) {
				resultExpr = typevisitor.getStaticContext().createVariable(Constants.DECLARATION_BINDINGTYPE, arg, Variable.CAN_CREATOR);
				newVars.add(resultExpr);
			}
		}
	}

	public void visit(Variable arg) throws XQueryException {
		resultExpr = null;
		if (!knownVars.contains(arg)) {
			resultExpr = typevisitor.getStaticContext().createVariable(Constants.DECLARATION_BINDINGTYPE, arg, Variable.CAN_CREATOR);
			newVars.add(resultExpr);
		}
	}

	public void visit(ExternalVariable arg) throws XQueryException {
		resultExpr = null;
		if (!knownVars.contains(arg)) {
			resultExpr = typevisitor.getStaticContext().createVariable(Constants.DECLARATION_BINDINGTYPE, arg, Variable.CAN_CREATOR);
			newVars.add(resultExpr);
		}
	}

}
