/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator;

import java.lang.reflect.InvocationTargetException;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.xquark.xml.xdbc.*;

public class MediatorDriver implements XMLDriver {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.10 $";
	private static final String RCSName = "$Name:  $";
	
	public static final String MEDIATOR_URL_PREFIX = "xdbc:xquark:mediator:";
	// **********************************************************************
	// * CLASS VARIABLE
	// **********************************************************************
	private static MediatorDriver instance = new MediatorDriver();

	static private Map datasources = Collections.synchronizedMap(new HashMap());

	// ************************************************************************
	// * INITIALISATION
	// ************************************************************************
	private MediatorDriver() {
		XMLDriverManager.registerDriver(this);
	}

	// ************************************************************************
	// * XMLDriver IMPLEMENTATION
	// ************************************************************************

	public boolean acceptsURI(String uri) throws XMLDBCException {
		return uri != null && uri.startsWith(MEDIATOR_URL_PREFIX);
	}

	public String getSpecificPart(String uri) throws XMLDBCException {
		if (acceptsURI(uri)) return uri.substring(MEDIATOR_URL_PREFIX.length());
		else throw new XMLDBCException("Unrecognized URI "+uri);
	}

	/**
	 * To obtain a handle to a data source with the specified URI.
	 * @param uri an URI corresponding to a data source.
	 * @return an instance of the specified data source,
	 *         or null if the driver does not know this type of data source
	 * @throws XMLDBCException if a data source access error occurs.
	 */
	public XMLDataSource getDataSource(String uri) throws XMLDBCException {
		if (!acceptsURI(uri)) return null;
		String confURL = getSpecificPart(uri);
		XMLDataSource source = (XMLDataSource) datasources.get(confURL);
		if (source != null) return source;
        try {
            Class dsImpl = Class.forName("org.xquark.mediator.Mediator");
            if (confURL.length() == 0) {
				source = (XMLDataSource)dsImpl.newInstance();
            } else {
				Class[] paramTypes = new Class[] { String.class };
				Object[] params = new Object[] { confURL };
				source = (XMLDataSource)dsImpl.getConstructor(paramTypes).newInstance(params);
            }
			datasources.put(confURL, source);
			return source;
        } catch (ClassNotFoundException e) {
			return null;
        } catch (IllegalArgumentException e) {
			return null;
        } catch (SecurityException e) {
			return null;
        } catch (InstantiationException e) {
			return null;
        } catch (IllegalAccessException e) {
			return null;
        } catch (InvocationTargetException e) {
			return null;
        } catch (NoSuchMethodException e) {
			return null;
        }
	}

	/**
	 * To obtain a handle to a data source with the specified URI, user name and password.
	 * @param uri a URI corresponding to a data source.
	 * @param user a user name
	 * @param password a user password
	 * @return an instance of the specified data source,
	 *         or null if the driver does not know this type of data source
	 * @throws XMLDBCException if a data source access error occurs.
	 */
	 public XMLDataSource getDataSource(String uri, String user, String password) throws XMLDBCException {
		 return null;
	 }
    
}
