/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator;

import org.xquark.xml.xdbc.*;

public class XDBCWrapper {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.4 $";
	private static final String RCSName = "Name";

	// **********************************************************************
	// * CLASS VARIABLE
	// **********************************************************************
	private Mediator mediator = null;
	private XMLDataSource dataSource = null;
	
	private String name;
	private String driver;
	private String uri;
	private String login = null;
	private String passwd = null;
	private boolean isConnected;


	// ************************************************************************
	// * INITIALISATION
	// ************************************************************************
	/**
	 *
	 */
	public XDBCWrapper(String name, String driver, String connection) {
		this(name, driver, connection, null, null);
	}

	/**
	 *
	 */
	public XDBCWrapper(String name, String driver, String connection, String login, String passwd) {
		this.name = name;
		this.driver = driver;
		this.uri = connection;
		this.login = login;
		this.passwd = passwd;
		this.isConnected = false;
	}
	
	public XDBCWrapper(String name, XMLDataSource source) {
		this.name = name;
		dataSource = source;
	}
	
	void setMediator(Mediator mediator) {
		this.mediator = mediator;
	}

	// ************************************************************************
	// * CONNECTION/DISCONNECTION METHODS
	// ************************************************************************

	public synchronized XMLDataSource getDataSource() throws XMLDBCException {
		if (dataSource == null) {
			if (login != null && passwd != null) {
				dataSource = XMLDriverManager.getDataSource(uri, login, passwd);
			} else {
				dataSource = XMLDriverManager.getDataSource(uri);
			}
		}
		return dataSource;
	}

	/**
	 * Load the accessor driver and open a connection.
	 */
	public XMLConnection getConnection() throws XMLDBCException {
		return getDataSource().getConnection();
	}

	/**
	 * Close the accessor's connection.
	 */
	public void close() {
		isConnected = false;
	}

	// ************************************************************************
	// * GET/SET METHODS
	// ************************************************************************
	/**
	 * Return the name of this accessor.
	 */
	public String getName() {
		return name;
	}
}
