/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.algebra;

import org.xquark.mediator.plan.*;
import org.xquark.mediator.runtime.MediatorException;
import org.xquark.xquery.parser.Variable;
import org.xquark.xquery.parser.XQueryExpression;

/**
 * This class represent a terminal leaf of the dependancy tree on first step
 * of creation (before atomization).
 *
 */
public class AlgArithFunction extends Algebra {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.7 $";
	private static final String RCSName = "$Name:  $";
	// **********************************************************************
	// * CLASS VARIABLES
	// **********************************************************************

	// ************************************************************************
	// * INITIALIZATION
	// ************************************************************************
	/**
	 *
	 * @param typevisitor
	 * @param expression
	 * @param var
	 * @param parent
	 */
	public AlgArithFunction(XQueryExpression expression, Variable var, AlgebraManager depmanager, boolean islet) throws MediatorException {
		super(expression, var, depmanager, islet);
		fillVarLists();
	}

	// ******************************************************************
	// * METHODS
	// ******************************************************************

	// ******************************************************************
	// * CREATE EXECUTION PLAN
	// ******************************************************************
	/**
	 *
	 * @param plan
	 * @return
	 */
	public Operator createOperator(ExecutionPlan plan) throws MediatorException {

		//if (true) throw new MediatorException("createAlgebra not supported in DepArithFunction");
		Operator operator = null;

		if (algChildren != null) {
			for (int i = 0; i < algChildren.size(); i++) {
				// getting child
				Algebra depchildi = (Algebra) algChildren.get(i);
//				if (log.isDebugEnabled()) {
//					log.debug("*********** DEPARITHFUNCTION CHILD STUFF *************************");
//					log.debug("ClassName = " + depchildi.getClass().getName());
//					log.debug("is let = " + depchildi.isLet());
//					log.debug("depchildiexpr = " + depchildi.getExpression());
//					log.debug("depchildi sources = " + depchildi.getExpression().getSourceNames());
//					log.debug("vari = " + depchildi.getVariables());
//					log.debug("definition vars depending on = " + depchildi.getVarsDependingOn());
//					log.debug("where vars depending on = " + depchildi.getVarsWhereOn());
//					log.debug("vars referenced by = " + depchildi.getVarsReferencedBy());
//					log.debug("children = " + depchildi.getChildren());
//				}
				if (operator == null)
					operator = (Operator) depchildi.createOperator(plan);
				else
					operator = new OpMerge(plan, null, operator, (Operator) depchildi.createOperator(plan));
			}
			operator = new OpNotSource(plan, null, (Variable) variables.get(0), operator, this.islet);
		} else
			operator = new OpEval(plan, this, null, hasIdentifier);

		operator.isLet(islet);
		return operator;
	}

	public Object clone() throws CloneNotSupportedException {
		AlgArithFunction newobj = (AlgArithFunction) super.clone();
		return newobj;
	}

	public void execute(ExecutionPlan plan) throws MediatorException {}

	// ******************************************************************
	// * DEBUGGING
	// ******************************************************************
}
