/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.algebra;

import org.xquark.mediator.plan.*;
import org.xquark.mediator.plan.primitivefunctions.OpFuncDISTINCT_VALUES;
import org.xquark.mediator.runtime.MediatorException;
import org.xquark.xquery.parser.*;
import org.xquark.xquery.parser.primitivefunctions.fnfunctions.FunctionDISTINCT_VALUES;

/**
 * This class represent a terminal leaf of the dependancy tree on first step
 * of creation (before atomization).
 *
 */
public class AlgFunction extends Algebra {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.7 $";
	private static final String RCSName = "$Name:  $";
	// **********************************************************************
	// * CLASS VARIABLES
	// **********************************************************************
	/**
	 * references on Functions (will be filled later when called locateFunctions)
	 */
	//private ArrayList sources ;
	//private ArrayList depnodes ;

	// ************************************************************************
	// * INITIALIZATION
	// ************************************************************************
	/**
	 *
	 * @param typevisitor
	 * @param expression
	 * @param var
	 * @param parent
	 */
	public AlgFunction(XQueryExpression expression, Variable var, AlgebraManager depmanager, boolean islet) throws MediatorException {
		super(expression, var, depmanager, islet);
		//paths = depmanager.getPathsfromVar(var);
		// fill var lists
		fillVarLists();
	}

	// ******************************************************************
	// * METHODS
	// ******************************************************************

	public void ParentHasIdentifier() {
		hasIdentifier = true;
	}

	// ******************************************************************
	// * CREATE EXECUTION PLAN
	// ******************************************************************
	/**
	 *
	 * @param plan
	 * @return
	 */
	public Operator createOperator(ExecutionPlan plan) throws MediatorException {

		//if (true) throw new MediatorException("createAlgebra not supported in DepFunction");
		
		Operator algebra = null;
		if (algChildren != null) {
			for (int i = 0; i < algChildren.size(); i++) {
			// getting child
				Algebra depchildi = (Algebra) algChildren.get(i);
//				if (log.isDebugEnabled()) {
//				log.debug("*********** DEPFUNCTION CHILD STUFF *************************");
//				log.debug("ClassName = " + depchildi.getClass().getName());
//				log.debug("is let = " + depchildi.isLet());
//				log.debug("depchildiexpr = " + depchildi.getExpression());
//				log.debug("depchildi sources = " + depchildi.getExpression().getSourceNames());
//				log.debug("vari = " + depchildi.getVariables());
//				log.debug("definition vars depending on = " + depchildi.getVarsDependingOn());
//				log.debug("where vars depending on = " + depchildi.getVarsWhereOn());
//				log.debug("vars referenced by = " + depchildi.getVarsReferencedBy());
//				log.debug("children = " + depchildi.getChildren());
//				}
				if (algebra == null)
					algebra = (Operator) depchildi.createOperator(plan);
				else
					algebra = new OpMerge(plan, null, algebra, (Operator) depchildi.createOperator(plan));
			}

			// TODO: later add the Agregate operator here.
	
			if (expression instanceof AggregateFunctionCall)
				algebra = new OpAggregateFunc(plan, expression, algebra);
			else if (expression instanceof FunctionDISTINCT_VALUES)
				//algebra = new AlgAggregateFunc(plan, expression, algebra);
				algebra = new OpFuncDISTINCT_VALUES(plan, expression, algebra);
			else if (expression instanceof PrimitiveFunctionCall)
				algebra = new OpFunc(plan, expression, algebra);
			else
				throw new MediatorException("Not yet implemented : create algebra for function -> " + expression.getClass().getName());
			algebra = new OpNotSource(plan, null, (Variable) variables.get(0), algebra, this.islet);
		}
		else
			algebra = new OpEval(plan,this,null,hasIdentifier);

		/*
		if (expression instanceof FunctionAVG)
		    root = new AlgFuncAVG(plan, expression, tmproot) ;
		else if (expression instanceof FunctionCEILING)
		    root = new AlgFuncCEILING(plan, expression, tmproot) ;
		else if (expression instanceof FunctionCOLLECTION)
		    root = new AlgFuncCOLLECTION(plan, expression, tmproot) ;
		else if (expression instanceof FunctionCONCAT)
		    root = new AlgFuncCONCAT(plan, expression, tmproot) ;
		else if (expression instanceof FunctionCONTAINS)
		    root = new AlgFuncCONTAINS(plan, expression, tmproot) ;
		else if (expression instanceof FunctionCOUNT)
		    root = new AlgFuncCOUNT(plan, expression, tmproot) ;
		else if (expression instanceof FunctionCURRENT_DATETIME)
		    root = new AlgFuncCURRENT_DATETIME(plan, expression, tmproot) ;
		else if (expression instanceof FunctionDATA)
		    root = new AlgFuncDATA(plan, expression, tmproot) ;
		else if (expression instanceof FunctionDATE)
		    root = new AlgFuncDATE(plan, expression, tmproot) ;
		else if (expression instanceof FunctionDATETIME)
		    root = new AlgFuncDATETIME(plan, expression, tmproot) ;
		else if (expression instanceof FunctionDECIMAL)
		    root = new AlgFuncDECIMAL(plan, expression, tmproot) ;
		else if (expression instanceof FunctionDISTINCT_VALUES)
		    root = new AlgFuncDISTINCT_VALUES(plan, expression, tmproot) ;
		else if (expression instanceof FunctionDOUBLE)
		    root = new AlgFuncDOUBLE(plan, expression, tmproot) ;
		else if (expression instanceof FunctionEMPTY)
		    root = new AlgFuncEMPTY(plan, expression, tmproot) ;
		else if (expression instanceof FunctionENDS_WITH)
		    root = new AlgFuncENDS_WITH(plan, expression, tmproot) ;
		else if (expression instanceof FunctionEXISTS)
		    root = new AlgFuncEXISTS(plan, expression, tmproot) ;
		else if (expression instanceof FunctionFALSE)
		    root = new AlgFuncFALSE(plan, expression, tmproot) ;
		else if (expression instanceof FunctionFLOOR)
		    root = new AlgFuncFLOOR(plan, expression, tmproot) ;
		else if (expression instanceof FunctionINTEGER)
		    root = new AlgFuncINTEGER(plan, expression, tmproot) ;
		else if (expression instanceof FunctionLOWER_CASE)
		    root = new AlgFuncLOWER_CASE(plan, expression, tmproot) ;
		else if (expression instanceof FunctionMATCH)
		    root = new AlgFuncMATCH(plan, expression, tmproot) ;
		else if (expression instanceof FunctionMAX)
		    root = new AlgFuncMAX(plan, expression, tmproot) ;
		else if (expression instanceof FunctionMIN)
		    root = new AlgFuncMIN(plan, expression, tmproot) ;
		else if (expression instanceof FunctionNAME)
		    root = new AlgFuncNAME(plan, expression, tmproot) ;
		else if (expression instanceof FunctionNOT)
		    root = new AlgFuncNOT(plan, expression, tmproot) ;
		else if (expression instanceof FunctionNUMBER)
		    root = new AlgFuncNUMBER(plan, expression, tmproot) ;
		else if (expression instanceof FunctionROUND)
		    root = new AlgFuncROUND(plan, expression, tmproot) ;
		else if (expression instanceof FunctionSTARTS_WITH)
		    root = new AlgFuncSTARTS_WITH(plan, expression, tmproot) ;
		else if (expression instanceof FunctionSTRING)
		    root = new AlgFuncSTRING(plan, expression, tmproot) ;
		else if (expression instanceof FunctionSTRING_LENGTH)
		    root = new AlgFuncSTRING_LENGTH(plan, expression, tmproot) ;
		else if (expression instanceof FunctionSUBSTRING)
		    root = new AlgFuncSUBSTRING(plan, expression, tmproot) ;
		else if (expression instanceof FunctionSUM)
		    root = new AlgFuncSUM(plan, expression, tmproot) ;
		else if (expression instanceof FunctionTIME)
		    root = new AlgFuncTIME(plan, expression, tmproot) ;
		else if (expression instanceof FunctionTRUE)
		    root = new AlgFuncTRUE(plan, expression, tmproot) ;
		else if (expression instanceof FunctionUPPER_CASE)
		    root = new AlgFuncUPPER_CASE(plan, expression, tmproot) ;
		*/

		algebra.isLet(islet);
		return algebra;
	}

	public Object clone() throws CloneNotSupportedException {
		AlgFunction newobj = (AlgFunction) super.clone();
		return newobj;
	}

	public void execute(ExecutionPlan plan) throws MediatorException {}

	// ******************************************************************
	// * DEBUGGING
	// ******************************************************************
}
