/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.algebra;

import java.util.ArrayList;

import org.xquark.mediator.decomposer.VariableDependencies;
import org.xquark.mediator.plan.*;
import org.xquark.mediator.runtime.MediatorException;
import org.xquark.xquery.parser.*;

public class AlgLocated extends Algebra {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.6 $";
	private static final String RCSName = "$Name:  $";

	// **********************************************************************
	// * CLASS VARIABLES
	// **********************************************************************
	
	// ************************************************************************
	// * INITIALIZATION
	// ************************************************************************

	/**
	 *
	 * @param typevisitor
	 * @param expression
	 * @param var
	 * @param parent
	 */
	public AlgLocated(LocatedExpression expression, Variable addvar, Variable var, AlgebraManager depmanager, boolean islet) {
		super(expression, var, depmanager, islet);
		this.addDependSourceVariable(addvar);

	}

	// ******************************************************************
	// * METHODS
	// ******************************************************************

	public void ParentHasIdentifier() {
		hasIdentifier = true;
	}

	// ******************************************************************
	// * CREATE EXECUTION PLAN
	// ******************************************************************

	/**
	 *
	 * @param plan
	 * @return
	 */
	public Operator createOperator(ExecutionPlan plan) throws MediatorException {
		Variable var = (Variable) variables.get(0);
		//Algebra algebra = new AlgNotSource(plan, expression, var, isLet(), this.hasIdentifier);
		Operator algebra = new OpNotSource(plan, this);
		
		if (parent != null && parent instanceof AlgFLWR) {
			AlgFLWR parentdep = (AlgFLWR)parent;
			VariableDependencies vardepend = parentdep.getVarDependencies();
			ArrayList tmplist = null;
			if (vardepend != null) {
				// change LARS 11/03/04 all condition -> complex condition
				ArrayList conds = vardepend.getAllComplexConditionsWithVar(var);
				if (conds != null) 
					for (int i=0;i<conds.size();i++) {
						ArrayList vars = vardepend.getVarsOfCond((XQueryExpression)conds.get(i));
						if (algManager.getScopeVariables().containsAll(vars)) {
							if (tmplist == null)
								tmplist = new ArrayList();
							tmplist.add(conds.get(i));
						}				
					}
			}
			if (tmplist != null) {
				XQueryExpression tmpexpr = null;
				if (tmplist.size() == 1) {
					tmpexpr = (XQueryExpression)tmplist.get(0);
				}
				else {
					try {
						tmpexpr = new BinOpANDExpression((XQueryExpression) tmplist.get(0), (XQueryExpression) tmplist.get(1), expression.getParentModule());
						for (int i = 2; i < tmplist.size(); i++)
							tmpexpr = new BinOpANDExpression(tmpexpr, (XQueryExpression) tmplist.get(i), expression.getParentModule());
					} catch (XQueryException e) {
						throw new MediatorException("Could not build where clause", e);
					}
				}
				algebra = new OpRestrict(plan,tmpexpr,algebra);				
			}
		}
		return algebra;

		
		//algebra.isLet(islet);
		//algebra.setDepends(depends);
		//return null;
	}

	public Object clone() throws CloneNotSupportedException {
		AlgLocated newobj = (AlgLocated) super.clone();

		return newobj;
	}

	public void execute(ExecutionPlan plan) throws MediatorException {}

	// ******************************************************************
	// * DEBUGGING
	// ******************************************************************

}
