/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 XQuark Group.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.algebra;

import java.util.ArrayList;

import org.xquark.mediator.decomposer.GetVariablePathsVisitor;
import org.xquark.mediator.decomposer.IsolateWhereVisitor;
import org.xquark.mediator.plan.ExecutionPlan;
import org.xquark.mediator.plan.OpSourceEval;
import org.xquark.mediator.plan.Operator;
import org.xquark.mediator.runtime.MediatorException;
import org.xquark.xquery.parser.*;
import org.xquark.xquery.typing.*;
import org.xquark.xquery.typing.QTypeAtom;

/**
 * This class represent the terminal leaf of the dependancy tree.
 * It is generated after atomization (and is resulted from the transformation
 * of DepSource that disappear).
 *
 */
public class AlgSourceEval extends Algebra {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.10 $";
	private static final String RCSName = "$Name:  $";
	// **********************************************************************
	// * CLASS VARIABLES
	// **********************************************************************
	//private boolean hasIdentifier = false;
	// ************************************************************************
	// * INITIALIZATION
	// ************************************************************************
	/**
	 * Create a node from a variable and an expression.
	 *
	 * To create the node, we take the original expression, and we
	 * extract everything relative to variable given by the leaf.
	 * from that, we create a new flower expression that will be associated
	 * to this DepNodeSource, and the old XQueryExpression that is
	 * referenced by it's DepNode is simplified.
	 * (bon, en gros, a sert a faire une optimisation en faisant
	 * redescendre les restrictions (et plus tard les projection) vers les
	 * sources.
	 *
	 * @param typevisitor
	 * @param origexp
	 * @param varleaf
	 * @param depmanager
	 */
	public AlgSourceEval(XQueryExpression origexp, ArrayList variables, AlgebraManager depmanager, boolean islet) throws MediatorException {
		// to satisfy the super constructor, but expression and variable will be changed in following step.
		super(origexp, variables, depmanager, islet);
	}

	public AlgSourceEval(XQueryExpression origexp, Variable varleaf, AlgebraManager depmanager, boolean islet) throws MediatorException {
		// to satisfy the super constructor, but expression and variable will be changed in following step.
		super(origexp, varleaf, depmanager, islet);
	}

	// ******************************************************************
	// * METHODS
	// ******************************************************************

	public void ParentHasIdentifier() {
		hasIdentifier = true;
	}

	/**
	 * Create the FLWRExpression like this :
	 * - F : is the variable varleaf that we consider with no changes
	 * (even the expression associated to it is kept ;
	 * - L : there is no Let at this point ;
	 * - W : is the major part, we must scan the origexp to see what
	 *       to keep
	 * - R : for now, we return just the variable, but later, when
	 *       we will care about prjection, we will make good
	 *       LocatedExpression with it.
	 * @param origexp
	 * @param varleaf
	 */

	private void createFLWRExpression() throws MediatorException {
		if (!(expression instanceof FLWRExpression))
			throw new MediatorException("DepNodeEval : " + expression + " is not a FLWR.");

		try {
			FLWRExpression origflwr = (FLWRExpression) expression;

			// Variables
			ArrayList newvariables = (ArrayList)variables.clone();
			newvariables.addAll(this.parent.getVarsDependingOn());
			newvariables.addAll(this.getVarsDependingOn());
			//newvariables.addAll(this.parent.getVarsWhereOn());
			newvariables.addAll(this.getVarsWhereOn());

			// Where
			XQueryExpression newWhereClause = null;
			if (origflwr.getWhereClause() != null) {
				newWhereClause = (XQueryExpression) origflwr.getWhereClause().clone();
				IsolateWhereVisitor visitor = new IsolateWhereVisitor(newvariables);
				newWhereClause.accept(visitor);
				newWhereClause = visitor.getIsolatedExpression();
				// erase isolated where clause from original where clause
				 ((FLWRExpression) expression).setWhereClause(visitor.getRemainingExpression());
			}
			
			// update paths because where clause is not calculate in flwr
			if (newWhereClause != null) {
				GetVariablePathsVisitor gvpv = new GetVariablePathsVisitor(true);
				newWhereClause.accept(gvpv);
				ArrayList tmplist = gvpv.getPaths();
				if (tmplist != null)
					for (int i = 0; i < tmplist.size(); i++)
						if (!this.algManager.getPathsList().contains(tmplist.get(i).toString()))
							paths.add(tmplist.get(i));
			}
			// OrderBy
			ArrayList newOrderbys = null;
//			if (origflwr.getOrderBy() != null) {
//				newOrderbys = (ArrayList) origflwr.getOrderBy().clone();
//				for (int i = 0, j = 0; i < newOrderbys.size(); i++, j++) {
//					XQueryExpression exp = (XQueryExpression) newOrderbys.get(i);
//					IsolateWhereVisitor visitor = new IsolateWhereVisitor(variables);
//					exp.accept(visitor);
//					exp = visitor.getIsolatedExpression();
//					if (exp != null) {
//						newOrderbys.set(i, exp);
//						//						if (visitor.getRemainingExpression() != null)
//						//							origflwr.getOrderBy().set(j,visitor.getRemainingExpression());
//						//						else {
//						//							origflwr.getOrderBy().remove(j);
//						//							j--;
//						//						}
//					} else {
//						newOrderbys.remove(i);
//						i--;
//					}
//				}
//				if (newOrderbys.isEmpty())
//					newOrderbys = null;
//				//				if (origflwr.getOrderBy().isEmpty())
//				//					origflwr.setOrderBy(null);
//				//if (origOrderby.isEmpty()) origflwr.setOrderBy(null);
//			}
			// create new FLWRExpression
			// keep only single variable
			// origflwr.setVariables(vars);
			// recalculate sources if necessary
			//if (vars.size() != ((FLWRExpression)expression).getVariables().size()) {
			//}
			// make return expression
			FLWRExpression newFlwr = null;
//			if (doReturn) {
//				returncreated = true;
				if (newWhereClause != null && newWhereClause.getQType() == null)
					newWhereClause.setQType(new QTypeAtom(this.algManager.getPlan().getSchemaManager().getBooleanType()));
				newFlwr = new FLWRExpression(variables, newOrderbys, newWhereClause, new XQueryVoid(expression.getParentModule()), expression.getParentModule());
				newFlwr.setParentModule(origflwr.getParentModule());
//			} else
//				newFlwr = new FLWRExpression(variables, newOrderbys, newWhereClause, new XQueryVoid(null), null);
//			flwrcreated = true;
			expression = newFlwr;
		} catch (TypeException te) {
			throw new MediatorException("DepNodeSource.createFLWRExpression : " + te.getMessage(), te);
		} catch (CloneNotSupportedException e) {
			throw new MediatorException("DepNodeSource.createFLWRExpression : " + e.getMessage(), e);
		} catch (XQueryException e) {
			throw new MediatorException("DepNodeSource.createFLWRExpression : " + e.getMessage(), e);
		}
	}

	// ******************************************************************
	// * ATOMIZE
	// ******************************************************************
	/**
	 *
	 * @return
	 */
	/*
	public ArrayList atomize () throws MediatorException
	{
			throw new MediatorException ("atomize should never been called at this step of the algorithm") ;
	}
	 */
	// ******************************************************************
	// * INFORMATION
	// ******************************************************************
	/**
	 * See what sources are associated to this DepNodeSources
	 */

	// ******************************************************************
	// * CREATE EXECUTION PLAN
	// ******************************************************************
	/**
	 *
	 * @param plan
	 * @return
	 */
	public Operator createOperator(ExecutionPlan plan) throws MediatorException {
		createFLWRExpression();
		Operator algebra = new OpSourceEval(plan, expression, paths, hasIdentifier);
		algebra.isLet(islet);		
		return algebra;
	}

	public Object clone() throws CloneNotSupportedException {
		AlgSourceEval newobj = (AlgSourceEval) super.clone();
		newobj.hasIdentifier = this.hasIdentifier;
		return newobj;
	}

	// ******************************************************************
	// * GET/SET METHODS
	// ******************************************************************
	/**
	 *
	 * @return
	 */
	//public ArrayList getPaths() { return paths ; }

	/**
	 *
	 * @return
	 */
	// public ArrayList getVarsThatDepend() throws MediatorException { return varsthatdepend ; }

	// ******************************************************************
	// * DEBUGGING
	// ******************************************************************
	/**
	 *
	 * @return
	 */
//	public String toCompleteString(int indent) {
//		StringBuffer buf = new StringBuffer();
//
//		buf.append(Utils.makeIndent(indent) + "<NodeEval variables=\"" + variables + " sources=\"" + getSources() + "\" let=" + isLet() + "\" depends=" + depends + "\" mono source=" + isMonoSource() + "\">\n");
//		buf.append(Utils.makeIndent(indent + 1) + "<Expression>\n");
//		buf.append(Utils.makeIndent(indent + 2) + expression + "\n");
//		buf.append(Utils.makeIndent(indent + 1) + "</Expression>\n");
//		//buf.append("\t\t<Function>" + funcToString() + "</Function>\n") ;
//		if (depchildren != null && !depchildren.isEmpty()) {
//			buf.append(Utils.makeIndent(indent + 1) + "<Leaves>\n");
//			for (int i = 0; i < depchildren.size(); i++) {
//				buf.append(Utils.makeIndent(indent + 2) + "<Leave>\n");
//				DepNode depchild = (DepNode) depchildren.get(i);
//				buf.append(depchild.toCompleteString(indent + 3));
//				buf.append(Utils.makeIndent(indent + 2) + "</Leave>\n");
//			}
//			buf.append(Utils.makeIndent(indent + 1) + "</Leaves>\n");
//		}
//		if (referenceDefinitionVars != null && !referenceDefinitionVars.isEmpty()) {
//			buf.append(Utils.makeIndent(indent + 1) + "<referenceDefinitionVars>\n");
//			for (int i = 0; i < referenceDefinitionVars.size(); i++) {
//				Variable var = (Variable) referenceDefinitionVars.get(i);
//				buf.append(Utils.makeIndent(indent + 2) + "<Var>" + var + "</Var>\n");
//			}
//			buf.append(Utils.makeIndent(indent + 1) + "</referenceDefinitionVars>\n");
//		}
//		if (dependDefinitionVars != null && !dependDefinitionVars.isEmpty()) {
//			buf.append(Utils.makeIndent(indent + 1) + "<DependingOn>\n");
//			for (int i = 0; i < dependDefinitionVars.size(); i++) {
//				Variable var = (Variable) dependDefinitionVars.get(i);
//				buf.append(Utils.makeIndent(indent + 2) + "<Var>" + var + "</Var>\n");
//			}
//			buf.append(Utils.makeIndent(indent + 1) + "</dependDefinitionVars>\n");
//		}
//		if (paths != null) {
//			buf.append(Utils.makeIndent(indent + 1) + "<Paths>\n");
//			for (int i = 0; i < paths.size(); i++)
//				buf.append(Utils.makeIndent(indent + 2) + "<Path>" + paths.get(i) + "</Path>\n");
//			buf.append(Utils.makeIndent(indent + 1) + "</Paths>\n");
//		}
//		buf.append(Utils.makeIndent(indent) + "</NodeEval>\n");
//		return buf.toString();
//	}

	public void execute(ExecutionPlan plan) throws MediatorException {}

}
