/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * Copyright (C) 2003 XQuark Group.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.decomposer;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Stack;

import org.xquark.mediator.algebra.*;
import org.xquark.mediator.runtime.MediatorException;
import org.xquark.xquery.parser.*;
import org.xquark.xquery.parser.primitivefunctions.fnfunctions.FunctionCOLLECTION;
import org.xquark.xquery.parser.primitivefunctions.fnfunctions.FunctionDOC;
import org.xquark.xquery.parser.util.Constants;
import org.xquark.xquery.typing.*;

/**
 * This abstract represent a Node/Leaf use in the construction
 * of the dependancy trees.
 *
 */

/** to be commented ... : objectif of this class
 * <code>AlgebraStructureVisitor</code> visit :
 * 1. call <code>DepManager</code> to create Dependence tree.
 * 2. */

public class AlgebraStructureVisitor extends DefaultParserVisitor {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.10 $";
	private static final String RCSName = "Name";

	// **********************************************************************
	// * CLASS VARIABLES
	// **********************************************************************
	private final Integer IN_LOCATED = new Integer(1);
	private final Integer IN_DEFINITION = new Integer(2);
	private final Integer IN_FLWR = new Integer(3);
	private final Integer IN_WHERE = new Integer(4);
	private final Integer IN_FUNCTION = new Integer(5);
	private final Integer IN_MAIN_DEFINITION = new Integer(6);
	private final Integer IN_SEQUENCE = new Integer(7);
	//private final Integer IN_PREDICATE = new Integer(3);

	// elements passed at construction time
	private AlgebraManager depmanager = null;
	private HashMap parentvariable = null;

	// private temporary elements
	private Stack varStack = new Stack();
	private AlgFLWR currentDepFLWR = null;
	private Algebra currentDepNode = null;
	private TypeVisitor typevisitor = null;
	private Stack stateStack = new Stack();

	private boolean ongoingLet = false;
	private LocatedExpression ongoingLocatedExpression = null;
	//private ArrayList currentSources = null;

	// ************************************************************************
	// * INITIALIZATION
	// ************************************************************************
	public AlgebraStructureVisitor(AlgebraManager depmanager) throws XQueryException {
		this.depmanager = depmanager;
		this.typevisitor = depmanager.getTypeVisitor();
		this.parentvariable = new HashMap();
		stateStack.push(IN_FLWR);
	}

	// ************************************************************************
	// * GET/SET METHODS
	// ************************************************************************

	private void addLeaf(Algebra leaf) throws MediatorException {
		if (currentDepNode != null)
			currentDepNode.addLeaf(leaf);
		else if (currentDepFLWR != null)
			currentDepFLWR.addLeaf(leaf);
	}

	// ************************************************************************
	// * METHODS
	// ************************************************************************

	public void visit(AggregateFunctionCall arg) throws XQueryException {
		Integer state = (Integer) stateStack.peek();
		if (state == IN_DEFINITION /*|| state == IN_FUNCTION*/
			) {
			AlgFunction depfunction = depmanager.addFunction(arg, (state == IN_DEFINITION) ? (Variable) varStack.peek() : null, true);
			addLeaf(depfunction);
			ongoingLet = false;
			ArrayList arguments = arg.getArguments();
			if (arguments != null) {
				Algebra tmpDepNode = currentDepNode;
				currentDepNode = depfunction;
				stateStack.push(IN_FUNCTION);
				for (int i = 0; i < arguments.size(); i++)
					arg.getArgument(i).accept(this);
				stateStack.pop();
				currentDepNode = tmpDepNode;
			}
		}
	}


	public void visit(ListOpArithExpression arg) throws XQueryException {
		Integer state = (Integer) stateStack.peek();
		if (state == IN_DEFINITION /*|| state == IN_FUNCTION*/
			) {
			AlgArithFunction deparithfunction = depmanager.addArithFunction(arg, (state == IN_DEFINITION) ? (Variable) varStack.peek() : null, true);
			addLeaf(deparithfunction);
			ongoingLet = false;
			Algebra tmpDepNode = currentDepNode;
			currentDepNode = deparithfunction;
			stateStack.push(IN_FUNCTION);
			arg.getExpression1().accept(this);
			arg.getExpression2().accept(this);
			stateStack.pop();
			currentDepNode = tmpDepNode;
		}
	}

	public void visit(ListOpUNIONExpression arg) throws XQueryException {
		Integer state = (Integer) stateStack.peek();
		if (state == IN_DEFINITION /*|| state == IN_FUNCTION*/
			) {
			AlgUnion depunion = depmanager.addUnion(arg, (state == IN_DEFINITION) ? (Variable) varStack.peek() : null, true);
			addLeaf(depunion);
			ongoingLet = false;
			Algebra tmpDepNode = currentDepNode;
			currentDepNode = depunion;
			stateStack.push(IN_FUNCTION);
			arg.getExpression1().accept(this);
			arg.getExpression2().accept(this);
			stateStack.pop();
			currentDepNode = tmpDepNode;
		}
	}

	public void visit(FLWRExpression arg) throws XQueryException {

		ArrayList variables = arg.getVariables();
		Integer currentstate = (Integer) stateStack.peek();
		//currentSources = null;

		Algebra tmpDepNode = currentDepNode;
		if (currentstate == IN_DEFINITION) {
			AlgFLWR depflwr = depmanager.addFlwr(arg, (Variable) varStack.peek(), ongoingLet);
			addLeaf(depflwr);
			currentDepNode = depflwr;
			ongoingLet = false;
		} else if (currentstate == IN_FUNCTION) {
			AlgFLWR depflwr = depmanager.addFlwr(arg, null, ongoingLet);
			addLeaf(depflwr);
			currentDepNode = depflwr;
			ongoingLet = false;
		}

		// TEST if sources is only one
		/*
		//		if (sources != null && sources.size() == 1) {
		//			for (int i=0;i<variables.size();i++) {
		//				Variable vari = (Variable)variables.get(i);
		//				Variable parentvari = (Variable)parentvariable.get(vari);
		//				if (parentvari != null) {
		//					currentDepMem.setDependency((DepMem) depmanager.getMapNode(parentvari));
		//					variables.remove(i);
		//					i--;
		//				}
		//				else {
		//					vari.setParentFLWRExpression(arg);
		//					parentvariable.put(vari,varStack.firstElement());
		//				}
		//			}
		//			//Utils.fillSources(arg,new UnitEnvironment(),this.depmanager.getMetaData());
		//			DepNodeSource newsource = depmanager.addNodeSource(arg,arg.getVariables(), ongoingLet);
		//			addLeaf(newsource);
		//			this.currentDepMem.setMonoSource(true);
		//			return;
		//		}
		*/
		/*
		ArrayList removedVariables = new ArrayList();
		for (int i=0;i<variables.size();i++) {
		    Variable vari = (Variable)variables.get(i);
		    Variable parentvari = (Variable)parentvariable.get(vari);
		    if (parentvari != null) {
		        currentDepFLWR.setDependency((DepFLWR) depmanager.getMapNode(parentvari));
		        removedVariables.add(vari);
		        variables.remove(i);
		        i--;
		    }
		}
		// remove conditions from where clause implicating only removed variables
		if (arg.getWhereClause() != null && !removedVariables.isEmpty()) {
		    IsolateWhereVisitor visitor = new IsolateWhereVisitor(removedVariables) ;
		    try {
		        arg.getWhereClause().accept(visitor) ;
		    }
		    catch (XQueryException e) {
		        throw new MediatorException("Can't isolate where expression : " + arg.getWhereClause().pprint(), e) ;
		    }
		    arg.setWhereClause(visitor.getRemainingExpression()) ;
		}
		*/
		// TEST if sources is only one
		/*
		Utils.fillSources(arg,this.depmanager.getMetaData());
		
		ArrayList sources = arg.getSources();
		if (sources != null && sources.size() == 1) {
			for (int i=0;i<variables.size();i++) {
				Variable vari = (Variable)variables.get(i);
				vari.setParentFLWRExpression(arg);
				parentvariable.put(vari,varStack.firstElement());
			}
			//Utils.fillSources(arg,new UnitEnvironment(),this.depmanager.getMetaData());
			DepNodeSource newsource = depmanager.addNodeSource(arg,arg.getVariables(), ongoingLet);
			addLeaf(newsource);
			if (currentstate == IN_MAIN_DEFINITION) currentDepFLWR.setMonoSource(true);
			else if (currentstate == IN_DEFINITION) currentDepNode = tmpDepNode;
			return;
		}
		*/
		for (int i = 0; i < variables.size(); i++) {
			Variable vari = (Variable) variables.get(i);
			vari.setParentFLWRExpression(arg);
			stateStack.push(IN_FLWR);
			vari.accept(this);
			stateStack.pop();
		}

		//		ArrayList varsOnTheSameSource = new ArrayList();
		//		String lastSource = null;
		//		ArrayList sourcesOfvari = null;
		//		for (int i = 0; i < variables.size(); i++) {
		//			Variable vari = (Variable)variables.get(i);
		//			vari.setParentFLWRExpression(arg);
		//			sourcesOfvari = vari.getSources();
		//
		//			boolean isMonoSource = (null != sourcesOfvari && 1 == sourcesOfvari.size());
		//			if (isMonoSource && sourcesOfvari.get(0).equals(lastSource)) {
		//				/* vari is defined on the same source as the precedant variable*/
		//				varsOnTheSameSource.add(vari);
		//			}
		//			else {
		//				if (!varsOnTheSameSource.isEmpty()) {
		//					/* Generate DepNodeSource for accumulated variables*/
		//					DepNodeSource newsource = depmanager.addNodeSource(arg,varsOnTheSameSource, ongoingLet);
		//					addLeaf(newsource);
		//
		//					varsOnTheSameSource = new ArrayList();
		//
		//				}
		//				/**/
		//				if (isMonoSource) {
		//					lastSource = (String)sourcesOfvari.get(0);
		//					varsOnTheSameSource.add(vari);
		//				}
		//				else {
		//					stateStack.push(IN_FLWR) ;
		//					vari.accept(this);
		//					stateStack.pop() ;
		//				}
		//			}
		//		}
		//
		//		/* treat last group of variables if there is */
		//		if (!varsOnTheSameSource.isEmpty()) {
		//			/* Generate DepNodeSource for accumulated variables*/
		//			DepNodeSource newsource = depmanager.addNodeSource(arg,varsOnTheSameSource, ongoingLet);
		//			addLeaf(newsource);
		//		}

		XQueryExpression whereclause = arg.getWhereClause();
		if (whereclause != null) {
			stateStack.push(IN_WHERE);
			whereclause.accept(this);
			stateStack.pop();
		}

		//ArrayList orderbys = arg.getOrderBy() ;
		if (currentstate == IN_DEFINITION)
			currentDepNode = tmpDepNode;

	}

	public void visit(FunctionCOLLECTION arg) throws XQueryException {

		Integer state = (Integer) stateStack.peek();
		AlgSource newsource = null;
		if (state == IN_DEFINITION)
			newsource = depmanager.addSource(arg, (Variable) varStack.peek(), ongoingLet);
		else if (state == IN_LOCATED)
			newsource = depmanager.addSource(ongoingLocatedExpression, (Variable) null, ongoingLet);
		else
			newsource = depmanager.addSource(arg, (Variable) null, ongoingLet);
		//if (!ongoingLet)
		addLeaf(newsource);
		ongoingLet = false;
	}

	public void visit(FunctionDOC arg) throws XQueryException {

		Integer state = (Integer) stateStack.peek();
		AlgSourceEval newsource = null;
		if (state == IN_DEFINITION)
			newsource = depmanager.addSourceEval(arg, (Variable) varStack.peek(), ongoingLet);
		else if (state == IN_FUNCTION)
			newsource = depmanager.addSourceEval(arg, (Variable) varStack.peek(), ongoingLet);
		else if (state == IN_LOCATED)
			newsource = depmanager.addSourceEval(ongoingLocatedExpression, (Variable) null, ongoingLet);
		else
			newsource = depmanager.addSourceEval(arg, (Variable) null, ongoingLet);
		//if (!ongoingLet)
		addLeaf(newsource);
		ongoingLet = false;
	}

	public void visit(PrimitiveFunctionCall arg) throws XQueryException {

		Integer state = (Integer) stateStack.peek();
		if (state == IN_DEFINITION /*|| state == IN_FUNCTION*/
			) {
			AlgFunction depfunction = depmanager.addFunction(arg, (state == IN_DEFINITION) ? (Variable) varStack.peek() : null, ongoingLet);
			addLeaf(depfunction);
			ongoingLet = false;
			ArrayList arguments = arg.getArguments();
			if (arguments != null) {
				Algebra tmpDepNode = currentDepNode;
				currentDepNode = depfunction;
				stateStack.push(IN_FUNCTION);
				for (int i = 0; i < arguments.size(); i++)
					arg.getArgument(i).accept(this);
				stateStack.pop();
				currentDepNode = tmpDepNode;
			}
		}

	}

	public void visit(LocatedExpression arg) throws XQueryException {
		Integer state = (Integer) stateStack.peek();
		ArrayList steps = arg.getSteps();
		if (state != IN_DEFINITION)
			stateStack.push(IN_LOCATED);
		ongoingLocatedExpression = arg;
		XQueryExpression exp = ((Step) steps.get(0)).getExpression();
		exp.accept(this);
		if (state != IN_DEFINITION)
			stateStack.pop();
		ongoingLocatedExpression = null;
	}

	public void visit(Value arg) throws XQueryException {
		Integer state = (Integer) stateStack.peek();
		if (state == IN_DEFINITION /*|| state == IN_FUNCTION*/
			) {
			AlgEval depeval = depmanager.addEval(arg, (state == IN_DEFINITION) ? (Variable) varStack.peek() : null, true);
			addLeaf(depeval);
			ongoingLet = false;
		}
	}

	private GetUsedVariablesVisitor guvvisitor = null;

	public void visit(Variable arg) throws XQueryException {
		// add   03/06/2003
		Integer currentstate = (Integer) stateStack.peek();
		if (currentstate == IN_FLWR && !this.depmanager.getPlan().getQAOnly()) {
			// add   02/06/2003
//			if (arg.getSourceNames() != null && arg.getSourceNames().size() == 1 && (arg.getUrls() == null || arg.getUrls().isEmpty())) {
//				/*
//				for (int i=0;i<variables.size();i++) {
//					Variable vari = (Variable)variables.get(i);
//					vari.setParentFLWRExpression(tmpflwr);
//					parentvariable.put(vari,varStack.firstElement());
//				}
//				*/
//				if (!(arg.getExpression() instanceof LocatedExpression) && !(arg.getExpression() instanceof Variable)) {
//					AlgSource newsource = depmanager.addSource(arg.getExpression(), arg, ongoingLet);
//					if (guvvisitor == null)
//						guvvisitor = new GetUsedVariablesVisitor();
//					guvvisitor.reset();
//					arg.getExpression().accept(guvvisitor);
//					newsource.addDependSourceVariables(guvvisitor.getVariables());
//					addLeaf(newsource);
//					/*
//					if (currentstate == IN_MAIN_DEFINITION) currentDepFLWR.setMonoSource(true);
//					else if (currentstate == IN_DEFINITION) currentDepNode = currentDepNode;
//					*/
//					parentvariable.put(arg, varStack.firstElement());
//					return;
//				}
//			}
			// end add
		}
		// end add
		// add   05/02/2003
		//		if (arg.getUrls() != null && arg.getUrls().size() == 1) {
		//			/*
		//			for (int i=0;i<variables.size();i++) {
		//				Variable vari = (Variable)variables.get(i);
		//				vari.setParentFLWRExpression(tmpflwr);
		//				parentvariable.put(vari,varStack.firstElement());
		//			}
		//			*/
		//			if (!(arg.getExpression() instanceof LocatedExpression) && !(arg.getExpression() instanceof Variable)) {
		//				DepNodeEval neweval = depmanager.addNodeEval(arg.getExpression(), arg, ongoingLet);
		//				if (guvvisitor == null)
		//					guvvisitor = new GetUsedVariablesVisitor();
		//				else
		//					guvvisitor.reset();
		//				arg.getExpression().accept(guvvisitor);
		//				neweval.addDependSourceVariables(guvvisitor.getVariables());
		//				addLeaf(neweval);
		//				/*
		//				if (currentstate == IN_MAIN_DEFINITION) currentDepFLWR.setMonoSource(true);
		//				else if (currentstate == IN_DEFINITION) currentDepNode = currentDepNode;
		//				*/
		//				return;
		//			}
		//		}
		// end add

		if (currentstate == IN_DEFINITION) {
			if (ongoingLocatedExpression == null) {
				AlgVar depvar = depmanager.addVar(arg, (Variable) varStack.peek(), ongoingLet);
				//if (!ongoingLet)
				addLeaf(depvar);
				ongoingLet = false;
			} else {
				AlgLocated deplocated = depmanager.addLocated(ongoingLocatedExpression, arg, (Variable) varStack.peek(), ongoingLet);
				//if (!ongoingLet)
				addLeaf(deplocated);
				ongoingLet = false;
			}
		} else if (currentstate == IN_FLWR) {
			if (arg.getBindingType() == Constants.LET_BINDINGTYPE) {
				ongoingLet = true;
				//depmanager.addAlias(arg, arg.getExpression ()) ;
				//currentDepMem.isLet(true) ;
			}
			varStack.push(arg);
			if (arg.getParentFLWRExpression() == null) {
				// remove variables if necessary
				FLWRExpression tmpflwr = (FLWRExpression) arg.getExpression();
				ArrayList variables = tmpflwr.getVariables();

				// change   27/03/2003 not needed anymore!!!
				//ArrayList removedVariables = null;
				Variable lastremovedvar = null;
				Variable lastparentvar = null;
				for (int i = 0; i < variables.size(); i++) {
					Variable vari = (Variable) variables.get(i);
					Variable parentvari = (Variable) parentvariable.get(vari);
					if (parentvari == null)
						break;
					lastremovedvar = vari;
					if (parentvari != lastparentvar) {
						lastparentvar = parentvari;
						if (tmpflwr.getWhereClause() != null) {
							XQueryExpression tmpwhere = ((FLWRExpression) depmanager.getMapNode(lastparentvar).getExpression()).getWhereClause();
							if (tmpflwr.getWhereClause() != null)
								tmpflwr.setWhereClause(tmpflwr.getWhereClause().erasePart(tmpwhere));
						}
					}
					//currentDepFLWR.setDependency((DepFLWR) depmanager.getMapNode(parentvari));
					//					if (removedVariables == null) 
					//						removedVariables = new ArrayList(1);
					//					removedVariables.add(vari);
					variables.remove(i);
					i--;
				}

				// remove conditions from where clause implicating only removed variables
				//				if (tmpflwr.getWhereClause() != null && lastremovedvar != null) {
				//					
				//					XQueryExpression tmpwhere = ((FLWRExpression)depmanager.getMapNode((Variable) parentvariable.get(lastremovedvar)).getExpression()).getWhereClause();					
				//					if (tmpwhere != null) 
				//						tmpflwr.setWhereClause(((BinOpANDExpression)tmpflwr.getWhereClause()).erasePart(tmpwhere));					
				//					
				//					// change   27/03/2003 this was an ERROR !!!!!!!
				////					IsolateWhereVisitor visitor = new IsolateWhereVisitor(removedVariables);
				////					visitor.includeNone(false);
				////					try {
				////						tmpflwr.getWhereClause().accept(visitor);
				////					} catch (XQueryException e) {
				////						throw new MediatorException("Can't isolate where expression : " + tmpflwr.getWhereClause().pprint(), e);
				////					}
				////					tmpflwr.setWhereClause(visitor.getRemainingExpression());
				//				}

				currentDepFLWR = depmanager.addQDB(arg, ongoingLet);
				/*
				for (int i=0;i<removedVariables.size();i++)
					currentDepFLWR.setDependency((DepFLWR) depmanager.getMapNode((Variable) parentvariable.get((Variable) removedVariables.get(i))));
				*/
				if (lastremovedvar != null)
					currentDepFLWR.setDependency((AlgFLWR) depmanager.getMapNode((Variable) parentvariable.get(lastremovedvar)));
				if (currentDepFLWR.getVarsDependingOn() != null)
					currentDepFLWR.updateVarDependencyList((ArrayList) currentDepFLWR.getVarsDependingOn().clone());

				//				Utils.fillSources(tmpflwr, this.depmanager.getMetaData(),false);
				//				ArrayList sources = tmpflwr.getSourceNames();
				//				ArrayList urls = tmpflwr.getUrls();

				// test if flwr is imbricated and flwr contains located expression starting from an above defined variable

				//				if (sources != null && sources.size() == 1) {
				//					for (int i = 0; i < variables.size(); i++) {
				//						Variable vari = (Variable) variables.get(i);
				//						vari.setParentFLWRExpression(tmpflwr);
				//						parentvariable.put(vari, varStack.firstElement());
				//					}
				//					DepNodeSource newsource = depmanager.addNodeSource(tmpflwr, tmpflwr.getVariables(), false);
				//					addLeaf(newsource);
				//					currentDepFLWR.setMonoSource(true);
				//					//if (currentstate == IN_MAIN_DEFINITION) currentDepFLWR.setMonoSource(true);
				//					//else if (currentstate == IN_DEFINITION) currentDepNode = currentDepNode;
				//					varStack.pop();
				//					return;
				//				}
				//				if (urls != null && urls.size() == 1) {
				//					for (int i = 0; i < variables.size(); i++) {
				//						Variable vari = (Variable) variables.get(i);
				//						vari.setParentFLWRExpression(tmpflwr);
				//						parentvariable.put(vari, varStack.firstElement());
				//					}
				//					DepNodeEval neweval = depmanager.addNodeEval(tmpflwr, tmpflwr.getVariables(), false);
				//					addLeaf(neweval);
				//					currentDepFLWR.setMonoSource(true);
				//					//if (currentstate == IN_MAIN_DEFINITION) currentDepFLWR.setMonoSource(true);
				//					//else if (currentstate == IN_DEFINITION) currentDepNode = currentDepNode;
				//					varStack.pop();
				//					return;
				//				}

				ongoingLet = false;
				stateStack.push(IN_MAIN_DEFINITION);
				//currentSources = null;
			} else {
				parentvariable.put(arg, varStack.firstElement());
				stateStack.push(IN_DEFINITION);
			}
			arg.getExpression().accept(this);
			stateStack.pop();
			varStack.pop();
		}
		//currentSources = sources;
	}

	public void visit(UnOpMinusExpression arg) throws XQueryException {

		Integer state = (Integer) stateStack.peek();
		if (state == IN_DEFINITION /*|| state == IN_FUNCTION*/
			) {
			AlgFunction depfunction = depmanager.addFunction(arg, (state == IN_DEFINITION) ? (Variable) varStack.peek() : null, ongoingLet);
			addLeaf(depfunction);
			ongoingLet = false;
			Algebra tmpDepNode = currentDepNode;
			currentDepNode = depfunction;
			stateStack.push(IN_FUNCTION);
			arg.getExpression().accept(this);
			stateStack.pop();
			currentDepNode = tmpDepNode;
		}
	}

	public void visit(XQueryExpressionSequence arg) throws XQueryException {

		Integer state = (Integer) stateStack.peek();
		if (state == IN_DEFINITION /*|| state == IN_FUNCTION*/
			) {
			AlgSequence depsequence = depmanager.addSequence(arg, (state == IN_DEFINITION) ? (Variable) varStack.peek() : null, ongoingLet);
			addLeaf(depsequence);
			ongoingLet = false;
			ArrayList expressions = arg.getSubExpressions();
			if (expressions != null) {
				stateStack.push(IN_FUNCTION);
				for (int i = 0; i < expressions.size(); i++) {
					Algebra tmpDepNode = currentDepNode;
					currentDepNode = depsequence;
					((XQueryExpression) expressions.get(i)).accept(this);
					currentDepNode = tmpDepNode;
				}
				stateStack.pop();
			}
		}
	}

	// add LARS 16/09/2003
	public void visit(Element arg) throws XQueryException {

		Integer state = (Integer) stateStack.peek();
		if (state == IN_DEFINITION) {
			AlgEval depeval = null;
			if (ongoingLocatedExpression == null) {
				depeval = depmanager.addEval(arg, (Variable) varStack.peek(), ongoingLet);
			} else {
				depeval = depmanager.addEval(ongoingLocatedExpression, (Variable) varStack.peek(), ongoingLet);
			}
			addLeaf(depeval);
		}
	}

	public void visit(AttributeValuePair arg) throws XQueryException {

		Integer state = (Integer) stateStack.peek();
		if (state == IN_DEFINITION) {
			AlgEval depeval = depmanager.addEval(arg, (Variable) varStack.peek(), ongoingLet);
			addLeaf(depeval);
		}
	}

	public void visit(XQueryExpression arg) throws XQueryException {
		//throw new XQueryException("CALL  AlgebraStructureVisitor for XQueryExpression : " + arg.getClass().getName());
	}

}
