/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * Copyright (C) 2003 XQuark Group.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.decomposer;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import org.xquark.xquery.normalize.*;
import org.xquark.xquery.parser.*;

public class AtomizeWhereVisitor extends DefaultParserVisitor {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.5 $";
	private static final String RCSName = "$Name:  $";
	// **********************************************************************
	// * CLASS VARIABLES
	// **********************************************************************
	// for each var all simple conditions in which this var appears
	private HashMap varSimpleConditionsMap = null; // { var1 [c1, c2], var2 [c3, c4] ...
	// for each var a map of (var,conditions) reprensenting simple joins
	private HashMap varJoinConditionsMap = null; // { var1 { var2 [c121, c122], var3 [c131, c132] ... }, var2 { var1 [c211, c212] ...
	// for each var a list of complex conditions in which it appears
	private HashMap varComplexConditionsMap = null; // { var1 [c1, c2], var2 [c3, c4] ...
	// for each var all conditions in which this var appears
	private HashMap varAllConditionsMap = null; // { var1 [c1, c2], var2 [c3, c4] ...
	// for each var all variables appearing in common conditions
	private HashMap varAssociatedVariablesMap = null; // { var1 [var2, var3], var2 [var3, var4] ...
	// for each condition all variables appearing
	private HashMap condAssociatedVariablesMap = null; // { c1 [var2, var3], c2 [var3, var4] ...
	// a list of all simple conditions
	private ArrayList allSimpleConditionsList = null;
	// a list of all join conditions
	private ArrayList allJoinConditionsList = null;
	// a list of all complex conditions
	private ArrayList allComplexConditionsList = null;
	// a list of all conditions without variables !!!
	private ArrayList allNoVariablesConditionsList = null;
	// a list of all conditions
	private ArrayList allConditionsList = null;
	// visitor
	private GetBoundVariablesVisitor gbv_visitor = null;

	// **********************************************************************
	// * INITIALIZATION
	// **********************************************************************
	public AtomizeWhereVisitor() {
		gbv_visitor = new GetBoundVariablesVisitor(true, false);
		varSimpleConditionsMap = new HashMap();
		varJoinConditionsMap = new HashMap();
		varComplexConditionsMap = new HashMap();
		varAllConditionsMap = new HashMap();
		varAssociatedVariablesMap = new HashMap();
		condAssociatedVariablesMap = new HashMap();
		allSimpleConditionsList = new ArrayList();
		allJoinConditionsList = new ArrayList();
		allComplexConditionsList = new ArrayList();
		allNoVariablesConditionsList = new ArrayList();
		allConditionsList = new ArrayList();
	}

	public HashMap getVarSimpleConditionsMap() {
		return varSimpleConditionsMap;
	}
	public HashMap getVarJoinConditionsMap() {
		return varJoinConditionsMap;
	}
	public HashMap getVarComplexConditionsMap() {
		return varComplexConditionsMap;
	}
	public HashMap getVarAllConditionsMap() {
		return varAllConditionsMap;
	}
	public HashMap getVarAssociatedVariablesMap() {
		return varAssociatedVariablesMap;
	}
	public HashMap getCondAssociatedVariablesMap() {
		return condAssociatedVariablesMap;
	}
	public ArrayList getAllSimpleConditionsList() {
		return allSimpleConditionsList;
	}
	public ArrayList getAllJoinConditionsList() {
		return allJoinConditionsList;
	}
	public ArrayList getAllComplexConditionsList() {
		return allComplexConditionsList;
	}
	public ArrayList getAllNoVariablesConditionsList() {
		return allNoVariablesConditionsList;
	}
	public ArrayList getAllConditionsList() {
		return allConditionsList;
	}

	//	public void reset() {
	//		varSimpleConditionsMap.clear();
	//		varJoinConditionsMap.clear();
	//		varComplexConditionsMap.clear();
	//		varAllConditionsMap.clear();
	//		allSimpleConditionsList.clear();
	//		allJoinConditionsList.clear();
	//		allComplexConditionsList.clear();
	//		allConditionsList.clear();
	//	}
	// **********************************************************************
	// * VISITATION
	// **********************************************************************
	//	public void visit(AttributeValuePair arg) throws XQueryException

	public void visit(BinOpANDExpression arg) throws XQueryException {
		arg.getExpression1().accept(this);
		arg.getExpression2().accept(this);
	}

	public void visit(ListOpCompExpression arg) throws XQueryException {
		allConditionsList.add(arg);
		gbv_visitor.reset();
		arg.accept(gbv_visitor);
		ArrayList vars = gbv_visitor.getAllVariables();
		fillVarAssociatedMap(vars);
		condAssociatedVariablesMap.put(arg,vars);
		if (vars.isEmpty()) {
			allNoVariablesConditionsList.add(arg);
		} else if (vars.size() == 1) {
			allSimpleConditionsList.add(arg);
			Variable var0 = (Variable) vars.get(0);
			addExprToVarMap(var0,arg,varAllConditionsMap);
			addExprToVarMap(var0,arg,varSimpleConditionsMap);
		} else if (vars.size() == 2) {
			gbv_visitor.reset();
			arg.getExpression1().accept(gbv_visitor);
			ArrayList vars1 = gbv_visitor.getAllVariables();
			if (vars1.size() != 1) {
				addExprToVarsMap(vars,arg,varAllConditionsMap);
				allComplexConditionsList.add(arg);
				// TODO add to complex map for each variable
			} else { // (vars1.size() == 1)
				gbv_visitor.reset();
				arg.getExpression2().accept(gbv_visitor);
				ArrayList vars2 = gbv_visitor.getAllVariables();
				if (vars2.size() != 1) {
					addExprToVarsMap(vars,arg,varAllConditionsMap);
					allComplexConditionsList.add(arg);
					// TODO add to complex map for each variable
				} else { // vars2.size() == 1
					allJoinConditionsList.add(arg);
					Map tmpmap = (Map)varJoinConditionsMap.get(vars1.get(0));
					if (tmpmap == null) {
						tmpmap = new HashMap();
						varJoinConditionsMap.put(vars1.get(0),tmpmap);
					}
					ArrayList tmplist = (ArrayList) tmpmap.get(vars2.get(0));
					if (tmplist == null) {
						tmplist = new ArrayList(1);
						tmpmap.put(vars2.get(0), tmplist);
					}
					tmplist.add(arg);
					tmpmap = (Map)varJoinConditionsMap.get(vars2.get(0));
					if (tmpmap == null) {
						tmpmap = new HashMap();
						varJoinConditionsMap.put(vars2.get(0),tmpmap);
					}
					tmplist = (ArrayList) tmpmap.get(vars1.get(0));
					if (tmplist == null) {
						tmplist = new ArrayList(1);
						tmpmap.put(vars1.get(0), tmplist);
					}
					tmplist.add(arg);
				}
			}
		} else { // vars.size() > 2
			for (int i = 0; i < vars.size(); i++) {
				Variable var0 = (Variable) vars.get(0);
				ArrayList tmplist = (ArrayList) varAllConditionsMap.get(var0);
				if (tmplist == null) {
					tmplist = new ArrayList(1);
					varAllConditionsMap.put(var0, tmplist);
				}
				tmplist.add(arg);
			}
			allComplexConditionsList.add(arg);
			// TODO add to complex map for each variable
		}

		//		if (vars.isEmpty())
		//			return;
		//		if (vars.size() > 1) {
		//			allComplexConditionsList.add(arg);
		//		}
		//		else if (vars.size() == 1) {
		//			allSimpleConditionsList.add(arg);			
		//		}
		//		for (int i = 0; i < vars.size(); i++)
		//			if (!varVarMap.containsKey(vars.get(i)))
		//				varVarMap.put(vars.get(i), null);
		//		if (vars.size() < 3) {
		//			Set values = varVarMap.keySet();
		//			Variable var1 = (Variable) vars.get(0);
		//			Variable var2 = (Variable) vars.get((vars.size()>1)?1:0);
		//			HashMap varMap = (HashMap) varVarMap.get(var1);
		//			if (varMap == null) {
		//				varMap = new HashMap();
		//				ArrayList tmpList = new ArrayList(1);
		//				tmpList.add(arg);
		//				varMap.put(var2, tmpList);
		//				varVarMap.put(var1,varMap);
		//			} else {
		//				ArrayList tmpList = (ArrayList) varMap.get(var2);
		//				if (tmpList == null) {
		//					tmpList = new ArrayList(1);
		//					varMap.put(var2,tmpList);
		//				}
		//				tmpList.add(arg);
		//			}
		//			if (vars.size() > 1) {
		//				varMap = (HashMap) varVarMap.get(var2);
		//				if (varMap == null) {
		//					varMap = new HashMap();
		//					ArrayList tmpList = new ArrayList(1);
		//					tmpList.add(arg);
		//					varMap.put(var1, tmpList);
		//					varVarMap.put(var2,varMap);
		//				} else {
		//					ArrayList tmpList = (ArrayList) varMap.get(var1);
		//					if (tmpList == null) {
		//						tmpList = new ArrayList(1);
		//						varMap.put(var1,tmpList);
		//					}
		//					tmpList.add(arg);
		//				}
		//			}
		//		}
	}

	public void visit(XQueryExpression arg) throws XQueryException {
		allConditionsList.add(arg);
		gbv_visitor.reset();
		arg.accept(gbv_visitor);
		ArrayList vars = gbv_visitor.getAllVariables();
		fillVarAssociatedMap(vars);
		condAssociatedVariablesMap.put(arg,vars);
		if (vars.isEmpty()) {
			allNoVariablesConditionsList.add(arg);
		} else if (vars.size() == 1) {
			allSimpleConditionsList.add(arg);
			Variable var0 = (Variable) vars.get(0);
			addExprToVarMap(var0,arg,varAllConditionsMap);
			addExprToVarMap(var0,arg,varSimpleConditionsMap);
		} else { //vars.size() > 1)
			allComplexConditionsList.add(arg);
			addExprToVarsMap(vars,arg,varAllConditionsMap);
			addExprToVarsMap(vars,arg,varComplexConditionsMap);
		}
	}

	private void addExprToVarMap(Variable var, XQueryExpression expr, Map map) {
		ArrayList tmplist = (ArrayList) map.get(var);
		if (tmplist == null) {
			tmplist = new ArrayList(1);
			map.put(var, tmplist);
		}
		tmplist.add(expr);
	}
	private void addExprToVarsMap(ArrayList vars, XQueryExpression expr, Map map) {
		for (int i = 0; i < vars.size(); i++) {
			addExprToVarMap((Variable) vars.get(i), expr,map);
		}
	}

	private void fillVarAssociatedMap(ArrayList vars) {
		if (vars.size() < 2) return;
		for (int i = 0; i < vars.size(); i++) {
			ArrayList tmplist = (ArrayList) varAssociatedVariablesMap.get(vars.get(i));
			if (tmplist == null) {
				tmplist = (ArrayList)vars.clone();
				tmplist.remove(i);
				varAssociatedVariablesMap.put(vars.get(i), tmplist);
			}
			else {
				for (int j = 0; j < vars.size(); j++) {
					if (j == i) continue;
					if (!tmplist.contains(vars.get(j)))
						tmplist.add(vars.get(j));
				}
			}
		}
	}

}
