/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.decomposer;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import org.xquark.xquery.parser.*;
import org.xquark.xquery.parser.util.Constants;

public class GetVariablePathsVisitor extends DefaultParserVisitor {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.8 $";
	private static final String RCSName = "$Name:  $";
	// **********************************************************************
	// * CLASS VARIABLES
	// **********************************************************************
	private HashMap varpaths = null;
	//private HashMap varjoinpaths = new HashMap();
	private ArrayList pathstrlist = null;
	private boolean eliminateDuplicates = false;
	private ArrayList allDefinedVars = null;
	
	private ArrayList currentlist = null;
	private boolean isrootqdb = false;
	private boolean isqdb = false;
	private boolean ismonosource = false;
	private boolean inWhere = false;
	private Variable defvar = null;
	//private HashMap aliasmap = new HashMap();
	//private HashMap dependvarmap = new HashMap();
	private ArrayList onlyvariables = null;
	private GetUsedVariablesVisitor guvv = new GetUsedVariablesVisitor();
	
	/*
	private final Integer IN_DEFINITION = new Integer(1);
	private final Integer IN_FLWR = new Integer(2);
	private final Integer IN_WHERE = new Integer(3);
	private final Integer IN_PREDICATE = new Integer(4);
	private final Integer IN_FUNCTION = new Integer(5);
	 */
	// **********************************************************************
	// * INITIALIZATION
	// **********************************************************************
	/**
	 *
	 */

	//public GetVariablePathsVisitor() { varpaths = new HashMap(); pathstrlist = new ArrayList(); }

	public GetVariablePathsVisitor(boolean eliminateDuplicates) {
		this.eliminateDuplicates = eliminateDuplicates;
		varpaths = new HashMap();
		pathstrlist = new ArrayList();
	}

	public GetVariablePathsVisitor(boolean eliminateDuplicates, HashMap varpaths, ArrayList pathstrlist) {
		this.eliminateDuplicates = eliminateDuplicates;
		this.varpaths = varpaths;
		this.pathstrlist = pathstrlist;
	}

	public ArrayList getPathStrList() {
		return pathstrlist;
	}
	public HashMap getVarPaths() {
		return varpaths;
	}
	public ArrayList getVarPaths(Variable var) {
		return (ArrayList) varpaths.get(var);
	}
	public ArrayList getPaths() {
		ArrayList retlist = new ArrayList();
		for (Iterator it = varpaths.keySet().iterator(); it.hasNext();)
			retlist.addAll((ArrayList) varpaths.get(it.next()));
		if (retlist.isEmpty())
			return null;
		return retlist;
	}

	public ArrayList getPaths(ArrayList variables) {
		if (variables == null) return null;
		ArrayList retlist = new ArrayList();
		for (Iterator it = varpaths.keySet().iterator(); it.hasNext();) {
			Object tmpvar = it.next();
			if (variables.contains(tmpvar))			
				retlist.addAll((ArrayList) varpaths.get(tmpvar));
		}
		if (retlist.isEmpty())
			return null;
		return retlist;
	}

//	public HashMap getAliasMap() {
//		return this.aliasmap;
//	}

	public void reset() {
		varpaths.clear();
		pathstrlist.clear();
//		aliasmap.clear();
//		dependvarmap.clear();
	}

	public void resetPaths() {
		varpaths.clear();
		pathstrlist.clear();
		onlyvariables = null;
	}

	public void setRootQDB(boolean isrootqdb) {
		this.isrootqdb = isrootqdb;
	}
	public void setQDB(boolean isqdb) {
		this.isqdb = isqdb;
	}
	public void setMonoSource(boolean ismonosource) {
		this.ismonosource = ismonosource;
	}

	public void setVariables(ArrayList variables) {
		this.onlyvariables = variables;
	}
	public void setCurrentList(ArrayList currentlist) {
		this.currentlist = currentlist;
	}
	private void addVarList(ArrayList addlist) {
		if (this.allDefinedVars == null)
			this.allDefinedVars = new ArrayList();
		this.allDefinedVars.addAll(addlist);
	}
	private void addVar(Variable var) {
		if (this.allDefinedVars == null)
			this.allDefinedVars = new ArrayList();
		this.allDefinedVars.add(var);
	}
	private void removeVarList(ArrayList removelist) {
		if (this.allDefinedVars != null)
			this.allDefinedVars.removeAll(removelist);
	}	
	public void resetCurrentVarList() {
		this.currentlist = null;
	}
//	private void addAliasMap(Variable defvar, Variable var, XQueryExpression expr) {
//		//dependvarmap.put(defvar, var);
//		aliasmap.put(defvar, expr);
//	}

	private void addVarPath(Variable var, XQueryExpression expr) {
		if (onlyvariables != null && !onlyvariables.contains(var))
			return;
		if (allDefinedVars != null) {
			if (!allDefinedVars.contains(var))
				return;
		}
		ArrayList tmplist = (ArrayList) varpaths.get(var);
		if (tmplist == null) {
			tmplist = new ArrayList();
			tmplist.add(expr);
			pathstrlist.add(expr.getStringValue());
			varpaths.put(var, tmplist);
		} else if (!pathstrlist.contains(expr.getStringValue()) || !eliminateDuplicates) {
			tmplist.add(expr);
			pathstrlist.add(expr.getStringValue());
		}
	}

	// **********************************************************************
	// * VISITATION
	// **********************************************************************
	// public void visit(AttributeValuePair arg) throws XQueryException
	// public void visit(BinOpANDExpression arg) throws XQueryException
	// public void visit(BinOpORExpression arg) throws XQueryException
	//	public void visit(CastTreatExpression arg) throws XQueryException
	//	public void visit(ContextDeclaration arg) throws XQueryException
	//	public void visit(Dereference arg) throws XQueryException
	public void visit(Element arg) throws XQueryException {
		ArrayList al = arg.getAttributes();
		if (al != null)
			for (int i = 0; i < al.size(); i++)
				 ((AttributeValuePair) al.get(i)).accept(this);
		al = arg.getSubExpressions();
		if (al != null)
			for (int i = 0; i < al.size(); i++)
				 ((XQueryExpression) al.get(i)).accept(this);
	}
	public void visit(FLWRExpression arg) throws XQueryException {
		boolean isqdb = this.isqdb;
		boolean isrootqdb = this.isrootqdb;
		this.isqdb = false;
		this.isrootqdb = false;
		//ArrayList prevlist = varlist;
		//varlist = arg.getVariables();
		for (int i = 0; i < arg.getVariables().size(); i++) {
			Variable vari = (Variable) arg.getVariables().get(i);
//			if (varlist != null && !varlist.contains(vari))
//				continue;
			addVar(vari);
			XQueryExpression expr = vari.getExpression();
//			if (expr instanceof RootNodeFunctionCall || (expr instanceof LocatedExpression && ((LocatedExpression)expr).getStep(0).getExpression() instanceof RootNodeFunctionCall))
//			if (expr instanceof Variable)
//				this.addAliasMap(vari, (Variable) expr, expr);
//			else if (expr instanceof LocatedExpression && ((LocatedExpression) expr).getStep(0).getExpression() instanceof Variable)
//				this.addAliasMap(vari, (Variable) ((LocatedExpression) expr).getStep(0).getExpression(), expr);
			expr.accept(this);
		}
		if (!isqdb) {
			ArrayList prevlist = allDefinedVars;
			allDefinedVars = null;
			arg.getReturnClause().accept(this);
			allDefinedVars = prevlist;
		}
		XQueryExpression qi = arg.getWhereClause();
		if (qi != null) {
//			if (!isqdb)
//				this.removeVarList(arg.getVariables());
//			guvv.reset();
//			qi.accept(guvv);
//			ArrayList reslist = guvv.getVariables();
//			// change   18/06/2003
//			if (reslist != null && !reslist.isEmpty()) {
			//if (reslist != null && reslist.size() > 1) {
				boolean prevInWhere = inWhere;
				inWhere = true;
				qi.accept(this);
				inWhere = prevInWhere;
//			}
//			if (!isqdb)
//				this.addVarList(arg.getVariables());
		}
		ArrayList al = arg.getOrderBy();
		if (al != null)
			for (int i = 0; i < al.size(); i++) {
				XQueryExpression expri = (XQueryExpression) al.get(i);
				if (expri.getOrder(0) != Constants.PLACE_ORDER)
				 	expri.accept(this);
			}
		this.isrootqdb = isrootqdb;
		this.isqdb = isqdb;
	}

	public void visit(FunctionCall arg) throws XQueryException {
		// change   18/06/2003
//		if (inWhere) {
//			guvv.reset();
//			arg.accept(guvv);
//			ArrayList reslist = guvv.getVariables();
//			if (reslist == null || reslist.size() < 2)
//				return;
//		}
		ArrayList al = arg.getArguments();
		if (al != null)
			for (int i = 0; i < al.size(); i++) {
				((XQueryExpression) al.get(i)).accept(this);
			}
	}

	//	public void visit(FunctionDefinition arg) throws XQueryException
	//	public void visit(InstanceOfExpression arg) throws XQueryException
	//	public void visit(InternalFunctionCall arg) throws XQueryException
	//	public void visit(ITEExpression arg) throws XQueryException
	//	public void visit(LibraryFunctionCall arg) throws XQueryException
	//	public void visit(ListOpAFTERExpression arg) throws XQueryException
	//	public void visit(ListOpArithExpression arg) throws XQueryException
	//	public void visit(ListOpBEFOREExpression arg) throws XQueryException

	//	public void visit(ListOpCompExpression arg) throws XQueryException {
	//		GetVariablePathsVisitor visitor = new GetVariablePathsVisitor(true);
	//		arg.getExpression1().accept(visitor);
	//		arg.getExpression2().accept(visitor);
	//		HashMap map = visitor.getVarPaths();
	//		int num = 0;
	//		boolean addvars = false;
	//		if (varlist != null) for (int i=0;i<varlist.size();i++) if (map.keySet().contains(varlist.get(i))) num++;
	//		if (num > 1 || (null !=onlyvariables && !onlyvariables.isEmpty())) addvars = true;
	//		for (Iterator it = map.keySet().iterator();it.hasNext();) {
	//			Variable var = (Variable)it.next();
	//			if (addvars || (varlist != null && !varlist.contains(var))) {
	//				ArrayList list = (ArrayList)map.get(var);
	//				for (int i=0;i<list.size();i++) this.addVarPath(var,(XQueryExpression)list.get(i));
	//			}
	//		}
	//	}
	public void visit(ListOpCompExpression arg) throws XQueryException {
		// change   18/06/2003
//		if (inWhere) {
//			guvv.reset();
//			arg.accept(guvv);
//			ArrayList reslist = guvv.getVariables();
//			if (reslist == null || reslist.size() < 2)
//				return;
//		}
		arg.getExpression1().accept(this);
		arg.getExpression2().accept(this);
	}

	//	public void visit(ListOpEXCEPTExpression arg) throws XQueryException
	//	public void visit(ListOpINTERSECTExpression arg) throws XQueryException
	//	public void visit(ListOpUNIONExpression arg) throws XQueryException
	public void visit(LocatedExpression arg) throws XQueryException {
		if (arg.getExpression() instanceof Variable)
			addVarPath((Variable) arg.getExpression(), arg);
		for (int i = 0; i < arg.getSteps().size(); i++)
			arg.getStepNum(i).accept(this);
	}

	//	public void visit(NodeType arg) throws XQueryException
	//	public void visit(Parameter arg) throws XQueryException
	//	public void visit(PrimitiveFunctionCall arg) throws XQueryException
	//  public void visit (QName arg) throws XQueryException
	//	public void visit(QuantifiedExpression arg) throws XQueryException
	//	public void visit(RangeExpression arg) throws XQueryException
	//	public void visit(RootNodeFunctionCall arg) throws XQueryException
	//	public void visit(SortedExpression arg) throws XQueryException
	public void visit(Step arg) throws XQueryException {
		ArrayList predicates = arg.getPredicates();
		if (predicates == null)
			return;
		for (int i = 0; i < predicates.size(); i++)
			 ((XQueryExpression) predicates.get(i)).accept(this);
	}
	//	public void visit(TypeSwitchExpression arg) throws XQueryException
	//	public void visit(UnOpMinusExpression arg) throws XQueryException
	//	public void visit(Value arg) throws XQueryException
	//	public void visit(ValueBoolean arg) throws XQueryException
	//	public void visit(ValueFloat arg) throws XQueryException
	//	public void visit(ValueInteger arg) throws XQueryException
	//	public void visit(ValueString arg) throws XQueryException
	//	public void visit(ValueText arg) throws XQueryException
	public void visit(Variable arg) throws XQueryException {
		addVarPath(arg, arg);
	}

	public void visit(XQueryBinaryOperatorExpression arg) throws XQueryException {
		arg.getExpression1().accept(this);
		arg.getExpression2().accept(this);
	}

	//	public void visit(XQueryBooleanOperatorExpression arg) throws XQueryException
	// public void visit (XQueryExpression arg) throws XQueryException
	public void visit(XQueryExpressionSequence arg) throws XQueryException {
		ArrayList al = arg.getSubExpressions();
		if (al != null)
			for (int i = 0; i < al.size(); i++)
				 ((XQueryExpression) al.get(i)).accept(this);
	}

	// public void visit(XQueryListBinaryOperatorExpression arg) throws XQueryException
	// public void visit(XQueryListUnaryOperatorExpression arg) throws XQueryException

	public void visit(XQueryUnaryOperatorExpression arg) throws XQueryException {
		arg.getExpression().accept(this);
	}
}
