/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * Copyright (C) 2003 XQuark Group.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.decomposer;

import java.util.ArrayList;

import org.xquark.xquery.normalize.*;
import org.xquark.xquery.parser.*;

public class IsolateWhereVisitor extends DefaultParserVisitor {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.9 $";
	private static final String RCSName = "$Name:  $";
	// **********************************************************************
	// * CLASS VARIABLES
	// **********************************************************************
	private ArrayList vars = null;
	private ArrayList varsstr = null;
	private boolean putvar = false;
	boolean includenone = true;
	private GetBoundVariablesVisitor gbv_visitor = null;

	// the expresion that will be generated for child expression
	private XQueryExpression isolatedExpression = null;
	private XQueryExpression remainingExpression = null;
	// the original expression (that will be pruned)
	// private XQueryExpression root = null;
	//private TypeVisitor typevisitor = null;

	// **********************************************************************
	// * INITIALIZATION
	// **********************************************************************
	/**
	 *
	 */
	public IsolateWhereVisitor() {
		init(null, new GetBoundVariablesVisitor(true, false));
	}
	public IsolateWhereVisitor(ArrayList vars) {
		init(vars, new GetBoundVariablesVisitor(true, false));
	}
	public IsolateWhereVisitor(ArrayList vars, GetBoundVariablesVisitor gbv_visitor) {
		init(vars, gbv_visitor);
	}

	private void init(ArrayList vars, GetBoundVariablesVisitor gbv_visitor) {
		this.isolatedExpression = null;
		this.remainingExpression = null;
		this.vars = vars;
		if (this.vars == null)
			this.vars = new ArrayList();
		varsstr = new ArrayList(this.vars.size());
		for (int i = 0; i < this.vars.size(); i++)
			varsstr.add(((Variable) this.vars.get(i)).getStringValue());
		this.gbv_visitor = gbv_visitor;
		gbv_visitor.reset();
	}

	public void includeNone(boolean includenone) {
		this.includenone = includenone;
	}

	public void putVar(boolean putvar) {
		this.putvar = putvar;
	}
	public XQueryExpression getIsolatedExpression() {
		return isolatedExpression;
	}
	public XQueryExpression getRemainingExpression() {
		return remainingExpression;
	}
	public void reset(ArrayList vars) {
		init(vars, this.gbv_visitor);
	}

	// **********************************************************************
	// * VISITATION
	// **********************************************************************
	//	public void visit(AttributeValuePair arg) throws XQueryException

	public void visit(BinOpANDExpression arg) throws XQueryException {
		arg.getExpression1().accept(this);
		//if (isolatedExpression == null && remainingExpression == null) return;
		XQueryExpression isolated1 = isolatedExpression;
		XQueryExpression remaining1 = remainingExpression;
		arg.getExpression2().accept(this);
		//if (isolatedExpression == null && remainingExpression == null) return;
		XQueryExpression isolated2 = isolatedExpression;
		XQueryExpression remaining2 = remainingExpression;

		if (isolated1 == arg.getExpression1() && isolated2 == arg.getExpression2()) {
			isolatedExpression = arg;
			remainingExpression = null;
		} else {
			if (isolated1 != null)
				if (isolated2 != null)
					isolatedExpression = new BinOpANDExpression(isolated1, isolated2, null);
				else
					isolatedExpression = isolated1;
			else
				isolatedExpression = isolated2;
			if (remaining1 != null)
				if (remaining2 != null)
					remainingExpression = new BinOpANDExpression(remaining1, remaining2, null);
				else
					remainingExpression = remaining1;
			else
				remainingExpression = remaining2;
		}
	}

	public void visit(BinOpORExpression arg) throws XQueryException {

		arg.getExpression1().accept(this);
		//if (isolatedExpression == null && remainingExpression == null) return;
		//XQueryExpression isolated1 = isolatedExpression;
		XQueryExpression remaining1 = remainingExpression;
		if (remaining1 != null) {
			isolatedExpression = null;
			remainingExpression = arg;
			return;
		}

		arg.getExpression2().accept(this);
		//if (isolatedExpression == null && remainingExpression == null) return;
		//XQueryExpression isolated2 = isolatedExpression;
		XQueryExpression remaining2 = remainingExpression;

		if (remaining2 == null) {
			isolatedExpression = arg;
			remainingExpression = null;
		} else {
			isolatedExpression = null;
			remainingExpression = arg;
		}

		//		if (isolated1 == arg.getExpression1() && isolated2 == arg.getExpression2()) {
		//			isolatedExpression = arg;
		//			remainingExpression = null;
		//		} else {
		//			if (isolated1 != null)
		//				if (isolated2 != null)
		//					isolatedExpression = new BinOpORExpression(isolated1, isolated2, null);
		//				else
		//					isolatedExpression = isolated1;
		//			else
		//				isolatedExpression = isolated2;
		//			if (remaining1 != null)
		//				if (remaining2 != null)
		//					remainingExpression = new BinOpORExpression(remaining1, remaining2, null);
		//				else
		//					remainingExpression = remaining1;
		//			else
		//				remainingExpression = remaining2;
		//		}
	}

	// ************************************************************************

	//	public void visit(CastTreatExpression arg) throws XQueryException
	//	public void visit(ContextDeclaration arg) throws XQueryException
	//	public void visit(Dereference arg) throws XQueryException
	//  public void visit (Element arg) throws XQueryException
	// for DepMem.createAlgebra
	public void visit(FLWRExpression arg) throws XQueryException {
		ArrayList variables = arg.getVariables();
		// variable happens somewhere
		if (putvar)
			for (int i = 0; i < variables.size(); i++) {
				Variable tmpvar = (Variable) variables.get(i);
				if (!varsstr.contains(tmpvar.getStringValue())) {
					vars.add(tmpvar);
					varsstr.add(tmpvar.getStringValue());
				}
			}
		isolatedExpression = null;
		XQueryExpression wexp = arg.getWhereClause();
		if (wexp != null)
			wexp.accept(this);
		// if (isolatedExpression == null && remainingExpression == null) return;
		XQueryExpression newwexp = isolatedExpression;

		// variable happens somewhere
		XQueryExpression rexp = arg.getReturnClause();
		rexp.accept(gbv_visitor);
		XQueryExpression newrexp = new XQueryExpressionSequence(gbv_visitor.getVariables(), null);

		ArrayList flwrvars = arg.getVariables();

		if (putvar)
			isolatedExpression = new FLWRExpression(variables, newwexp, newrexp, null);
		else
			isolatedExpression = new FLWRExpression(vars, newwexp, newrexp, null);
		isolatedExpression.setParentModule(arg.getParentModule());
		remainingExpression = null;
	}

	public void visit(FunctionCall arg) throws XQueryException {
		if (arg.getArguments() != null) {
			for (int i = 0; i < arg.getArguments().size(); i++) {
				XQueryExpression expi = (XQueryExpression) arg.getArgument(i);
				expi.accept(this);
				if (isolatedExpression == null && remainingExpression == null)
					return;
				if (remainingExpression != null) {
					isolatedExpression = null;
					remainingExpression = arg;
					return;
				}
			}
		}
		isolatedExpression = arg;
		remainingExpression = null;
	}

	//	public void visit(FunctionDefinition arg) throws XQueryException
	//	public void visit(InstanceOfExpression arg) throws XQueryException
	//	public void visit(InternalFunctionCall arg) throws XQueryException
	//	public void visit(ITEExpression arg) throws XQueryException
	//	public void visit(LibraryFunctionCall arg) throws XQueryException
	//	public void visit(ListOpAFTERExpression arg) throws XQueryException
	//	public void visit(ListOpArithExpression arg) throws XQueryException
	//	public void visit(ListOpBEFOREExpression arg) throws XQueryException
	//  public void visit(ListOpCompExpression arg) throws XQueryException {
	//	public void visit(ListOpEXCEPTExpression arg) throws XQueryException
	//	public void visit(ListOpINTERSECTExpression arg) throws XQueryException
	//	public void visit(ListOpUNIONExpression arg) throws XQueryException
	//  public void visit(LocatedExpression arg) throws XQueryException {    
	//	public void visit(NodeType arg) throws XQueryException
	//	public void visit(Parameter arg) throws XQueryException
	//	public void visit(PrimitiveFunctionCall arg) throws XQueryException
	//  public void visit (QName arg) throws XQueryException
	public void visit(QuantifiedExpression arg) throws XQueryException {
		// not supported
		isolatedExpression = null;
		remainingExpression = null;
		/*
		vars.add(arg.getVariable());
		makeSimple(arg) ;
		vars.remove(arg.getVariable());
		*/
	}
	//	public void visit(RangeExpression arg) throws XQueryException
	//	public void visit(RootNodeFunctionCall arg) throws XQueryException
	//	public void visit(SortedExpression arg) throws XQueryException
	//	public void visit(Step arg) throws XQueryException
	//	public void visit(TypeSwitchExpression arg) throws XQueryException
	//	public void visit(UnOpMinusExpression arg) throws XQueryException
	//	public void visit(Value arg) throws XQueryException
	//	public void visit(ValueBoolean arg) throws XQueryException
	//	public void visit(ValueFloat arg) throws XQueryException
	//	public void visit(ValueInteger arg) throws XQueryException
	//	public void visit(ValueString arg) throws XQueryException
	public void visit(Variable arg) throws XQueryException {
		if (varsstr.contains(arg.getStringValue())) {
			isolatedExpression = arg;
			remainingExpression = null;
		} else {
			isolatedExpression = null;
			remainingExpression = arg;
		}
	}

	// public void visit (XQueryBinaryOperatorExpression arg) throws XQueryException
	//	public void visit(XQueryBooleanOperatorExpression arg) throws XQueryException
	public void visit(XQueryExpression arg) throws XQueryException {
		// default behavior
		gbv_visitor.reset();
		arg.accept(gbv_visitor);
		ArrayList boundvars = gbv_visitor.getVariables();

		if (boundvars != null && !boundvars.isEmpty())
			for (int i = 0; i < boundvars.size(); i++)
				if (!varsstr.contains(((Variable) boundvars.get(i)).getStringValue())) {
					isolatedExpression = null;
					remainingExpression = arg;
					return;
				}
		if ((boundvars == null || boundvars.isEmpty()) && !includenone) {
			isolatedExpression = null;
			remainingExpression = arg;
		} else {
			isolatedExpression = arg;
			remainingExpression = null;
		}
	}

	public void visit(XQueryExpressionSequence arg) throws XQueryException {
		ArrayList sequence = arg.getSubExpressions();
		ArrayList alnew = new ArrayList();
		ArrayList alold = new ArrayList();
		for (int i = 0; i < sequence.size(); i++) {
			XQueryExpression expi = (XQueryExpression) sequence.get(i);
			expi.accept(this);
			XQueryExpression isolated = isolatedExpression;
			XQueryExpression remaining = remainingExpression;
			if (isolated != null)
				alnew.add(isolated);
			if (remaining != null)
				alold.add(remaining);
		}

		if (alnew.isEmpty())
			isolatedExpression = null;
		else {
			isolatedExpression = new XQueryExpressionSequence(alnew, null);
			isolatedExpression.setParenthesis(true);
		}
		if (alold.isEmpty())
			remainingExpression = null;
		else {
			remainingExpression = new XQueryExpressionSequence(alold, null);
			remainingExpression.setParenthesis(true);
		}
	}

	// public void visit(XQueryListBinaryOperatorExpression arg) throws XQueryException {    
	// public void visit(XQueryListUnaryOperatorExpression arg) throws XQueryException
	// public void visit(XQueryUnaryOperatorExpression arg) throws XQueryException {
}
