/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * Copyright (C) 2003 XQuark Group.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.decomposer;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Set;

import org.xquark.xquery.metadata.MetaDataImpl;
import org.xquark.xquery.metadata.resolver.PathResolver;
import org.xquark.xquery.parser.*;

public class StepLists {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.6 $";
	private static final String RCSName = "$Name:  $";
	// **********************************************************************
	// * CLASS VARIABLES
	// **********************************************************************
	private ArrayList stepslists = null;
	private ArrayList cur_stepslists = null;

	private HashMap mapvar = null;
	private MetaDataImpl metadata = null;
	
	private PathResolver pathresolver = null;
	private MakeStepsListVisitor stepslistvisitor = null;
	
	// **********************************************************************
	// * INITIALIZATION
	// **********************************************************************
	/**
	 *
	 */
	public StepLists(MetaDataImpl metadata) {
		stepslists = new ArrayList();
		mapvar = new HashMap();
		this.metadata = metadata;
		pathresolver = new PathResolver(metadata);
		stepslistvisitor = new MakeStepsListVisitor();
	}
	
	public StepLists(HashMap mapvar, MetaDataImpl metadata) {
		stepslists = new ArrayList();
		this.mapvar = mapvar;
		this.metadata = metadata;
		pathresolver = new PathResolver(metadata);
		stepslistvisitor = new MakeStepsListVisitor();
	}

	// **********************************************************************
	// * METHODS
	// **********************************************************************
	public void newStepslist() throws XQueryException {
		if ((cur_stepslists != null) && (cur_stepslists.isEmpty()))
			return;
		cur_stepslists = new ArrayList(1);
		cur_stepslists.add(new ArrayList());
		stepslists.addAll(cur_stepslists);
	}

	public ArrayList getStepsLists() {
		return stepslists;
	}

	public Set getRegisteredVariables() {
		return mapvar.keySet();
	}

	public void registerVars(ArrayList vars) throws XQueryException {
		if (vars == null)
			return;
		for (int i = 0; i < vars.size(); i++)
			this.registerVar((Variable) vars.get(i));
	}
	public void registerVar(Variable var) throws XQueryException {

		// ADD   11/04/2003
		if ((var.getSourceNames() != null && !var.getSourceNames().isEmpty()) || (var.getUrls() != null && !var.getUrls().isEmpty())) {
			if (mapvar.get(var) != null)
				return;
			mapvar.put(var, null);
			return;
		}
		// END ADD

		XQueryExpression exp = var.getExpression();

		// ADD   11/04/2003
		if ((exp.getSourceNames() != null && !exp.getSourceNames().isEmpty()) || (exp.getUrls() != null && !exp.getUrls().isEmpty())) {
			var.setSourceNames(exp.getSourceNames());
			var.setUrls(exp.getUrls());
			if (mapvar.get(var) != null)
				return;
			mapvar.put(var, null);
			return;
		}
		// END ADD

		StepLists tmplist = new StepLists(this.mapvar, this.metadata);		
		stepslistvisitor.setStepsLists(tmplist);
		exp.accept(stepslistvisitor);

		// ADD   11/04/2003
		if ((exp.getSourceNames() != null && !exp.getSourceNames().isEmpty()) || (exp.getUrls() != null && !exp.getUrls().isEmpty())) {
			var.setSourceNames(exp.getSourceNames());
			var.setUrls(exp.getUrls());
			if (mapvar.get(var) != null)
				return;
			mapvar.put(var, null);
			return;
		}
		// END ADD

		//		if (exp.getSourceNames() != null && !exp.getSourceNames().isEmpty())
		//			var.setSourceNames(exp.getSourceNames());
		//		else {
		ArrayList al = tmplist.getStepsLists();

		if (al != null && !al.isEmpty()) {
			// there should be only one path for each variable
			/*
			if (al.size() != 1) 
				throw new XQueryException("More than one path associated to variable : " + var);
			*/
			ArrayList endlist = (ArrayList) al.get(al.size() - 1);
			mapvar.put(var, endlist);
			
			if (!endlist.isEmpty()) {
				if (((Step)endlist.get(0)).getExpression() instanceof Variable) {
					Variable frontvar = (Variable)((Step)endlist.get(0)).getExpression();
					var.setSourceNames(frontvar.getSourceNames());
					var.setUrls(frontvar.getUrls());
					exp.setSourceNames(frontvar.getSourceNames());
					exp.setUrls(frontvar.getUrls());
				} else if (metadata != null) {				
					pathresolver.resolveSteps(endlist);
					var.setSourceNames(pathresolver.getSourceNames());
					var.setUrls(pathresolver.getUrls());
					exp.setSourceNames(pathresolver.getSourceNames());
					exp.setUrls(pathresolver.getUrls());
				}
			}
		}
		//		}
		//return;
		//        ArrayList al = visitor.getStepsLists().getStepsLists();
		//        if (al != null && !al.isEmpty()) {
		//            // there should be only one path for each variable
		//			/*
		//			if (al.size() != 1) 
		//				throw new XQueryException("More than one path associated to variable : " + var);
		//            */
		//            if (al.size() == 1)
		//				mapvar.put(var, al.get(0)) ;
		//						
		//			if (metadata != null) {
		//				PathResolver pathresolver = new PathResolver(metadata);
		//				pathresolver.resolveStepsList(al);
		//				var.setSources(pathresolver.getArraySources());
		//			}
		//		}

	}

	public void addReplaceVar(Variable var) throws XQueryException {
		ArrayList prefix = (ArrayList) mapvar.get(var);
		if (prefix == null)
			return; // throw new Exception(var + " was not registered before. Something went wrong") ;
		for (int i = 0; i < prefix.size(); i++) {
			Step stepi = (Step) prefix.get(i);
			addStep(stepi);
		}
	}

	public void addStep(XQueryExpression step) throws XQueryException {
		if (step == null)
			return;

		if (cur_stepslists == null)
			newStepslist();

		if (step instanceof Variable) {
			newStepslist();
			addReplaceVar((Variable) step);
			return;
		}

		if (!(step instanceof Step))
			throw new XQueryException("Error: step is not type Step or Variable");

		Step stepp = (Step) step;
		/*
		if (stepp.getExpression() instanceof NodeType) {
			// do nothing
		} else if (stepp.getExpression() instanceof Variable)
			addReplaceVar((Variable) stepp.getExpression());
		else if (stepp.getExpression() instanceof QName)
			for (int i = 0; i < cur_stepslists.size(); i++)
				 ((ArrayList) cur_stepslists.get(i)).add(stepp);
		else if (stepp.getExpression() instanceof FunctionCOLLECTION)
			for (int i = 0; i < cur_stepslists.size(); i++)
				 ((ArrayList) cur_stepslists.get(i)).add(stepp);
		else if (stepp.getExpression() instanceof FunctionDOC)
			for (int i = 0; i < cur_stepslists.size(); i++)
				 ((ArrayList) cur_stepslists.get(i)).add(stepp);
		else if (stepp.getExpression() instanceof Value)
			for (int i = 0; i < cur_stepslists.size(); i++)
				 ((ArrayList) cur_stepslists.get(i)).add(stepp);
		else 
		*/
		if (stepp.getExpression() instanceof ListOpUNIONExpression) {
			// special case union operators are issued from normalization (located expressions)
			// handle first located expression
			ArrayList steps = ((LocatedExpression)((ListOpUNIONExpression)stepp.getExpression()).getExpression1()).getSteps();
			// clone current list (depth 2)
			ArrayList tmplist = new ArrayList(cur_stepslists.size());
			for (int i=0;i<cur_stepslists.size();i++)
				tmplist.add(((ArrayList)cur_stepslists.get(i)).clone());
			for (int i=0;i<steps.size();i++)
				addStep((Step)steps.get(i));
			// create new current list and remember previous
			ArrayList oldlist = cur_stepslists;
			stepslists.addAll(tmplist);
			cur_stepslists = tmplist;
			// handle second located expression
			steps = ((LocatedExpression)((ListOpUNIONExpression)stepp.getExpression()).getExpression2()).getSteps();
			for (int i=0;i<steps.size();i++)
				addStep((Step)steps.get(i));
			// set new current list (addition fof both lists
			cur_stepslists.addAll(oldlist);
		}
		else {
			for (int i = 0; i < cur_stepslists.size(); i++)
				 ((ArrayList) cur_stepslists.get(i)).add(stepp);
		}
//		else
//			throw new XQueryException("Error: step.getExpression : " + ((Step)step).getExpression() + " is not type QName or Variable or a FunctionCOLLECTION or a FunctionDOCUMENT");
	}

//	public String toString() {
//		StringBuffer sb = new StringBuffer();
//
//		sb.append("{");
//		for (int i = 0; i < stepslists.size(); i++) {
//			sb.append(" [");
//			ArrayList stepslist = (ArrayList) stepslists.get(i);
//			for (int j = 0; j < stepslist.size(); j++) {
//				Step step = (Step) stepslist.get(j);
//				if (j != 0)
//					sb.append(", ");
//				sb.append(step.toString());
//			}
//			sb.append("] ");
//		}
//		sb.append("}");
//		return sb.toString();
//	}
}
