/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * Copyright (C) 2003 XQuark Group.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.decomposer;

import java.util.ArrayList;
import java.util.HashMap;

import org.xquark.mediator.runtime.MediatorException;
import org.xquark.xquery.parser.Variable;
import org.xquark.xquery.parser.XQueryException;
import org.xquark.xquery.parser.XQueryExpression;

public class VariableDependencies {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.5 $";
	private static final String RCSName = "$Name:  $";
	// **********************************************************************
	// * CLASS VARIABLES
	// **********************************************************************
	// object containing
	// for each var all simple conditions in which this var appears
	private HashMap varSimpleConditionsMap = null;
	// for each var a map of (var,conditions) reprensenting simple joins
	private HashMap varJoinConditionsMap = null;
	// for each var a list of complex conditions in which it appears
	private HashMap varComplexConditionsMap = null;
	// for each var all conditions in which this var appears
	private HashMap varAllConditionsMap = null;
	// for each var all variables appearing in common conditions
	private HashMap varAssociatedVariablesMap = null; // { var1 [var2, var3], var2 [var3, var4] ...
	// for each condition all variables appearing
	private HashMap condAssociatedVariablesMap = null; // { c1 [var2, var3], c2 [var3, var4] ...
	// a list of all simple conditions
	private ArrayList allSimpleConditionsList = null;
	// a list of all join conditions
	private ArrayList allJoinConditionsList = null;
	// a list of all complex conditions
	private ArrayList allComplexConditionsList = null;
	// a list of all conditions
	private ArrayList allConditionsList = null;
	// a list of all conditions without variables !!!
	private ArrayList allNoVariablesConditionsList = null;
	// a visitor
	AtomizeWhereVisitor awvisitor = new AtomizeWhereVisitor();
	// **********************************************************************
	// * INITIALIZATION
	// **********************************************************************

	public VariableDependencies(XQueryExpression expr) throws MediatorException {
		try {
			expr.accept(awvisitor);
		} catch (XQueryException e) {
			throw new MediatorException("Could not atomize where clause : " + expr, e);
		}
		varSimpleConditionsMap = awvisitor.getVarSimpleConditionsMap();
		varJoinConditionsMap = awvisitor.getVarJoinConditionsMap();
		varComplexConditionsMap = awvisitor.getVarComplexConditionsMap();
		varAllConditionsMap = awvisitor.getVarAllConditionsMap();
		varAssociatedVariablesMap = awvisitor.getVarAssociatedVariablesMap();
		condAssociatedVariablesMap = awvisitor.getCondAssociatedVariablesMap();
		allNoVariablesConditionsList = awvisitor.getAllNoVariablesConditionsList();
		allSimpleConditionsList = awvisitor.getAllSimpleConditionsList();
		allJoinConditionsList = awvisitor.getAllJoinConditionsList();
		allComplexConditionsList = awvisitor.getAllComplexConditionsList();
		allConditionsList = awvisitor.getAllConditionsList();
	}
	
	// access methods
	public ArrayList getAllAssociatedVariables (Variable var) {
		return (ArrayList)varAssociatedVariablesMap.get(var);
	}
	public HashMap getJoinMap(Variable var) {
		return (HashMap)varJoinConditionsMap.get(var);
	}
	public boolean hasJoins() {
		if (allJoinConditionsList.isEmpty()) return false;
		return true; 
	}
	public HashMap getJoinsMap() {
		return varJoinConditionsMap; 
	}
//	public ArrayList getAllConditionsWithVar(Variable var) {
//		return (ArrayList)varAllConditionsMap.get(var);
//	}
	public ArrayList getAllComplexConditionsWithVar(Variable var) {
		return (ArrayList)varComplexConditionsMap.get(var);
	}
	public ArrayList getVarsOfCond(XQueryExpression expr) {
		return (ArrayList)condAssociatedVariablesMap.get(expr);
	}
	// debug methods
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append("varSimpleConditionsMap : ");
		sb.append(varSimpleConditionsMap);
		sb.append("\nvarJoinConditionsMap : ");
		sb.append(varJoinConditionsMap);
		sb.append("\nvarComplexConditionsMap : ");
		sb.append(varComplexConditionsMap);
		sb.append("\nvarAllConditionsMap : ");
		sb.append(varAllConditionsMap);
		sb.append("\nvarAssociatedVariablesMap : ");
		sb.append(varAssociatedVariablesMap);
		sb.append("\ncondAssociatedVariablesMap : ");
		sb.append(condAssociatedVariablesMap);
		sb.append("\nallNoVariablesConditionsList : ");
		sb.append(allNoVariablesConditionsList);
		sb.append("\nallSimpleConditionsList : ");
		sb.append(allSimpleConditionsList);
		sb.append("\nallJoinConditionsList : ");
		sb.append(allJoinConditionsList);
		sb.append("\nallComplexConditionsList : ");
		sb.append(allComplexConditionsList);		
		sb.append("\nallConditionsList : ");
		sb.append(allConditionsList);
		return sb.toString();
	}

}
