/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2004 XQuark Group.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.plan;

import java.util.ArrayList;

import org.xquark.mediator.DOMUtils.DOMUtils;
import org.xquark.mediator.DOMUtils.EvaluationVisitor;
import org.xquark.mediator.DOMUtils.Tuple;
import org.xquark.mediator.runtime.MediatorStatement;
import org.xquark.mediator.runtime.PreparedMediatorStatement;
import org.xquark.mediator.runtime._MediatorConnection;
import org.xquark.schema.SchemaManager;
import org.xquark.xml.xdbc.PreparedXMLStatement;
import org.xquark.xml.xdbc.XMLDBCException;
import org.xquark.xpath.datamodel.TypedNode;
import org.xquark.xquery.parser.Variable;
import org.xquark.xquery.parser.XQueryException;

public class DynamicContext {

	private MediatorStatement statement = null;
	private ArrayList currentTuples = new ArrayList();
	private ExecutionPlan plan = null;
	private EvaluationVisitor evalVisitor = null;

	// other element can be added later like TimeZone, currentDate ...

	public DynamicContext(ExecutionPlan plan, MediatorStatement statement) {
		this.statement = statement;
		this.plan = plan;
		this.evalVisitor = new EvaluationVisitor(plan.getSchemaManager());
	}

	public _MediatorConnection getConnection() {
		return statement.getMediatorConnection();
	}
	public void addCurrentTuple(Tuple tuple) {
		this.currentTuples.add(0, tuple);
	}

	public void deleteCurrentTuple(Tuple tuple) {
		this.currentTuples.remove(tuple);
	}

	public ArrayList getCurrentTuples() {
		return this.currentTuples;
	}

	public SchemaManager getSchemaManager() {
		return plan.getTypeVisitor().getSchemaManager();
	}

	public String getBaseURI() {
		return statement.getBaseURI();
	}

//	public boolean hasPreparedXMLStatement(OpSource opSource) {
//		return statement.hasPreparedXMLStatement(opSource);
//	}

	public PreparedXMLStatement getPreparedXMLStatement(OpSource opSource, String sourceName, String requestStr) throws XMLDBCException {
		return statement.getPreparedXMLStatement(opSource, sourceName, requestStr);
	}

	public Object getVariableValue(Variable var) throws XQueryException {		
	    Object strvalue = null;
		if (var.getExpression() == null) {
		    
			return ((PreparedMediatorStatement) statement).getVariables().get(var.getStringValue());
		}
		for (int j = 0; j < currentTuples.size(); j++) {
			evalVisitor.reset((Tuple) currentTuples.get(j));

			var.getExpression().accept(evalVisitor);
			ArrayList tmplist = evalVisitor.getResNodes();
			if (tmplist != null && !tmplist.isEmpty()) {
				if (tmplist.size() == 1) {
					TypedNode tmpnode = (TypedNode) tmplist.get(0);
					strvalue = tmpnode.getTypedValue();
					if (strvalue == null) {
						strvalue = DOMUtils.getTextLeaf(tmpnode);
					}
				} else {
					ArrayList seqlist = new ArrayList();
					for (int k = 0; k < tmplist.size(); k++) {
						TypedNode tmpnodek = (TypedNode) tmplist.get(k);
						strvalue = tmpnodek.getTypedValue();
						if (strvalue == null) {
							strvalue = DOMUtils.getTextLeaf(tmpnodek);
						}
						if (strvalue != null)
						    seqlist.add(strvalue);
					}
					if (!seqlist.isEmpty())
					    strvalue = seqlist;
					else
					    strvalue = null;
				}
				break;
			}
		}
		return strvalue;
	}
	
	public Object clone() {
		DynamicContext cloned = new DynamicContext(this.plan,this.statement);
		cloned.currentTuples = (ArrayList)currentTuples.clone();
		return cloned;
	}

}
