/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.plan;

import java.util.*;

import org.xquark.mediator.DOMUtils.BufferTuple;
import org.xquark.mediator.DOMUtils.Tuple;
import org.xquark.mediator.runtime.MediatorException;
import org.xquark.mediator.runtime.MediatorResultSet;
import org.xquark.mediator.runtime.MediatorStatement;
import org.xquark.schema.*;
import org.xquark.util.NamespaceContextStack;
import org.xquark.xml.xdbc.*;
import org.xquark.xpath.datamodel.TypedNode;
import org.xquark.xpath.datamodel.TypedValue;
import org.xquark.xquery.metadata.MetaDataImpl;
import org.xquark.xquery.parser.*;
import org.xquark.xquery.parser.hinttree.HintTree;
import org.xquark.xquery.typing.*;
import org.xquark.xquery.xdbc.XDBCResultSetInterface;

public class ExecutionPlan {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.20 $";
	private static final String RCSName = "$Name:  $";

	// **********************************************************************
	// * CLASS VARIABLES
	// **********************************************************************

	private Operator root = null;
	private ResultSet resultset = null;
	private TypeVisitor typevisitor;
	private String declarations = null;
	private ArrayList idvars = null;
	private HashMap aliases = null;
	private String baseURI = null;
	private boolean qaOnly = false;
	private XQueryExpression qmem;
	//private XQueryExpression origExpr = null;
	private String queryStringImports = null;
	private String queryString = null;
	private XQueryModule module = null;
	private MetaDataImpl metadata = null;

	private final int defaultsubqueryhandling = HintTree.MERGE_TYPE;
	//private final int defaultsubqueryhandling = HintTree.NESTED_TYPE;

	// to handle types generated by depnodesource
	SchemaManager schemamanager = null;
	Schema schema = null;
	// ***********************************************************************
	// * INITIALIZATION
	// ***********************************************************************
	public ExecutionPlan(MetaDataImpl metadata, TypeVisitor typevisitor, ArrayList idvars, String queryStringImports, String queryString, XQueryModule module, String baseURI, boolean qaOnly) throws MediatorException {
		if (typevisitor == null)
			throw new MediatorException("TypeVisitor in plan constructor cannot be null");
		this.metadata = metadata;
		this.typevisitor = typevisitor;
		this.idvars = idvars;
		//this.origExpr = origExpr;
		this.queryString = queryString;
		this.queryStringImports = queryStringImports;
		this.module = module;
		this.baseURI = baseURI;
		this.qaOnly = qaOnly;
		// compute declaration string
		NamespaceContextStack declarations = module.getDeclarations();
		if (declarations == null)
			this.declarations = null;
		else {
			StringBuffer buf = new StringBuffer();
			Collection declPrefixes = declarations.getPrefixes();
			//			Iterator itValues = declPrefixes.iterator();
			//			while (itValues.hasNext()) { buf.append(" "); buf.append((()itValues.next()).pprint()); }
			//			this.declarations = buf.toString();
			Iterator itValues = declPrefixes.iterator();
			String prefix = null;
			while (itValues.hasNext()) {
				prefix = (String) itValues.next();
				buf.append("declare ");
				if (prefix != null) {
					buf.append("namespace ");
					buf.append(prefix);
				} else
					buf.append("default element namespace");
				buf.append(" = \"");
				buf.append(declarations.getNamespaceURI(prefix));
				buf.append("\";\n");
			}
			this.declarations = buf.toString();
		}
		// create schemamanager and schema
		schemamanager = typevisitor.getTypingSchemaManager();
		schema = schemamanager.getSchema("http://www.xquark.org/EXECUTIONPLAN");
	}

	// ***********************************************************************
	// * GET/SET METHODS
	// ***********************************************************************
	public XDBCResultSetInterface getResultSet() throws MediatorException {
		try {
			PlanResultSet presultset = new PlanResultSet(resultset);
			return presultset;
		} catch (XMLDBCException xmldbce) {
			throw new MediatorException("ExecutionPlan.getResultSet: " + xmldbce.getMessage(), xmldbce);
		}
	}

	public SchemaManager getSchemaManager() {
		return schemamanager;
	}

	public String getDeclarations() {
		return declarations;
	}
	public TypeVisitor getTypeVisitor() {
		return typevisitor;
	}
	public Operator getRoot() {
		return root;
	}
	public void setRoot(Operator algebra) throws MediatorException {
		root = algebra;
	}
	public ArrayList getIdVars() {
		return idvars;
	}
	public void setIdVars(ArrayList idvars) {
		this.idvars = idvars;
	}

	public String getAlias(String varstr) {
		if (aliases == null)
			return null;
		for (Iterator it = aliases.keySet().iterator(); it.hasNext();) {
			String stri = (String) it.next();
			if (varstr.startsWith(stri))
				return (String) aliases.get(stri) + varstr.substring(stri.length());
		}
		return null;
	}

	public byte getDefaultSubHandling() {
		return defaultsubqueryhandling;
	}
	// ***********************************************************************
	// * METHODS
	// ***********************************************************************
	public void makeAlias(Variable var, XQueryExpression path) {
		if (aliases == null)
			aliases = new HashMap();
		aliases.put(var.getStringValue(), path.getStringValue());
	}

	/*
	public void addReplaceValues(Tuple tuple) {    	
		BufferTuple bt = tuple.getParent();
		for (int i=0;i<tuple.size();i++) {
			ArrayList tmplist = tuple.getNodesAtIndex(i);
			if (tmplist.size() != 1) continue;
			this.replaceValues.put(bt.getPathList().get(i),tmplist.get(0));
		}
	}
	
	public void deleteReplaceValues(Tuple tuple) {
		BufferTuple bt = tuple.getParent();
		for (int i=0;i<tuple.size();i++) {
			this.replaceValues.remove(bt.getPathList().get(i));
		}
	}
	
	public HashMap getCurrentValues() {
		return this.replaceValues;
	}
	*/

	public String getBaseURI() {
		return baseURI;
	}

	public boolean getQAOnly() {
		return qaOnly;
	}

	public XQueryModule getModule() {
		return module;
	}

	// ***********************************************************************
	// * EXECUTE QUERY
	// ***********************************************************************
	/**
	 *
	 */
	public XMLResultSet executeQuery(MediatorStatement statement) throws XQueryException, XMLDBCException {
		ResultSet resultset = null;
		// test source number
		DynamicContext context = new DynamicContext(this, statement);

		if (queryStringImports != null) {
			// create String to add for external variables
			PreparedXMLStatement preparedStatement = null;
			Iterator it = module.getSourceNames().iterator();
			String sourceName = (String) it.next();
			try {
				preparedStatement = context.getPreparedXMLStatement(null, sourceName, queryStringImports);
			} catch (XMLDBCException xe) {
				throw new MediatorException(xe.getMessage(), xe);
			}
			if (preparedStatement != null) {
				preparedStatement.setBaseURI(baseURI);
				HashMap varMap = statement.getVariables();
				if (varMap != null) {
					for (Iterator itns = varMap.keySet().iterator(); itns.hasNext();) {
						String nsStr = (String) itns.next();
						HashMap nsMap = (HashMap) varMap.get(nsStr);
						for (Iterator itvn = nsMap.keySet().iterator(); itvn.hasNext();) {
							String vnStr = (String) itvn.next();
							TypedValue typedValue = (TypedValue) nsMap.get(vnStr);
							Object value = typedValue.getTypedValue();
							preparedStatement.setObject(nsStr, vnStr, value);
						}
					}
				}
				return preparedStatement.executeQuery();
			} else {
				XMLConnection connection = null;
				connection = context.getConnection().getSubConnectionByName(sourceName);
				if (connection == null)
					throw new MediatorException("AlgSource: couldn't established connection to " + sourceName);
				XMLStatement stat = connection.createStatement();
				stat.setBaseURI(baseURI);
				// add to quryString all external variables defined in statment
				StringBuffer sb = new StringBuffer();
				HashMap varMap = statement.getVariables();
				if (varMap != null) {
					for (Iterator itns = varMap.keySet().iterator(); itns.hasNext();) {
						String nsStr = (String) itns.next();
						HashMap nsMap = (HashMap) varMap.get(nsStr);
						HashMap moduleNsMap = (HashMap) module.getVariables().get(nsStr);
						for (Iterator itvn = nsMap.keySet().iterator(); itvn.hasNext();) {
							String vnStr = (String) itvn.next();
							TypedValue typedValue = (TypedValue) nsMap.get(vnStr);
							Object value = typedValue.getTypedValue();
							Variable var = (Variable) moduleNsMap.get(vnStr);
							if (value != null) {
								sb.append("declare variable $");
								if (nsStr != null) {
									sb.append(" {");
									sb.append(nsStr);
									sb.append("}");
								}
								sb.append(vnStr);
								if (var != null && var.getTypeDeclaration() != null) {
									sb.append(" as ");
									sb.append(var.getTypeDeclaration());
									sb.append("");
								}
								sb.append(" {");
								if (value instanceof String) {
									sb.append("\"");
									sb.append(value.toString());
									sb.append("\"");
								} else
									sb.append(value.toString());
								sb.append("};\n");
							}
						}
					}
				}
				sb.append(queryString);
				return stat.executeQuery(sb.toString());
			}
		} else {
			// create tuple in context
			HashMap varMap = statement.getVariables();
			if (varMap != null) {
				ArrayList paths = new ArrayList();
				ArrayList nodes = new ArrayList();
				for (Iterator itns = varMap.keySet().iterator(); itns.hasNext();) {
					String nsStr = (String) itns.next();
					HashMap nsMap = (HashMap) varMap.get(nsStr);
					HashMap moduleNsMap = (HashMap) module.getVariables().get(nsStr);
					for (Iterator itvn = nsMap.keySet().iterator(); itvn.hasNext();) {
						String vnStr = (String) itvn.next();
						TypedNode typedNode = (TypedNode) nsMap.get(vnStr);
						Variable var = (Variable) moduleNsMap.get(vnStr);
						if (var != null) {
							paths.add(var);
							nodes.add(typedNode);
						}
					}
				}
				BufferTuple buf = new BufferTuple(paths, 0);
				Tuple tuple = buf.newTuple();
				for (int i = 0; i < nodes.size(); i++)
					tuple.addNodeAtIndex(i, (TypedNode) nodes.get(i));
				buf.add(tuple);
				context.addCurrentTuple(tuple);
			}
			if (root != null) {
				resultset = root.executeQuery(context);
			} else
				resultset = new FakeResultSet(null, context);
			return new MediatorResultSet(new PlanResultSet(resultset), getQMEM(), getIdVars(), statement);
		}
	}
	// **********************************************************************
	// * OPTIMIZE
	// **********************************************************************
	/**
	 *
	 */
	public void optimize() throws MediatorException {
		//root = root.optimize() ;
	}

	// type name handling
	private final String TYPE_NAME = "PlanType";
	private int typeIndex = 0;
	public void setTypeName(Type type) throws SchemaException {
		type.setName(TYPE_NAME + (++typeIndex));
		schema.register(type);
		this.schema.initialize();
	}
	// **********************************************************************
	// * DEBUG
	// **********************************************************************
	/**
	 *
	 */
	public String toString() {
		StringBuffer buf = new StringBuffer();
		buf.append("<Plan>\n");
		if (root != null)
			buf.append(root.toCompleteString(1));
		buf.append("</Plan>\n");
		return buf.toString();
	}

	public void setQMEM(XQueryExpression expression) {
		qmem = expression;
	}
	public XQueryExpression getQMEM() {
		return qmem;
	}

	public void appendExternalVariables(StringBuffer sbvars, ArrayList externalVariables) {
		int length = sbvars.length();
		for (int i = 0; i < externalVariables.size(); i++) {
			Variable vari = (Variable) externalVariables.get(i);
			sbvars.append("declare variable ");
			sbvars.append(vari.getStringValue(module.getDeclarations()));
			Type typ = vari.getQType().getType();
			if (typ == null) {
				typ = schemamanager.getAnySimpleType();
			}
			if (typ.getValueType() == null) {
				sbvars.setLength(length);
				return;
			} else {
				sbvars.append(" as xs:");
				sbvars.append((typ.getValueType()).getPrimitive().getName());
			}
			sbvars.append(" external;\n");
		}
	}

	/**
	 * @return
	 */
	public MetaDataImpl getMetadata() {
		return metadata;
	}

}
