/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.plan;

import java.util.ArrayList;

import org.xquark.mediator.DOMUtils.BufferTuple;
import org.xquark.mediator.DOMUtils.EvaluationVisitor;
import org.xquark.mediator.DOMUtils.Tuple;
import org.xquark.mediator.runtime.MediatorException;
import org.xquark.xquery.parser.*;
import org.xquark.xquery.parser.primitivefunctions.fnfunctions.FunctionCOUNT;

public class OpAggregateFunc extends OpUn {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.9 $";
	private static final String RCSName = "$Name:  $"; // * INITIALIZATION
	// ***********************************************************************
	/**
	 *
	 */
	public OpAggregateFunc(ExecutionPlan plan, XQueryExpression expression, Operator childexp) throws MediatorException {
		super(plan, expression, childexp);
		// the path number can be only one (otherwise distinct values might be incorrect)
		if (paths != null)
			paths.clear();
		// calculate paths
		if (paths == null)
			paths = new ArrayList();
		getRetPath(((FunctionCall) expression).getArgument(0), paths);
		if (paths.isEmpty() || paths.size() > 1)
			throw new MediatorException("Could not find paths for aggregate function call");
		// ADD LARS 30/03/04
		 ((FunctionCall) expression).getArguments().set(0, paths.get(0));
		size = 1;
		idsize = 0;
	}

	// TODO better!!!
	private void getRetPath(XQueryExpression expr, ArrayList reslist) {
		if (expr instanceof XQueryExpressionSequence) {
			for (int i = 0; i < ((XQueryExpressionSequence) expr).getSubExpressions().size(); i++) {
				getRetPath((XQueryExpression) ((XQueryExpressionSequence) expr).getSubExpressions().get(i), reslist);
			}
		}
		if (expr instanceof LocatedExpression || expr instanceof Value || expr instanceof Variable) {
			reslist.add(expr);
		} else if (expr instanceof FLWRExpression) {
			getRetPath(((FLWRExpression) expr).getReturnClause(), reslist);
		} else if (expr instanceof FunctionCall) {
			getRetPath(((FunctionCall) expr).getArgument(0), reslist);
		}
	}

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(OperatorVisitor visitor) throws MediatorException {
		visitor.visit(this);
	}

	// ***********************************************************************
	// * EXECUTE QUERY
	// ***********************************************************************
	/**
	 *
	 */
	public ResultSet getResultSet(DynamicContext context, OperatorRunnable child) throws MediatorException {
		return new AggregateFuncResultSet(this, context, child);
	}

	// **********************************************************************
	// * OPTIMIZE
	// **********************************************************************
	/**
	 * FuncMAX *this* node with the subnode(s). As it should be called
	 * only from source, the returned node must be of type AlgSource.
	 * The return node is a AlgSource where the XQueryExpression is the merging
	 * of all child(ren) nodes' expression.
	 */
	// protected Algebra mergeWithSub() throws MediatorException { return this ; }

	// **********************************************************************
	// * DEBUG
	// **********************************************************************
	/**
	 *
	 */
}

class AggregateFuncResultSet extends UnResultSet {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.9 $";
	private static final String RCSName = "$Name:  $"; // * CLASS VARIABLES
	// ***********************************************************************

	// ***********************************************************************
	// * INITIALIZATION
	// ***********************************************************************
	public AggregateFuncResultSet(OpAggregateFunc algebra, DynamicContext context, OperatorRunnable child) throws MediatorException {
		super(algebra, context, child);
	}

	// ***********************************************************************
	// * EVALUATE IMPLEMENTATION
	// ***********************************************************************
	/**
	 * In a merge product, we can all do in parallel, so we
	 * must generate all results of both source and then make the merge
	 * product.
	 *
	 * @param non_blocking if the evaluation is blocking, we must
	 *        generate all the results before passing.
	 */
	protected void evaluate(boolean non_blocking) throws MediatorException {
		//ArrayList childpaths = null ;
		//Algebra child = ((AlgUnOp) algebra).getChildAlgebra() ;
		//childpaths = child.getPaths() ;

		// change LARS 09/03/04 to avoid exit when result set is empty but aggregage function can be calculated
		if (!(this.getOperator().getExpression() instanceof FunctionCOUNT) && !resultset.hasNext()) {
			//if (!resultset.hasNext()) {
			resultset.close();
			setFinishedWhenEmpty();
			return;
		}

		BufferTuple tmpBufferTuple = new BufferTuple(resultset.getOperator().getPaths(), 0);
		if (resultset.hasNext()) {
			while (resultset.hasNext()) {
				Tuple tuple = resultset.next();
				tuple.eraseIdSize();
				tmpBufferTuple.add(tuple);
			}
		}

		resultset.close();
		try {
			EvaluationVisitor visitor = new EvaluationVisitor(this.operator.getPlan().getSchemaManager());
			visitor.reset(tmpBufferTuple);
			visitor.setReturnType(EvaluationVisitor.TUPLE_TYPE);
			this.getOperator().getExpression().accept(visitor);
			Tuple tuple = visitor.getResultTuple();
			if (tuple != null)
				this.buftuples.add(tuple);
		} catch (XQueryException xqe) {
			throw new MediatorException(xqe.getMessage(), xqe);
		}

		setFinishedWhenEmpty();
	}
}
