/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.plan;

import java.util.ArrayList;

import org.xquark.mediator.DOMUtils.Tuple;
import org.xquark.mediator.runtime.MediatorException;
import org.xquark.xml.xdbc.XMLDBCException;
import org.xquark.xquery.parser.XQueryExpression;

public class OpMerge extends OpBin {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.8 $";
	private static final String RCSName = "$Name:  $";

	// **********************************************************************
	// * CLASS VARIABLES
	// **********************************************************************
	// ***********************************************************************
	// * INITIALIZATION
	// ***********************************************************************
	/**
	 *
	 */
	public OpMerge(ExecutionPlan plan, XQueryExpression expression, Operator left, Operator right) throws MediatorException {
		super(plan, expression, left, right);
		init();
	}

//	public OpMerge(ExecutionPlan plan, XQueryExpression expression, ArrayList algebralist) throws MediatorException {
//		super(plan, expression, algebralist);
//		init();
//	}

	private void init() throws MediatorException {
		// Sometimes, it can happens that the two members of merge product have common attributes. In that case, we must mark them
		// as common. So when we will generate the tuples, we can access them efficiently.
		ArrayList childpaths1 = getLeftAlgebra().getPathsList();
		ArrayList childpaths2 = getRightAlgebra().getPathsList();
		size = getLeftAlgebra().getSize() + getRightAlgebra().getSize();

//		equivalences = new ArrayList();
//		int index = -1;
//		for (int i = 0; i < childpaths2.size(); i++) {
//			if ((index = childpaths1.indexOf(childpaths2.get(i))) != -1) {
//				addEquivalence(equivalences, index, i);
//				size--;
//				break;
//			}
//		}

//		idequivalences = new ArrayList();
//		ArrayList childvars1 = getLeftAlgebra().getVariables();
//		ArrayList childvars2 = getRightAlgebra().getVariables();
		idsize = getLeftAlgebra().getIdSize() + getRightAlgebra().getIdSize();
//		for (int i = 0; i < childvars2.size(); i++) {
//			Variable var2i = (Variable) childvars2.get(i);
//			if ((index = childvars1.indexOf(var2i)) != -1) {
//				addEquivalence(idequivalences, index, i);
//				idsize--;
//			}
//		}

		ordertype = ORDER_PARALLELIZATION;
	}

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(OperatorVisitor visitor) throws MediatorException {
		visitor.visit(this);
	}

	// ***********************************************************************
	// * EXECUTE QUERY
	// ***********************************************************************
	/**
	 *
	 */
	protected ResultSet getResultSet(DynamicContext context, OperatorRunnable[] children) throws MediatorException {
		try {
			return new MergeResultSet(this, context, children);
		} catch (XMLDBCException e) {
			throw new MediatorException("Can't construct " + getClass() + ": " + e.getMessage(), e);
		}
	}

	// **********************************************************************
	// * OPTIMIZE
	// **********************************************************************
	/**
	 * Merge *this* node with the subnode(s). As it should be called
	 * only from source, the returned node must be of type AlgSource.
	 * The return node is a AlgSource where the XQueryExpression is the merging
	 * of all child(ren) nodes' expression.
	 */
	// protected Algebra mergeWithSub() throws MediatorException { return this ; }

	// **********************************************************************
	// * DEBUG
	// **********************************************************************
	/**
	 *
	 */
}

class MergeResultSet extends BinResultSet {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.8 $";
	private static final String RCSName = "$Name:  $"; // * CLASS VARIABLES
	// ***********************************************************************
	
	private Tuple halftuple2 = null;
	// ***********************************************************************
	// * INITIALIZATION
	// ***********************************************************************
	public MergeResultSet(OpMerge operator, DynamicContext context, OperatorRunnable[] children) throws XMLDBCException {
		super(operator, context, children);
	}

	// ***********************************************************************
	// * EVALUATE IMPLEMENTATION
	// ***********************************************************************
	/**
	 * In a merge product, we can all do in parallel, so we
	 * must generate all results of both source and then make the merge
	 * product.
	 * @param non_blocking if the evaluation is blocking, we must
	 *        generate all the results before passing.
	 *			A merge is necessarly blocking.
	 */
	protected void evaluate(boolean non_blocking) throws MediatorException {
		
		if (!children[0].getResultSet().hasNext()) {
			children[0].getResultSet().close();
			if (halftuple2 == null) 
				children[1].getResultSet().close();
			setFinishedWhenEmpty();
			return;
		}
		
		if (halftuple2 == null) {
			if (children[1].getResultSet().hasNext())
				halftuple2 = children[1].getResultSet().next() ;
			children[1].getResultSet().close();
		}		

        while (true) {
            if (children[0].getResultSet().hasNext()) {
                Tuple halftuple1 = children[0].getResultSet().next() ;
                if (halftuple2 != null)
					buftuples.add(halftuple1.merge(halftuple2, operator));
				else
					buftuples.add(halftuple1.completeTupleForLeft(operator.getSize()));
                if (non_blocking) {
                    // can make higher buffering by modifying this test
                    if ((buftuples != null) && (! buftuples.isEmpty()))
                        break ;
                }
            }
            else {
        		children[0].getResultSet().close();
                setFinishedWhenEmpty() ;
                return ;
            }
        }
        if (! children[0].getResultSet().hasNext()) {
        	children[0].getResultSet().close();
            setFinishedWhenEmpty() ;
        }
	}


}
