/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.plan;

import java.util.ArrayList;

import org.xquark.mediator.DOMUtils.DOMUtils;
import org.xquark.mediator.DOMUtils.EvaluationVisitor;
import org.xquark.mediator.DOMUtils.Tuple;
import org.xquark.mediator.algebra.Algebra;
import org.xquark.mediator.runtime.MediatorException;
import org.xquark.xpath.datamodel.TypedNode;
import org.xquark.xquery.parser.Variable;
import org.xquark.xquery.parser.XQueryException;
import org.xquark.xquery.parser.XQueryExpression;

public class OpNotSource extends OpUn {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.12 $";
	private static final String RCSName = "$Name:  $"; // * INITIALIZATION
	// ***********************************************************************
	private boolean onlyVar = false;

	public OpNotSource(ExecutionPlan plan, XQueryExpression expression, Variable var, Operator childexp, boolean isLet) throws MediatorException {
		super(plan, expression, childexp);
		init(var);
		this.isLet(isLet);
		idsize = childOperator.getIdSize();
		if (plan.getIdVars().contains(var)) {
			idsize++;
		}
	}

	//	public AlgNotSource(ExecutionPlan plan, XQueryExpression expression, Variable var, boolean isLet, boolean hasIdentifier) throws MediatorException {
	//		super(plan, expression, null);
	//		init(var);
	//		this.isLet(isLet);
	//		if (hasIdentifier) {
	//			idsize = 1;
	//		}
	//	}

	public OpNotSource(ExecutionPlan plan, Algebra depnode) throws MediatorException {
		super(plan, depnode.getExpression(), null);
		if (depnode.getPaths().isEmpty())
			init((Variable) depnode.getVariables().get(0));
		else {
			addPaths(depnode.getPaths());
			size = paths.size();
		}
		this.isLet(depnode.isLet());
		if (depnode.hasIdentifier()) {
			idsize = 1;
		}
	}

	private void init(Variable var) {
		onlyVar = true;
		resetPaths();
		addPath(var);
		size = 1;
	}

	public boolean isOnlyVar() {
		return onlyVar;
	}

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(OperatorVisitor visitor) throws MediatorException {
		visitor.visit(this);
	}

	// ***********************************************************************
	// * EXECUTE QUERY
	// ***********************************************************************
	/**
	 *
	 */
	protected ResultSet getResultSet(DynamicContext context, OperatorRunnable child) throws MediatorException {
		return new NotSourceResultSet(this, context, child);
	}

	// **********************************************************************
	// * OPTIMIZE
	// **********************************************************************

	// **********************************************************************
	// * DEBUG
	// **********************************************************************
	/**
	 *
	 */
}

class NotSourceResultSet extends UnResultSet {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.12 $";
	private static final String RCSName = "$Name:  $"; // * INITIALIZATION
	// ***********************************************************************

	EvaluationVisitor visitor = null;
	/**
	 *
	 */
	public NotSourceResultSet(OpNotSource operator, DynamicContext context, OperatorRunnable child) throws MediatorException {
		super(operator, context, child);
	}

	// ***********************************************************************
	// * EVALUATE IMPLEMENTATION
	// ***********************************************************************
	/*
	 * @param non_blocking if the evaluation is blocking, we must
	 *        generate all the results before passing.
	 */
	protected void evaluate(boolean non_blocking) throws MediatorException {

		boolean islet = this.operator.isLet();
		if (islet)
			non_blocking = false;
		Tuple newtuple = null;
		if (resultset == null) {
			if (expression != null) {
				ArrayList nodes = DOMUtils.evalNodes(context, expression, null);
				if (nodes != null) {
					if (islet) {
						if (((OpNotSource) operator).isOnlyVar()) {
							newtuple = buftuples.newTuple();
							for (int i = 0; i < nodes.size(); i++) {
								newtuple.addNodeAtIndex(0, (TypedNode) nodes.get(i));
							}
							newtuple.fillIdentifiers();
							buftuples.add(newtuple);
						} else {
							newtuple = buftuples.newTuple();
							for (int i = 0; i < nodes.size(); i++) {
								ArrayList nodelist = new ArrayList(1);
								nodelist.add(nodes.get(i));
								for (int j = 0; j < paths.size(); j++) {
									ArrayList reslist = DOMUtils.getSteps(nodelist, (XQueryExpression) paths.get(j));
									newtuple.addNodesAtIndex(j, reslist);
								}
							}
							newtuple.fillIdentifiers();
							buftuples.add(newtuple);
						}
					} else {
						if (((OpNotSource) operator).isOnlyVar()) {
							for (int i = 0; i < nodes.size(); i++) {
								newtuple = buftuples.newTuple();
								newtuple.addNodeAtIndex(0, (TypedNode) nodes.get(i));
								newtuple.fillIdentifiers();
								buftuples.add(newtuple);
							}
						} else {
							for (int i = 0; i < nodes.size(); i++) {
								ArrayList nodelist = new ArrayList(1);
								nodelist.add(nodes.get(i));
								newtuple = buftuples.newTuple();
								for (int j = 0; j < paths.size(); j++) {
									ArrayList reslist = DOMUtils.getSteps(nodelist, (XQueryExpression) paths.get(j));
									newtuple.addNodesAtIndex(j, reslist);
								}
								newtuple.fillIdentifiers();
								buftuples.add(newtuple);
							}
						}
					}
				}
			}
			setFinishedWhenEmpty();
		} else {
			while (true) {
				if (resultset.hasNext()) {
					Tuple tuple = resultset.next();
					if (expression == null) {
						if (islet) {
							if (newtuple == null) {
								newtuple = tuple.getPart(0, operator.getIdSize());
								buftuples.add(newtuple);
							} else
								newtuple.addNodesAtIndex(0, tuple.getNodesAtIndex(0));
						} else {
							int index = 0;
							while ((newtuple = tuple.getPart(0, index++, operator.getIdSize())) != null) {
								buftuples.add(newtuple);
							}
						}
					} else { // expression is not null
						try {
							ArrayList tuples = context.getCurrentTuples();
							tuples.add(tuple);
							if (visitor == null)
								visitor = new EvaluationVisitor(this.operator.getPlan().getSchemaManager());
							visitor.reset(tuples);
							visitor.setReturnType(EvaluationVisitor.NODE_TYPE);
							expression.accept(visitor);
							ArrayList nodes = visitor.getResNodes();
							if (nodes != null) {
								if (islet) {
									if (((OpNotSource) operator).isOnlyVar()) {
										if (newtuple == null) {
											newtuple = buftuples.newTuple();
											newtuple.addNodesAtIndex(0, nodes);
											newtuple.fillIdentifiers();
											buftuples.add(newtuple);
										} else
											newtuple.addNodesAtIndex(0, nodes);
									} else {
										if (newtuple == null) {
											newtuple = buftuples.newTuple();
											for (int i = 0; i < nodes.size(); i++) {
												ArrayList nodelist = new ArrayList(1);
												nodelist.add(nodes.get(i));
												for (int j = 0; j < paths.size(); j++) {
													ArrayList reslist = DOMUtils.getSteps(nodelist, (XQueryExpression) paths.get(j));
													newtuple.addNodesAtIndex(j, reslist);
												}
											}
											newtuple.fillIdentifiers();
											buftuples.add(newtuple);
										} else {
											for (int i = 0; i < nodes.size(); i++) {
												ArrayList nodelist = new ArrayList(1);
												nodelist.add(nodes.get(i));
												for (int j = 0; j < paths.size(); j++) {
													ArrayList reslist = DOMUtils.getSteps(nodelist, (XQueryExpression) paths.get(j));
													newtuple.addNodesAtIndex(j, reslist);
												}
											}
										}
									}
								} else {
									if (((OpNotSource) operator).isOnlyVar()) {
										for (int i = 0; i < nodes.size(); i++) {
											newtuple = buftuples.newTuple();
											newtuple.addNodeAtIndex(0, (TypedNode) nodes.get(i));
											newtuple.fillIdentifiers();
											buftuples.add(newtuple);
										}
									} else {
										for (int i = 0; i < nodes.size(); i++) {
											ArrayList nodelist = new ArrayList(1);
											nodelist.add(nodes.get(i));
											newtuple = buftuples.newTuple();
											for (int j = 0; j < paths.size(); j++) {
												ArrayList reslist = DOMUtils.getSteps(nodelist, (XQueryExpression) paths.get(j));
												newtuple.addNodesAtIndex(j, reslist);
											}
											newtuple.fillIdentifiers();
											buftuples.add(newtuple);
										}
									}
								}
							}
							tuples.remove(tuple);
						} catch (XQueryException e) {
							throw new MediatorException("AlgNotSource.evaluate: " + e.getMessage(), e);
						}
					}
					if (non_blocking) {
						// can make higher buffering by modifying this test
						if ((buftuples != null) && (!buftuples.isEmpty()))
							break;
					}
				} else {
					setFinishedWhenEmpty();
					resultset.close();
					return;
				}
			}
			if (!resultset.hasNext()) {
				setFinishedWhenEmpty();
				resultset.close();
			}
		}
	}
}
