/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.plan ;

import org.xquark.mediator.DOMUtils.EvaluationVisitor;
import org.xquark.mediator.DOMUtils.Tuple;
import org.xquark.mediator.runtime.MediatorException;
import org.xquark.xquery.parser.XQueryException;
import org.xquark.xquery.parser.XQueryExpression;


public class OpRestrict extends OpUn {
    // **********************************************************************
    // * VERSIONING
    // **********************************************************************
    private static final String RCSRevision = "$Revision: 1.12 $";
    private static final String RCSName = "$Name:  $";    // * INITIALIZATION
    // ***********************************************************************
    /**
     *
     */
    public OpRestrict(ExecutionPlan plan, XQueryExpression expression,Operator childexp) throws MediatorException {
        super(plan, expression, childexp) ;
        size = childexp.getSize();
        idsize = childexp.getIdSize();
    }

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(OperatorVisitor visitor) throws MediatorException {
		visitor.visit(this);
	}

    // ***********************************************************************
    // * EXECUTE QUERY
    // ***********************************************************************
    /**
     *
     */
    protected ResultSet getResultSet(DynamicContext context, OperatorRunnable child) throws MediatorException {
    	return new RestrictResultSet(this, context, child) ;
    }

    // **********************************************************************
    // * OPTIMIZE
    // **********************************************************************

    // **********************************************************************
    // * DEBUG
    // **********************************************************************
}



class RestrictResultSet extends UnResultSet {
    // **********************************************************************
    // * VERSIONING
    // **********************************************************************
    private static final String RCSRevision = "$Revision: 1.12 $";
    private static final String RCSName = "$Name:  $";    // * INITIALIZATION
    // ***********************************************************************

	private EvaluationVisitor evalvisitor = null;

    /**
     *
     */
    public RestrictResultSet(OpRestrict operator, DynamicContext context, OperatorRunnable child) throws MediatorException {
        super(operator, context, child) ;
    }

    // ***********************************************************************
    // * EVALUATE IMPLEMENTATION
    // ***********************************************************************
    /**
     * @param non_blocking if the evaluation is blocking, we must
     *        generate all the results before passing.
     */
    protected void evaluate(boolean non_blocking) throws MediatorException {
        
        while (true) {
            if (resultset.hasNext()) {
                Tuple tuple = resultset.next() ;
                try {
                    if (expression == null) buftuples.add(tuple) ;
                    else {
                        if (evalvisitor == null) 
                        	evalvisitor = new EvaluationVisitor(this.operator.getPlan().getSchemaManager()) ;
						context.addCurrentTuple(tuple);
						//evalvisitor.reset(tuple);
						evalvisitor.reset(context.getCurrentTuples());
						evalvisitor.setReturnType(EvaluationVisitor.BOOLEAN_TYPE);
                        expression.accept(evalvisitor) ;
						context.deleteCurrentTuple(tuple);
                        if (evalvisitor.getVerdict()) {
                            buftuples.add(tuple) ;
                        }
                        else if (! resultset.hasNext()) setFinishedWhenEmpty() ;
                    }
                }
                catch (XQueryException e) {
                    throw new MediatorException("AlgRestrict.evaluate: " + e.getMessage(), e) ;
                }

                if (non_blocking) {
                    // can make higher buffering by modifying this test
                    if ((buftuples != null) && (! buftuples.isEmpty()))
                        break ;
                }
            }
            else {
                setFinishedWhenEmpty() ;
				resultset.close();
                return ;
            }
        }
        if (! resultset.hasNext()) {
            setFinishedWhenEmpty() ;
			resultset.close();
        }
		return;
    }
}
