/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.plan;

import java.util.ArrayList;

import org.xquark.mediator.DOMUtils.DOMUtils;
import org.xquark.mediator.DOMUtils.EvaluationVisitor;
import org.xquark.mediator.DOMUtils.Tuple;
import org.xquark.mediator.algebra.Algebra;
import org.xquark.mediator.runtime.MediatorException;
import org.xquark.xpath.datamodel.TypedNode;
import org.xquark.xquery.parser.Variable;
import org.xquark.xquery.parser.XQueryException;
import org.xquark.xquery.parser.XQueryExpression;
import org.xquark.xquery.parser.primitivefunctions.fnfunctions.FunctionCOUNT;

public class OpSimpleMerge extends OpUn {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.14 $";
	private static final String RCSName = "$Name:  $"; // * INITIALIZATION
	// ***********************************************************************
	private boolean onlyVar = false;

	public OpSimpleMerge(ExecutionPlan plan, XQueryExpression expression, Variable var, Operator childexp, boolean isLet) throws MediatorException {
		super(plan, expression, childexp);
		init(var);
		this.isLet(isLet);
	}

	public OpSimpleMerge(ExecutionPlan plan, Algebra algebra, Operator childexp) throws MediatorException {
		super(plan, algebra.getExpression(), childexp);
		Variable tmpVar = (Variable) algebra.getVariables().get(0);
		if (algebra.getPaths().isEmpty())
			init(tmpVar);
		else {
			ArrayList tmpPaths = algebra.getPaths();
			addPaths(tmpPaths);
			size = childOperator.getSize() + tmpPaths.size();
			idsize = childOperator.getIdSize();
			if (plan.getIdVars().contains(tmpVar))
				idsize++;
			if (tmpPaths.size() == 1 && ((XQueryExpression) tmpPaths.get(0)).getStringValue().equals(tmpVar.getStringValue()))
				onlyVar = true;
		}
		this.isLet(algebra.isLet());
	}

	private void init(Variable var) {
		//resetPaths();
		addPath(var);
		size = childOperator.getSize() + 1;
		idsize = childOperator.getIdSize();
		if (plan.getIdVars().contains(var))
			idsize++;
		onlyVar = true;
	}

	public boolean isOnlyVar() {
		return onlyVar;
	}

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(OperatorVisitor visitor) throws MediatorException {
		visitor.visit(this);
	}

	// ***********************************************************************
	// * EXECUTE QUERY
	// ***********************************************************************
	/**
	 *
	 */
	protected ResultSet getResultSet(DynamicContext context, OperatorRunnable child) throws MediatorException {
		return new SimpleMergeResultSet(this, context, child);
	}

	// **********************************************************************
	// * OPTIMIZE
	// **********************************************************************
	/**
	 *
	 */

	// **********************************************************************
	// * DEBUG
	// **********************************************************************
	/**
	 *
	 */
}

class SimpleMergeResultSet extends UnResultSet {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.14 $";
	private static final String RCSName = "$Name:  $"; // * INITIALIZATION
	// ***********************************************************************
	EvaluationVisitor visitor = null;
	/**
	 *
	 */
	public SimpleMergeResultSet(OpSimpleMerge operator, DynamicContext context, OperatorRunnable child) throws MediatorException {
		super(operator, context, child);
	}

	// ***********************************************************************
	// * EVALUATE IMPLEMENTATION
	// ***********************************************************************
	/*
	 * @param non_blocking if the evaluation is blocking, we must
	 *        generate all the results before passing.
	 */
	protected void evaluate(boolean non_blocking) throws MediatorException {

		ArrayList idxsteps = new ArrayList();
		idxsteps.add(new Integer(0));
		boolean islet = this.operator.isLet();
		XQueryExpression expression = this.operator.getExpression();
 
		if (!resultset.hasNext()) {
			try {
				EvaluationVisitor visitor = new EvaluationVisitor(this.operator.getPlan().getSchemaManager());
				Tuple newtuple = new Tuple(operator.getSize()-resultset.getOperator().getSize(),operator.getIdSize()-resultset.getOperator().getIdSize());
				newtuple.setParent(buftuples);
				visitor.reset(newtuple);
				visitor.setReturnType(EvaluationVisitor.NODE_TYPE);
				expression.accept(visitor);
				ArrayList nodes = visitor.getResNodes();
				if (nodes != null)
					merge(islet,newtuple,nodes);
			} catch (XQueryException xqe) {
				throw new MediatorException(xqe.getMessage(), xqe);
			}
			setFinishedWhenEmpty();
			resultset.close();
			return;
		}

		while (true) {
			if (resultset.hasNext()) {
				Tuple tuple = resultset.next();
				ArrayList nodes = tuple.getPath(expression.getStringValue());
				if (nodes == null) {
					try {
						if (visitor == null)
							visitor = new EvaluationVisitor(this.operator.getPlan().getSchemaManager());
						visitor.reset(tuple);
						visitor.setReturnType(EvaluationVisitor.NODE_TYPE);
						expression.accept(visitor);
						nodes = visitor.getResNodes();
					} catch (XQueryException e) {
						throw new MediatorException(e.getMessage(), e);
					}
				}
				merge(islet, tuple, nodes);
				if (non_blocking) {
					// can make higher buffering by modifying this test
					if ((buftuples != null) && (!buftuples.isEmpty())) {
						break;
					}
				}
			} else {
				setFinishedWhenEmpty();
				resultset.close();
				return;
			}
		}
	}

	private void merge(boolean islet, Tuple tuple, ArrayList nodes) throws MediatorException {
		Tuple newtuple = null;
		if (islet) {
			if (((OpSimpleMerge) operator).isOnlyVar()) {
				tuple.mergeInPlace(nodes, this.operator);
				buftuples.add(tuple);
			} else {
				newtuple = buftuples.newTuple();
				for (int j = 0; j < paths.size(); j++) {
					String pathStrj = ((XQueryExpression) paths.get(j)).getStringValue();
					// test if paths already in tuple
					int index = tuple.getParent().getPathIndex(pathStrj);
					if (index != -1) {
						newtuple.addNodesAtIndex(j, tuple.getNodesAtIndex(index));
					} else if (pathStrj.equals(expression.getStringValue())) {
						newtuple.addNodesAtIndex(j, nodes);
					} else {
						for (int i = 0; i < nodes.size(); i++) {
							ArrayList reslist = DOMUtils.getSteps((TypedNode) nodes.get(i), (XQueryExpression) paths.get(j));
							if (reslist != null)
								newtuple.addNodesAtIndex(j, reslist);
						}
					}
				}
				newtuple.fillIdentifiers();
				buftuples.add(newtuple);
			}
		} else {
			if (nodes != null) {
				if (((OpSimpleMerge) operator).isOnlyVar()) {
					for (int i = 0; i < nodes.size(); i++) {
						newtuple = new Tuple(tuple);
						newtuple.mergeInPlace((TypedNode) nodes.get(i), this.operator);
						buftuples.add(newtuple);
					}
				} else {
					for (int i = 0; i < nodes.size(); i++) {
						ArrayList nodelist = new ArrayList(1);
						nodelist.add(nodes.get(i));
						newtuple = buftuples.newTuple();
						// get paths but test if path is expression !!!
						for (int j = 0; j < paths.size(); j++) {
							String pathStrj = ((XQueryExpression) paths.get(j)).getStringValue();
							// test if paths already in tuple
							int index = tuple.getParent().getPathIndex(pathStrj);
							if (index != -1) {
								newtuple.addNodesAtIndex(j, tuple.getNodesAtIndex(index));
							} else if (pathStrj.equals(expression.getStringValue())) {
								newtuple.addNodesAtIndex(j, nodes);
							} else {
								ArrayList reslist = DOMUtils.getSteps(nodelist, (XQueryExpression) paths.get(j));
								if (reslist != null)
									newtuple.addNodesAtIndex(j, reslist);
							}
						}
						newtuple.fillIdentifiers();
						buftuples.add(newtuple);
					}
				}

			}
		}
	}

}
