/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.plan;

import java.util.ArrayList;

import org.xquark.mediator.DOMUtils.DOMUtils;
import org.xquark.mediator.DOMUtils.Tuple;
import org.xquark.mediator.decomposer.Utils;
import org.xquark.mediator.runtime.MediatorException;

public class OpSort extends OpUn {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.8 $";
	private static final String RCSName = "$Name:  $";
	private ArrayList orderbys = null;
	// ***********************************************************************
	// * INITIALIZATION
	// ***********************************************************************
	/**
	 *
	 */

	public OpSort(ExecutionPlan plan, Operator algebra, ArrayList orderbys) throws MediatorException {
		super(plan, null, algebra);
		this.orderbys = orderbys;
		init();
	}

	private void init() {
	    islet = childOperator.isLet();
		size = childOperator.getSize();
		idsize = childOperator.getIdSize();
		//ordertype = ORDER_PARALLELIZATION ;
		//ordertype = this.ORDER_SERIALIZATION ;
	}

	public ArrayList getOrderBys() {
		return this.orderbys;
	}

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(OperatorVisitor visitor) throws MediatorException {
		visitor.visit(this);
	}

	// ***********************************************************************
	// * EXECUTE QUERY
	// ***********************************************************************
	/**
	 *
	 */
	protected ResultSet getResultSet(DynamicContext context, OperatorRunnable child) throws MediatorException {
		return new SortResultSet(this, context, child);
	}

	// **********************************************************************
	// * OPTIMIZE
	// **********************************************************************
	/**
	 * Merge *this* node with the subnode(s). As it should be called
	 * only from source, the returned node must be of type AlgSource.
	 * The return node is a AlgSource where the XQueryExpression is the merging
	 * of all child(ren) nodes' expression.
	 */
	// protected Algebra mergeWithSub() throws MediatorException { return this ; }

	// **********************************************************************
	// * DEBUG
	// **********************************************************************

	public String toCompleteString(int indent) {
		StringBuffer buf = new StringBuffer();
		buf.append(Utils.makeIndent(indent) + "<" + getClass().getName() + " isLet=\"" + isLet() + "\">\n");
		buf.append(Utils.makeIndent(indent + 1) + "<Expression>\n");
		buf.append(Utils.makeIndent(indent + 2) + expression + "\n");
		buf.append(Utils.makeIndent(indent + 1) + "</Expression>\n");
		//		if (variables != null) {
		//			buf.append(Utils.makeIndent(indent + 1) + "<Variables>\n") ;
		//			for (int i = 0 ; i < variables.size() ; i ++) {
		//				buf.append(Utils.makeIndent(indent + 2) + "<Variable>" + variables.get(i) + "</Variable>\n") ;
		//			}
		//			buf.append(Utils.makeIndent(indent + 1) + "</Variables>\n") ;
		//		}
		if (paths != null) {
			buf.append(Utils.makeIndent(indent + 1) + "<Paths>\n");
			for (int i = 0; i < paths.size(); i++) {
				buf.append(Utils.makeIndent(indent + 2) + "<Path>" + paths.get(i) + "</Path>\n");
			}
			buf.append(Utils.makeIndent(indent + 1) + "</Paths>\n");
		}
		if (orderbys != null) {
			buf.append(Utils.makeIndent(indent + 1) + "<Orderbys>\n");
			for (int i = 0; i < orderbys.size(); i++) {
				buf.append(Utils.makeIndent(indent + 2) + "<Orderby>" + orderbys.get(i) + "</Orderby>\n");
			}
			buf.append(Utils.makeIndent(indent + 1) + "</Orderbys>\n");
		}
		buf.append(Utils.makeIndent(indent + 1) + "<Element>\n");
		if (this.childOperator != null)
			buf.append(this.childOperator.toCompleteString(indent + 2));
		buf.append(Utils.makeIndent(indent + 1) + "</Element>\n");
		buf.append(Utils.makeIndent(indent) + "</" + getClass().getName() + ">\n");
		return buf.toString();
	}

}

class SortResultSet extends UnResultSet {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.8 $";
	private static final String RCSName = "$Name:  $";
	private int index = 0;
	// ***********************************************************************
	// * INITIALIZATION
	// ***********************************************************************
	/**
	 *
	 */
	public SortResultSet(OpSort operator, DynamicContext context, OperatorRunnable child) throws MediatorException {
		super(operator, context, child);
	}

	// ***********************************************************************
	// * EVALUATE IMPLEMENTATION
	// ***********************************************************************
	/**
	 *
	 * @param non_blocking if the evaluation is blocking, we must
	 *        generate all the results before passing.
	 *		  A cartesian product is necessary blocking.
	 */
	protected void evaluate(boolean non_blocking) throws MediatorException {

		//non_blocking = false;

		boolean islet = this.operator.isLet();
		ArrayList orderbys = ((OpSort) operator).getOrderBys();
		int index = 0;
		if (!resultset.hasNext()) {
			resultset.close();
			setFinishedWhenEmpty();
			return;
		}
		this.buftuples.add(resultset.next());
		while (resultset.hasNext()) {
			Tuple tuple = resultset.next();
			int i;
			boolean inserted = false;
			for (i = this.buftuples.size() - 1; i >= 0; i--) {
				Tuple tuplei = (Tuple) this.buftuples.get(i);
				if (!DOMUtils.orderTuple(tuple, tuplei, orderbys)) {
					this.buftuples.add(i + 1, tuple);
					inserted = true;
					break;
				}
			}
			if (!inserted)
				this.buftuples.add(0, tuple);
		}
		this.setFinishedWhenEmpty();
		return;
	}
}
