/*
 * This file belongs to the XQuark distribution. Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307. You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.plan;

import java.util.ArrayList;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xquark.mediator.DOMUtils.GetUnknownPathsVisitor;
import org.xquark.mediator.DOMUtils.SAX2DOM;
import org.xquark.mediator.DOMUtils.Tuple;
import org.xquark.mediator.decomposer.Utils;
import org.xquark.mediator.runtime.MediatorException;
import org.xquark.schema.validation.PSVInfoSetProvider;
import org.xquark.schema.validation.SchemaValidationContext;
import org.xquark.schema.validation.ValidatingSchemaHandler;
import org.xquark.util.NamespaceContextStack;
import org.xquark.xml.xdbc.*;
import org.xquark.xquery.parser.*;
import org.xquark.xquery.typing.QType;

public class OpSource extends ZeroOp {
    // **********************************************************************
    // * VERSIONING
    // **********************************************************************
    private static final String RCSRevision = "$Revision: 1.51 $";

    private static final String RCSName = "$Name:  $";

    // **********************************************************************
    // * CLASS VARIABLES
    // **********************************************************************
    //private XMLConnection connection = null;
    private PreparedXMLStatement preparedStatement = null;

    private ArrayList externalVariables = null;

    private String requestStr = null;

    private String preparedRequestStr = null;

    private String source = null;

    private boolean cannotBePrepared = false;

    // ***********************************************************************
    // * INITIALIZATION
    // ***********************************************************************
    /**
     *  
     */
    public OpSource(ExecutionPlan plan, XQueryExpression expression, ArrayList origpaths, String source, boolean hasIdentifier) throws MediatorException {
        super(plan, expression);
        try {
            for (int i = 0; i < origpaths.size(); i++) {
                addPath((XQueryExpression) ((XQueryExpression) origpaths.get(i)).clone());
            }
        } catch (CloneNotSupportedException cnse) {
        }
        //addPaths(origpaths);
        this.source = source;
        this.size = this.getPaths().size();
        if (hasIdentifier)
            this.idsize = 1;
        if (source == null)
            throw new MediatorException("AlgSource: no source !");
        // creating if necessary and if possible prepared statement
        createRequestString();
        GetUnknownPathsVisitor gupv = new GetUnknownPathsVisitor(getSchemaManager(), getTypeVisitor());
        try {
            expression.accept(gupv);
        } catch (XQueryException xqe) {
            throw new MediatorException(xqe.getMessage());
        }
        externalVariables = gupv.getNewVariables();
        // test if an external variable can be multivaluated
        if (externalVariables != null)
            for (int i = 0; i < externalVariables.size(); i++) {
                Variable vari = (Variable) externalVariables.get(i);
                QType qtypei = vari.getQType();
                if (qtypei.isMultiple()) {
                    cannotBePrepared = true;
                    break;
                }
            }
    }

    // #############################################################################
    // VISITOR STUFF
    // #############################################################################

    public void accept(OperatorVisitor visitor) throws MediatorException {
        visitor.visit(this);
    }

    // ***********************************************************************
    // * EXECUTE QUERY
    // ***********************************************************************
    /**
     *  
     */
    protected ResultSet getResultSet(DynamicContext context) throws MediatorException {
        ResultSet resultset = null;
        if (!cannotBePrepared)
            if (preparedRequestStr != null) {
                try {
                    preparedStatement = context.getPreparedXMLStatement(this, source, preparedRequestStr);
                } catch (XMLDBCException xe) {
                    preparedRequestStr = null;
                    preparedStatement = null;
                    throw new MediatorException(xe.getMessage(), xe);
                }
            }
        if (preparedStatement != null)
            resultset = new SourceResultSet(this, context, preparedStatement);
        else {
            XMLConnection connection = null;
            connection = context.getConnection().getSubConnectionByName(source);
            if (connection == null)
                throw new MediatorException("AlgSource: couldn't established connection to " + source);
            resultset = new SourceResultSet(this, context, connection);
        }
        return resultset;
    }

    /**
     * Return all the sources depending of this node.
     */
    public ArrayList getSources() {
        if (sources == null) {
            sources = new ArrayList();
            sources.add(source);
        }
        return sources;
    }

    public String getSourceName() {
        return source;
    }

    public void setPrepared() throws MediatorException {
        if (!cannotBePrepared || prepared)
            return;
        // find in FLWRExpression (only in where clause) all expression that do not start with a variable of the FLWRExpression
        StringBuffer sbvars = new StringBuffer();
        sbvars.append(requestStr);
        if (externalVariables != null && !externalVariables.isEmpty()) {
            plan.appendExternalVariables(sbvars, externalVariables);
            if (sbvars.length() == 0) {
                preparedStatement = null;
                prepared = false;
                return;
            }
        }
        // TODO find a better way to modify prefixes of included modules
        sbvars.append(expression.getStringValue(plan.getModule().getDeclarations()));
        preparedRequestStr = sbvars.toString();
        prepared = true;
    }

    // creates request string
    private void createRequestString() {
        if (requestStr == null) {
            StringBuffer sb = new StringBuffer();
            sb.append("declare validation preserve;\n");
            String tmpstr = plan.getDeclarations();
            if (tmpstr != null)
                sb.append(tmpstr);
            sb.append("declare namespace xsi = \"http://www.w3.org/2001/XMLSchema-instance\";\n");
            requestStr = sb.toString();
        }
    }

    public ArrayList getExternalVariables() {
        return externalVariables;
    }

    public String getRequestStr() {
        return requestStr;
    }

    public String getPreparedRequestStr() {
        return preparedRequestStr;
    }

    // **********************************************************************
    // * PRINT
    // **********************************************************************
    public String toCompleteString(int indent) {
        StringBuffer buf = new StringBuffer();
        buf.append(Utils.makeIndent(indent) + "<" + getClass().getName() + " isLet=\"" + islet + "\" valueDepends=\"" + valueDepends + "\" whereDepends=\"" + whereDepends + "\">\n");
        buf.append(Utils.makeIndent(indent + 1) + "<Expression>\n");
        buf.append(Utils.makeIndent(indent + 2) + expression + "\n");
        buf.append(Utils.makeIndent(indent + 1) + "</Expression>\n");
        buf.append(Utils.makeIndent(indent + 1) + "<Source>" + source + "</Source>\n");
        //		if (variables != null) {
        //			buf.append(Utils.makeIndent(indent + 1) + "<Variables>\n");
        //			for (int i = 0; i < variables.size(); i++) {
        //				buf.append(Utils.makeIndent(indent + 2) + "<Variable>" + variables.get(i) + "</Variable>\n");
        //			}
        //			buf.append(Utils.makeIndent(indent + 1) + "</Variables>\n");
        //		}
        if (paths != null) {
            buf.append(Utils.makeIndent(indent + 1) + "<Paths>\n");
            for (int i = 0; i < paths.size(); i++) {
                buf.append(Utils.makeIndent(indent + 2) + "<Path>" + paths.get(i) + "</Path>\n");
            }
            buf.append(Utils.makeIndent(indent + 1) + "</Paths>\n");
        }
        buf.append(Utils.makeIndent(indent) + "</" + getClass().getName() + ">\n");
        return buf.toString();
    }

}

class SourceResultSet extends ZeroResultSet {
    // **********************************************************************
    // * VERSIONING
    // **********************************************************************
    private static final String RCSRevision = "$Revision: 1.51 $";

    private static final String RCSName = "$Name:  $";

    private static Log log = LogFactory.getLog(SourceResultSet.class);

    // **********************************************************************
    // * CLASS VARIABLES
    // **********************************************************************
    private XMLStatement statement = null;

    private XMLResultSet resultset = null;

    private PSVInfoSetProvider psvip = null;

    //private SchemaValidationContext svc = null;
    private ValidatingSchemaHandler vsh = null;

    private SAX2DOM thandler = null;

    // ***********************************************************************
    // * INITIALIZATION
    // ***********************************************************************
    /**
     *  
     */
    public SourceResultSet(OpSource operator, DynamicContext context, XMLConnection connection) throws MediatorException {
        super(operator, context);
        try {
            statement = connection.createStatement();
        } catch (XMLDBCException xe) {
            throw new MediatorException(xe.getMessage(), xe);
        }

        StringBuffer sb = new StringBuffer();

        sb.append(operator.getRequestStr());

        ArrayList externalVariables = operator.getExternalVariables();
        if (externalVariables != null) {
            try {
                for (int i = 0; i < externalVariables.size(); i++) {
                    Variable vari = (Variable) externalVariables.get(i);
                    Object strvalue = context.getVariableValue(vari);
                    if (strvalue == null)
                        throw new MediatorException("Could not find expression while replacing variable");
                    sb.append("declare variable $");
                    //sb.append(vari.getStringValue());
                    QName qname = vari.getVarName();
                    NamespaceContextStack contextSatck = operator.getPlan().getModule().getDeclarations();
                    if (contextSatck != null) {
                        String prefix = contextSatck.getPrefix(qname.getNameSpace());          
                        if (prefix != null && prefix.length() != 0) {
                            sb.append(prefix);
                            sb.append(":");
                        }
                    }
                    sb.append(qname.getLocalName());
                    sb.append(" := {");
                    if (strvalue instanceof ArrayList) {
                        sb.append("(");
                        ArrayList list = (ArrayList)strvalue;
                        for (int j=0;j<list.size();j++) {
                            if (j!=0)
                                sb.append(",");
                            strvalue = list.get(j);
                            if (strvalue instanceof String) {
                                sb.append("\"");
                                sb.append(strvalue.toString());
                                sb.append("\"");
                            } else
                                sb.append(strvalue.toString());
                        }
                        sb.append(")");
                    } else {
                        if (strvalue instanceof String) {
                            sb.append("\"");
                            sb.append(strvalue.toString());
                            sb.append("\"");
                        } else
                            sb.append(strvalue.toString());
                    }
                    sb.append("};\n");
                }
            } catch (XQueryException xqe) {
                throw new MediatorException(xqe.getMessage(), xqe);
            }
        }
        // TODO find a better way to modify prefixes of included modules
        sb.append(expression.getStringValue(operator.getPlan().getModule().getDeclarations()));

        if (log.isDebugEnabled())
            log.debug("SOURCE EXECUTING \n" + sb.toString());

        try {
            resultset = statement.executeQuery(sb.toString());

            if (!resultset.hasNext()) {
                resultset.close();
                resultset = null;
                statement.close();
                setFinishedWhenEmpty();
            }
        } catch (XMLDBCException xe) {
            throw new MediatorException(xe.getMessage(), xe);
        }
    }

    public SourceResultSet(OpSource operator, DynamicContext context, PreparedXMLStatement statement) throws MediatorException {
        super(operator, context);

        try {
            ArrayList externalVariables = operator.getExternalVariables();
            if (externalVariables != null) {
                try {
                    for (int i = 0; i < externalVariables.size(); i++) {
                        Variable vari = (Variable) externalVariables.get(i);
                        Object strvalue = context.getVariableValue(vari);
                        if (strvalue == null)
                            throw new MediatorException("Could not find expression while replacing variable");
                        ((PreparedXMLStatement) statement).setObject(vari.getVarName().getNameSpace(), vari.getVarName().getName(), strvalue);
                    }
                } catch (XQueryException xqe) {
                    throw new XMLDBCException(xqe.getMessage());
                }
            }
            resultset = ((PreparedXMLStatement) statement).executeQuery();

            if (!resultset.hasNext()) {
                resultset.close();
                resultset = null;
                setFinishedWhenEmpty();
            }
        } catch (XMLDBCException xe) {
            throw new MediatorException(xe.getMessage(), xe);
        }
    }

    // ***********************************************************************
    // * EVALUATE IMPLEMENTATION
    // ***********************************************************************
    /**
     * 
     * @param non_blocking
     *            if the evaluation is blocking, we must generate all the results before passing.
     */
    protected void evaluate(boolean non_blocking) throws MediatorException {

        if (resultset == null)
            return;

        //non_blocking = false;

        if (this.operator.isLet())
            non_blocking = false;
        try {
            if (thandler == null) {
                psvip = this.resultset.getPSVInfoSetProvider();
                if (psvip == null) {
                    SchemaValidationContext svc = new SchemaValidationContext(this.operator.getPlan().getSchemaManager());
                    vsh = new ValidatingSchemaHandler(svc, false);
                    thandler = new SAX2DOM(svc);
                    vsh.setContentHandler(thandler);
                    vsh.setErrorHandler(new ErrorHandler() {
                        public void warning(SAXParseException ex) throws SAXException {
                        }

                        public void error(SAXParseException ex) throws SAXException {
                            System.err.println(ex);
                        }

                        public void fatalError(SAXParseException ex) throws SAXException {
                        }
                    });
                    resultset.setContentHandler(vsh);
                } else {
                    thandler = new SAX2DOM(psvip);
                    resultset.setContentHandler(thandler);
                }
            }
            while (true) {
                Tuple tuple = new Tuple(this);
                thandler.init(tuple);
                if (resultset.hasNext()) {
                    if (vsh != null)
                        vsh.startDocument();
                    else
                        thandler.startDocument();
                    resultset.nextAsSAX();
                    if (vsh != null)
                        vsh.endDocument();
                    else
                        thandler.endDocument();
                    if (this.operator.isLet()) {
                        if (this.buftuples.size() == 0) {
                            tuple.fillIdentifiers();
                            buftuples.add(tuple);
                        } else {
                            Tuple first = (Tuple) this.buftuples.get(0);
                            first.appendTuple(tuple);
                        }
                    } else {
                        tuple.fillIdentifiers();
                        buftuples.add(tuple);
                    }
                    // can make higher buffering by modifying this test
                    if (non_blocking && !buftuples.isEmpty()) {
                        if (resultset != null && !resultset.hasNext()) {
                            resultset.close();
                            resultset = null;
                            if (statement != null)
                                statement.close();
                            setFinishedWhenEmpty();
                        }
                        break;
                    }
                } else {
                    resultset.close();
                    resultset = null;
                    if (statement != null)
                        statement.close();
                    setFinishedWhenEmpty();
                    return;
                }
            }
        } catch (XMLDBCException xe) {
            throw new MediatorException("AlgSource.evaluate: " + xe.getMessage(), xe);
        } catch (SAXException saxe) {
            throw new MediatorException("AlgSource.evaluate: " + saxe.getMessage(), saxe);
        }
    }

    protected void finalize() {
        try {
            if (null != resultset) {
                resultset.close();
                if (statement != null)
                    statement.close();
            }
        } catch (Exception ex) {
        }
        ;
    }
}