/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 XQuark Group.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.plan;

import java.util.ArrayList;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.xquark.mediator.decomposer.Utils;
import org.xquark.mediator.runtime.MediatorException;
import org.xquark.xml.xdbc.XMLResultSet;
import org.xquark.xml.xqueryevaluator.eval.EvalFilter;
import org.xquark.xquery.parser.FLWRExpression;
import org.xquark.xquery.parser.XQueryExpression;

public class OpSourceEval extends ZeroOp {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.11 $";
	private static final String RCSName = "$Name:  $";

	// **********************************************************************
	// * CLASS VARIABLES
	// **********************************************************************
	//private XMLConnection connection = null;
	//private String source = null;

	// ***********************************************************************
	// * INITIALIZATION
	// ***********************************************************************
	/**
	 *
	 */
	public OpSourceEval(ExecutionPlan plan, XQueryExpression expression, ArrayList origpaths, boolean hasIdentifier) throws MediatorException {
		super(plan, expression);

		try {
			for (int i = 0; i < origpaths.size(); i++) {
				addPath((XQueryExpression) ((XQueryExpression) origpaths.get(i)).clone());
			}
		} catch (CloneNotSupportedException cnse) {
		}
		//addPaths(origpaths);
		this.size = this.getPaths().size();
		if (hasIdentifier)
			this.idsize = 1;
	}

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(OperatorVisitor visitor) throws MediatorException {
		visitor.visit(this);
	}

	// ***********************************************************************
	// * EXECUTE QUERY
	// ***********************************************************************
	/**
	 *
	 */
	protected ResultSet getResultSet(DynamicContext context) throws MediatorException {
		return new SourceEvalResultSet(this, context);
	}

	// **********************************************************************
	// * OPTIMIZE
	// **********************************************************************
	/**
	 * Return all the sources depending of this node.
	 */
	public ArrayList getSources() {
		return null;
	}

	public String toCompleteString(int indent) {
		StringBuffer buf = new StringBuffer();
		buf.append(Utils.makeIndent(indent) + "<" + getClass().getName() + " size=" + size + " id size=" + idsize + " isLet=\"" + islet + "\" valueDepends=\"" + valueDepends + "\" whereDepends=\"" + whereDepends + "\">\n");
		buf.append(Utils.makeIndent(indent + 1) + "<Expression>\n");
		buf.append(Utils.makeIndent(indent + 2) + expression + "\n");
		buf.append(Utils.makeIndent(indent + 1) + "</Expression>\n");
		//		buf.append(Utils.makeIndent(indent + 1) + "<Source>" + source + "</Source>\n");
		//		if (variables != null) {
		//			buf.append(Utils.makeIndent(indent + 1) + "<Variables>\n");
		//			for (int i = 0; i < variables.size(); i++) {
		//				buf.append(Utils.makeIndent(indent + 2) + "<Variable>" + variables.get(i) + "</Variable>\n");
		//			}
		//			buf.append(Utils.makeIndent(indent + 1) + "</Variables>\n");
		//		}
		if (paths != null) {
			buf.append(Utils.makeIndent(indent + 1) + "<Paths>\n");
			for (int i = 0; i < paths.size(); i++) {
				buf.append(Utils.makeIndent(indent + 2) + "<Path>" + paths.get(i) + "</Path>\n");
			}
			buf.append(Utils.makeIndent(indent + 1) + "</Paths>\n");
		}
		buf.append(Utils.makeIndent(indent) + "</" + getClass().getName() + ">\n");
		return buf.toString();
	}

}

class SourceEvalResultSet extends ZeroResultSet {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.11 $";
	private static final String RCSName = "$Name:  $";

	private static Log log = LogFactory.getLog(SourceEvalResultSet.class);

	// **********************************************************************
	// * CLASS VARIABLES
	// **********************************************************************
	private XMLResultSet resultset = null;
	private EvalFilter evalFilter;

	// ***********************************************************************
	// * INITIALIZATION
	// ***********************************************************************
	/**
	 *
	 */
	public SourceEvalResultSet(OpSourceEval operator, DynamicContext context) throws MediatorException {
		super(operator, context);
		if (log.isDebugEnabled())
			log.debug("MEMORY EXECUTING \n" + expression.getStringValue());
		evalFilter = new EvalFilter(this.buftuples, context, this.paths, (FLWRExpression) this.expression);
	}

	// ***********************************************************************
	// * EVALUATE IMPLEMENTATION
	// ***********************************************************************
	/**
	 *
	 * @param non_blocking if the evaluation is blocking, we must
	 *        generate all the results before passing.
	 */
	//int tupleindex = 0;
	protected void evaluate(boolean non_blocking) throws MediatorException {
		boolean isDone = false;
		isDone = evalFilter.evaluate();
		if (isDone)
			setFinishedWhenEmpty();
	}

}
