/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.plan;

import java.util.ArrayList;
import java.util.HashMap;

import org.xquark.mediator.DOMUtils.*;
import org.xquark.mediator.decomposer.Utils;
import org.xquark.mediator.runtime.MediatorException;
import org.xquark.xquery.parser.ListOpCompExpression;
import org.xquark.xquery.parser.XQueryException;
import org.xquark.xquery.parser.XQueryExpression;
import org.xquark.xquery.parser.hinttree.HintTree;
import org.xquark.xquery.parser.util.Constants;

public class OpSubQuery extends OpMult {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.17 $";
	private static final String RCSName = "$Name:  $";
	// ***********************************************************************
	// * VARIABLES
	// ***********************************************************************
	private int[] locations = null;
	private int[] idlocations = null;
	//private HashMap subelementsrestrict = new HashMap() ;
	private boolean pivot = false;
	private boolean imbricated = false;
	protected int ordertype = ORDER_SERIALIZATION;
	protected int outertype = HintTree.NESTED_TYPE;
	protected ArrayList mergeRestrictions = null;
	protected ArrayList outsideRestrictions = null;

	// ***********************************************************************
	// * INITIALIZATION
	// ***********************************************************************
	/**
	 *
	 */

	public OpSubQuery(ExecutionPlan plan, XQueryExpression expression, Operator[] children, boolean pivot, boolean imbricated, ArrayList mergeRestrictions, ArrayList outsideRestrictions) throws MediatorException {
		super(plan, expression, children);
		ordertype = ORDER_PARALLELIZATION;
		this.pivot = pivot;
		this.imbricated = imbricated;
		this.outertype = HintTree.MERGE_TYPE;
		this.mergeRestrictions = mergeRestrictions;
		this.outsideRestrictions = outsideRestrictions;
		init();
	}

	public OpSubQuery(ExecutionPlan plan, XQueryExpression expression, Operator[] children, boolean pivot, boolean imbricated) throws MediatorException {
		super(plan, expression, children);
		ordertype = ORDER_PARALLELIZATION;
		this.pivot = pivot;
		this.imbricated = imbricated;
		init();
	}

	public boolean getPivot() {
		return this.pivot;
	}
	//public void setPivot(boolean pivot) { this.pivot = pivot; }
	public boolean getImbricated() {
		return this.imbricated;
	}
	//public void setImbricated(boolean imbricated) { this.imbricated = imbricated; }
	public int getOuterType() {
		return this.outertype;
	}

	public ArrayList getMergeRestrictions() {
		return this.mergeRestrictions;
	}
	public ArrayList getOutsideRestrictions() {
		return this.outsideRestrictions;
	}

	public void init() throws MediatorException {
		// Sometimes, it can happens that the two members of merge
		// product have common attributes. In that case, we must mark them
		// as common. So when we will generate the tuples, we can access
		// them efficiently.
		if (childrenOperator == null)
			return;
		this.resetPaths();
		if (this.paths == null || this.paths.isEmpty()) {
			for (int i = 0; i < childrenOperator.length; i++) {
				Operator operatori = childrenOperator[i];
				operatori.setParentOperator(this);
				if (i != 0 && pivot && operatori.getDepends())
					operatori.setPrepared();
				addPaths(operatori.getPaths());
			}
		}
		size = childrenOperator[0].getSize(); //.getPaths().size() ;
		idsize = childrenOperator[0].getIdSize();
		locations = new int[childrenOperator.length - 1];
		idlocations = new int[childrenOperator.length - 1];
		for (int k = 1; k < childrenOperator.length; k++) {
			locations[k - 1] = size;
			idlocations[k - 1] = idsize;
			size += childrenOperator[k].getSize(); //.getPaths().size();
			idsize += childrenOperator[k].getIdSize();
		}
	}

	public int[] getIdLocations() {
		return idlocations;
	}
	public int[] getLocations() {
		return locations;
	}

	/**
	 * Return true if the node is of type source.
	 * AlgBinOp is not a source.
	 *
	 * @return false ;
	 */
	public boolean isSource() {
		return false;
	}

	/**
	 * Return all the sources depending of this node.
	 * @return
	 */
	public ArrayList getSources() {
		/*
		if (sources == null) {
			sources = new ArrayList() ;
			for (int i=0;i<algebralist.size();i++) {
				ArrayList tmplist = ((Algebra)algebralist.get(i)).getSources();
				for (int j=0;j<tmplist.size();j++) {
					Object sourcej = tmplist.get(j) ;
					if (! sources.contains(sourcej)) sources.add(sourcej) ;
				}
			}
		}
		return sources ;
		 */
		return null;
	}

	protected boolean shareSameSources() {
		/*
		for (int i=0;i<algebralist.size();i++)
			if (((Algebra)algebralist.get(i)).shareSameSources() == false) return false;
		if (compareSource()) return true ;
		 */
		return false;
	}

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(OperatorVisitor visitor) throws MediatorException {
		visitor.visit(this);
	}

	// ***********************************************************************
	// * EXECUTE QUERY
	// ***********************************************************************
	/**
	 *
	 */
	public ResultSet executeQuery(DynamicContext context) throws MediatorException {
		try {
			OperatorRunnable[] runnables = new OperatorRunnable[childrenOperator.length];
			if (ordertype == ORDER_SERIALIZATION) {
				for (int i = 0; i < childrenOperator.length; i++) {
					OperatorRunnable ori = childrenOperator[i].getOperatorRunnable(context);
					runnables[i] = ori;
					if (!childrenOperator[i].getDepends() || i == 0 || this.outertype == HintTree.MERGE_TYPE)
						ori.run();
				}
			} else if (ordertype == ORDER_PARALLELIZATION) {
				Thread[] threads = new Thread[childrenOperator.length];
				for (int i = 0; i < childrenOperator.length; i++) {
					OperatorRunnable ori = childrenOperator[i].getOperatorRunnable((DynamicContext)context.clone());
					runnables[i] = ori;
					if (!childrenOperator[i].getDepends() || i == 0 || this.outertype == HintTree.MERGE_TYPE) {
						threads[i] = new Thread(ori);
						threads[i].start();
					}
				}
				// wait for all threads to exit
				for (int i = 0; i < threads.length; i++) {
					Thread thread = threads[i];
					if (thread != null && thread.isAlive())
						thread.join();
				}
			}
			return getResultSet(context, runnables);
		} catch (InterruptedException e) {
			throw new MediatorException("AlgMultOp.executeQuery: " + e.getMessage(), e);
		}
	}

	protected ResultSet getResultSet(DynamicContext context, OperatorRunnable[] children) throws MediatorException {
		return new SubQueryNoPivotResultSet(this, context, children, pivot, imbricated);
	}

	// **********************************************************************
	// * OPTIMIZE
	// **********************************************************************

	// **********************************************************************
	// * DEBUG
	// **********************************************************************
	public String toCompleteString(int indent) {
		StringBuffer buf = new StringBuffer();
		buf.append(Utils.makeIndent(indent) + "<" + getClass().getName() + " hasPivot=\"" + pivot + "\" isImbricated=\"" + this.imbricated + " size=" + size + " id size=" + idsize + " isLet=\"" + islet + "\" valueDepends=\"" + valueDepends + "\" whereDepends=\"" + whereDepends + "\">\n");
		buf.append(Utils.makeIndent(indent + 1) + "<mergeRestrictions>\n");
		buf.append(Utils.makeIndent(indent + 2) + mergeRestrictions + "\n");
		buf.append(Utils.makeIndent(indent + 1) + "</mergeRestrictions>\n");
		buf.append(Utils.makeIndent(indent + 1) + "<outsideRestrictions>\n");
		buf.append(Utils.makeIndent(indent + 2) + outsideRestrictions + "\n");
		buf.append(Utils.makeIndent(indent + 1) + "</outsideRestrictions>\n");
		/*
		buf.append(Utils.makeIndent(indent + 1) + "<Expression>\n") ;
		buf.append(Utils.makeIndent(indent + 2) + expression + "\n") ;
		buf.append(Utils.makeIndent(indent + 1) + "</Expression>\n") ;
		if (variables != null) {
			buf.append(Utils.makeIndent(indent + 1) + "<Variables>\n") ;
			for (int i = 0 ; i < variables.size() ; i ++) {
				buf.append(Utils.makeIndent(indent + 2) + "<Variable>" + variables.get(i) + "</Variable>\n") ;
			}
			buf.append(Utils.makeIndent(indent + 1) + "</Variables>\n") ;
		}
		 */
		if (paths != null) {
			buf.append(Utils.makeIndent(indent + 1) + "<Paths>\n");
			for (int i = 0; i < paths.size(); i++) {
				buf.append(Utils.makeIndent(indent + 2) + "<Path>" + paths.get(i) + "</Path>\n");
			}
			buf.append(Utils.makeIndent(indent + 1) + "</Paths>\n");
		}
		/*
		if (subelementsrestrict != null && !subelementsrestrict.isEmpty()) {
			buf.append(Utils.makeIndent(indent + 1) + "<Restrictions>\n") ;
			for (Iterator it = subelementsrestrict.keySet().iterator();it.hasNext();) {
				Integer index = (Integer)it.next();
				buf.append(Utils.makeIndent(indent + 2) + "<Index>" + index + "</Index>\n") ;
				ArrayList tmplist = (ArrayList)subelementsrestrict.get(index);
				if (tmplist == null) continue;
				for (int i = 0 ; i < tmplist.size() ; i ++)
					buf.append(Utils.makeIndent(indent + 2) + "<Restriction>" + tmplist.get(i) + "</Restriction>\n") ;
			}
			buf.append(Utils.makeIndent(indent + 1) + "</Restrictions>\n") ;
		}
		 */
		for (int i = 0; i < childrenOperator.length; i++) {
			buf.append(Utils.makeIndent(indent + 1) + "<Element" + (i + 1) + ">\n");
			buf.append(childrenOperator[i].toCompleteString(indent + 2));
			buf.append(Utils.makeIndent(indent + 1) + "</Element" + (i + 1) + ">\n");
		}
		buf.append(Utils.makeIndent(indent) + "</" + getClass().getName() + ">\n");
		return buf.toString();
	}
}

abstract class SubQueryResultSet extends MultResultSet {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.17 $";
	private static final String RCSName = "$Name:  $";
	// ***********************************************************************
	// * VARIABLES
	// ***********************************************************************
	protected int[] locations = null;
	protected int[] idlocations = null;
	protected boolean pivot;
	protected boolean imbricated;
	// ***********************************************************************
	// * INITIALIZATION
	// ***********************************************************************
	/**
	 *
	 */
	//public SubQueryResultSet(AlgSubQuery algebra, HashMap subelementsrestrict, boolean pivot) throws XMLDBCException {
	public SubQueryResultSet(OpSubQuery operator, DynamicContext context, OperatorRunnable[] runnables, boolean pivot, boolean imbricated) throws MediatorException {
		super(operator, context);
		this.children = runnables;
//		resultsets = new ResultSet[children.length];
//		for (int i = 0; i < children.length; i++) {
//			Operator operatori = operator.getSubElement(i);
//			if (!operatori.getDepends() || i == 0 || operator.getOuterType() == HintTree.MERGE_TYPE)
//				resultsets[i] = children[i].getResultSet();
//		}
		this.pivot = pivot;
		this.imbricated = imbricated;
		// initLocations(algebra);
		this.locations = operator.getLocations();
		this.idlocations = operator.getIdLocations();
	}

	//	public int[] getLocations() {
	//		return locations;
	//	}
	//
	//	public int[] getIdLocations() {
	//		return idlocations;
	//	}

}

class SubQueryNoPivotResultSet extends SubQueryResultSet {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.17 $";
	private static final String RCSName = "$Name:  $";

	private int index = 0;
	//private boolean hasinformation = false;
	private boolean allLets = true;

	private ResultSet pivotResultSet = null;
	private ArrayList allTuples = null;
	private int[] indexes = null;
	private Tuple tuplepivot = null;
	private boolean is_used = false;
	private ArrayList currentTuples = null;
	private int size = -1;
	private boolean endindex = false;
	private int outertype = HintTree.NESTED_TYPE;
	private SortedKeyMap tm0 = null;
	private SortedKeyMap tm1 = null;
	private HashMap pivotindex = null;
	private int indexindex = 0;

	public SubQueryNoPivotResultSet(OpSubQuery operator, DynamicContext context, OperatorRunnable[] children, boolean pivot, boolean imbricated) throws MediatorException {
		super(operator, context, children, pivot, imbricated);
		init();
	}

	protected void init() throws MediatorException {
		size = children.length;
		if (pivot) {
			pivotResultSet = children[0].getResultSet();
			if (pivotResultSet == null || !pivotResultSet.hasNext()) {
				pivotResultSet.close();
				setFinishedWhenEmpty();
				for (int i = 1; i < size; i++)
					if (!children[i].getOperator().getDepends())
						children[i].getResultSet().close();
				//this.hasinformation = true;
				return;
			}
			outertype = ((OpSubQuery) operator).getOuterType();
			if (outertype == HintTree.NESTED_TYPE) {
				allTuples = new ArrayList(size - 1);
				indexes = new int[size - 1];
				for (int i = 1; i < size; i++) {
					indexes[i - 1] = -1;
					if (allLets && (!children[i].getOperator().isLet() || children[i].getOperator().getWhereDepends()))
						allLets = false;
				}
			} else {
				allTuples = new ArrayList(size);
				indexes = new int[size];
				indexes[0] = -1;
				for (int i = 1; i < size; i++) {
					indexes[i] = -1;
					if (allLets && (!children[i].getOperator().isLet() || children[i].getOperator().getWhereDepends()))
						allLets = false;
				}
			}
		}
	}

	// ***********************************************************************
	// * EVALUATE IMPLEMENTATION
	// ***********************************************************************
	/**
	 * 	
	 * @param non_blocking if the evaluation is blocking, we must
	 *        generate all the results before passing.
	 */
	protected void evaluate(boolean non_blocking) throws MediatorException {

		//non_blocking = false;

		// DERNIERE VERSION teste		

		if (pivot) {
			if (outertype == HintTree.NESTED_TYPE) {
				boolean added = false;
				while (!added) {
					if (endindex) {
						endindex = false;
						if (is_used == false && (imbricated || allLets)) {
							Tuple newtuple = new Tuple(operator);
							newtuple.appendTuple(tuplepivot);
							newtuple.fillIdentifiers(0, tuplepivot);
							buftuples.add(newtuple);
							if (!added && non_blocking)
								added = true;
							tuplepivot = null;
							continue;
						}
						tuplepivot = null;
					}
					if (tuplepivot == null) {
						if (pivotResultSet.hasNext()) {
							tuplepivot = pivotResultSet.next();
							is_used = false;
							index = 0;
						} else {
							pivotResultSet.close();
							setFinishedWhenEmpty();
							return;
						}
					}
					Tuple tuple = null;
					Operator operatori = children[index + 1].getOperator();
					if (operatori.getDepends()) {
						ResultSet subresultset = children[index + 1].getResultSet();
						if (subresultset == null) {
							children[index + 1].getContext().addCurrentTuple(tuplepivot);
							children[index + 1].run();
							subresultset = children[index + 1].getResultSet();
						}
						if (subresultset != null && subresultset.hasNext()) { // more tuples
							tuple = subresultset.next();
						} else { // no more tuples
							children[index + 1].getContext().deleteCurrentTuple(tuplepivot);
							if (subresultset != null)
								subresultset.close();
							allTuples.add(null);
							children[index + 1].setResultSet(null);
							if (index == size - 2)
								endindex = true;
							else
								index++;
							continue;
						}
					} else { // no dependency					
						if (indexes[index] == -1) { // first time round
							ResultSet subresultset = children[index + 1].getResultSet();
							if (subresultset.hasNext()) { // more tuples
								tuple = subresultset.next();
								if (currentTuples == null)
									currentTuples = new ArrayList();
								currentTuples.add(tuple);
							} else { // no more tuples
								subresultset.close();
								allTuples.add(currentTuples);
								currentTuples = null;
								indexes[index] = 0;
								if (index == size - 2)
									endindex = true;
								else
									index++;
								continue;
							}
						} else { // not first time round
							if (indexes[index] == 0)
								currentTuples = (ArrayList) allTuples.get(index);
							if (indexes[index] == ((currentTuples == null) ? 0 : currentTuples.size())) {
								indexes[index] = 0;
								if (index == size - 2)
									endindex = true;
								else
									index++;
								continue;
							}
							if (currentTuples != null)
								tuple = (Tuple) currentTuples.get(indexes[index]);
							else
								tuple = null;
							indexes[index]++;
						}
					}

					Tuple newtuple = new Tuple(operator);
					newtuple.appendTuple(tuplepivot);
					newtuple.appendTupleAtIndex(locations[index], tuple);
					newtuple.fillIdentifiers(0, tuplepivot);
					newtuple.fillIdentifiers(idlocations[index], tuple);
					if (newtuple != null) {
						buftuples.add(newtuple);
						if (!added && non_blocking)
							added = true;
						is_used = true;
					}
				}
			} else if (outertype == HintTree.MERGE_TYPE) {
				//this is blocking!
				// get all pivot elements and index them with row id
				if (allTuples.isEmpty()) {
					pivotindex = new HashMap();
					ArrayList tmplist = new ArrayList();
					int tmpint = 0;
					while (pivotResultSet.hasNext()) {
						Tuple tmptuple = pivotResultSet.next();
						tmplist.add(tmptuple);
						pivotindex.put(tmptuple, new Integer(tmpint++));
					}
					// do not close (it puts the buftuples to null)
					// pivotResultSet.close();
					allTuples.add(tmplist);
					indexes[0] = 0;
					index = 1;
					indexindex = 0;
				}
				// make sort merge if necessary
				boolean added = false;
				while (!added) {
					if (endindex) {
						endindex = false;
						indexes[0]++;
						if (is_used == false && (imbricated || allLets)) {
							Tuple newtuple = new Tuple(operator);
							newtuple.appendTuple(tuplepivot);
							newtuple.fillIdentifiers(0, tuplepivot);
							buftuples.add(newtuple);
							if (!added && non_blocking)
								added = true;
							tuplepivot = null;
							continue;
						}
						tuplepivot = null;
					}
					if (tuplepivot == null) {
						if (indexes[0] < ((ArrayList) allTuples.get(0)).size()) {
							tuplepivot = (Tuple) ((ArrayList) allTuples.get(0)).get(indexes[0]);
							is_used = false;
							index = 1;
						} else {
							setFinishedWhenEmpty();
							pivotResultSet.close();
							return;
						}
					}
					//Tuple tuple = null;
					if (children[index].getOperator().getDepends()) {
						if (indexes[index] == -1) { // merge not calculate yet
							// make sort merge
							// fill alltuples with hashMap						
							ResultSet subalgi = children[index].getResultSet();
							if (!subalgi.hasNext()) {
								subalgi.close();
								// add empty resultset
								allTuples.add(null);
								// set index for no recalculation
								indexes[index] = 0;
								continue;
							}
							while (subalgi.hasNext()) {
								Tuple tuplei = subalgi.next();
							}
							// do not close (it puts the buftuples to null)
							tm0 = children[0].getResultSet().getIndex(indexindex);
							tm0.makeIterator();
							tm1 = children[index].getResultSet().getIndex(0);
							tm1.makeIterator();
							subalgi.close();

							HashMap tmpmap = new HashMap();

							// initialize variables
							ArrayList mergeRestrictions = (ArrayList) ((ArrayList) ((OpSubQuery) operator).getMergeRestrictions()).get(indexindex);
							XQueryExpression outsideRestrictions = (XQueryExpression) ((ArrayList) ((OpSubQuery) operator).getOutsideRestrictions()).get(indexindex);
							// increment index of index
							indexindex++;
							int size = mergeRestrictions.size();
							ArrayList leftstrings = new ArrayList(size);
							ArrayList rightstrings = new ArrayList(size);
							for (int i = 0; i < size; i++) {
								leftstrings.add(((ListOpCompExpression) mergeRestrictions.get(i)).getExpression1().getStringValue());
								rightstrings.add(((ListOpCompExpression) mergeRestrictions.get(i)).getExpression2().getStringValue());
							}
							int operatorNEQ = ((ListOpCompExpression) mergeRestrictions.get(size - 1)).getOperator();
							ArrayList leftbuffer = new ArrayList();
							ArrayList rightbuffer = new ArrayList();
							Tuple left = null;
							Tuple right = null;
							int leftindex = -1;
							Tuple leftfront = null;
							Tuple rightfront = null;
							Tuple nextright = null;

							if (operatorNEQ == 0) {
								while (true) {
									if (left == null || right == null)
										if (!tm0.hasNext() || !tm1.hasNext()) {
											break;
										}
									// find first equality
									if (left == null) {
										left = tm0.next();
									}
									if (right == null) {
										right = tm1.next();
									}
									while (left != null && right != null) {
										int num = 0;
										while (num < size) {
											int res = DOMUtils.simpleCompareNodeValue(left.getPath((String) leftstrings.get(num)),right.getPath((String) rightstrings.get(num)));
											if (res == 0)
												num++;
											else {
												if (res < 0) {
													if (tm0.hasNext()) {
														left = tm0.next();
													} else
														left = null;
												} else if (res > 0) {
													if (tm1.hasNext()) {
														right = tm1.next();
													} else
														right = null;
												}
												break;
											}
										}
										if (num == size)
											break;
									}
									if (left == null || right == null) {
										continue;
									}
									// clean buffers
									leftbuffer.clear();
									leftbuffer.add(left);
									rightbuffer.clear();
									rightbuffer.add(right);
									// get all left elements equal to another and bufferize them
									while (true) {
										leftfront = null;
										if (tm0.hasNext()) {
											leftfront = tm0.next();
											int num = 0;
											while (num < size) {
												int res = DOMUtils.simpleCompareNodeValue(left.getPath((String) leftstrings.get(num)),leftfront.getPath((String) leftstrings.get(num)));
												if (res == 0)
													num++;
												else
													break;
											}
											if (num == size) {
												leftbuffer.add(leftfront);
												left = leftfront;
											} else
												break;
										} else
											break;

									}
									// get all right elements equal to another and bufferize them
									while (true) {
										rightfront = null;
										if (tm1.hasNext()) {
											rightfront = tm1.next();
											int num = 0;
											while (num < size) {
												int res = DOMUtils.simpleCompareNodeValue(right.getPath((String) rightstrings.get(num)),rightfront.getPath((String) rightstrings.get(num)));
												if (res == 0)
													num++;
												else
													break;
											}
											if (num == size) {
												rightbuffer.add(rightfront);
												right = rightfront;
											} else {
												break;
											}
										} else
											break;
									}
									// add all necessary tuples to buffertuple result
									for (int i = 0; i < leftbuffer.size(); i++) {
										for (int j = 0; j < rightbuffer.size(); j++) {
											// TEST outside restrictions
											if (outsideRestrictions != null) {
												ArrayList tmpList = new ArrayList();
												tmpList.addAll(context.getCurrentTuples());
												tmpList.add(leftbuffer.get(i));
												tmpList.add(rightbuffer.get(j));
												try {
													EvaluationVisitor visitor = new EvaluationVisitor(operator.getPlan().getSchemaManager(), tmpList);
													outsideRestrictions.accept(visitor);
													if (!visitor.getVerdict())
														continue;
												} catch (XQueryException e) {
													throw new MediatorException(e.getMessage(), e);
												}
											}
											Integer tmpint = (Integer) pivotindex.get(leftbuffer.get(i));
											ArrayList tmplist = (ArrayList) tmpmap.get(tmpint);
											if (tmplist == null) {
												tmplist = new ArrayList();
												tmpmap.put(tmpint, tmplist);
											}
											tmplist.add(rightbuffer.get(j));
											//this.buftuples.add(((Tuple) leftbuffer.get(i)).merge((Tuple) rightbuffer.get(j), algebra));
										}
									}
									left = leftfront;
									right = rightfront;
								}
							} else { // with inequality

								while (true) {
									if (!tm0.hasNext()) {
										break;
									} else if (right == null && nextright == null && !tm1.hasNext() && rightbuffer.isEmpty()) {
										break;
									}
									int num;
									// find first equality
									if (left == null) {
										left = tm0.next();
									} else {
										leftfront = tm0.next();
										num = 0;
										while (num < size - 1) {
											int res = DOMUtils.simpleCompareNodeValue(left.getPath((String) leftstrings.get(num)),leftfront.getPath((String) leftstrings.get(num)));
											if (res == 0)
												num++;
											else
												break;
										}
										if (num != size - 1) {
											rightbuffer.clear();
										}
										left = leftfront;
									}
									if (right == null) {
										if (nextright != null) {
											right = nextright;
											nextright = null;
										} else if (tm1.hasNext()) {
											right = tm1.next();
										}
									}
									// find first match
									boolean add = false;
									while (right != null) {
										num = 0;
										while (num < size) {
											int res = DOMUtils.simpleCompareNodeValue(left.getPath((String) leftstrings.get(num)), right.getPath((String) rightstrings.get(num)));
											if (num != size - 1) {
												if (res == 0)
													num++;
												else {
													if (res < 0) {
														num = size + 1;
														break;
													} else if (res > 0) {
														if (tm1.hasNext()) {
															right = tm1.next();
														} else
															right = null;
													}
													break;
												}
											} else {
												if ((res == 0 && (operatorNEQ == Constants.GEQ_COMPOP || operatorNEQ == Constants.LEQ_COMPOP)) || (res < 0 && (operatorNEQ == Constants.LT_COMPOP || operatorNEQ == Constants.LEQ_COMPOP)) || (res > 0 && (operatorNEQ == Constants.GEQ_COMPOP || operatorNEQ == Constants.GT_COMPOP))) {
													num++;
												} else {
													num = size + 1;
													break;
												}
											}
										}
										if (num == size) {
											add = true;
											rightbuffer.add(right);
											break;
										}
										if (num > size)
											break;
									}

									if (rightbuffer.isEmpty()) {
										continue;
									}
									// get all right elements equal to another and bufferize them
									if (add) {
										rightfront = nextright;
										while (true) {
											if (rightfront == null)
												if (tm1.hasNext()) {
													rightfront = tm1.next();
												}
											if (rightfront != null) {
												num = 0;
												while (num < size) {
													if (num != size - 1) {
														int res = DOMUtils.simpleCompareNodeValue(right.getPath((String) rightstrings.get(num)), rightfront.getPath((String) rightstrings.get(num)));
														if (res == 0)
															num++;
														else {
															nextright = rightfront;
															rightfront = null;
															break;
														}
													} else {
														int res = DOMUtils.simpleCompareNodeValue(left.getPath((String) leftstrings.get(num)), rightfront.getPath((String) rightstrings.get(num)));
														if ((res == 0 && (operatorNEQ == Constants.GEQ_COMPOP || operatorNEQ == Constants.LEQ_COMPOP)) || (res < 0 && (operatorNEQ == Constants.LT_COMPOP || operatorNEQ == Constants.LEQ_COMPOP)) || (res > 0 && (operatorNEQ == Constants.GEQ_COMPOP || operatorNEQ == Constants.GT_COMPOP)))
															num++;
														else {
															nextright = rightfront;
															rightfront = null;
															break;
														}
													}
												}
												if (num == size) {
													rightbuffer.add(rightfront);
													right = rightfront;
													rightfront = null;
												} else {
													break;
												}
											} else
												break;
										}
										right = rightfront;
									}
									// add all necessary tuples to buffertuple result
									for (int j = 0; j < rightbuffer.size(); j++) {
										// TEST outside restrictions
										if (outsideRestrictions != null) {
											ArrayList tmpList = new ArrayList();
											tmpList.addAll(context.getCurrentTuples());
											tmpList.add(left);
											tmpList.add(rightbuffer.get(j));
											try {
												EvaluationVisitor visitor = new EvaluationVisitor(operator.getPlan().getSchemaManager(), tmpList);
												outsideRestrictions.accept(visitor);
												if (!visitor.getVerdict())
													continue;
											} catch (XQueryException e) {
												throw new MediatorException(e.getMessage(), e);
											}
										}
										Integer tmpint = (Integer) pivotindex.get(left);
										ArrayList tmplist = (ArrayList) tmpmap.get(tmpint);
										if (tmplist == null) {
											tmplist = new ArrayList();
											tmpmap.put(tmpint, tmplist);
										}
										tmplist.add(rightbuffer.get(j));
										//this.buftuples.add(left.merge((Tuple) rightbuffer.get(j), algebra));
									}
								}
							}
							// add hashmap to alltuples
							if (tmpmap.isEmpty())
								allTuples.add(null);
							else
								allTuples.add(tmpmap);
							// set index for no recalculation
							indexes[index] = 0;
						}
						if (indexes[index] == 0) { // merge already calculated
							HashMap map = (HashMap) allTuples.get(index);
							if (map != null) {
								ArrayList tmplist = (ArrayList) map.get(new Integer(indexes[0]));
								if (tmplist != null) {
									for (int i = 0; i < tmplist.size(); i++) {
										Tuple newtuple = new Tuple(operator);
										newtuple.appendTuple(tuplepivot);
										newtuple.appendTupleAtIndex(locations[index - 1], (Tuple) tmplist.get(i));
										newtuple.fillIdentifiers(0, tuplepivot);
										newtuple.fillIdentifiers(idlocations[index - 1], (Tuple) tmplist.get(i));
										if (newtuple != null) {
											buftuples.add(newtuple);
											if (!added && non_blocking)
												added = true;
											is_used = true;
										}
									}
								}
							}
							if (index == size - 1)
								endindex = true;
							else
								index++;
						}
						continue;
					} else { // no dependency
						Tuple tuple = null;
						if (indexes[index] == -1) { // first time round
							ResultSet subresultset = children[index].getResultSet();
							if (subresultset.hasNext()) { // more tuples
								tuple = subresultset.next();
								if (currentTuples == null)
									currentTuples = new ArrayList();
								currentTuples.add(tuple);
							} else { // no more tuples
								subresultset.close();
								allTuples.add(currentTuples);
								currentTuples = null;
								indexes[index] = 0;
								if (index == size - 1)
									endindex = true;
								else
									index++;
								continue;
							}
						} else { // not first time round
							if (indexes[index] == 0)
								currentTuples = (ArrayList) allTuples.get(index);
							if (indexes[index] == currentTuples.size()) {
								indexes[index] = 0;
								if (index == size - 1)
									endindex = true;
								else
									index++;
								continue;
							}
							tuple = (Tuple) currentTuples.get(indexes[index]);
							indexes[index]++;
						}
						Tuple newtuple = new Tuple(operator);
						newtuple.appendTuple(tuplepivot);
						newtuple.appendTupleAtIndex(locations[index - 1], tuple);
						newtuple.fillIdentifiers(0, tuplepivot);
						newtuple.fillIdentifiers(idlocations[index - 1], tuple);
						if (newtuple != null) {
							buftuples.add(newtuple);
							if (!added && non_blocking)
								added = true;
							is_used = true;
						}
					}
				}

			}
		} else { // NORMAL CODE for this class!!!	
			while (index < children.length && !children[index].getResultSet().hasNext())
				index++;
			while (index < children.length) {
				if (children[index].getResultSet().hasNext()) {
					Tuple tuple = children[index].getResultSet().next();
					Tuple newtuple = new Tuple(operator);
					if (index == 0)
						newtuple.appendTupleAtIndex(0, tuple);
					else
						newtuple.appendTupleAtIndex(locations[index - 1], tuple);
					if (index == 0)
						newtuple.fillIdentifiers(0, tuple);
					else
						newtuple.fillIdentifiers(idlocations[index - 1], tuple);
					buftuples.add(newtuple);

					if (non_blocking) {
						if ((buftuples != null) && (!buftuples.isEmpty()))
							break;
					}
				} else {
					children[index].getResultSet().close();
					index++;
				}
			}
			if (index == children.length)
				setFinishedWhenEmpty();
		}
	}
}

//class SubQueryPivotNoImbResultSet extends SubQueryResultSet {
//
//	public SubQueryPivotNoImbResultSet(OpSubQuery algebra) throws XMLDBCException {
//		super(algebra, true, false);
//	}
//
//	protected void evaluate(boolean non_blocking) throws XMLDBCException {
//	}
//
//
//}
//
//class SubQueryPivotImbResultSet extends SubQueryPivotNoImbResultSet {
//
//	public SubQueryPivotImbResultSet(OpSubQuery algebra) throws XMLDBCException {
//		super(algebra);
//		this.imbricated = false;
//	}
//
//	protected void evaluate(boolean non_blocking) throws XMLDBCException {
//	}
//}
