/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.plan ;

import org.xquark.mediator.DOMUtils.Tuple;
import org.xquark.mediator.runtime.MediatorException;
import org.xquark.xml.xdbc.XMLDBCException;

public class OpUnion extends OpMult  {
    // **********************************************************************
    // * VERSIONING
    // **********************************************************************
    private static final String RCSRevision = "$Revision: 1.8 $";
    private static final String RCSName = "$Name:  $";    // * INITIALIZATION
    // ***********************************************************************
    /**
     *
     */

    public OpUnion(ExecutionPlan plan, Operator[] operators) throws MediatorException {
        super(plan, null, operators) ;
        init();
    }

    private void init() {
        size = childrenOperator[0].getSize();
        idsize = childrenOperator[0].getIdSize();
        ordertype = ORDER_PARALLELIZATION ;
        //ordertype = this.ORDER_SERIALIZATION ;
    }

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(OperatorVisitor visitor) throws MediatorException {
		visitor.visit(this);
	}

    // ***********************************************************************
    // * EXECUTE QUERY
    // ***********************************************************************
    /**
     *
     */
    protected ResultSet getResultSet(DynamicContext context, OperatorRunnable[] children) throws MediatorException {
        try {
            return new UnionResultSet(this, context, children) ;
        }
        catch (XMLDBCException e) {
            throw new MediatorException("Can't construct " + getClass() + ": " + e.getMessage(), e) ;
        }
    }

    // **********************************************************************
    // * OPTIMIZE
    // **********************************************************************
    /**
     * Merge *this* node with the subnode(s). As it should be called
     * only from source, the returned node must be of type AlgSource.
     * The return node is a AlgSource where the XQueryExpression is the merging
     * of all child(ren) nodes' expression.
     */
    // protected Algebra mergeWithSub() throws MediatorException { return this ; }

    // **********************************************************************
    // * DEBUG
    // **********************************************************************
}

class UnionResultSet extends MultResultSet {
    // **********************************************************************
    // * VERSIONING
    // **********************************************************************
    private static final String RCSRevision = "$Revision: 1.8 $";
    private static final String RCSName = "$Name:  $";

    private int index = 0;
    // ***********************************************************************
    // * INITIALIZATION
    // ***********************************************************************
    /**
     *
     */
    public UnionResultSet(OpUnion operator, DynamicContext context, OperatorRunnable[] children) throws XMLDBCException {
        super(operator, context, children) ;
    }

    // ***********************************************************************
    // * EVALUATE IMPLEMENTATION
    // ***********************************************************************
    /**
     *
     * @param non_blocking if the evaluation is blocking, we must
     *        generate all the results before passing.
     *		  A cartesian product is necessary blocking.
     */
    protected void evaluate(boolean non_blocking) throws MediatorException {
    	
		//non_blocking = false;
    	
        boolean islet = operator.isLet();
        Tuple restuple = null;
        while (true) {
            while (index < children.length && (children[index].getResultSet() == null || !children[index].getResultSet().hasNext())) {
                if (children[index].getResultSet() != null)
					children[index].getResultSet().close();
                index++;
            }
            if (index == children.length) {
                setFinishedWhenEmpty() ;
                break;
            }
            	
            Tuple tuple = children[index].getResultSet().next() ;
            if (islet) {
                if (restuple == null) restuple = tuple;
                else restuple.appendTuple(tuple);
            }
            else buftuples.add(tuple) ;
            if (non_blocking) {
                if (!islet && buftuples != null && ! buftuples.isEmpty()) break ;
            }
        }
        if (islet && restuple != null) buftuples.add(restuple);
    }
}
