/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.plan ;

import org.xquark.mediator.runtime.MediatorException;

public abstract class UnResultSet extends ResultSet {
    // **********************************************************************
    // * VERSIONING
    // **********************************************************************
    private static final String RCSRevision = "$Revision: 1.6 $";
    private static final String RCSName = "$Name:  $";
    // **********************************************************************
    // * CLASS VARIABLES
    // **********************************************************************
    protected ResultSet resultset = null ;
    
    // ***********************************************************************
    // * INITIALIZATION
    // ***********************************************************************
    /**
     *
     */
    public UnResultSet(Operator operator, DynamicContext context, OperatorRunnable child) throws MediatorException {
        super(operator, context);
        // if it is a leaf (for AlgSource)
        if (child != null)         
        	resultset = child.getResultSet() ;
    }
    
    /**
     *
     */
    protected void setFinishedWhenEmpty() throws MediatorException {
        super.setFinishedWhenEmpty() ;
        if (resultset != null) {
        	resultset.getOperator().terminate();
        	resultset.close() ;
        }
    }
    
    /**
     *
     */
    public void close() throws MediatorException {
        super.close() ;
        //if (! resultset.isClosed ())
		if (resultset != null)
			resultset.close() ;
    }
    
    /** Fill the buffer buftuples with tuple.
     *
     * @param non_blocking if the evaluation is blocking, we must
     *        generate all the results before passing.
     */
    protected void evaluate(boolean non_blocking) throws MediatorException {
    }
    
}
