/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 XQuark Group.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xml.xqueryevaluator.eval;


public class XPathFilter implements Cloneable {
	private static final String RCSRevision = "$Revision: 1.1 $"; 
	private static final String RCSName     = "$Name:  $";
    private int index = 0;
    private short[] skipCounts;
    XPathExpr xpath;
    
    public XPathFilter(XPathExpr xpath) {
      this.xpath = xpath;
	  skipCounts = new short[xpath.getStepCount()];
    }
    
    public boolean isDone() {
      return index == xpath.getStepCount();
    }
    
    protected Object clone() {
    	XPathFilter result = null;
    	try {
    		result = (XPathFilter) super.clone();
    		result.skipCounts = new short[skipCounts.length];
    		for (int i = 0; i < skipCounts.length; i++) {
				result.skipCounts[i] = skipCounts[i];
    		}
    	} catch (CloneNotSupportedException ex) {}
    	return result;
    }
    
    public boolean pushElement(String namespaceURI, String localName) {
      if (isDone()) return false;
	  if (xpath.isAnyLevel(index)) {
		skipCounts[index]++;
		return true;
	  } 
	  if (xpath.isSelf(index)) return false;
      if (xpath.matchElement(namespaceURI, localName, index)) {
        index++;
        skipSelfStep(namespaceURI, localName);
        return true;
      }
      return false;
    }
    
    public boolean matchDescendantOrSelfStep(String namespaceURI, String localName) {
    	return xpath.isAnyLevel(index) && (xpath.isSelf(index) || skipCounts[index] > 0) && xpath.matchElement(namespaceURI, localName, index);
    }
    
    public XPathFilter fork(String namespaceURI, String localName) {
    	XPathFilter next = (XPathFilter) clone();
    	if (next.skipCounts[index] > 0) next.skipCounts[index]--;
    	next.index++;
    	next.skipSelfStep(namespaceURI, localName);
    	return next;
    }
    
    public void skipSelfStep(String namespaceURI, String localName) {
    	while (!xpath.isAnyLevel(index) && xpath.isSelfElement(index) && xpath.matchElement(namespaceURI, localName, index)) {
    		index++;
    	}
    }
    
	public boolean matchSelf(String uri, String localName) {
		if (isDone()) return false;
		for (int i = index; i < xpath.getStepCount(); i++) {
			if (!xpath.isSelf(i) || !xpath.matchElement(uri, localName, i))
			    return false;
		}
		return true;
	}
    
    public boolean matchAttribute(String namespaceURI, String localName) {
      if (isDone()) return false;
      if (xpath.matchAttribute(namespaceURI, localName, index)) {
        return true;
      }
      return false;
    }
    
    public boolean matchText() {
		if (isDone()) return false;
		if (!xpath.isAnyLevel(index) && xpath.isSelf(index)) return false;
		if (xpath.matchText(index)) {
			for (int i = index+1; i < xpath.getStepCount(); i++) {
				if (!xpath.isSelf(i) || !xpath.matchText(i))
					return false;
			}
			return true;
		}
		return false;
    }
    
	public boolean matchSelfText() {
		if (isDone()) return false;
		for (int i = index; i < xpath.getStepCount(); i++) {
			if (!xpath.isSelf(i) || !xpath.matchText(i))
				return false;
		}
		return true;
	}
    
	public boolean matchSelfNode() {
		if (isDone()) return false;
		for (int i = index; i < xpath.getStepCount(); i++) {
			if (!xpath.isSelf(i) || !xpath.isSelfNode(i))
				return false;
		}
		return true;
	}
    
    public void popElement() {
    	if (!isDone() && skipCounts[index] > 0) skipCounts[index]--;
    	else if (index > 0) index--;
    }
    
  }
