/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * Copyright (C) 2003 XQuark Group.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

/**
 * Created on 15 nov. 2002
 *
 * To change this generated comment edit the template variable "filecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of file comments go to
 * Window>Preferences>Java>Code Generation.
 */
package org.xquark.jdbc.typing;

import java.sql.*;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.*;

/**
 * Object containing DBMS specific information
 *
 */
public class DBMSInfo
{
	private static final String RCSRevision = "$Revision: 1.2 $";
	private static final String RCSName = "$Name:  $";

    private short dbms;
	private int systemCollectionsDataLength = 255;
    private int maxUserCollectionsDataLength = 255;	// indexable size
    private String rowId = null;
	private boolean useDoubleQuotes4DDLNames = false;
	private SQLStatementCustomizer selectLockStatement = null;
	private SQLStatementCustomizer temporaryTableStatement = null;
	private SQLStatementCustomizer indexOrganizedStatement = null;
    private TypeMap typeMap = null;

	/**
	 * Returns the typeMap.
	 * @return TypeMap
	 */
	public TypeMap getTypeMap()
	{
		return typeMap;
	}

	/**
	 * Returns the dbms.
	 * @return short
	 */
	public short getDbms()
	{
		return dbms;
	}

	/**
	 * Returns the selectLockStatement.
	 * @return String
	 */
	public SQLStatementCustomizer getSelectLockStatement()
	{
		return selectLockStatement;
	}

	/**
	 * Returns the maxUserCollectionsDataLength.
	 * @return int
	 */
	public int getMaxUserCollectionsDataLength()
	{
		return maxUserCollectionsDataLength;
	}

	/**
	 * Returns the useDoubleQuotesForDatabaseObjects.
	 * @return boolean
	 */
	public boolean useDoubleQuotes4DDLNames()
	{
		return useDoubleQuotes4DDLNames;
	}

	/**
	 * Returns the rowId.
	 * @return String
	 */
	public String getRowId()
	{
		return rowId;
	}

	/**
	 * Returns the systemCollectionsDataLength.
	 * @return int
	 */
	public int getSystemCollectionsDataLength()
	{
		return systemCollectionsDataLength;
	}

	/**
	 * Sets the typeMap.
	 * @param typeMap The typeMap to set
	 */
	public void setTypeMap(TypeMap typeMap)
	{
		this.typeMap = typeMap;
	}

	/**
	 * Sets the dbms.
	 * @param dbms The dbms to set
	 */
	public void setDbms(short dbms)
	{
		this.dbms = dbms;
	}

	/**
	 * Sets the selectLockStatement.
	 * @param selectLockStatement The selectLockStatement to set
	 */
	public void setSelectLockStatement(SQLStatementCustomizer selectLockStatement)
	{
		this.selectLockStatement = selectLockStatement;
	}

	/**
	 * Sets the maxUserCollectionsDataLength.
	 * @param maxUserCollectionsDataLength The maxUserCollectionsDataLength to set
	 */
	public void setMaxUserCollectionsDataLength(int maxUserCollectionsDataLength)
	{
		this.maxUserCollectionsDataLength = maxUserCollectionsDataLength;
	}

	/**
	 * Sets the useDoubleQuotesForDatabaseObjects.
	 * @param useDoubleQuotesForDatabaseObjects The useDoubleQuotesForDatabaseObjects to set
	 */
	public void setUseDoubleQuotes4DDLNames(boolean useDoubleQuotesForDatabaseObjects)
	{
		this.useDoubleQuotes4DDLNames = useDoubleQuotesForDatabaseObjects;
	}

	/**
	 * Sets the rowId.
	 * @param rowId The rowId to set
	 */
	public void setRowId(String rowId)
	{
		this.rowId = rowId;
	}

	/**
	 * Sets the systemCollectionsDataLength.
	 * @param systemCollectionsDataLength The systemCollectionsDataLength to set
	 */
	public void setSystemCollectionsDataLength(int systemCollectionsDataLength)
	{
		this.systemCollectionsDataLength = systemCollectionsDataLength;
	}

	/**
	 * Returns the indexOrganizedStatement.
	 * @return SQLStatementCustomizer
	 */
	public SQLStatementCustomizer getIndexOrganizedStatement()
	{
		return indexOrganizedStatement;
	}

	/**
	 * Returns the temporaryTableStatement.
	 * @return SQLStatementCustomizer
	 */
	public SQLStatementCustomizer getTemporaryTableStatement()
	{
		return temporaryTableStatement;
	}

	/**
	 * Sets the indexOrganizedStatement.
	 * @param indexOrganizedStatement The indexOrganizedStatement to set
	 */
	public void setIndexOrganizedStatement(SQLStatementCustomizer indexOrganizedHint)
	{
		this.indexOrganizedStatement = indexOrganizedHint;
	}

	/**
	 * Sets the temporaryTableStatement.
	 * @param temporaryTableStatement The temporaryTableStatement to set
	 */
	public void setTemporaryTableStatement(SQLStatementCustomizer temporaryTableHint)
	{
		this.temporaryTableStatement = temporaryTableHint;
	}
	
	/**
	 * Retrieves org.xquark.repository.mapping.ColumnMetadata from the
	 * underlying database.
	 * @param tableName The table name
	 * @throws SQLException If a database error occurs while creation
	 */
	public List getTableMetadata(String catalogName, String schemaName,
									String tableName, Connection connection,
								 		JDBCProperties properties)
	throws SQLException {
	    if (properties.useResultSet4ColumnMetadata())
	        return getTableMetadataUsingRSMetadata(catalogName, schemaName,
	                tableName, connection, properties);
	    else
	        return getTableMetadataUsingDBMetadata(catalogName, schemaName,
	                tableName, connection, properties);
	}

	private List getTableMetadataUsingDBMetadata(String catalogName, String schemaName,
							String tableName, Connection connection, JDBCProperties properties)
	throws SQLException {
		List columns = new ArrayList();
		DatabaseMetaData metadata = connection.getMetaData();
		ResultSet rs = metadata.getColumns(catalogName, schemaName, tableName, null);
		try {
			while (rs.next()) {
				columns.add(new ColumnMetaDataImpl(rs, getTypeMap(), properties));
			}
		} finally {
			rs.close();
		}
		return columns;
	}

	private List getTableMetadataUsingRSMetadata(String catalogName, String schemaName,
							String tableName, Connection connection, JDBCProperties properties)
	throws SQLException {
		List columns = new ArrayList();
		Statement stmt = null;
		ResultSetMetaData rsetdata = null;
		if (dbms == DBMSConstants.MYSQL || dbms == DBMSConstants.MYSQL323) {
			stmt = connection.createStatement();
			stmt.execute(
				createQuery(catalogName, schemaName, tableName));
			rsetdata = stmt.getResultSet().getMetaData();
		}
		else {
			stmt =
			connection.prepareStatement(
					createQuery(catalogName, schemaName, tableName));
			((PreparedStatement) stmt).execute();
			rsetdata = ((PreparedStatement) stmt).getMetaData();
		}
		
		try {
			for (int i = 1; i <= rsetdata.getColumnCount(); i++) {
				columns.add(new ColumnMetaDataImpl(rsetdata, i, getTypeMap(), properties));
			}
		} finally {
		    if (stmt != null)
		        stmt.close();
		}
		return columns;
	}

    /**
     * Create a dummy costless query in order to explore result set JDBC
     * metadata that seems more reliable on most drivers than exploring
     * database metadata.
     * @param catalogName
     * @param schemaName
     * @param origTableName
     * @return
     */
    protected String createQuery(String catalogName, String schemaName, String origTableName) {
    	StringBuffer retVal = new StringBuffer();
    
    	retVal.append("SELECT * FROM ");
    	if (null != catalogName && 0 < catalogName.trim().length()) {
    		retVal.append(catalogName.trim());
    		retVal.append(".");
    	}
    	if (null != schemaName && 0 < schemaName.trim().length()) {
    		retVal.append(schemaName.trim());
    		retVal.append(".");
    	}
    	if (null != origTableName && 0 < origTableName.trim().length()) {
    		retVal.append(origTableName.trim());
    	}
    	retVal.append(" WHERE 0=1");
    	return retVal.toString();
    }
}
