/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * Copyright (C) 2003 XQuark Group.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.jdbc.typing;

import java.io.StringReader;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xquark.schema.*;

/** This class describes the XML type of a datum to map.
 * <p>This class is to be used in a custom mapping context to indicate the type
 * of the datum returned by the user generator.</p>
 */
public class XMLTypeInfo implements SchemaConstants
{
	private static final String RCSRevision = "$Revision: 1.1 $"; 
	private static final String RCSName     = "$Name:  $";
	
    //////////////////////////////////////////////////////////////////////////
    //                          DATA
    //////////////////////////////////////////////////////////////////////////
    /** The XML Schema API simple type
     */
    protected SimpleType sType;
    
	protected String stringType;

	//////////////////////////////////////////////////////////////////////////
    //                 CONSTRUCTORS & ACCESSORS
    //////////////////////////////////////////////////////////////////////////
	protected XMLTypeInfo() {}
	
    /** Constructor.
     * @param XMLType an XML Schema type as an XML fragment.
     * @see org.xquark.repository.UserGenerator#getXMLType()
     */
    public XMLTypeInfo(String XMLType)
    {
        setXMLType(XMLType);
    }
    
    /** Constructor.
     * @param XMLType an XML Schema API SimpleType object.
     */
    public XMLTypeInfo(SimpleType XMLType)
    {
        setXMLType(XMLType);
    }
    
    /** Accessor.
     * @param XMLType an XML Schema API SimpleType object.
     */
    public void setXMLType(SimpleType XMLType)
    {
        sType = XMLType;
    }
    
    /** Accessor.
     * @param XMLType an XML Schema API SimpleType object.
     */
    public SimpleType getXMLType()
    {
        return sType;
    }
    
	public String getStringXMLType()
	{
		return stringType;
	}
	
    /** Accessor.
     * @param XMLType an XML Schema type as an XML fragment.
     * @see getXMLType()
     */
    public void setXMLType(String XMLType)
    {
        XMLType = XMLType.trim();
        stringType = XMLType;
        
        SchemaManager dummyManager = new SchemaManager();
        int offset = XMLType.indexOf(SIMPLE_TYPE_TAG);
        final String DUMMY_TYPE = "dummyType";
        
        if (offset >= 0)  // a fragment to adapt to add a name
        {
            StringBuffer dummySchemaString = new StringBuffer("<");
            dummySchemaString.append(SCHEMA_TAG);
            dummySchemaString.append(" xmlns=\"");
            dummySchemaString.append(XMLSCHEMA_URI);
            dummySchemaString.append("\" ");
            dummySchemaString.append(TARGET_NAMESPACE_ATTR);
            dummySchemaString.append("=\"RepositoryTemp\">");
            dummySchemaString.append(XMLType.substring(0, offset + SIMPLE_TYPE_TAG.length()));
            dummySchemaString.append(" name=\"");
            dummySchemaString.append(DUMMY_TYPE);
            dummySchemaString.append('"');
            dummySchemaString.append(XMLType.substring(offset + SIMPLE_TYPE_TAG.length()));
            dummySchemaString.append("</");
            dummySchemaString.append(SCHEMA_TAG);
            dummySchemaString.append('>');
            
            // loadSchema
            Schema dummySchema;
            try {
                dummySchema = dummyManager.loadSchema(new InputSource(new StringReader(dummySchemaString.toString())));
            }
            catch (SAXException e) {
				throw new RuntimeException(e.getMessage());
            }
            sType = (SimpleType)dummySchema.getType(DUMMY_TYPE);
        }
        else    // a simple string representing an XML Schema predefined datatype
        {
            sType = (SimpleType)dummyManager.getType(XMLSCHEMA_URI, XMLType);
            if (sType == null)
                throw new RuntimeException("Unknown XML Schema predefined datatype \"" + XMLType + "\" in user generator");
        }
    }
    
    public String toString()
    {
        return sType.toString();
    }
        
}
