/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.schema.datatypes;

import java.net.URL;

import org.xquark.schema.SchemaException;
import org.xquark.schema.datatypes.URI.MalformedURIException;
import org.xquark.schema.validation.ValidationContextProvider;


class AnyURIType extends MesureableType {
    private static final String RCSRevision = "$Revision: 1.3 $";
    private static final String RCSName = "$Name:  $";

    AnyURIType() {
        super("anyURI", PrimitiveType.ANY_URI);
    }

	protected Object toValidType(Object data) {
		if (data instanceof URL) {
			try {
				return toURI(data.toString());
			} catch (SchemaException e) {
				// should not happen
				return (URI) data;
			}
		} else {
			return (URI) data;
		}
	}

    protected URI toURI(String value) throws SchemaException {
        try {
        	return new URI(value);
        } catch (MalformedURIException e) {
            super.invalidValue(value);
            return null;
        }
    }

    public void checkFacets(Object valueSpace) throws SchemaException {
        super.checkFacets(valueSpace);
        super.checkLength(((URI) valueSpace).getLength(), valueSpace);
    }

    protected Object toValueSpace(String value, ValidationContextProvider context) throws SchemaException {
        return toURI(value);
    }
}
