/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.schema.datatypes;

import org.xquark.schema.SchemaException;
import org.xquark.schema.validation.ValidationContextProvider;


class BooleanType extends PrimitiveType {
    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName = "$Name:  $";

    BooleanType() {
        super("boolean", PrimitiveType.BOOLEAN);
    }

	protected Object toValidType(Object data) {
		return (Boolean) data;
	}

    public void checkFacets(Object valueSpace) throws SchemaException {
        return;
    }

    protected Object toValueSpace(String value, ValidationContextProvider context) throws SchemaException {
        Boolean result = null;
        if (value.equals("true") || value.equals("1"))
            result = Boolean.TRUE;
        else if (value.equals("false") || value.equals("0"))
            result = Boolean.FALSE;
        else
            super.invalidValue(value.toString());
        return result;
    }

    public String toXMLString(Object data, ValidationContextProvider context) {
        if (data == null)
            return null;
        if (data instanceof Boolean)
            return data.toString();
        else if (data instanceof Number) {
            if (((java.lang.Number) data).intValue() == 1)
                return "true";
            else
                return "false";
        } else if (data instanceof String) {
            if (("1".equals(data)))
                return "true";
            else
                return "false";
        } else
            return "false";
    }

}
