/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.schema.datatypes;


public class ByteArray {
	
	private byte[] data;
	private int hashCode;

	public ByteArray(byte[] data) {
		this.data = data;
		this.hashCode = computeHashCode(data);
	}
	
	public ByteArray(byte[] data, int hashCode) {
		this.data = data;
		this.hashCode = hashCode;
	}
	
	public byte[] getData() {
		return data;
	}
	
	public int getLength() {
		return data.length;
	}
	
	public boolean equals(Object obj) {
		if (obj == this) return true;
		if (!checkClass(obj)) return false;
		ByteArray other = (ByteArray)obj;
		int len = getLength();
		if (other.getLength() != len) return false;
		for (int i = 0; i < len; i++) 
			if (data[i] != other.data[i]) return false;
		return true;
	}
	
	protected boolean checkClass(Object obj) {
		return obj instanceof ByteArray; 
	}
	
	public int hashCode() {
		return hashCode;
	}
	
	public static int computeHashCode(byte[] data) {
		int len = data.length/4;
		int result = 0;
		int tmp = 0;
		for (int i = 0; i < len; i++) {
			tmp = data[4*i] << 24;
			tmp |= data[4*i+1] << 16;
			tmp |= data[4*i+2] << 8;
			tmp |= data[4*i+3];
			result ^= tmp;
		}
		tmp = 0;
		switch (data.length%4) {
			case 3: 
				tmp |= data[4*len+2] << 8;
				// pass through
			case 2:
				tmp |= data[4*len+1] << 16;
				// pass through
			case 1:
				tmp |= data[4*len] << 24;
				// pass through
			default:
				result ^= tmp;
		}
		return result;
	}
}
