/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.schema.datatypes;

import java.text.DecimalFormat;
import java.util.Locale;

import org.xquark.schema.SchemaException;
import org.xquark.schema.validation.ValidationContextProvider;


class FloatType extends ComparableType {
    private static final String RCSRevision = "$Revision: 1.3 $";
    private static final String RCSName = "$Name:  $";

	private static final String NAN = "NaN";
	private static final String INF = "INF";
	private static final String POSITIVE_INF = "+INF";
	private static final String NEGATIVE_INF = "-INF";
	
	private DecimalFormat format = null;
	
    FloatType() {
        super("float", PrimitiveType.FLOAT);
        format = (DecimalFormat)DecimalFormat.getInstance(Locale.US);
        format.applyPattern("0.0######E0");
    }

	public Object toValidType(Object data) {
		if (data instanceof Float) {
			return data;
		} else if (data instanceof Number) {
			return new Float(((Number)data).floatValue());
		} else {
			return (Float)data;
		}
	}
	
    protected Comparable toComparable(String value) throws SchemaException {
        // special float
        if (INF.equals(value))
            return new Float(Float.POSITIVE_INFINITY);
        else if (POSITIVE_INF.equals(value))
            return new Float(Float.POSITIVE_INFINITY);
        else if (NEGATIVE_INF.equals(value))
            return new Float(Float.NEGATIVE_INFINITY);
        else if (NAN.equals(value))
            return new Float(Float.NaN);
        // normal float
        try {
            return new Float(value);
        } catch (NumberFormatException nfe) {
            super.invalidValue(value);
            return null;
        }
    }

    public String toXMLString(Object obj, ValidationContextProvider context) {
		if (obj == null) return null;   
        Float data = (Float)obj;
        if (data.isNaN()) return NAN;
        else if (data.isInfinite()) {
        	if (data.floatValue() > 0) return INF;
        	else return NEGATIVE_INF;  
        } else {
        	return format.format(data.doubleValue());
        }
    }
}
