/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.schema.loader;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xquark.schema.IdentityConstraint;
import org.xquark.schema.SchemaConstants;
import org.xquark.schema.SchemaException;
import org.xquark.util.DefaultElementHandler;
import org.xquark.util.ElementHandler;

class IdentityConstraintHandler extends DefaultElementHandler implements SchemaConstants {
private static final String RCSRevision = "$Revision: 1.1 $"; 
private static final String RCSName     = "$Name:  $";
  private Loader loader = null;
  private IdentityConstraint identityConstraint = null;
  
  IdentityConstraintHandler(Loader loader, IdentityConstraint identityConstraint)
  throws SAXException {
    this.loader = loader;
    this.identityConstraint = identityConstraint;
  }
  
  public ElementHandler startElement(String namespaceURI, String localName, Attributes atts)
  throws SAXException {
        if ( namespaceURI.equals(XMLSCHEMA_URI) ) {
            // selector
            if ( localName.equals(SELECTOR_TAG) ) {
              String xpath = atts.getValue("", XPATH_ATTR);
              try {
                  identityConstraint.setSelector(loader.buildXPaths(xpath));
              }
              catch ( SchemaException se ) {
                  String errMsg = "Error while processing localName -> " + localName; 
                  loader.reportLoadingError(errMsg, se);
              }     
              return this;
            }

            // field
            if ( localName.equals(FIELD_TAG) ) {
              String xpath = atts.getValue("", XPATH_ATTR);
              try {
                  identityConstraint.addField(loader.buildXPaths(xpath));
              }
              catch ( SchemaException se ) {
                  String errMsg = "Error while processing localName -> " + localName; 
                  loader.reportLoadingError(errMsg, se);
               }     
              return this;
            }

            if ( localName.equals(ANNOTATION_TAG) ) {
              return new AnnotationHandler();
            }
        }

        loader.notifyUnknownElement(namespaceURI, localName);
        return this;
  }
  
}
