/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.schema.loader;

import java.util.StringTokenizer;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xquark.schema.*;
import org.xquark.util.DefaultElementHandler;
import org.xquark.util.ElementHandler;

class SimpleTypeHandler extends DefaultElementHandler implements SchemaConstants
{
private static final String RCSRevision = "$Revision: 1.1 $"; 
private static final String RCSName     = "$Name:  $";
  private Loader loader = null;
  private Type type = null;
  private boolean simpleContent;
  private boolean redefine;
  private boolean alreadyRedefined = false;

  SimpleTypeHandler(Loader loader, Type type, boolean simpleContent, boolean redefine) 
    throws SAXException 
  {
			this.loader = loader;
			this.type = type;
      this.simpleContent = simpleContent;
      this.redefine = redefine;
	}
    	
	public ElementHandler startElement(String strNamespaceURI, String localName, Attributes atts) 
	throws SAXException
	{       
	  if ( strNamespaceURI.equals(XMLSCHEMA_URI) ) {     
        // restriction
        if ( localName.equals(RESTRICTION_TAG) ) {
          if ( !redefine ) {
            // set base type
            Type baseType = null;
            try {
              baseType = loader.getType(atts.getValue("", BASE_ATTR));
            } 
            catch ( SchemaException se ) {
              String errMsg = "Error while processing localName -> " + localName; 
              loader.reportLoadingError(errMsg, se);
            }
            type.setBaseType(baseType);
            if ( simpleContent ) ((ComplexType)type).setExtension(false);
          } 
          else {
            // redefine simpel Type : special treating
            alreadyRedefined = true;
            ((SimpleType)type).setVariety(SimpleType.VARIETY_ATOMIC);
            // get base type name
            String baseTypeName = atts.getValue("", BASE_ATTR);
            if ( !this.type.getName().equals(baseTypeName) ) {
              String errCode = "src-redefine.5";
              String errMsg = "Error while processing localName -> " + localName; 
              loader.reportLoadingError(errMsg, new SchemaException(errCode));
            }
          }
          return this;
        }

        // in a redefinition, simpleContent is always false
        // facet
        Facet facet = loader.buildFacet(localName, atts);
        if (facet != null) {
          if ( simpleContent ) {
            SimpleType contentType = null;
            if (((ComplexType)type).getContentModel().getModel() == null) {
              contentType = new SimpleType(type.getSchema(), null, new SimpleBaseTypeRef(((ComplexType)type)));
              ((ComplexType)type).getContentModel().setModel(contentType);
            } else {
              contentType = (SimpleType)((ComplexType)type).getContentModel().getModel();
            }
            try {
              contentType.setFacet(facet);
            } catch ( SchemaException e ) {
              String errMsg = "Error while processing localName -> " + localName; 
              loader.reportLoadingError(errMsg, e);
            }
          }
          else {
            if ( redefine && !alreadyRedefined ) {
              String errCode = "src-redefine.5";
              String errMsg = "Error while processing localName -> " + localName; 
              loader.reportLoadingError(errMsg, new SchemaException(errCode));
            }
            try {
              ((SimpleType)type).setFacet(facet);
            } catch ( SchemaException e ) {
              String errMsg = "Error while processing localName -> " + localName; 
              loader.reportLoadingError(errMsg, e);
            }
          }

          return this;
        }

        // redefined simpleType can't have the following tags
        if ( redefine ) {
          String errCode = "src-redefine.5";
          String errMsg = "Error while processing localName -> " + localName; 
          loader.reportLoadingError(errMsg, new SchemaException(errCode));
        }

        // this is only when simpleContent is true
        // in a redefinition, simpleContent is always false
        if ( simpleContent && localName.equals(EXTENSION_TAG) ) {
          // get base type
          Type baseType = null;
          try {
            baseType = loader.getType(atts.getValue("", BASE_ATTR));
          } 
          catch ( SchemaException se ) {
            String errMsg = "Error while processing localName -> " + localName; 
            loader.reportLoadingError(errMsg, se);
          }
          type.setBaseType(baseType);
          ((ComplexType)type).setExtension(true);
          return this;
        }

        else if (localName.equals(SIMPLE_TYPE_TAG)) {
          // inline simple type definition
          SimpleType inlineType = null;
          try {
            inlineType = loader.buildSimpleType(atts);
          } 
          catch ( SchemaException se ) {
            String errMsg = "Error while processing localName -> " + localName; 
            loader.reportLoadingError(errMsg, se);
          }
          if ( simpleContent ) {
            ((ComplexType)type).getContentModel().setModel(inlineType);
            return new SimpleTypeHandler(loader, inlineType, false, false);
          }
          else {
            switch ( ((SimpleType)type).getVariety() ) {
              case SimpleType.VARIETY_ATOMIC:
              // chack for restriction
              if ( type.getBaseType() != null ) {
                String errCode = "src-simple-type.2";
                String errMsg = "Error while processing localName -> " + localName; 
                loader.reportLoadingError(errMsg, new SchemaException(errCode));
              }
              type.setBaseType(inlineType);

              case SimpleType.VARIETY_LIST:
              if ( ((SimpleType)type).getItemType() != null ) {
                String errCode = "src-simple-type.3";
                String errMsg = "Error while processing localName -> " + localName; 
                loader.reportLoadingError(errMsg, new SchemaException(errCode));
              }
              ((SimpleType)type).setItemType(inlineType);

              case SimpleType.VARIETY_UNION:
              ((SimpleType)type).addMemberType(inlineType);
            }
            return new SimpleTypeHandler(loader, inlineType, false, false);  
          }
        } 

        // list, only simpleContent is false
        // it's verificated by Schema of Schemas
        else if ( localName.equals(LIST_TAG) ) {
          ((SimpleType)type).setVariety(SimpleType.VARIETY_LIST);
          // check itemType attribute
          String itemTypeName = atts.getValue("", ITEM_TYPE_ATTR);
          if ( itemTypeName != null ) {
            try {
              ((SimpleType)type).setItemType(loader.getType(itemTypeName));
            } 
            catch ( SchemaException se ) {
              String errMsg = "Error while processing localName -> " + localName; 
              loader.reportLoadingError(errMsg, se);
            }
          }
          return this;
        }

        // union, only simpleContent is false
        // it's verificated by Schema of Schemas
        else if ( localName.equals(UNION_TAG) ) {
          ((SimpleType)type).setVariety(SimpleType.VARIETY_UNION);        
          // check memberTypes attribute
          String memberTypes = atts.getValue("", MEMBER_TYPES_ATTR);
          if ( memberTypes != null ) {
            StringTokenizer stMemberTypes = new StringTokenizer(memberTypes);
            while (stMemberTypes.hasMoreTokens()) {
              String memberTypeName = stMemberTypes.nextToken();
              try {
                ((SimpleType)type).addMemberType(loader.getType(memberTypeName));
              } 
              catch ( SchemaException se ) {
                String errMsg = "Error while processing localName -> " + localName; 
                loader.reportLoadingError(errMsg, se);
              }
            }
          }
          return this;
        }

        // attribute tag, only when simpleContent is true
        // it's verificated by Schema of Schemas
        else if ( localName.equals(ATTRIBUTE_TAG) ) {
          // case 3: attribute declaration
          AttributeDeclaration decl = null;
          try {
            decl = loader.buildAttributeDeclaration(atts, ((ComplexType)type));
            ((ComplexType)type).register(decl);
          } catch ( SchemaException e ) {
            String errMsg = "Error while processing localName -> " + localName; 
            loader.reportLoadingError(errMsg, e);
          }
          return new AttributeTypeHandler(loader, decl);
        } 

        // attribute group, only when simpleContent is true
        // it's verificated by Schema of Schemas
        else if ( localName.equals(ATTRIBUTE_GROUP_TAG) ) {
          // case 4: attribute group reference
          String refName = atts.getValue("", REF_ATTR);
          // refName is verified by Schema of Schemas
          try {
            SchemaComponent ref = loader.getRef(refName, Schema.ATTRIBUTE_GROUP_DEFINITION);
            ((ComplexType)type).register(ref);
          } catch ( SchemaException e ) {
            String errMsg = "Error while processing localName -> " + localName; 
            loader.reportLoadingError(errMsg, e);
          }
          return this;
        } 

        // anyAttribute, only when simpleContent is true
        // it's verificated by Schema of Schemas
        else if (localName.equals(ANY_ATTRIBUTE_TAG)) {
          // case 5: any attribute reference
          Wildcard wildcard = null;
          try {
            wildcard = loader.buildWildcard(atts);
          } catch ( SchemaException e ) {
            String errMsg = "Error while processing localName -> " + localName; 
            loader.reportLoadingError(errMsg, e);
          }
          ((ComplexType)type).setAttributeWildcard(wildcard);
          return this;
        } 

        else if ( localName.equals(ANNOTATION_TAG) ) {
          return new AnnotationHandler();
        }
	  }
        
	  loader.notifyUnknownElement(strNamespaceURI, localName);
	  return this;
	}
	
}
