/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.schema.validation;

import org.xquark.schema.*;

public class AllContentIterator extends ContentIterator {
private static final String RCSRevision = "$Revision: 1.1 $"; 
private static final String RCSName     = "$Name:  $";

  private AllModelGroup group;
  private AllModelGroup current;
  private ContentIterator iterator;
  private boolean inUse = false;
  
  public AllContentIterator(Particle particle)
  {
    super(particle);
    group = (AllModelGroup)particle.getTerm();
    this.current = (AllModelGroup)group.clone();
  }

  public ElementDeclaration getMatchedDeclaration() {
    if (iterator != null) 
      return iterator.getMatchedDeclaration();
    else return null;
  }

  public ElementDeclaration getModelDeclaration() {
    if (iterator != null) 
      return iterator.getModelDeclaration();
    else return null;
  }

  public int getProcessContents() {
    if (iterator != null) 
      return iterator.getProcessContents();
    else return LAX;
  }
  
  private ContentIterator selectContentIterator(String namespace, String localName) 
  {
    java.util.Iterator it = current.iterator();
    while (it.hasNext()) {
      Particle particle = (Particle)it.next();
      if (particle.hasNextElement(namespace, localName)) {
        inUse = true;
	it.remove();
	return ContentIteratorFactory.createIterator(particle, reporter);
      }
    }
    return null;
  }
  
  protected int nextElement(String namespace, String localName)
  {
    iterator = selectContentIterator(namespace, localName);
    if (iterator == null)
      return stopProcessing();
    switch (iterator.nextElement(namespace, localName)) {
    case MATCHED:
      return MATCHED;
    case UNKNOWN:
      return UNKNOWN;
    case INVALID:
    case TOO_MANY:
    case DONE:
    default:
      if (iterator.getMatchedDeclaration() != null) 
	return invalidElement(iterator.getExceptions());
      else
	return invalidElement("cvc-particle.3.3", parentParticle, iterator.getExceptions());
    }
  }

  public int stopProcessing() {
    if (!inUse) {
      if ( minOccurs != 0 && groupMinExtent() !=0 ) {
        return invalidElement("cvc-particle.3.1", parentParticle);
      } else if (reporter != null) {
        reporter.skipped(parentParticle);
      }
      return DONE;
    }
    java.util.Iterator it = current.iterator();
    while (it.hasNext()) {
      Particle particle = (Particle)it.next();
      if (particle.getMinExtent() > 0) {
	SchemaException se = SchemaException.computeMinExtentException(particle);
	return invalidElement("cvc-particle.3.3", parentParticle, se);
      } else if (reporter != null) {
	reporter.skipped(particle);
      }
    }
    return DONE;
  }

  protected boolean nextValidElements(java.util.List elements) {
    // for All model group, it's better to add the particle whose minExtent is 0 
    // in the top of the elements list, and the particle whose minExtent is 1
    // int the end of the elements list. If not, ElementGenrator will generate more elements
    // than necessary
    // noted  Nov. 27, 2001
    java.util.Iterator it = current.iterator();
    boolean allOptional = true;
    while (it.hasNext()) {
      Particle particle = (Particle)it.next();
      if (elements != null) {
        if (particle.getMinExtent() > 0)          
          elements.addAll(particle.firstValidElements());
        else
          elements.addAll(0, particle.firstValidElements());
      }
      if (particle.getMinExtent() > 0) allOptional = false;
    }
    return allOptional;
  }
  
  private int groupMinExtent() {
    if ( group == null || group.size() == 0 ) return 0;
    java.util.Iterator it = group.iterator();
    int minExtent = 0;
    while ( it.hasNext() ) {
      Particle p = (Particle)it.next();
      minExtent += p.getMinExtent();
    }
    return minExtent;
  }
    
}




