/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.schema.validation;

import org.xquark.schema.*;

public class ContentIteratorFactory {
private static final String RCSRevision = "$Revision: 1.1 $"; 
private static final String RCSName     = "$Name:  $";
  
  public static ContentIterator createIterator(Particle particle, ContentReporter reporter)
  {
    ContentIteratorFactory.Visitor visitor = new ContentIteratorFactory.Visitor(particle);
    try {
      ((SchemaVisitable)particle.getTerm()).accept(visitor);
    } 
    catch (SchemaException ex) {
      throw new RuntimeException(ex.getMessage());
    }
    ContentIterator result = visitor.getResult();
    result.setContentReporter(reporter);
    return result;
  }

  public static ContentIterator createIterator(Particle particle) {
    return createIterator(particle, null);
  }

  static class Visitor extends DefaultSchemaVisitor {
	private static final String RCSRevision = "$Revision: 1.1 $"; 
	private static final String RCSName     = "$Name:  $";
    private ContentIterator result = null;
    private Particle particle;
    
    Visitor(Particle particle) {
      this.particle = particle;
    }

    ContentIterator getResult() { return result; }
    
    public void visit(ElementDeclaration decl) {
      result = new ElementContentIterator(particle);
    }
    public void visit(SequenceModelGroup group) {
      result = new SequenceContentIterator(particle);
    }
    public void visit(ChoiceModelGroup group) {
      result = new ChoiceContentIterator(particle);
    }
    public void visit(AllModelGroup group) {
      result = new AllContentIterator(particle);
    }
    public void visit(Wildcard wildcard) {
      result = new WildcardContentIterator(particle);
    }
  }

}
