/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.schema.validation;

import org.xquark.schema.*;

public class WildcardContentIterator extends ContentIterator 
  implements SchemaConstants
{
private static final String RCSRevision = "$Revision: 1.1 $"; 
private static final String RCSName     = "$Name:  $";
  private Wildcard wildcard;
  private int count = 0;
  private int process;
  private ElementDeclaration matchedDecl = null;

  public WildcardContentIterator(Particle particle) 
  {
    super(particle.getMinOccurs(), particle.getMaxOccurs());
    this.parentParticle = particle;
    wildcard = (Wildcard)particle.getTerm();
    process = wildcard.getProcessContents();
  }

  public ElementDeclaration getMatchedDeclaration() {
    return matchedDecl;
  }

  public int getProcessContents() {
    return process;
  }

  protected int nextElement(String namespace, String localName)
  {
    if (namespace != null && namespace.length() == 0)
      namespace = null;
    
    String errCode = wildcard.checkNamespaceAllowed(namespace);
    if (errCode != null) {
      if ( count == 0 ) 
        return invalidElement("cvc-particle.1.3", parentParticle, new SchemaException(errCode, wildcard));
      else
        return stopProcessing();
    }

    if (count == maxOccurs) {
      invalidElement("cvc-particle.1.2", parentParticle);
      return TOO_MANY;
    }
    if (reporter != null)
      reporter.matched(parentParticle, count);
    count++;

    if (process == Wildcard.SKIP) return ContentIterator.UNKNOWN;

    matchedDecl = wildcard.getManager().getElementDeclaration(namespace, localName);
    if (matchedDecl != null) 
      return MATCHED;
    else
      return ContentIterator.UNKNOWN;
  }
  
  public int stopProcessing() {
    if (count < minOccurs) 
      return invalidElement("cvc-particle.1.1", parentParticle);
    else {
      if (reporter != null && parentParticle != null && count < maxOccurs)
        reporter.skipped(parentParticle);
      return DONE;
    }
  }

  protected boolean nextValidElements(java.util.List elements) {
    return count >= minOccurs;
  }
  
}


