/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.util;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Define a SAX ContentHandler or/and LexicalHandler writing a DOM-tree inside a given document.
 * Such writing doesn't erase previous content but append to it.
 * So, if the document is not empty, this handler will try to add nodes anyway, and
 * probably cause a hierarchy error due to try to add root node or doc type in addition
 * of a previous one... <BR>
 * <BR>
 * WARNING: if this class is used for two differents types of handler to build a DOMTree from
 * the same parsing, the parser must reference the SAME DOMBuilder object for the two types of
 * handlers unless the DOM-Tree will not correspond to the waited DOM-Tree.
 *
 * WARNING: This class is designed to be used with a SAX2 XMLReader set with the default
 * features (namespaces = true, prefixes = false).
 *
 * @see org.xml.sax.ContentHandler, org.xml.sax.LexicalHandler, org.w3c.dom.Document
 */
public class DOMBuilder extends NSPrefixDecorator
{
    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName     = "$Name:  $";

    private BasicDOMBuilder builder;
    
    public DOMBuilder(Document doc)
    {
        builder = new BasicDOMBuilder(doc);
        plugBuilder();
    }
    
    public DOMBuilder(Element node)
    {
        builder = new BasicDOMBuilder(node);
        plugBuilder();
    }
    
    private void plugBuilder()
    {
		setContentHandler(builder);
		setLexicalHandler(builder);
		setEnabled(true);
    }
    
    public void setDocument(Document doc)
    {
        builder.setDocument(doc);
    }
    
    public void setElement(Element root)
    {
        builder.setElement(root);
    }
}
