/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xml.xdbc;

/**
 * The interface that every driver class must implement.<BR><BR>
 * It allows new connections from XMLDBC client.<BR><BR>
 */
public interface XMLDriver {
 
 	/**
 	 * Returns true if the driver understand the specified connection string
 	 * @param uri the connection string
 	 * @return a boolean
 	 * @throws XMLDBCException
 	 */
	public boolean acceptsURI(String uri) throws XMLDBCException;
	
	/**
	 * Returns the driver-specific part of the URI
	 * @param uri
	 * @return a string
	 * @throws XMLDBCException if the URI is not recognized by the driver
	 */
	public String getSpecificPart(String uri) throws XMLDBCException;
	
    /**
     * To obtain a handle to a data source with the specified URI.
     * @param uri an URI corresponding to a data source.
     * @return an instance of the specified data source,
     *         or null if the driver does not know this type of data source
     * @throws XMLDBCException if a data source access error occurs.
     */
    public XMLDataSource getDataSource(String uri) throws XMLDBCException;

	/**
	 * To obtain a handle to a data source with the specified URI, user name and password.
	 * @param uri a URI corresponding to a data source.
	 * @param user a user name
	 * @param password a user password
	 * @return an instance of the specified data source,
	 *         or null if the driver does not know this type of data source
	 * @throws XMLDBCException if a data source access error occurs.
	 */
	public XMLDataSource getDataSource(String uri, String user, String password) throws XMLDBCException;
}
