/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xml.xdbc;

/**
 * Interface for XMLDBC storage error handlers.
 *
 * <p>If an XMLDBC application needs to implement customized error
 * handling, it must implement this interface and then register an
 * instance with the XMLDocumentFiler using the
 * {@link org.xquark.xml.xdbc.XMLDocumentFiler#setErrorHandler setErrorHandler}
 * method.  The document filer will then report all errors and warnings
 * through this interface.</p>
 *
 * <p>Fatal errors, i.e., errors making storage impossible are not reported
 * through this interface but carry out exceptions.</p>
 *
 * @see org.xquark.xml.xdbc.XMLDocumentFiler#setErrorHandler
 * @see org.xquark.xml.xdbc.XMLDBCException
 */
public interface XMLErrorHandler
{
 

    /**
     * Receive notification of a warning. For instance the risk of data loss
     *
     * @param exception The warning information encapsulated in an XMLDBC exception.
     * @exception XMLDBCException Any XMLDBC exception, possibly
     *            wrapping another exception.
     * @see org.xquark.xml.xdbc.XMLDBCException 
     */
    public void warning (XMLDBCException exception)
	throws XMLDBCException;
    
    
    /**
     * Receive notification of an error that did not prevent the document storage.
     *
     * @param exception The error information encapsulated in an XMLDBC exception.
     * @exception XMLDBCException Any XMLDBC exception, possibly
     *            wrapping another exception.
     * @see org.xquark.xml.xdbc.XMLDBCException 
     */
    public void error (XMLDBCException exception)
	throws XMLDBCException;
}
