/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 XQuark Group.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xpath.datamodel;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

public class ValidationContextProvider
    implements org.xquark.schema.validation.ValidationContextProvider {
    public static final String RCSRevision = "$Revision: 1.1 $";
    public static final String RCSName = "$Name:  $";

    protected HashMap namespaces;
    protected HashMap notations;
    protected String documentBase;
    protected HashMap reverseNamespaces = null;

    /**
     * Constructor for ValidationContextProvider.
     */
    public ValidationContextProvider(Map namespaces, HashMap notations, 
                                     String documentBase) {
        this.namespaces = new HashMap(namespaces);

        if (notations != null) {
            this.notations = new HashMap(notations);
        }

        this.documentBase = documentBase;
    }

    /**
     * @see ValidationContextProvider#getDocumentBase()
     */
    public String getDocumentBase() {
        return documentBase;
    }

    /**
     * @see ValidationContextProvider#getNotationDeclarations()
     */
    public Map getNotationDeclarations() {
        return notations;
    }

    /**
     * @see ValidationContextProvider#getNamespaceURI(String)
     */
    public String getNamespaceURI(String prefix) {
        return (String) namespaces.get(prefix);
    }

    /**
     * @see ValidationContextProvider#getPrefix(String)
     */
    public String getPrefix(String uri) {
        if (reverseNamespaces == null) {
            reverseNamespaces = new HashMap(namespaces.size());

            Iterator iterator = namespaces.keySet().iterator();
            Object pre;
            Object name;

            while (iterator.hasNext()) {
                pre = iterator.next();
                name = namespaces.get(pre);
                reverseNamespaces.put(name, pre);
            }
        }

        return (String) reverseNamespaces.get(uri);
    }
}
