/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2004 XQuark Group.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.metadata;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.Date;
import java.util.TimeZone;

import org.xquark.xquery.JavaModule;
import org.xquark.xquery.parser.*;

public class DynamicContext {

	private Date currentDateTime = null;
	private TimeZone currentTimeZone = null;

	private JavaInvoker javaInvoker = new JavaInvoker();
	
	public DynamicContext() {}
	
	public Object invoke(LibraryFunctionCall lfc, Object[] params) throws XQueryException {
		QName funcName = lfc.getFuncName();
		int index = funcName.getNameSpace().indexOf(':');
		String type = funcName.getNameSpace().substring(0,index);
		if (type.equals("java"))
			return javaInvoker.invoke(lfc,params);
		return null;
	}

	public void addImportModule(XQueryModule module) throws XQueryException {
		if (module instanceof JavaModule) {
			try {
				JavaModule mod = (JavaModule) module;
				Class javaClass = mod.getJavaClass();
				Class[] classes = null;
				Constructor constructor = javaClass.getConstructor(classes);
				Object obj = constructor.newInstance(null);
				javaInvoker.addClass(javaClass.getName(), obj);
			} catch (InvocationTargetException e) {
				throw new XQueryException("Could not import module " + module.getNamespace());
			} catch (IllegalAccessException e) {
				throw new XQueryException("Could not import module " + module.getNamespace());
			} catch (InstantiationException e) {
				throw new XQueryException("Could not import module " + module.getNamespace());
			} catch (NoSuchMethodException e) {
				throw new XQueryException("Could not import module " + module.getNamespace());
			}
		}
	}
}
