/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2004 XQuark Group.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.metadata;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;

import org.xquark.xquery.parser.LibraryFunctionCall;
import org.xquark.xquery.parser.XQueryException;

public class JavaInvoker implements InvokerInterface {

	private DynamicContext dc = null;
	private HashMap javaClassMap = new HashMap();
	
	public Object invoke(String funcName, Object[] params) {
		return null;
	}
	
	public Object invoke(LibraryFunctionCall lfc, Object[] params) throws XQueryException {
		try {
			Object javaObj = (Object) javaClassMap.get(lfc.getFuncName().getNameSpace().substring(5));
			Method method = lfc.getFunctionDefinition().getMethod();
			Class[] paramTypes = method.getParameterTypes();
			// translate dataModel objects to java objects
			// cast to match parmeter types of method
			Object[] javaObjs = new Object[params.length];
			for (int i=0;i<params.length;i++) {
				Object obj = getJavaObjectFromDataModelObject(params[i],paramTypes[i]);
				javaObjs[i] = obj;
			}
			Object obj = method.invoke(javaObj, javaObjs);
			return obj;
		} catch (IllegalAccessException e) {
			throw new XQueryException(e.getMessage());
		} catch (InvocationTargetException e) {
			throw new XQueryException(e.getMessage());
		}
	}

	public void setDC(DynamicContext dc) {
		this.dc = dc;
	}

	public void addClass(String name, Object obj) {
		javaClassMap.put(name,obj);
	}

/*
	Simple Type -> Java Type
	xsd:string -> java.lang.String
	xsd:integer -> java.math.BigInteger
	xsd:int -> int
	xsd:long -> long
	xsd:short -> short
	xsd:decimal -> java.math.BigDecimal
	xsd:float -> float
	xsd:double -> double
	xsd:boolean -> boolean
	xsd:byte -> byte
	xsd:unsignedInt -> long
	xsd:unsignedShort -> int
	xsd:unsignedByte -> short
	xsd:QName -> javax.xml.namespace.QName
	xsd:dateTime -> java.util.Calendar
	xsd:date -> java.util.Calendar
	xsd:time -> java.util.Calendar
	xsd:anyURI -> java.net.URI (J2SE 1.4 only) java.lang.String
	xsd:base64Binary -> byte[]
	xsd:hexBinary -> byte[]
	xsd:anySimpleType -> java.lang.String
*/
	private Object getJavaObjectFromDataModelObject(Object obj, Class cl) {
		if (obj instanceof ArrayList)
			return null;
		if (cl == String.class) {
			return obj.toString();
		} else if (cl == int.class) {
			return new Integer(((Number)obj).intValue());
		}
		return null;
	}


}
