/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.metadata;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import org.xquark.schema.*;
import org.xquark.xpath.NodeKind;
import org.xquark.xpath.XTree;
import org.xquark.xpath.XTreeNode;
import org.xquark.xpath.schema.SchemaNode;
import org.xquark.xpath.schema.SchemaTreeBuilder;
import org.xquark.xquery.metadata.resolver.CollectionMetadata;

// ***************************************************************************
// * Class 'MetaCollection'
// ***************************************************************************

public class MetaCollection implements CollectionMetadata {
	// **********************************************************************
	// * Constant
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.2 $";
	private static final String RCSName = "$Name:  $";

	// **********************************************************************
	// * Fields
	// **********************************************************************
	private String collection_name = null;
	private MetaDataImpl metadata = null;
	private XTree schematree = null;

	private SchemaTreeBuilder factory = null;
	private SchemaContext schemacontext = null;
	private SchemaNode parentnode = null;

	private Declaration declaration = null;

	private String namespace = null;

	private Schema schema = null;
	private String strschema = null;
	private boolean inAttribute = false;
	// ADD   27/05/2003
	private ArrayList documents = null; // list of strings representing document paths (relative or absolute)

	// **********************************************************************
	// * Constructor
	// **********************************************************************

	/**
	 * Construct the class MetaCollection
	 *
	 * @param
	 */
	public MetaCollection(MetaDataImpl metadata, String collection_name) {
		this.metadata = metadata;
		this.collection_name = collection_name;
		factory = new SchemaTreeBuilder();
		schematree = factory.createTree();
		schemacontext = new SchemaContext(metadata.getSchemaManager());
		parentnode = (SchemaNode) schematree.getRoot();
	}

	// used when cloning for importing 
	public MetaCollection(MetaDataImpl metadata, MetaCollection oldcol) {
		this.metadata = metadata;
		this.collection_name = oldcol.getCollectionName();
		factory = new SchemaTreeBuilder();
		this.schematree = oldcol.getXTree();
		schemacontext = new SchemaContext(metadata.getSchemaManager());
		parentnode = (SchemaNode) schematree.getRoot();
	}

	// **********************************************************************
	// * Methods
	// **********************************************************************
	// ONLY for QA
	public void setDocuments(ArrayList documents) {
		this.documents = documents;
	}
	public ArrayList getDocuments() {
		return documents;
	}

	/**
	 * 
	 *
	 * @param
	 *
	 * @return
	 */
	public void addMetaElement(String namespace, String name, boolean isAttribute) {
		if (namespace != null) {
			this.namespace = namespace;
		}

		SchemaNode schemanode = null;
		if (isAttribute) {
			AttributeDeclaration attDecl = schemacontext.getAttributeDeclaration(namespace, name);
			if ((attDecl != null) && (attDecl.getSchema() != null))
				schema = attDecl.getSchema();
			//			if (attDecl == null) {
			//				attDecl = new AttributeDeclaration(schema, name, null);
			//				attDecl.setType(metadata.getSchemaManager().getType("http://www.w3.org/2001/XMLSchema", "anyAttribute"));
			//			}
			if (hasChild(parentnode, name, namespace) == null) {
				if (attDecl == null)
					schemanode = (SchemaNode) factory.createNamedNode(parentnode, namespace, name, NodeKind.ATTRIBUTE);
				else
					schemanode = (SchemaNode) factory.createTypedNode(parentnode, attDecl);
			}
			inAttribute = true;
		} else {
			declaration = schemacontext.getElementDeclaration(namespace, name);

			if ((declaration != null) && (declaration.getSchema() != null))
				schema = declaration.getSchema();
			//			if (declaration == null)
			//			{
			//				declaration = new ElementDeclaration(schema, name, null);
			//				declaration.setType(metadata.getSchemaManager ().getType("http://www.w3.org/2001/XMLSchema","anyType"));				
			//			}
			if (declaration instanceof ElementDeclaration) {
				schemacontext.push((ElementDeclaration) declaration);
			}

			SchemaNode childnode = null;
			if ((childnode = (SchemaNode) hasChild(parentnode, name, namespace)) != null) {
				parentnode = childnode;
			} else {
				if (declaration == null)
					schemanode = (SchemaNode) factory.createNamedNode(parentnode, namespace, name, NodeKind.ELEMENT);
				else
					schemanode = (SchemaNode) factory.createTypedNode(parentnode, declaration);
				//				if (declaration instanceof ElementDeclaration)
				parentnode = schemanode;
			}

		}
	}

	private XTreeNode hasChild(XTreeNode parentnode, String name, String ns) {
		if (ns != null && ns.equals(""))
			ns = null;
		if (name == null || name.equals(""))
			return null;
		if (parentnode == null)
			return null;
		Collection children = parentnode.getChildren();
		if (children == null)
			return null;
		for (Iterator iterator = children.iterator(); iterator.hasNext();) {
			XTreeNode child = (XTreeNode) iterator.next();
			String childns = child.getNamespace();
			if (childns != null && childns.equals(""))
				childns = null;
			String childname = child.getLocalName();
			if (childname == null || childname.equals(""))
				return null;
			if (((ns == null && childns == null) || ((ns != null) && (childns != null) && (ns.equalsIgnoreCase(childns)))) && ((name != null) && (childname != null) && (name.equalsIgnoreCase(childname))))
				return child;
		}
		return null;
	}

	public void addEndElement() {
		if (inAttribute) {
			inAttribute = false;
			return;
		}
		if ((declaration != null) && ((declaration instanceof ElementDeclaration))) {
			declaration = schemacontext.pop();
		}

		if (parentnode != null) {
			parentnode = (SchemaNode) parentnode.getParent();
		}
	}

	/**
	 * Return the XTree associated to this metacollection (defined from
	 * abstract CollectionMetadata defined in parent class).
	 *
	 * @return XTree associated to this metacollection.
	 */
	public XTree getXTree() {
		return schematree;
	}

	/**
	 * Return the collection_name associated to this metacollection 
	 * (defined from abstract CollectionMetadata defined in parent class).
	 *
	 * @return the collection_name of the collection
	 */
	public String getCollectionName() {
		return collection_name;
	}

	public Collection getRootNodes() {
		if (schematree != null) {
			return schematree.getRoot().getChildren();
		} else
			return null;
	}

	// **********************************************************************
	// * Destructor
	// **********************************************************************

	/**
	 * Dereference allocated object which will be further be destroy
	 * by the garbage collector.
	 */
	protected void finalize() throws Throwable {}
}
