/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.metadata.resolver;

import java.util.Collection;

import org.xquark.xpath.XTree;

/**
 * Interface that provides a common access to the metadata of a collection
 * in an XML data source.
 */
public interface CollectionMetadata {

	/**
	 * Accessor to the collection name.
	 * @return String
	 */
	public String getCollectionName();

	/**
	 * Accessor to the collection main metadata as an XPath data model tree
	 * with potential XMLSchema type information.
	 * @return a memory structure which is in fact a tree which is based on
	 * the XPath data model.
	 */
	public XTree getXTree();

	/**
	 * Accessor to the collection of root element declarations
	 * @return If colllection has restricted root element declarations returns them, otherwise null
	 */
	public Collection getRootNodes();

}
