/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package  org.xquark.xquery.metadata.resolver;

import java.util.Collection;

import org.xquark.schema.SchemaManager;
import org.xquark.xquery.ModuleManager;

/**
 * Interface that provides an access to a set of sources and the attached
 * metadata.
 */
public interface MetadataAccess
{


    
    /** returns a collection of metadata objects for all reachable sources.
     * @return a Collection object containing SourceMetadata objects.
     */
    public Collection getSources();
    
    /** Provide access to a particular data source.
     * <B>Warning:</B> implementation may be case unsensitive.
     * @param sourceName the ID of the source
     * @return a SourceMetadata object
     */    
    public SourceMetadata getSourceMetadata(String sourceName);
 
 	/**
 	 * Provides access to the SchemaManager holding schemas used in source metadata
 	 * @return a SchemaManager
 	 */
 	public SchemaManager getSchemaManager();

	/**
	 * Provides access to the ModuleManager holding modules used in source metadata
	 * @return a ModuleManager
	 */
	public ModuleManager getModuleManager();
 	  
}
