/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * Copyright (C) 2003 XQuark Group.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.normalize;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Stack;

import org.xquark.xquery.parser.*;
import org.xquark.xquery.parser.util.Constants;
import org.xquark.xquery.typing.TypeVisitor;

public class CanonizeVisitor extends DefaultParserVisitor {

	private static final String RCSRevision = "$Revision: 1.4 $";
	private static final String RCSName = "$Name:  $";

	private Stack parents = null;
	private Stack FLWRParents = null;
	private XQueryExpression resultExpr = null;

	private ArrayList QDB = null;
	private XQueryExpression QMEM = null;
	private int inReturn = 0;
	private int inWhere = 0;
	private Variable inVarDef = null;

	protected TypeVisitor typeVisitor = null;

	//	public CanonizeVisitor()
	public CanonizeVisitor(TypeVisitor typeVisitor) {
		parents = new Stack();
		FLWRParents = new Stack();
		this.typeVisitor = typeVisitor;
	}

	public XQueryExpression getResult() {
		return resultExpr;
	}
	public ArrayList getQDB() {
		return QDB;
	}
	public XQueryExpression getQMEM() {
		return QMEM;
	}

	// visitors

	public void visit(Element arg) throws XQueryException {
		resultExpr = null;
		parents.push(arg);
		ArrayList attributes = arg.getAttributes();
		if (attributes != null) {
			for (int i = 0; i < attributes.size(); i++) {
				XQueryExpression expri = (XQueryExpression) attributes.get(i);
				expri.accept(this);
				while (resultExpr != null) {
					attributes.set(i, resultExpr);
					expri = (XQueryExpression) attributes.get(i);
					resultExpr = null;
					expri.accept(this);
				}
			}
		}
		ArrayList subExpressions = arg.getSubExpressions();
		if (subExpressions != null) {
			for (int i = 0; i < subExpressions.size(); i++) {
				XQueryExpression expri = (XQueryExpression) subExpressions.get(i);
				expri.accept(this);
				while (resultExpr != null) {
					subExpressions.set(i, resultExpr);
					expri = (XQueryExpression) subExpressions.get(i);
					resultExpr = null;
					expri.accept(this);
				}
			}
		}
		parents.pop();
	}

	public void visit(FLWRExpression arg) throws XQueryException {
		resultExpr = null;
		int i, j, k;
		// test if it has a parent FLWR
		if (inReturn > 0 && !FLWRParents.isEmpty()) {
			FLWRExpression parentFLWRExpr = (FLWRExpression) FLWRParents.peek();
			// look at dependencies between variables! (LATER!!!)
			// create new variable
			Variable var = typeVisitor.getStaticContext().createVariable(Constants.OUTERLET_BINDINGTYPE, arg, Variable.CAN_CREATOR);
			parentFLWRExpr.addVariable(var);
			XQueryExpression retClause = parentFLWRExpr.getReturnClause();
			if (retClause == arg)
				parentFLWRExpr.setReturnClause(var);
			else {
				// substiture FLWR by var in it's parent
				HashMap map = new HashMap();
				map.put(arg, var);
				retClause.substitute(map, typeVisitor);
			}
			var.accept(typeVisitor);
			var.accept(this);
			return;
		}

		parents.push(arg);
		FLWRParents.push(arg);

		XQueryExpression whereExpr = arg.getWhereClause();
		if (whereExpr != null) {
			inWhere++;
			whereExpr.accept(this);
			inWhere--;
			whereExpr = arg.getWhereClause();
		}

		XQueryExpression returnExpr = arg.getReturnClause();
		inReturn++;
		returnExpr.accept(this);
		inReturn--;
		returnExpr = arg.getReturnClause();

		//HintTree hinttree = arg.getHintTree();
		//ArrayList orderby = arg.getOrderBy();
		/*
		for (i=0;i<orderby.size();i++) {
		    XQueryExpression orderbyi = (XQueryExpression)orderby.get(i);
		    inReturn++;
		    orderbyi.accept(this);
		    inReturn--;
		}
		*/
		ArrayList variables = arg.getVariables();
		Variable tmpVar = null;
		int varsize = variables.size();
		for (i = 0; i < varsize; i++) {
			tmpVar = (Variable) variables.get(i);
			Variable tmpVarDef = inVarDef;
			inVarDef = tmpVar;
			tmpVar.accept(this);
			inVarDef = tmpVarDef;
		}

		ArrayList qdb = new ArrayList();
		ArrayList forvarlist = new ArrayList();
		ArrayList letvarlist = new ArrayList();
		// ArrayList usedvarlist = new ArrayList();
		// get variables used in return clause
		// put variables in different lists
		for (i = 0; i < variables.size(); i++) {
			tmpVar = (Variable) variables.get(i);
			if (tmpVar.getBindingType() == Constants.OUTERLET_BINDINGTYPE)
				letvarlist.add(tmpVar);
			else
				forvarlist.add(tmpVar);
		}

		// XQueryExpressionSequence tmpSeq = new XQueryExpressionSequence(usedvarlist,typeVisitor);
		XQueryExpressionSequence tmpSeq = new XQueryExpressionSequence(forvarlist, arg.getParentModule());
		tmpSeq.setParenthesis(true);
		// change   23/01/2003
		//FLWRExpression tmpFLWR = new FLWRExpression(forvarlist, orderby, whereExpr, tmpSeq, typeVisitor);
		FLWRExpression tmpFLWR = new FLWRExpression(forvarlist, arg.getHintTree(), arg.getOrderBy(), whereExpr, tmpSeq, arg.getParentModule());
		tmpFLWR.setParentModule(arg.getParentModule());
		tmpVar = typeVisitor.getStaticContext().createVariable(Constants.LET_BINDINGTYPE, tmpFLWR, Variable.CAN_CREATOR);
		qdb.add(tmpVar);
		XQueryExpression qmem = returnExpr;

		ArrayList allinnerqdb = new ArrayList();
		for (i = 0; i < letvarlist.size(); i++) {
			Variable letVarListi = (Variable) letvarlist.get(i);
			FLWRExpression letVarListiExpr = (FLWRExpression) letVarListi.getExpression();
			letVarListiExpr.accept(this);
			ArrayList innerqdb = QDB;
			XQueryExpression innerqmem = QMEM;
			// put depends
			ArrayList vars = letVarListiExpr.getVariables();
			if (!vars.isEmpty()) {
				ArrayList dependids = new ArrayList();
				for (j = 0; j < vars.size(); j++) {
					Variable varj = (Variable) vars.get(j);
					if (varj.getBindingType() == Constants.FOR_BINDINGTYPE)
						dependids.add(varj);
				}
				if (letVarListiExpr.getReturnClause().getDependIDs() == null && !dependids.isEmpty())
					letVarListiExpr.getReturnClause().setDependIDs(dependids);
			}

			// add for variables from qdb into innerqmem
			for (j = 0; j < innerqdb.size(); j++) {
				Variable innerQdbVari = (Variable) innerqdb.get(j);
				FLWRExpression innerQdbVariExpr = (FLWRExpression) innerQdbVari.getExpression();
				for (k = 0; k < qdb.size(); k++) {
					Variable qdbVari = (Variable) qdb.get(k);
					XQueryExpression qdbVariExpr = qdbVari.getExpression();
					tmpVar = typeVisitor.getStaticContext().createVariable(Constants.FOR_BINDINGTYPE, qdbVari, Variable.CAN_CREATOR);
					innerQdbVariExpr.insertVariable(tmpVar);
				}
			}
			// add innerqdb to allinnerqdb
			allinnerqdb.addAll(innerqdb);
			// replace in qmem variable by innerqmem
			// handle special case when entire return expression must be replaced
			if (qmem == letVarListi) {
				arg.setReturnClause(innerqmem);
				qmem = arg.getReturnClause();
			} else {
				ArrayList parentList = letVarListi.getParentExpression();
				HashMap map = new HashMap();
				map.put(letVarListi, innerqmem);
				for (j = 0; j < parentList.size(); j++) {
					XQueryExpression parenti = (XQueryExpression) parentList.get(j);
					parenti.substitute(map, typeVisitor);
				}
			}
		}

		// add allinnerqdb to qdb
		qdb.addAll(allinnerqdb);
		// put skolems
		if (letvarlist.size() > 0) {
			ArrayList tmpList = new ArrayList(forvarlist.size());
			for (i = 0; i < forvarlist.size(); i++) {
				Variable var = (Variable) forvarlist.get(i);
				// change   27/03/2002
				if (var.getBindingType() != Constants.LET_BINDINGTYPE)
					tmpList.add(var);
			}
			qmem.putSkolemIDs(tmpList);
		}

		// set return values
		QDB = qdb;
		QMEM = qmem;

		FLWRParents.pop();
		parents.pop();
	}

	public void visit(Variable arg) throws XQueryException {
		resultExpr = null;
		parents.push(arg);
		if (inReturn > 0) {
			if (inVarDef != null) {
				XQueryExpression expr = arg.getExpression();
				expr.accept(this);
			}
		}
		parents.pop();
	}

	public void visit(XQueryBinaryOperatorExpression arg) throws XQueryException {
		resultExpr = null;
		parents.push(arg);
		XQueryExpression tmpExpr = arg.getExpression1();
		tmpExpr.accept(this);
		while (resultExpr != null) {
			arg.setExpression1(resultExpr);
			tmpExpr = arg.getExpression1();
			resultExpr = null;
			tmpExpr.accept(this);
		}
		tmpExpr = arg.getExpression2();
		tmpExpr.accept(this);
		while (resultExpr != null) {
			arg.setExpression2(resultExpr);
			tmpExpr = arg.getExpression2();
			resultExpr = null;
			tmpExpr.accept(this);
		}
		parents.pop();
	}
	//	public void visit(XQueryBooleanOperatorExpression arg) throws XQueryException;

	public void visit(XQueryModule arg) throws XQueryException {
		// TODO danger expressions must be of size 1
		ArrayList expressions = arg.getExpressions();
		if (expressions != null && expressions.size() == 1)
			((XQueryExpression)expressions.get(0)).accept(this);
		else
			resultExpr = null;
	}

	public void visit(XQueryExpression arg) throws XQueryException {
		resultExpr = null;
	}

	public void visit(XQueryExpressionSequence arg) throws XQueryException {
		resultExpr = null;
		parents.push(arg);
		ArrayList subExpressions = arg.getSubExpressions();
		ArrayList newSubExpressions = new ArrayList();
		XQueryExpression rexExpr = null;
		if (subExpressions != null) {
			for (int i = 0; i < subExpressions.size(); i++) {
				XQueryExpression expri = (XQueryExpression) subExpressions.get(i);
				expri.accept(this);
				while (resultExpr != null) {
					subExpressions.set(i, resultExpr);
					expri = (XQueryExpression) subExpressions.get(i);
					resultExpr = null;
					expri.accept(this);
				}
			}
		}
		parents.pop();
	}

	public void visit(XQueryUnaryOperatorExpression arg) throws XQueryException {
		resultExpr = null;
		parents.push(arg);
		XQueryExpression expr = arg.getExpression();
		expr.accept(this);
		while (resultExpr != null) {
			arg.setExpression(resultExpr);
			expr = arg.getExpression();
			resultExpr = null;
			expr.accept(this);
		}
		parents.pop();
	}

}
