/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.normalize;

import org.xquark.xquery.parser.XQueryException;

/**
 * This class describes a normalized exception for various use in classe ExpressionNormalizeVisitor.<BR>
 * <BR>
 */
public class NormalizeException extends XQueryException {//extends java.lang.RuntimeException {

    //*************************** Constants declaration:

    /** Error code for exception that have no associated error code. */
    public static final int NO_CODE = 0;
    /** TODO */
    public static final int TODO1 = 1;
    /** TODO */
    public static final int TODO2 = 2;

    //*************************** Properties declaration:

    /** The error code of this exception. */
    private int errorCode;

    /** The underlying exception (or error) of this exception. */
    private java.lang.Throwable exception;

    //*************************** Constructor declaration:

    /**
     * Constructor with error code.
     * @param code the code featuring the kind of error (see constants).
     */
    public NormalizeException(int code) {
        this(code, "",null);
    }

    /**
     * Constructor with message.
     * @param msg a message to describe the exception.
     */
    public NormalizeException(java.lang.String msg) {
        this(NormalizeException.NO_CODE, msg, null);
    }

    /**
     * Constructor with error code and message.
     * @param code the code featuring the kind of error (see constants).
     * @param msg a message to describe the exception.
     */
    public NormalizeException(int code, java.lang.String msg) {
        this(code, msg, null);
    }

    /**
     * Constructor with message and underlying exception.
     * @param msg a message to describe the exception.
     * @param exception the underlying exception (or error).
     */
    public NormalizeException(java.lang.String msg, java.lang.Throwable exception) {
        this(NormalizeException.NO_CODE, msg, exception);
    }

    /**
     * Constructor with error code, message and underlying exception.
     * @param code the code featuring the kind of error (see constants).
     * @param msg a message to describe the exception.
     * @param exception the underlying exception (or error).
     */
    public NormalizeException(int code, java.lang.String msg, java.lang.Throwable exception) {
        super(msg);
        this.errorCode = code;
        this.exception = exception;
    }

    /**
     * Constructor with error code, message and underlying exception.
     * @param code the code featuring the kind of error (see constants).
     * @param msg a message to describe the exception.
     * @param exception the underlying exception (or error).
     */
    public NormalizeException(int code, java.lang.Throwable exception) {
        super(exception.getMessage());
        this.errorCode = code;
        this.exception = exception;
    }

    //*************************** Other methods declaration:

    /**
     * To get the code of this exception/error.
     * @return the error code.
     */
    public int getCode() {
        return this.errorCode;
    }

    /**
     * To get the exception of this exception/error.
     * @return the underlying exception.
     */
    public java.lang.Throwable getException() {
        return this.exception;
    }
}
