/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * Copyright (C) 2003 XQuark Group.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.normalize;

// import org.xquark.xml.xqueryparser.common.primitivefunctions.*;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;

import org.xquark.schema.AttributeDeclaration;
import org.xquark.schema.ElementDeclaration;
import org.xquark.schema.Type;
import org.xquark.xpath.Axis;
import org.xquark.xquery.parser.*;
import org.xquark.xquery.typing.*;

public class SubExpressionVisitor extends DefaultParserVisitor {

	private static final String RCSRevision = "$Revision: 1.2 $";
	private static final String RCSName = "$Name:  $";

	private ArrayList resultExprs = new ArrayList(1);
	private XQueryExpression subExpression = null;
	private byte axis = Axis.NONE;
	private int depth = 1;
	private boolean last = false;
	private String subNameSpace = null;
	private String subLocalName = null;
	private TypeVisitor typeVisitor = null;

	public ArrayList getResultExprs() {
		return resultExprs;
	}

	public SubExpressionVisitor(TypeVisitor typeVisitor) {
		this.typeVisitor = typeVisitor;
	}

	public void setSubExpression(byte axis) {
		setSubExpression(null, axis, 1, false);
	}
	public void setSubExpression(byte axis, int depth) {
		setSubExpression(null, axis, depth, false);
	}
	public void setSubExpression(QName qname) {
		setSubExpression(qname, Axis.NONE, 1, false);
	}
	public void setSubExpression(QName qname, boolean last) {
		setSubExpression(qname, Axis.CHILD, 1, last);
	}
	public void setSubExpression(QName qname, byte axis) {
		setSubExpression(qname, axis, 1, false);
	}
	public void setSubExpression(QName qname, byte axis, int depth, boolean last) {
		subExpression = qname;
		this.axis = axis;
		this.depth = depth;
		this.last = last;
		if (subExpression != null) {
			subNameSpace = ((QName) subExpression).getNameSpace();
			subLocalName = ((QName) subExpression).getLocalName();
		} else {
			subNameSpace = null;
			subLocalName = null;
		}
		resultExprs.clear();
	}

	/*
	public void visit(AggregateFunctionCall arg) throws XQueryException;
	 */

	public void visit(AttributeValuePair arg) throws XQueryException {
		resultExprs.clear();
		//		if (arg.getQType().getOccurence() != QType.OCC_1_1) return;
		if (depth == 0 && axis == Axis.ATTRIBUTE) {
			XQueryExpression attValPairName = arg.getAttributeName();
			if (!(attValPairName instanceof QName))
				return;
			String nameSpace = ((QName) attValPairName).getNameSpace();
			String localName = ((QName) attValPairName).getLocalName();
			if (compareQNames(nameSpace, localName, subNameSpace, subLocalName)) {
				XQueryExpression attValPairValue = arg.getAttributeValue();
				if (attValPairValue instanceof XQueryExpressionSequence)
					resultExprs.add((XQueryExpression) ((XQueryExpressionSequence) attValPairValue).getSubExpressions().get(0));
				else
					resultExprs.add(attValPairValue);
			}
		}
	}

	/*
	public void visit(BinOpANDExpression arg) throws XQueryException;
	public void visit(BinOpORExpression arg) throws XQueryException;
	public void visit(CastTreatExpression arg) throws XQueryException;
	public void visit(ContextDeclaration arg) throws XQueryException;
	public void visit(Dereference arg) throws XQueryException;
	 */

	public void visit(Element arg) throws XQueryException {
		resultExprs.clear();
		//		if (arg.getQType().getOccurence() != QType.OCC_1_1) return;
		ArrayList subExpressions = arg.getSubExpressions();
		ArrayList attributes = arg.getAttributes();
		QType qtype = arg.getQType();
		Type type = qtype.getType();
		if (subExpression == null) {
			if (subExpressions != null && subExpressions.size() == 1) {
				XQueryExpression tmpExpr = (XQueryExpression) subExpressions.get(0);
				// verify that QType is QTypeElement with simple type
				switch (axis) {
					case Axis.SELF :
						if (type != null && type.getValueType() != null) {
							if (tmpExpr instanceof XQueryExpressionSequence)
								subExpressions = ((XQueryExpressionSequence) tmpExpr).getSubExpressions();
							resultExprs.add(subExpressions.get(0));
						}
						break;
				}
			}
			return;
		}
		XQueryExpression startTag = arg.getStartTag();
		if (!(startTag instanceof QName)) {
			return;
		}
		if (depth == 0) {
			String nameSpace = null;
			String localName = null;
			switch (axis) {
				case Axis.NONE :
				case Axis.CHILD :
					QName startTagName = (QName) startTag;
					nameSpace = startTagName.getNameSpace();
					localName = startTagName.getLocalName();
					if (compareQNames(nameSpace, localName, subNameSpace, subLocalName)) {
						if (last) {
							if (qtype.getType().isSimpleType()) {
								if (subExpressions != null && subExpressions.size() == 1) {
									XQueryExpression tmpExpr = (XQueryExpression) subExpressions.get(0);
									if (tmpExpr instanceof XQueryExpressionSequence)
										subExpressions = ((XQueryExpressionSequence) tmpExpr).getSubExpressions();
									resultExprs.add(subExpressions.get(0));
								}
							}
						} else {
							if (subExpressions != null)
								for (int i = 0; i < subExpressions.size(); i++)
									resultExprs.add(subExpressions.get(i));
							if (attributes != null)
								for (int i = 0; i < attributes.size(); i++)
									resultExprs.add(attributes.get(i));
						}
						return;
					}
					break;
					/*
					case Constants.ATTRIBUTE :
					    for (int i=0;i<attributes.size();i++) {
					        AttributeValuePair attValPair = (AttributeValuePair)attributes.get(i);
					        XQueryExpression attValPairName = attValPair.getAttributeName();
					        if (!(attValPairName instanceof QName)) continue;
					        nameSpace = ((QName)attValPairName).getNameSpace();
					        localName = ((QName)attValPairName).getLocalName();
					        if (compareQNames(nameSpace,localName,subNameSpace,subLocalName)) {
					            XQueryExpression attValPairValue = attValPair.getAttributeValue();
					            if (attValPairValue instanceof XQueryExpressionSequence) resultExprs.add((XQueryExpression)((XQueryExpressionSequence)attValPairValue).getSubExpressions().get(0));
					            else resultExprs.add(attValPairValue);
					            break;
					        }
					    }
					 */
			}
		} else {
			if (depth == 1 && axis == Axis.ATTRIBUTE) {
				if (attributes != null) {
					for (int i = 0; i < attributes.size(); i++) {
						AttributeValuePair attValPair = (AttributeValuePair) attributes.get(i);
						XQueryExpression attValPairName = attValPair.getAttributeName();
						if (!(attValPairName instanceof QName))
							continue;
						String nameSpace = ((QName) attValPairName).getNameSpace();
						String localName = ((QName) attValPairName).getLocalName();
						if (compareQNames(nameSpace, localName, subNameSpace, subLocalName)) {
							XQueryExpression attValPairValue = attValPair.getAttributeValue();
							if (attValPairValue instanceof XQueryExpressionSequence)
								resultExprs.add((XQueryExpression) ((XQueryExpressionSequence) attValPairValue).getSubExpressions().get(0));
							else
								resultExprs.add(attValPairValue);
							break;
						}
					}
				}
			} else {
				if (depth == -1) {
					Set tmpResults = new HashSet();
					if (attributes != null) {
						for (int i = 0; i < attributes.size(); i++) {
							AttributeValuePair attValPair = (AttributeValuePair) attributes.get(i);
							depth = 0;
							attValPair.accept(this);
							if (resultExprs != null)
								tmpResults.addAll(resultExprs);
							depth = -1;
						}
					}
					if (subExpressions != null) {
						for (int i = 0; i < subExpressions.size(); i++) {
							XQueryExpression subExpr = (XQueryExpression) subExpressions.get(i);
							depth = 0;
							subExpr.accept(this);
							if (resultExprs != null)
								tmpResults.addAll(resultExprs);
							depth = -1;
							subExpr.accept(this);
							if (resultExprs != null)
								tmpResults.addAll(resultExprs);
						}
					}
					resultExprs = new ArrayList(tmpResults);
				} else {
					if (subExpressions != null) {
						ArrayList tmpResults = new ArrayList();
						for (int i = 0; i < subExpressions.size(); i++) {
							XQueryExpression subExpr = (XQueryExpression) subExpressions.get(i);

							depth--;
							resultExprs.clear();
							subExpr.accept(this);
							if (resultExprs != null)
								tmpResults.addAll(resultExprs);
							depth++;
						}
						resultExprs = new ArrayList(tmpResults);
					}
				}
			}
		}

	}

	// utility
	boolean compareQNames(String nameSpace1, String localName1, String nameSpace2, String localName2) {
		if ((((nameSpace1 == null || nameSpace1.equals("")) && (nameSpace2 == null || nameSpace2.equals(""))) || (nameSpace1 != null && nameSpace1.equals("*")) || (nameSpace2 != null) && (nameSpace2.equals("*") || nameSpace2.equals(nameSpace1))) && localName1 != null && localName2 != null && localName1.equals(localName2))
			return true;
		return false;
	}

	/*	
	public void visit(FLWRExpression arg) throws XQueryException
	public void visit(FunctionCall arg) throws XQueryException;
	public void visit(FunctionDefinition arg) throws XQueryException;
	public void visit(InstanceOfExpression arg) throws XQueryException;
	public void visit(InternalFunctionCall arg) throws XQueryException;
	public void visit(ITEExpression arg) throws XQueryException;
	public void visit(LibraryFunctionCall arg) throws XQueryException;
	public void visit(ListOpAFTERExpression arg) throws XQueryException;
	public void visit(ListOpArithExpression arg) throws XQueryException;
	public void visit(ListOpBEFOREExpression arg) throws XQueryException;
	public void visit(ListOpCompExpression arg) throws XQueryException;
	public void visit(ListOpEXCEPTExpression arg) throws XQueryException;
	public void visit(ListOpINTERSECTExpression arg) throws XQueryException;
	public void visit(ListOpUNIONExpression arg) throws XQueryException;
	 */

	public void visit(LocatedExpression arg) throws XQueryException {
		if (depth != -1)
			resultExprs.clear();
		//		if (arg.getQType().getOccurence() != QType.OCC_1_1) return;
		// get elements to work with and verify some stuff
		QType qtype = arg.getQType();
		if (!(qtype instanceof QTypeElement)) {
			return;
		}
		QTypeElement qtypeelt = (QTypeElement) qtype;
		XQueryExpression name = qtypeelt.getName();
		if (!(name instanceof QName)) {
			return;
		}
		QName qname = (QName) name;
		Type type = qtypeelt.getType();
		// special case .
		if (subExpression == null) {
			if (axis == Axis.SELF) {
				if (!type.isSimpleType()) {
					return;
				}
				resultExprs.add(arg);
			}
			return;
		}
		if (depth == 0) {
			if (!type.isSimpleType() && !type.getName().equals(Type.ANY_TYPE)) {
				return;
			}
			if (!compareQNames(qname.getNameSpace(), qname.getLocalName(), subNameSpace, subLocalName)) {
				return;
			}
			resultExprs.add(arg);
		} else if (depth == 1) {
			if (type.isSimpleType()) {
				return;
			}
			if (axis == Axis.ATTRIBUTE) {
				AttributeDeclaration attDecl = type.getAttributeDeclaration(subNameSpace, subLocalName);
				if (attDecl != null) {
					ArrayList steps = (ArrayList) arg.getSteps().clone();
					steps.add(new Step(true, Axis.ATTRIBUTE, subExpression, null, arg.getParentModule()));
					resultExprs.add(new LocatedExpression(steps, false, arg.getParentModule()));
					return;
				}
			} else {
				ElementDeclaration eltDecl = type.getElementDeclaration(subNameSpace, subLocalName);
				if (eltDecl != null) {
					ArrayList steps = (ArrayList) arg.getSteps().clone();
					steps.add(new Step(true, Axis.CHILD, subExpression, null, arg.getParentModule()));
					resultExprs.add(new LocatedExpression(steps, false, arg.getParentModule()));
					return;
				}
			}

		} else if (depth == -1) {
		}
	}

	//    public void visit(NodeType arg) throws XQueryException;
	//    public void visit(Parameter arg) throws XQueryException;
	//    public void visit(PrimitiveFunctionCall arg) throws XQueryException;
	//    public void visit(QName arg) throws XQueryException;
	//    public void visit(QuantifiedExpression arg) throws XQueryException;
	//    public void visit(RangeExpression arg) throws XQueryException;
	//    public void visit(RootNodeFunctionCall arg) throws XQueryException;
	//    public void visit(SortedExpression arg) throws XQueryException;
	//    public void visit(Step arg) throws XQueryException;
	//    public void visit(TypeSwitchExpression arg) throws XQueryException;
	//    public void visit(UnOpMinusExpression arg) throws XQueryException;
	//    public void visit(Value arg) throws XQueryException;
	//    public void visit(ValueBoolean arg) throws XQueryException;
	//    public void visit(ValueFloat arg) throws XQueryException;
	//    public void visit(ValueInteger arg) throws XQueryException;
	//    public void visit(ValueString arg) throws XQueryException;
	//    public void visit(ValueText arg) throws XQueryException;

	public void visit(Variable arg) throws XQueryException {
		resultExprs.clear();
		//		if (arg.getQType().getOccurence() != QType.OCC_1_1) return;
		QType qtype = arg.getQType();
		Type type = qtype.getType();
		// special case .
		if (subExpression == null) {
			if (axis == Axis.SELF) {
				if (type != null && !type.isSimpleType()) {
					return;
				}
				resultExprs.add(arg);
			}
			return;
		}
		if (!(qtype instanceof QTypeElement || (qtype instanceof QTypeUnion && qtype.isElement()))) {
			return;
		}
		if (qtype instanceof QTypeElement) {
			QTypeElement qtypeelt = (QTypeElement) qtype;
			XQueryExpression name = qtypeelt.getName();
			if (!(name instanceof QName)) {
				return;
			}
			QName qname = (QName) name;
			if (depth == 0) {
				if (/* qtype != null && qtype instanceof QTypeElement && qtype.getName() instanceof QName && */
					subExpression instanceof QName) {
					String nameSpace = ((QName) qtype.getName()).getNameSpace();
					String localName = ((QName) qtype.getName()).getLocalName();
					if (compareQNames(nameSpace, localName, subNameSpace, subLocalName)) {
						resultExprs.add(arg);
						return;
					}
				}
			}
			//else if (depth == 1) {
			// this has been done for cases where variable contains several steps of the sort expression ....
			if (depth != -1) {
				if (subExpression instanceof QName) {
					QType qt = Utils.getSubType(QTypeDocument.ANY_ORIGIN, qtype, new Step(false, axis, subExpression, null, null), this.typeVisitor.getSchemaManager(), false);
					if (qt != null) {
						ArrayList steps = new ArrayList();
						steps.add(new Step(false, Axis.NONE, arg, null, arg.getParentModule()));
						steps.add(new Step(true, axis, subExpression, null, arg.getParentModule()));
						resultExprs.add(new LocatedExpression(steps, false, arg.getParentModule()));
						return;
					}
				}
			}
			/*
			else if (depth == -1) {
			}
			*/
		} else {
			QTypeUnion qtypeunion = (QTypeUnion) qtype;
			boolean add1 = true;
			boolean add2 = true;
			for (int i = 0; i < qtypeunion.getList().size() && (add1 || add2); i++) {
				QTypeElement qtypeelt = (QTypeElement) qtypeunion.getList().get(i);
				XQueryExpression name = qtypeelt.getName();
				if (!(name instanceof QName)) {
					add1 = false;
					add2 = false;
					break;
				}
				QName qname = (QName) name;
				if (add1 && depth == 0) {
					if (/* qtype != null && qtype instanceof QTypeElement && qtype.getName() instanceof QName && */
						subExpression instanceof QName) {
						String nameSpace = ((QName) qtypeelt.getName()).getNameSpace();
						String localName = ((QName) qtypeelt.getName()).getLocalName();
						if (!compareQNames(nameSpace, localName, subNameSpace, subLocalName))
							add1 = false;
					}
				}
				//else if (depth == 1) {
				// this has been done for cases where variable contains several steps of the sort expression ....
				if (!add1 && add2 && depth != -1) {
					if (subExpression instanceof QName) {
						Object obj = Utils.getSubType(QTypeDocument.ANY_ORIGIN, qtypeelt, new Step(false, axis, subExpression, null, null), this.typeVisitor.getSchemaManager(), false);
						if (obj == null)
							add2 = false;
					}
				}
				/*
				else if (depth == -1) {
				}
				*/
			}
			if (add1 || add2) {
				if (add1 && depth == 0) {
					resultExprs.add(arg);
					return;
				}
				//else if (depth == 1) {
				// this has benn done for cases wehere variable contains several steps of the sort expression ....
				if (add2 && depth != -1) {
					ArrayList steps = new ArrayList();
					steps.add(new Step(false, Axis.NONE, arg, null, arg.getParentModule()));
					steps.add(new Step(true, axis, subExpression, null, arg.getParentModule()));
					resultExprs.add(new LocatedExpression(steps, false, arg.getParentModule()));
					return;
				}
			}
		}
	}

	//    public void visit(XQueryBinaryOperatorExpression arg) throws XQueryException;
	//    public void visit(XQueryBooleanOperatorExpression arg) throws XQueryException;

	public void visit(XQueryExpression arg) throws XQueryException {
		resultExprs.clear();
	}

	public void visit(XQueryExpressionSequence arg) throws XQueryException {
		resultExprs.clear();
		//		if (arg.getQType().getOccurence() != QType.OCC_1_1) return;
		if (subExpression == null) {
			return;
		}
		ArrayList subExpressions = arg.getSubExpressions();
		for (int i = 0; i < subExpressions.size(); i++) {
			resultExprs.clear();
			XQueryExpression subExpr = (XQueryExpression) subExpressions.get(i);
			subExpr.accept(this);
			// if (depth != -1 && !resultExprs.isEmpty()) break;
		}
	}
	/*
	public void visit(XQueryFile arg) throws XQueryException;
	public void visit(XQueryListBinaryOperatorExpression arg) throws XQueryException;
	public void visit(XQueryListUnaryOperatorExpression arg) throws XQueryException;
	public void visit(XQueryUnaryOperatorExpression arg) throws XQueryException {
	public void visit(XQueryUnit arg) throws XQueryException;
	public void visit(XQueryVoid arg) throws XQueryException;
	 */
	// primitive functions
	/*
	public void visit(FunctionATTRIBUTE arg) throws XQueryException ;
	public void visit(FunctionAVG arg) throws XQueryException ;
	public void visit(FunctionCOLLECTION arg) throws XQueryException;
	public void visit(FunctionCONCAT arg) throws XQueryException;
	public void visit(FunctionCONTAINS arg) throws XQueryException;
	public void visit(FunctionCOUNT arg) throws XQueryException;
	public void visit(FunctionCURRENT_DATETIME arg) throws XQueryException;
	public void visit(FunctionDATA arg) throws XQueryException;
	public void visit(FunctionDATE arg) throws XQueryException;
	public void visit(FunctionDATETIME arg) throws XQueryException;
	public void visit(FunctionDISTINCT_VALUES arg) throws XQueryException;
	public void visit(FunctionDOCUMENT arg) throws XQueryException;
	public void visit(FunctionEMPTY arg) throws XQueryException;
	public void visit(FunctionENDS_WITH arg) throws XQueryException;
	public void visit(FunctionFILTER arg) throws XQueryException;
	public void visit(FunctionGET_GMONTH_FROM_DATE arg) throws XQueryException;
	public void visit(FunctionGET_GYEAR_FROM_DATE arg) throws XQueryException;
	public void visit(FunctionLOWER_CASE arg) throws XQueryException;
	public void visit(FunctionMATCH arg) throws XQueryException;
	public void visit(FunctionMAX arg) throws XQueryException;
	public void visit(FunctionMIN arg) throws XQueryException;
	public void visit(FunctionNAME arg) throws XQueryException;
	public void visit(FunctionNOT arg) throws XQueryException;
	public void visit(FunctionNUMBER arg) throws XQueryException;
	public void visit(FunctionPART arg) throws XQueryException;
	public void visit(FunctionPOSITION arg) throws XQueryException;
	public void visit(FunctionSHALLOW arg) throws XQueryException;
	public void visit(FunctionSTARTS_WITH arg) throws XQueryException;
	public void visit(FunctionSTRING arg) throws XQueryException;
	public void visit(FunctionSTRING_LENGTH arg) throws XQueryException;
	public void visit(FunctionSTRING_VALUE arg) throws XQueryException;
	public void visit(FunctionSUBSTRING arg) throws XQueryException;
	public void visit(FunctionSUM arg) throws XQueryException;
	// public void visit(FunctionTEXT arg) throws XQueryException;
	public void visit(FunctionTIME arg) throws XQueryException;
	public void visit(FunctionUPPER_CASE arg) throws XQueryException;
	 */
}
