/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.normalize;

import java.util.ArrayList;
import java.util.HashMap;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.xquark.xquery.parser.*;
import org.xquark.xquery.parser.util.Constants;

public class TransformXQueryExpression {

	private static final String RCSRevision = "$Revision: 1.3 $";
	private static final String RCSName = "$Name:  $";

	private static Log log = LogFactory.getLog("internal." + TransformXQueryExpression.class.getName());

	protected ArrayList QDB = null;
	protected XQueryExpression QMEM = null;
	protected ArrayList dependMaps = null;
	protected ArrayList varList = null;
	protected ArrayList returnVarList = null;
	protected ArrayList idVarList = null;

	//    public TransformXQueryExpression(ArrayList QDB, XQueryExpression QMEM, ArrayList dependMaps, ArrayList varList,  ArrayList returnVarList,  ArrayList idVarList) {
	//        this.QDB = QDB;
	//        this.QMEM = QMEM;
	//        this.dependMaps = dependMaps;
	//        this.varList = varList;
	//        this.returnVarList = returnVarList;
	//        this.idVarList = idVarList;
	//    }

	public TransformXQueryExpression(ArrayList QDB, XQueryExpression QMEM) throws ParseException {
		this.QDB = QDB;
		this.QMEM = QMEM;
		varList = new ArrayList();
		ArrayList keyVars = new ArrayList();
		dependMaps = new ArrayList();
		XQueryExpression remainingWhereExpr = null;
		try {
			if (log.isDebugEnabled())
				log.debug("START SPECIAL TREATMENT ");
			GetBoundVariablesVisitor gbvv = new GetBoundVariablesVisitor(true, true);
			QMEM.accept(gbvv);
			// list of all variables appearing in QMEM (eather idvariables or reconstruction variables)
			ArrayList boundVariables = gbvv.getVariables();
			// list of all reconstruction variables appearing in QMEM
			returnVarList = gbvv.getOnlyVariables();
			// list of all id variables appearing in QMEM
			idVarList = gbvv.getIDsVariables();

			/*
			ALGORITHM
			Iterating on QDBs from bottom to top
			1. elimination of for $x in 1 (added by restructuration or canonization) whenever possible
				conditions :
					must be unique variable
					no subpaths exist
				if let binding and more than one QDB
					do erase
					!!! DANGER !!!
					if first QDB erased and remaining where expression is empty
						put root label on QMEM for reconstruction
			2. replacing QDBs in FLWRs
			example
			let $1 := for $x1 in COLX1, $x2 in COLX2 where C1 order by x3, x4 return $x1, $x2
			let $2 := for $X in $1, $y1 in COLY1, $y2 in COLY2 where C2 order by y3, y4 return $X, $y1, $y2
			becomes
			let $1 := for $x in COL1 return $x
			let $2 := for $x1 in COLX1, $x2 in COLX2, $y1 in COLY1, $y2 in COLY2 where C1 & C2 orderby x3, x4, y4 return $x1, $x2, $y1, $y2 			 
			*/
			if (QDB != null) {
				for (int l = QDB.size() - 1; l >= 0; l--) {
					Variable qdbVari = (Variable) QDB.get(l);
					FLWRExpression qdbFlwri = (FLWRExpression) qdbVari.getExpression();
					ArrayList qbdFlwriVars = qdbFlwri.getVariables();
					ArrayList qbdFlwriOrders = qdbFlwri.getOrderBy();

					// some special treatment to avoid having for $x in 1 !!!! ATTENTION !!!
					if (qbdFlwriVars.size() == 1) {
						Variable qbdFlwriVar0 = (Variable) qbdFlwriVars.get(0);
						ArrayList paths = qbdFlwriVar0.getSubPathList();
						if (paths == null || paths.isEmpty()) {
							if (qbdFlwriVar0.getBindingType() == Constants.LET_BINDINGTYPE && QDB.size() > 1) {
								//erasing from QDB
								QDB.remove(l);
								l--;
								// remember where expression if it is the last QBD (it is a silly expression anyway)
								if (QDB.isEmpty())
									remainingWhereExpr = qdbFlwri.getWhereClause();
								// erasing from idvariables
								idVarList.remove(qbdFlwriVar0);
								// erasing from skolemIDs
								EraseSkolemIDVisitor eraseSkolemIDVisitor = new EraseSkolemIDVisitor(qbdFlwriVar0);
								QMEM.accept(eraseSkolemIDVisitor);
								if (l < 0 && remainingWhereExpr == null)
									QMEM.setRoot(true);
								continue;
							} else if (!returnVarList.contains(qbdFlwriVar0) && qbdFlwriVar0.getExpression().getQType().isAtom() && qdbFlwri.getWhereClause() == null) {
								// erasing from other variables
								for (int m = QDB.size() - 1; m > l; m--) {
									ArrayList tmpList = ((FLWRExpression) ((Variable) QDB.get(m)).getExpression()).getVariables();
									tmpList.remove(qbdFlwriVar0);
								}
								//erasing from QDB
								QDB.remove(l);
								l--;
								// erasing from idvariables
								idVarList.remove(qbdFlwriVar0);
								// erasing from skolemIDs
								EraseSkolemIDVisitor eraseSkolemIDVisitor = new EraseSkolemIDVisitor(qbdFlwriVar0);
								QMEM.accept(eraseSkolemIDVisitor);
								if (l < 0)
									QMEM.setRoot(true);
								continue;
							}
						}
					}

					for (int m = qbdFlwriVars.size() - 1; m >= 0; m--) {
						Variable qbdFlwriVari = (Variable) qbdFlwriVars.get(m);
						XQueryExpression qbdFlwriVariExpr = qbdFlwriVari.getExpression();
						if (QDB.contains(qbdFlwriVariExpr)) {
							FLWRExpression tmpFlwr = (FLWRExpression) ((Variable) qbdFlwriVariExpr).getExpression();
							ArrayList tmpList = tmpFlwr.getVariables();
							qbdFlwriVars.remove(m);
							int add = 0;
							for (int n = 0; n < tmpList.size(); n++) {
								Variable tmpVar = (Variable) tmpList.get(n);
								if (!qbdFlwriVars.contains(tmpVar)) {
									qdbFlwri.insertVariableAtPosition(tmpVar, m + add);
									qbdFlwriVars = qdbFlwri.getVariables();
									add++;
								}
							}
							tmpList = tmpFlwr.getOrderBy();
							if (tmpList != null)
								for (int n = 0; n < tmpList.size(); n++) {
									XQueryExpression tmpExpr = (XQueryExpression) tmpList.get(n);
									if (qbdFlwriOrders == null || !qbdFlwriOrders.contains(tmpExpr)) {
										qdbFlwri.addOrderBy(n, tmpExpr);
										qbdFlwriOrders = qdbFlwri.getOrderBy();
									}
								}
							m += add;
							if (add != 0) {
								qdbFlwri.addWhereClause(tmpFlwr.getWhereClause());
							}
						}
					}
				}
			}
			// creating variable dependencies
			// ATTENTION : this could have been done much easier (and faster!) in between the previous algorithms
			// TODO
			/*
			temporary HashMap
			QBD -> dependNode
				
			ALGORITHM
			Iterating on QDBs from bottom to top
				Iterating on QDBs from just above to top
				if all variables of above QDB are contained in QDB
					there is a parent relationship
					if son node does not exist create it with no parent
					if parent node does exist add the son node
					else create the parent node, add the son node
					add all parent variables to keyvars list				
			*/
			if (QDB != null && !QDB.isEmpty()) {
				// make dependency map
				HashMap tmpHash = new HashMap(QDB.size());
				for (int l = QDB.size() - 1; l >= 0; l--) {
					Variable qdbVarl = (Variable) QDB.get(l);
					FLWRExpression qdbFlwrl = (FLWRExpression) qdbVarl.getExpression();
					ArrayList qbdFlwrlVars = qdbFlwrl.getVariables();

					boolean foundParent = false;
					for (int m = l - 1; m >= 0; m--) {
						Variable qdbVarm = (Variable) QDB.get(m);
						FLWRExpression qdbFlwrm = (FLWRExpression) qdbVarm.getExpression();
						ArrayList qbdFlwrmVars = qdbFlwrm.getVariables();

						if (qbdFlwrlVars.containsAll(qbdFlwrmVars)) {
							foundParent = true;
							DependNode parentNode = (DependNode) tmpHash.get(qdbFlwrm);
							DependNode sonNode = (DependNode) tmpHash.get(qdbFlwrl);
							if (sonNode == null) {
								sonNode = new DependNode((DependNode) null, null, qdbFlwrl);
								tmpHash.put(qdbFlwrl, sonNode);
							}
							if (parentNode != null)
								parentNode.addSon(0, sonNode);
							else {
								parentNode = new DependNode((DependNode) null, null, qdbFlwrm);
								parentNode.addSon(0, sonNode);
								tmpHash.put(qdbFlwrm, parentNode);
							}
							// add parent Node variables to keyvars
							for (int n = qbdFlwrmVars.size() - 1; n >= 0; n--) {
								Variable parVarn = (Variable) qbdFlwrmVars.get(n);
								if (!keyVars.contains(parVarn))
									keyVars.add(0, parVarn);
							}
							break;
						}
					}
					if (!foundParent) {
						if (tmpHash.get(qdbFlwrl) != null)
							dependMaps.add(0, tmpHash.get(qdbFlwrl));
						else
							dependMaps.add(0, new DependNode((DependNode) null, null, qdbFlwrl));
					}
				}

				// TODO check if this code is useful?!!!
				/*
				adds variables to allvars
				adds variables to varlist (only for binding?)
				*/
				ArrayList allVars = new ArrayList();
				for (int l = 0; l < QDB.size(); l++) {
					FLWRExpression tmpExpr = (FLWRExpression) ((Variable) QDB.get(l)).getExpression();
					ArrayList tmpList = tmpExpr.getVariables();
					for (int m = 0; m < tmpList.size(); m++) {
						Variable tmpVar = (Variable) tmpList.get(m);
						if (!(allVars.contains(tmpVar)))
							allVars.add(tmpVar);
						//else keyVars.add(tmpVar);
						if (tmpVar.getBindingType() == Constants.FOR_BINDINGTYPE)
							if (!(varList.contains(tmpVar)))
								varList.add(tmpVar);
					}
				}

				// TODO check if this code is useful?!!!
				/*
				erases variables in return clause that are not bound or keys
				if no variables left keep the first
				*/
				for (int l = 0; l < QDB.size(); l++) {
					Variable vari = (Variable) QDB.get(l);
					FLWRExpression tmpExpr = (FLWRExpression) vari.getExpression();
					XQueryExpressionSequence tmpSeq = (XQueryExpressionSequence) tmpExpr.getReturnClause();
					ArrayList subExprs = (ArrayList) tmpSeq.getSubExpressions().clone();
					for (int m = 0; m < subExprs.size(); m++) {
						Variable tmpVar = (Variable) subExprs.get(m);
						if (!(boundVariables.contains(tmpVar)) && !keyVars.contains(tmpVar)) {
							subExprs.remove(m);
							m--;
						}
					}
					if (subExprs.isEmpty()) {
						subExprs.add(tmpSeq.getSubExpressions().get(0));
						tmpSeq.setSubExpressions(subExprs);
					} else
						tmpSeq.setSubExpressions(subExprs);
				}

				SetVariablePathsVisitor varPathVisitor = new SetVariablePathsVisitor();
				for (int l = 0; l < QDB.size(); l++) {
					((Variable) QDB.get(l)).getExpression().accept(varPathVisitor);
				}
				QMEM.accept(varPathVisitor);

			}
			// special verification for extractor : verify that all variables in varlist have at least one path associated
			// if not add the variable itself (but only for variable bound on Values)
			for (int l = 0; l < varList.size(); l++) {
				Variable var = (Variable) varList.get(l);
				ArrayList pathslist = var.getSubPathList();
				if (pathslist == null || pathslist.isEmpty())
					var.addSubPath(var);
			}

			if (log.isDebugEnabled())
				log.debug("STOP SPECIAL TREATMENT ");
		} catch (XQueryException e) {
			throw new ParseException("Invalid expression : " + this +" -> " + e.getMessage());
		}
		/***********************************/

		if (log.isTraceEnabled()) {
			log.trace("dependMaps = " + dependMaps);
			log.trace("varList = " + varList);
			//log.trace("keyVars = " + keyVars);
			log.trace("returnVariables = " + returnVarList);
			log.trace("idVariables = " + idVarList);
			if (varList != null)
				for (int l = 0; l < varList.size(); l++) {
					ArrayList trees = ((Variable) varList.get(l)).getXTrees();
					log.trace(((Variable) varList.get(l)) + " --> trees --> " + trees);
					ArrayList subPathList = ((Variable) varList.get(l)).getSubPathList();
					log.trace(((Variable) varList.get(l)) + " --> subPathList --> " + subPathList);
					if (subPathList != null)
						for (int m = 0; m < subPathList.size(); m++) {
							log.trace(((XQueryExpression) subPathList.get(m)) + " --> xtrees--> " + ((XQueryExpression) subPathList.get(m)).getXTrees());
							log.trace(((XQueryExpression) subPathList.get(m)) + " --> loopIDs --> " + ((XQueryExpression) subPathList.get(m)).getLoopIDs());
						}
				}
			for (int si = 0; si < QDB.size(); si++) {
				log.trace("QDB[" + si + "]: " + ((Variable) QDB.get(si)).getExpression());
			}
			log.trace("QMEM: " + QMEM);
		}
	}

	public ArrayList getQDB() {
		return QDB;
	}

	public XQueryExpression getQMEM() {
		return QMEM;
	}

	public ArrayList getDependMap() {
		return dependMaps;
	}

	public ArrayList getVarList() {
		return varList;
	}

	public ArrayList getReturnVarList() {
		return returnVarList;
	}

	public ArrayList getIdVarList() {
		return idVarList;
	}

	// For debugging
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append("\n%%% QDB: " + QDB + "\n");
		sb.append("%%% QMEM: " + QMEM + "\n");
		sb.append("%%% dependMap: " + dependMaps + "\n");
		sb.append("%%% varList: " + varList + "\n");
		sb.append("%%% returnVarList: " + returnVarList + "\n");
		sb.append("%%% idVarList: " + idVarList + "\n");
		return sb.toString();
	}

}
