/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.parser;

import org.xquark.xquery.typing.TypeException;


public class AttributeValuePair extends XQueryBinaryOperatorExpression implements Cloneable {

	private static final String RCSRevision = "$Revision: 1.9 $";
	private static final String RCSName = "$Name:  $";

	// false by default
	protected boolean computed = false;
	protected boolean xmlns = false;

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(ParserVisitor visitor) throws XQueryException {
		visitor.visit(this);
	}

	// #############################################################################
	// CONSTRUCTOR STUFF
	// #############################################################################

	public AttributeValuePair(XQueryExpression attributeName, XQueryExpression attributeValue, boolean isXmlns, XQueryModule parentModule) throws TypeException, XQueryException {
		super(attributeName, attributeValue);
		setIsComputed(false);
		setIsXmlns(isXmlns);
		setParentModule(parentModule);
		if (parentModule != null && parentModule.getStaticContext().getTypeVisitor() != null)
			accept(parentModule.getStaticContext().getTypeVisitor());
	}

	// #############################################################################
	// ATTRIBUTE ACCESS STUFF
	// #############################################################################

	// get
	public XQueryExpression getAttributeName() {
		return super.getExpression1();
	}

	public XQueryExpression getAttributeValue() {
		return super.getExpression2();
	}

	public boolean isComputed() {
		return computed;
	}

	public boolean isXmlns() {
		return xmlns;
	}

	// set
	public void setAttibuteName(XQueryExpression attributeName) throws XQueryException {
		super.setExpression1(attributeName);
	}

	public void setAttributeValue(XQueryExpression attributeValue) throws XQueryException {
		super.setExpression2(attributeValue);
	}

	public void setIsComputed(boolean computed) {
		this.computed = computed;
	}

	public void setIsXmlns(boolean xmlns) {
		this.xmlns = xmlns;
	}

}
