/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

/*
 * CaseClauseExpression.java
 *
 * Created on 26 septembre 2002, 14:10
 */

package org.xquark.xquery.parser;

import org.xquark.xquery.typing.TypeException;


public class CaseClauseExpression extends XQueryExpression implements Cloneable {
	
	private static final String RCSRevision = "$Revision: 1.9 $";
	private static final String RCSName     = "$Name:  $";
	
	protected SequenceType sequenceType = null;
	protected Variable variable = null;
	// cannot be null
	protected XQueryExpression returnExpression = null;
	
	// #############################################################################
	// VISITOR STUFF
	// #############################################################################
	
	public void accept(ParserVisitor visitor) throws XQueryException {
		visitor.visit(this);
	}
	
	// #############################################################################
	// CONSTRUCTOR STUFF
	// #############################################################################
	
	public CaseClauseExpression(SequenceType sequenceType, Variable variable, XQueryExpression returnExpression, XQueryModule parentModule) throws TypeException, XQueryException {
		setSequenceType(sequenceType);
		setVariable(variable);
		setReturnExpression(returnExpression);
		setParentModule(parentModule);
		if (parentModule != null && parentModule.getStaticContext().getTypeVisitor() != null)
			accept(parentModule.getStaticContext().getTypeVisitor());
	}
	
	// #############################################################################
	// ATTRIBUTE ACCESS STUFF
	// #############################################################################
	
	public void setParentModule(XQueryModule parentUnit) {
		if (parentUnit == null) return;
		this.parentModule = parentUnit;
		super.setParentModule(parentUnit);
		sequenceType.setParentModule(parentUnit);
		if (variable != null) variable.setParentModule(parentUnit);
		returnExpression.setParentModule(parentUnit);
	}
	
	public void setSequenceType(SequenceType sequenceType) throws XQueryException {
		this.sequenceType = sequenceType;
		if (sequenceType != null) {
			this.sequenceType.setParentModule(parentModule);
			this.sequenceType.setParentExpression(this);
		}
	}
	
	public SequenceType getSequenceType() { return sequenceType; }
	
	public void setVariable(Variable variable) {
		this.variable = variable;
		if (variable != null) {
			this.variable.setParentModule(parentModule);
			this.variable.setParentExpression(this);
		}
	}
	
	public Variable getVariable() { return variable; }
	
	public void setReturnExpression(XQueryExpression returnExpression) throws XQueryException {
		// cannot be null
		if (returnExpression == null) throw new XQueryException("ReturnExpression of a caseClause cannot be null.");
		this.returnExpression = returnExpression;
		this.returnExpression.setParentModule(parentModule);
		this.returnExpression.setParentExpression(this);
	}
	
	public XQueryExpression getReturnExpression() { return returnExpression; }
		
}
