/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.parser;

import java.util.*;

import org.xquark.xquery.metadata.StaticContext;
import org.xquark.xquery.parser.primitivefunctions.fnfunctions.FunctionCOLLECTION;

public class CloneVisitor extends DefaultParserVisitor {

	private static final String RCSRevision = "$Revision: 1.34 $";
	private static final String RCSName = "$Name:  $";

	private HashMap mapvars = new HashMap();
	//private boolean cloneNoneExistent = false;
	//private boolean cloneBindings = true;
	private XQueryExpression newobj = null;
	private Stack objectStack = new Stack();
	private boolean isBinding = false;
	private boolean renameVars = true;
	private StaticContext sc = null;
	
	public CloneVisitor(StaticContext sc) {
		this.sc = sc;
	}
	public CloneVisitor(StaticContext sc, boolean renameVars) {
		this.sc = sc;
		this.renameVars = renameVars;
	}

	public XQueryExpression getClone() {
		return (XQueryExpression) objectStack.peek();
	}

	private Object pop() {
		Object obj = objectStack.pop();
		if (!objectStack.isEmpty())
			newobj = (XQueryExpression) objectStack.peek();
		return obj;
	}

	private void push(XQueryExpression expr) {
		newobj = expr;
		objectStack.push(newobj);
	}

	// visit methods

	public void visit(AggregateFunctionCall arg) throws XQueryException {
		visit((PrimitiveFunctionCall) arg);
	}

	public void visit(AttributeValuePair arg) throws XQueryException {
		visit((XQueryBinaryOperatorExpression) arg);
	}

	public void visit(BinOpANDExpression arg) throws XQueryException {
		visit((XQueryBooleanOperatorExpression) arg);
	}

	public void visit(BinOpORExpression arg) throws XQueryException {
		visit((XQueryBooleanOperatorExpression) arg);
	}

	public void visit(CaseClauseExpression arg) throws XQueryException {
		visit((XQueryExpression) arg); //.accept(this);
		SequenceType tmpSequenceTypeExpression = arg.getSequenceType();
		if (tmpSequenceTypeExpression != null) {
			tmpSequenceTypeExpression.accept(this);
			tmpSequenceTypeExpression = (SequenceType) pop();
			((CaseClauseExpression) newobj).setSequenceType(tmpSequenceTypeExpression);
		}
		Variable tmpVariable = arg.getVariable();
		if (tmpVariable != null) {
			isBinding = true;
			tmpVariable.accept(this);
			tmpVariable = (Variable) pop();
			((CaseClauseExpression) newobj).setVariable(tmpVariable);
		}
		XQueryExpression tmpXQueryExpression = arg.getReturnExpression();
		if (tmpXQueryExpression != null) {
			tmpXQueryExpression.accept(this);
			tmpXQueryExpression = (XQueryExpression) pop();
			((CaseClauseExpression) newobj).setReturnExpression(tmpXQueryExpression);
		}
	}

	public void visit(CastTreatExpression arg) throws XQueryException {
		visit((XQueryUnaryOperatorExpression) arg); //.accept(this);
		SequenceType tmpSequenceTypeExpression = arg.getSequenceType();
		if (tmpSequenceTypeExpression != null) {
			tmpSequenceTypeExpression.accept(this);
			tmpSequenceTypeExpression = (SequenceType) pop();
			((CastTreatExpression) newobj).setSequenceType(tmpSequenceTypeExpression);
		}
		((CastTreatExpression) newobj).setOperator(arg.getOperator());
	}

	public void visit(CData arg) throws XQueryException {
		visit((XQueryExpression) arg); //.accept(this);
		if (arg.getCData() != null)
			 ((CData) newobj).setCData(new String(arg.getCData()));
	}

	public void visit(Element arg) throws XQueryException {
		visit((XQueryExpression) arg); //.accept(this);
		XQueryExpression tmpXQueryExpression = arg.getStartTag();
		if (tmpXQueryExpression != null) {
			tmpXQueryExpression.accept(this);
			tmpXQueryExpression = (XQueryExpression) pop();
			((Element) newobj).setStartTag(tmpXQueryExpression);
		}
		((Element) newobj).setLocalPrefixes(arg.getLocalPrefixes());
		((Element) newobj).setAttributes(totalClone(arg.getAttributes()));
		((Element) newobj).setSubExpressions(totalClone(arg.getSubExpressions()));
	}

	public void visit(FLWRExpression arg) throws XQueryException {
		visit((XQueryExpression) arg); //.accept(this);
		 ((FLWRExpression) newobj).setVariables(totalClone(arg.getVariables(), true));
		XQueryExpression tmpXQueryExpression = arg.getWhereClause();
		if (tmpXQueryExpression != null) {
			tmpXQueryExpression.accept(this);
			tmpXQueryExpression = (XQueryExpression) pop();
			((FLWRExpression) newobj).setWhereClause(tmpXQueryExpression);
		}
		tmpXQueryExpression = arg.getReturnClause();
		if (tmpXQueryExpression != null) {
			tmpXQueryExpression.accept(this);
			tmpXQueryExpression = (XQueryExpression) pop();
			((FLWRExpression) newobj).setReturnClause(tmpXQueryExpression, false);
		}
		((FLWRExpression) newobj).setOrderBy(totalClone(arg.getOrderBy()));
	}

	public void visit(FunctionCOLLECTION arg) throws XQueryException {
		visit((PrimitiveFunctionCall) arg); //.accept(this);
		((FunctionCOLLECTION) newobj).setArguments(totalClone(arg.getArguments()));
	}
	public void visit(FunctionCall arg) throws XQueryException {
		visit((XQueryExpression) arg); //.accept(this);
		QName tmpQName = arg.getFuncName();
		if (tmpQName != null) {
			tmpQName.accept(this);
			tmpQName = (QName) pop();
			((FunctionCall) newobj).setFuncName(tmpQName);
		}
		((FunctionCall) newobj).setArguments(totalClone(arg.getArguments()));
	}

	public void visit(FunctionDeclaration arg) throws XQueryException {
		visit((XQueryExpression) arg); //.accept(this);
		QName tmpQName = arg.getFuncName();
		if (tmpQName != null) {
			tmpQName.accept(this);
			tmpQName = (QName) pop();
			((FunctionDeclaration) newobj).setFuncName(tmpQName);
		}
		((FunctionDeclaration) newobj).setParameters(totalClone(arg.getParameters()));
		SequenceType tmpSequenceTypeExpression = arg.getReturnType();
		if (tmpSequenceTypeExpression != null) {
			tmpSequenceTypeExpression.accept(this);
			tmpSequenceTypeExpression = (SequenceType) pop();
			((FunctionDeclaration) newobj).setReturnType(tmpSequenceTypeExpression);
		}
		((FunctionDeclaration) newobj).setExpressions(totalClone(arg.getExpressions()));
	}

	public void visit(InstanceOfExpression arg) throws XQueryException {
		visit((XQueryUnaryOperatorExpression) arg); //.accept(this);
		SequenceType tmpSequenceTypeExpression = arg.getSequenceType();
		if (tmpSequenceTypeExpression != null) {
			tmpSequenceTypeExpression.accept(this);
			tmpSequenceTypeExpression = (SequenceType) pop();
			((InstanceOfExpression) newobj).setSequenceType(tmpSequenceTypeExpression);
		}
	}

	public void visit(InternalFunctionCall arg) throws XQueryException {
		visit((FunctionCall) arg);
	}

	public void visit(ITEExpression arg) throws XQueryException {
		XQueryExpression ifXQueryExpression = arg.getIfExpression();
		ifXQueryExpression.accept(this);
		ifXQueryExpression = (XQueryExpression) pop();
		XQueryExpression thenXQueryExpression = arg.getThenExpression();
		thenXQueryExpression.accept(this);
		thenXQueryExpression = (XQueryExpression) pop();
		XQueryExpression elseXQueryExpression = arg.getElseExpression();
		elseXQueryExpression.accept(this);
		elseXQueryExpression = (XQueryExpression) pop();
		visit((XQueryExpression) arg); //.accept(this);
		 ((ITEExpression) newobj).setIfExpression(ifXQueryExpression);
		((ITEExpression) newobj).setThenExpression(thenXQueryExpression);
		((ITEExpression) newobj).setElseExpression(elseXQueryExpression);
	}

	public void visit(ItemType arg) throws XQueryException {
		visit((XQueryExpression) arg); //.accept(this);
		XQueryExpression tmpXQueryExpression = arg.getExpression();
		if (tmpXQueryExpression != null) {
			tmpXQueryExpression.accept(this);
			tmpXQueryExpression = (XQueryExpression) pop();
			((ItemType) newobj).setExpression(tmpXQueryExpression);
		}
	}

	public void visit(LibraryFunctionCall arg) throws XQueryException {
		visit((FunctionCall) arg);
	}

	public void visit(ListOpAFTERExpression arg) throws XQueryException {
		visit((XQueryListBinaryOperatorExpression) arg);
	}

	public void visit(ListOpArithExpression arg) throws XQueryException {
		visit((XQueryListBinaryOperatorExpression) arg); //.accept(this);
		 ((ListOpArithExpression) newobj).setOperator(arg.getOperator());
	}

	public void visit(ListOpBEFOREExpression arg) throws XQueryException {
		visit((XQueryListBinaryOperatorExpression) arg);
	}

	public void visit(ListOpCompExpression arg) throws XQueryException {
		visit((XQueryListBinaryOperatorExpression) arg); //.accept(this);
		 ((ListOpCompExpression) newobj).setOperator(arg.getOperator());
	}

	public void visit(ListOpEXCEPTExpression arg) throws XQueryException {
		visit((XQueryListBinaryOperatorExpression) arg);
	}

	public void visit(ListOpINTERSECTExpression arg) throws XQueryException {
		visit((XQueryListBinaryOperatorExpression) arg);
	}

	public void visit(ListOpUNIONExpression arg) throws XQueryException {
		visit((XQueryListBinaryOperatorExpression) arg);
	}

	public void visit(LocatedExpression arg) throws XQueryException {
		visit((XQueryExpression) arg); //.accept(this);
		if (arg.getSteps() != null)
			 ((LocatedExpression) newobj).setSteps(totalClone(arg.getSteps()));
		((LocatedExpression) newobj).setInPredicate(arg.getInPredicate());
		((LocatedExpression) newobj).setNodeAccessor(arg.getNodeAccessor());
	}

	public void visit(NodeTest arg) throws XQueryException {
		visit((XQueryExpression) arg); //.accept(this);
		 ((NodeTest) newobj).setKind(arg.getKind());
		XQueryExpression tmpXQueryExpression = arg.getArg();
		if (tmpXQueryExpression != null) {
			tmpXQueryExpression.accept(this);
			tmpXQueryExpression = (XQueryExpression) pop();
			((NodeTest) newobj).setArg(tmpXQueryExpression);
		}
	}

	public void visit(PrimitiveFunctionCall arg) throws XQueryException {
		visit((FunctionCall) arg);
	}

	public void visit(QName arg) throws XQueryException {
		visit((XQueryExpression) arg);
		if (arg.getNameSpace() != null)
			 ((QName) newobj).setNameSpace(arg.getNameSpace());
		if (arg.getPrefixName() != null)
			 ((QName) newobj).setPrefixName(arg.getPrefixName());
		if (arg.getLocalName() != null)
			 ((QName) newobj).setLocalName(arg.getLocalName());
	}

	public void visit(QuantifiedExpression arg) throws XQueryException {
		visit((XQueryExpression) arg);
		((QuantifiedExpression) newobj).setVariables(totalClone(arg.getVariables(), true));
		XQueryExpression tmpXQueryExpression = arg.getExpression();
		if (tmpXQueryExpression != null) {
			tmpXQueryExpression.accept(this);
			tmpXQueryExpression = (XQueryExpression) pop();
			((XQueryUnaryOperatorExpression) newobj).setExpression(tmpXQueryExpression);
		}
		((QuantifiedExpression) newobj).setKind(arg.getKind());
	}

	public void visit(RangeExpression arg) throws XQueryException {
		visit((XQueryListBinaryOperatorExpression) arg);
	}

	public void visit(InputFunctionCall arg) throws XQueryException {
		visit((PrimitiveFunctionCall) arg);
	}

	public void visit(SequenceType arg) throws XQueryException {
		visit((XQueryExpression) arg);
		ItemType tmpItemType = arg.getItemType();
		if (tmpItemType != null) {
			tmpItemType.accept(this);
			tmpItemType = (ItemType) pop();
			((SequenceType) newobj).setItemType(tmpItemType);
		}
		((SequenceType) newobj).setOccurrence(arg.getOccurrence());
	}

	public void visit(SortedExpression arg) throws XQueryException {
		visit((XQueryListUnaryOperatorExpression) arg);
		 ((SortedExpression) newobj).setStable(arg.getStable());
		if (arg.getSortClauses() != null)
			 ((SortedExpression) newobj).setSortClauses(totalClone(arg.getSortClauses()));
	}

	public void visit(Step arg) throws XQueryException {
		visit((XQueryUnaryOperatorExpression) arg);
		if (arg.getPredicates() != null)
			 ((Step) newobj).setPredicates(totalClone(arg.getPredicates()));
		((Step) newobj).setHasSeparator(arg.hasSeparator());
		((Step) newobj).setAxis(arg.getAxis());
	}

	public void visit(TypeSwitchExpression arg) throws XQueryException {
		visit((XQueryBinaryOperatorExpression) arg);
		if (arg.getCases() != null)
			 ((TypeSwitchExpression) newobj).setCases(totalCloneHashMap(arg.getCases()));
		Variable tmpVariable = arg.getDefaultVariable();
		if (tmpVariable != null) {
			isBinding = true;
			tmpVariable.accept(this);
			tmpVariable = (Variable) pop();
			((TypeSwitchExpression) newobj).setDefaultVariable(tmpVariable);
		}
	}

	public void visit(UnOpMinusExpression arg) throws XQueryException {
		visit((XQueryUnaryOperatorExpression) arg);
	}

	public void visit(ValidateExpression arg) throws XQueryException {
		visit((XQueryUnaryOperatorExpression) arg);
		SchemaContextPath scp = arg.getSchemaContext();
		if (scp != null) {
			scp.accept(this);
			scp = (SchemaContextPath) pop();
			((ValidateExpression) newobj).setSchemaContext(scp);
		}
		((ValidateExpression) newobj).setScopeType(arg.getScopeType());
		((ValidateExpression) newobj).setValidationType(arg.getValidationType());
	}

	public void visit(Value arg) throws XQueryException {
		visit((XQueryExpression) arg);
		if (arg.getValue() != null)
			 ((Value) newobj).setValue(new String(arg.getValue()));
	}

	public void visit(ValueBoolean arg) throws XQueryException {
		visit((Value) arg);
	}

	public void visit(ValueDecimal arg) throws XQueryException {
		visit((Value) arg);
	}

	public void visit(ValueDouble arg) throws XQueryException {
		visit((Value) arg);
	}

	public void visit(ValueFloat arg) throws XQueryException {
		visit((Value) arg);
	}

	public void visit(ValueInteger arg) throws XQueryException {
		visit((Value) arg);
	}

	public void visit(ValueString arg) throws XQueryException {
		visit((Value) arg);
	}

	public void visit(ValueText arg) throws XQueryException {
		visit((Value) arg);
	}

	public void visit(Variable arg) throws XQueryException {
		Variable tmpVar = null;
		//if (mapvars != null)
		tmpVar = (Variable) mapvars.get(arg);
		if (tmpVar != null) {
			push(tmpVar);
			return;
		} else if (!isBinding /*&& !cloneNoneExistent*/) {
			push(arg);
			return;
		}

		visit((XQueryExpression) arg);
		isBinding = false;
		// W3C WD 16 August 2002
		SequenceType tmpSequenceTypeExpression = arg.getTypeDeclaration();
		if (tmpSequenceTypeExpression != null) {
			tmpSequenceTypeExpression.accept(this);
			tmpSequenceTypeExpression = (SequenceType) pop();
			((Variable) newobj).setTypeDeclaration(tmpSequenceTypeExpression);
		}
		// REMOVED: change the variable name!!!
		// change LARS 23/02/04
		if (renameVars) {
			((Variable) newobj).setVarName(sc.getVarName(arg.getParentModule()));
			((Variable) newobj).setCreator(Variable.CLONE_CREATOR);
		}

		XQueryExpression tmpXQueryExpression = arg.getExpression();
		if (tmpXQueryExpression != null) {
			tmpXQueryExpression.accept(this);
			tmpXQueryExpression = (XQueryExpression) pop();
			((Variable) newobj).setExpression(tmpXQueryExpression, false);
		}

		//OUf, le chef a dit qu'au niveau du mediateur, AlgebraExpression est toujours a nul !
		//Donc pas de clonage recursif
		//NON --->	newobj.setAlgebraExpression((getAlgebraExpression () == null) ? null : (Expression) getAlgebraExpression ().clone (mapvars));
		 ((Variable) newobj).setBindingType(arg.getBindingType());
		mapvars.put(arg, newobj);
	}

	public void visit(XMLComment arg) throws XQueryException {
		visit((XQueryUnaryOperatorExpression) arg);
		 ((XMLComment) newobj).setIsComputed(arg.isComputed());
	}

	public void visit(XMLProcessingInstruction arg) throws XQueryException {
		visit((XQueryBinaryOperatorExpression) arg);
		((XMLProcessingInstruction) newobj).setIsComputed(arg.isComputed());
	}

	public void visit(XQueryBinaryOperatorExpression arg) throws XQueryException {
		visit((XQueryExpression) arg);
		XQueryExpression tmpXQueryExpression = arg.getExpression1();
		if (tmpXQueryExpression != null) {
			tmpXQueryExpression.accept(this);
			tmpXQueryExpression = (XQueryExpression) pop();
			((XQueryBinaryOperatorExpression) newobj).setExpression1(tmpXQueryExpression);
		}
		tmpXQueryExpression = arg.getExpression2();
		if (tmpXQueryExpression != null) {
			tmpXQueryExpression.accept(this);
			tmpXQueryExpression = (XQueryExpression) pop();
			((XQueryBinaryOperatorExpression) newobj).setExpression2(tmpXQueryExpression);
		}
	}

	public void visit(XQueryBooleanOperatorExpression arg) throws XQueryException {
		visit((XQueryBinaryOperatorExpression) arg);
	}

	public void visit(XQueryExpression arg) throws XQueryException {
		newobj = arg.shallowClone();
		newobj.setParenthesis(arg.getParenthesis());
		newobj.setContext(arg.getContext());
		newobj.setOrder(arg.getOrder());

		newobj.setQType(arg.getQType());
		newobj.setXTrees(arg.getXTrees());
		newobj.setDependIDs(arg.getDependIDs());
		newobj.setParentModule(arg.getParentModule());
		newobj.setSkolemIDs(arg.getSkolemIDs());
		newobj.setRoot(arg.getRoot());
		newobj.setLoopIDs(arg.getLoopIDs());
		newobj.setSourceNames(arg.getSourceNames());
		newobj.setUrls(arg.getUrls());

		newobj.resetStringValue();
		push(newobj);
	}

	public void visit(XQueryExpressionSequence arg) throws XQueryException {
		visit((XQueryExpression) arg);
		if (arg.getSubExpressions() != null)
			 ((XQueryExpressionSequence) newobj).setSubExpressions(totalClone(arg.getSubExpressions()));
	}

	public void visit(XQueryListBinaryOperatorExpression arg) throws XQueryException {
		visit((XQueryBinaryOperatorExpression) arg);
	}

	public void visit(XQueryListUnaryOperatorExpression arg) throws XQueryException {
		visit((XQueryUnaryOperatorExpression) arg);
	}

	public void visit(XQueryUnaryOperatorExpression arg) throws XQueryException {
		visit((XQueryExpression) arg);
		XQueryExpression tmpXQueryExpression = arg.getExpression();
		if (tmpXQueryExpression != null) {
			tmpXQueryExpression.accept(this);
			tmpXQueryExpression = (XQueryExpression) pop();
			((XQueryUnaryOperatorExpression) newobj).setExpression(tmpXQueryExpression);
		}
	}

	public void visit(XQueryVoid arg) throws XQueryException {
		visit((XQueryExpression) arg);
	}

	// primitive functions

	// utilities
	private ArrayList totalClone(ArrayList origlist) throws XQueryException {
		return totalClone(origlist, false);
	}
	private ArrayList totalClone(ArrayList origlist, boolean isBinding) throws XQueryException {
		if (origlist == null)
			return null;
		ArrayList newlist = new ArrayList(origlist.size());
		for (int i = 0; i < origlist.size(); i++) {
			this.isBinding = isBinding;
			((XQueryExpression) origlist.get(i)).accept(this);
			newlist.add(pop());
		}
		return newlist;
	}

	private HashMap totalCloneHashMap(HashMap origmap) throws XQueryException {
		if (origmap == null)
			return null;
		HashMap newmap = new HashMap(origmap.size());
		Set keys = origmap.keySet();
		Iterator it = keys.iterator();
		while (it.hasNext()) {
			XQueryExpression objkey = (XQueryExpression) it.next();
			objkey.accept(this);
			XQueryExpression clonekey = (XQueryExpression) pop();

			XQueryExpression value = (XQueryExpression) origmap.get(objkey);
			XQueryExpression clonevalue = null;
			if (value != null) {
				value.accept(this);
				clonevalue = (XQueryExpression) pop();
			}
			newmap.put(clonekey, clonevalue);
		}
		return newmap;
	}

}
