/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.parser;

import java.lang.reflect.Method;
import java.util.ArrayList;

import org.xquark.xquery.typing.TypeException;

public class FunctionDeclaration extends XQueryExpression implements Cloneable {

	private static final String RCSRevision = "$Revision: 1.12 $";
	private static final String RCSName = "$Name:  $";

	// cannot be null
	protected QName funcName = null;
	protected ArrayList parameters = null;
	// cannot be null
	protected SequenceType returnType = null;
	// cannot be null or empty
	private ArrayList expressions = null;
	// home module when imported
	private XQueryModule homeModule = null;
	
	// for java functions
	private Method method = null;

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(ParserVisitor visitor) throws XQueryException {
		visitor.visit(this);
	}

	// #############################################################################
	// CONSTRUCTOR STUFF
	// #############################################################################

	public FunctionDeclaration(QName name, ArrayList parameters, SequenceType returnType, ArrayList expressions, XQueryModule parentModule) throws TypeException, XQueryException {
		setFuncName(name);
		setParameters(parameters);
		setReturnType(returnType);
		setExpressions(expressions);
		setParentModule(parentModule);
		homeModule = parentModule;
		if (parentModule != null && parentModule.getStaticContext().getTypeVisitor() != null)
			accept(parentModule.getStaticContext().getTypeVisitor());
	}

	// #############################################################################
	// ATTRIBUTE ACCESS STUFF
	// #############################################################################

	public XQueryModule getHomeModule() {
		return homeModule;
	}
	
	public QName getFuncName() {
		return funcName;
	}
	public void setFuncName(QName funcName) throws XQueryException {
		if (funcName == null || funcName.getName().equals(""))
			throw new XQueryException("funcName of FunctionDefinition cannot be null or empty string");
		this.funcName = funcName;
		funcName.setParentModule(parentModule);
	}

	public SequenceType getReturnType() {
		return returnType;
	}
	public void setReturnType(SequenceType returnType) {
		this.returnType = returnType;
		if (returnType != null) {
			this.returnType.setParentExpression(this);
			this.returnType.setParentModule(parentModule);
		}
	}

	public ArrayList getParameters() {
		return parameters;
	}

	public void setParameters(ArrayList parameters) {
		this.parameters = parameters;
		if (this.parameters != null)
			for (int i = 0; i < this.parameters.size(); i++) {
				Variable param = (Variable) this.parameters.get(i);
				param.setParentExpression(this);
				param.setParentModule(parentModule);
			}
	}

	public ArrayList getExpressions() {
		return expressions;
	}
	public void setExpressions(ArrayList expressions) throws XQueryException {
		this.expressions = expressions;
		if (expressions != null)
			for (int i = 0; i < this.expressions.size(); i++)
				 ((XQueryExpression) this.expressions.get(i)).setParentModule(parentModule);
	}

	public Method getMethod() {
		return method;
	}
	
	public void setMethod(Method method) {
		this.method = method;
	}
	
	// #############################################################################
	// PRINT STUFF
	// #############################################################################
	public void addParentExpression(XQueryExpression parentExpression) {
		addParentExpression(parentExpression);
		funcName.addParentExpression(parentExpression);
		if (parameters != null)
			for (int i = 0; i < parameters.size(); i++)
				 ((Variable) parameters.get(i)).addParentExpression(parentExpression);
		if (returnType != null)
			returnType.addParentExpression(parentExpression);
		if (expressions != null)
			for (int i = 0; i < this.expressions.size(); i++)
				 ((XQueryExpression) this.expressions.get(i)).addParentExpression(parentExpression);
	}

}
