/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */


package  org.xquark.xquery.parser;

import org.xquark.xquery.typing.TypeException;



public class ITEExpression extends XQueryExpression implements Cloneable {
    
    private static final String RCSRevision = "$Revision: 1.12 $";
    private static final String RCSName     = "$Name:  $";
    
    // cannot be null
    protected XQueryExpression ifExpression = null;
    // cannot be null
    protected XQueryExpression thenExpression = null;
    // cannot be null
    protected XQueryExpression elseExpression = null;
    
    // #############################################################################
    // VISITOR STUFF
    // #############################################################################
    
    public void accept(ParserVisitor visitor) throws XQueryException {
        visitor.visit(this);
    }
    
    // #############################################################################
    // CONSTRUCTOR STUFF
    // #############################################################################
    
    public ITEExpression(XQueryExpression ifExpression, XQueryExpression thenExpression, XQueryExpression elseExpression, XQueryModule parentModule) throws TypeException, XQueryException {
        setIfExpression(ifExpression);
        setThenExpression(thenExpression);
        setElseExpression(elseExpression);
		setParentModule(parentModule);
		if (parentModule != null && parentModule.getStaticContext().getTypeVisitor() != null)
			accept(parentModule.getStaticContext().getTypeVisitor());
    }
    
    // #############################################################################
    // ATTRIBUTE ACCESS STUFF
    // #############################################################################
        
    public XQueryExpression getIfExpression() { return ifExpression; }
    public void setIfExpression(XQueryExpression expression) throws XQueryException {
        // cannot be null
        if (expression == null) throw new XQueryException("ifExpression of ITEExpression cannot be null");
        this.ifExpression = expression;
        this.ifExpression.setParentModule(parentModule);
        this.ifExpression.setParentExpression(this);
    }
    
    public XQueryExpression getThenExpression() { return thenExpression; }
    public void setThenExpression(XQueryExpression expression) throws XQueryException {
        // cannot be null
        if (expression == null) throw new XQueryException("thenExpression of ITEExpression cannot be null");
        this.thenExpression = expression;
        this.thenExpression.setParentModule(parentModule);
        this.thenExpression.setParentExpression(this);
    }
    
    public XQueryExpression getElseExpression() { return elseExpression; }
    public void setElseExpression(XQueryExpression expression) throws XQueryException {
        // cannot be null
        if (expression == null) throw new XQueryException("elseExpression of ITEExpression cannot be null");
        this.elseExpression = expression;
        this.elseExpression.setParentModule(parentModule);
        this.elseExpression.setParentExpression(this);
    }
        
    // #############################################################################
    // CLONE STUFF
    // #############################################################################
    public void addParentExpression(XQueryExpression parentExpression) {
        addParentExpression(parentExpression) ;
        ifExpression.addParentExpression(parentExpression);
        thenExpression.addParentExpression(parentExpression);
        elseExpression.addParentExpression(parentExpression);
    }
    
}



