/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.parser;

import org.xquark.xquery.typing.TypeException;


public class InstanceOfExpression extends XQueryUnaryOperatorExpression implements Cloneable {

	private static final String RCSRevision = "$Revision: 1.11 $";
	private static final String RCSName = "$Name:  $";

	// cannot be null
	// W3C WD 16 August 2002
	// change QName in SequenceTypeExpression
	protected SequenceType sequenceType = null;

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(ParserVisitor visitor) throws XQueryException {
		visitor.visit(this);
	}

	// #############################################################################
	// CONSTRUCTOR STUFF
	// #############################################################################

	public InstanceOfExpression(XQueryExpression expression, SequenceType sequenceType, XQueryModule parentModule) throws TypeException, XQueryException {
		super(expression);
		setSequenceType(sequenceType);
		setParentModule(parentModule);
		if (parentModule != null && parentModule.getStaticContext().getTypeVisitor() != null)
			accept(parentModule.getStaticContext().getTypeVisitor());
	}

	// #############################################################################
	// ATTRIBUTE ACCESS STUFF
	// #############################################################################

	public SequenceType getSequenceType() {
		return sequenceType;
	}
	public void setSequenceType(SequenceType sequenceType) throws XQueryException {
		// cannot be null
		if (sequenceType == null)
			throw new XQueryException("sequenceType of InstanceOfExpression cannot be null");
		this.sequenceType = sequenceType;
		this.sequenceType.setParentModule(parentModule);
		this.sequenceType.setParentExpression(this);
	}

	// #############################################################################
	// CLONE STUFF
	// #############################################################################
	public void addParentExpression(XQueryExpression parentExpression) {
		addParentExpression(parentExpression);
		sequenceType.addParentExpression(parentExpression);
		if (expression != null)
			expression.addParentExpression(parentExpression);
	}

}
