/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.parser;

import java.util.ArrayList;
import java.util.HashMap;

import org.xquark.xquery.parser.hinttree.HintTree;
import org.xquark.xquery.typing.TypeException;
import org.xquark.util.NamespaceContext;

public class ParserFactory implements ParserFactoryInterface {

	private static final String RCSRevision = "$Revision: 1.21 $";
	private static final String RCSName = "$Name:  $";

	public AttributeValuePair createAttributeValuePair(XQueryExpression attributeName, XQueryExpression attributeValue, boolean isXmlns, XQueryModule parentModule) throws TypeException, XQueryException {
		return new AttributeValuePair(attributeName, attributeValue, isXmlns, parentModule);
	}

	public BinOpANDExpression createBinOpANDExpression(XQueryExpression mainExpression, XQueryExpression expression, XQueryModule parentModule) throws TypeException, XQueryException {
		return new BinOpANDExpression(mainExpression, expression, parentModule);
	}

	public BinOpORExpression createBinOpORExpression(XQueryExpression mainExpression, XQueryExpression expression, XQueryModule parentModule) throws TypeException, XQueryException {
		return new BinOpORExpression(mainExpression, expression, parentModule);
	}

	public CData createCData(String data, XQueryModule parentModule) throws TypeException, XQueryException {
		return new CData(data, parentModule);
	}

	public CaseClauseExpression createCaseClauseExpression(SequenceType sequenceType, Variable variable, XQueryExpression returnExpression, XQueryModule parentModule) throws TypeException, XQueryException {
		return new CaseClauseExpression(sequenceType, variable, returnExpression, parentModule);
	}

	public CastTreatExpression createCastTreatExpression(int kind, SequenceType sequenceType, XQueryExpression expression, XQueryModule parentModule) throws TypeException, XQueryException {
		return new CastTreatExpression(kind, sequenceType, expression, parentModule);
	}

	public Document createDocument(XQueryExpression element, XQueryModule parentModule) throws TypeException, XQueryException {
		return new Document(element, parentModule);
	}

	public Element createElement(XQueryExpression startTag, HashMap localPrefixes, ArrayList vectAttributes, ArrayList vectExpressions, XQueryModule parentModule) throws TypeException, XQueryException {
		return new Element(startTag, localPrefixes, vectAttributes, vectExpressions, parentModule);
	}

	public FLWRExpression createFLWRExpression(ArrayList bindings, HintTree hintClause, XQueryExpression whereClause, ArrayList orderbys, XQueryExpression returnClause, XQueryModule parentModule) throws TypeException, XQueryException {
		return new FLWRExpression(bindings, hintClause, whereClause, orderbys, returnClause, parentModule);
	}

	public FLWRExpression createFLWRExpression(ArrayList bindings, XQueryExpression whereClause, ArrayList orderbys, XQueryExpression returnClause, XQueryModule parentModule) throws TypeException, XQueryException {
		return new FLWRExpression(bindings, orderbys, whereClause, returnClause, parentModule);
	}

	public FLWRExpression createFLWRExpression(ArrayList bindings, XQueryExpression whereClause, XQueryExpression returnClause, XQueryModule parentModule) throws TypeException, XQueryException {
		return new FLWRExpression(bindings, whereClause, returnClause, parentModule);
	}

	public FunctionDeclaration createFunctionDeclaration(QName funcName, ArrayList tmpVect, SequenceType sequenceType, ArrayList vectExpression, XQueryModule parentModule) throws TypeException, XQueryException {
		return new FunctionDeclaration(funcName, tmpVect, sequenceType, vectExpression, parentModule);
	}

	public ITEExpression createITEExpression(XQueryExpression expressionIf, XQueryExpression expressionThen, XQueryExpression expressionElse, XQueryModule parentModule) throws TypeException, XQueryException {
		return new ITEExpression(expressionIf, expressionThen, expressionElse, parentModule);
	}

	public InstanceOfExpression createInstanceOfExpression(XQueryExpression expression, SequenceType sequenceType, XQueryModule parentModule) throws TypeException, XQueryException {
		return new InstanceOfExpression(expression, sequenceType, parentModule);
	}

	public InternalFunctionCall createInternalFunctionCall(QName funcName, ArrayList argsVect, XQueryModule parentModule) throws TypeException, XQueryException {
		return new InternalFunctionCall(funcName, argsVect, parentModule);
	}

	public ItemType createItemType(XQueryExpression expression, XQueryModule parentModule) throws TypeException, XQueryException {
		return new ItemType(expression, parentModule);
	}
	
	public HintTree createHintTree(byte type, byte side, Object left, Object right) {
		return new HintTree(type, side, left, right);
	}

	public LibraryFunctionCall createLibraryFunctionCall(QName funcName, ArrayList argsVect, FunctionDeclaration funcdef, XQueryModule parentModule) throws TypeException, XQueryException {
		return new LibraryFunctionCall(funcName, argsVect, funcdef, parentModule);
	}

	public ListOpAFTERExpression createListOpAFTERExpression(XQueryExpression mainExpression, XQueryExpression expression, XQueryModule parentModule) throws TypeException, XQueryException {
		return new ListOpAFTERExpression(mainExpression, expression, parentModule);
	}

	public ListOpArithExpression createListOpArithExpression(XQueryExpression mainExpression, XQueryExpression expression, int kind, XQueryModule parentModule) throws TypeException, XQueryException {
		return new ListOpArithExpression(mainExpression, expression, kind, parentModule);
	}

	public ListOpBEFOREExpression createListOpBEFOREExpression(XQueryExpression mainExpression, XQueryExpression expression, XQueryModule parentModule) throws TypeException, XQueryException {
		return new ListOpBEFOREExpression(mainExpression, expression, parentModule);
	}

	public ListOpCompExpression createListOpCompExpression(XQueryExpression mainExpression, XQueryExpression expression, int kind, XQueryModule parentModule) throws TypeException, XQueryException {
		return new ListOpCompExpression(mainExpression, expression, kind, parentModule);
	}

	public ListOpEXCEPTExpression createListOpEXCEPTExpression(XQueryExpression mainExpression, XQueryExpression expression, XQueryModule parentModule) throws TypeException, XQueryException {
		return new ListOpEXCEPTExpression(mainExpression, expression, parentModule);
	}

	public ListOpINTERSECTExpression createListOpINTERSECTExpression(XQueryExpression mainExpression, XQueryExpression expression, XQueryModule parentModule) throws TypeException, XQueryException {
		return new ListOpINTERSECTExpression(mainExpression, expression, parentModule);
	}

	public ListOpUNIONExpression createListOpUNIONExpression(XQueryExpression mainExpression, XQueryExpression expression, XQueryModule parentModule) throws TypeException, XQueryException {
		return new ListOpUNIONExpression(mainExpression, expression, parentModule);
	}

	public LocatedExpression createLocatedExpression(ArrayList pathVect, boolean inPredicate, XQueryModule parentModule) throws TypeException, XQueryException {
		return new LocatedExpression(pathVect, inPredicate, parentModule, false);
	}

	public ElementTest createElementTest(SchemaContextPath scp, QName elementQName, QName typeQName, boolean nillable, XQueryModule parentModule) throws TypeException, XQueryException {
		return new ElementTest(scp, elementQName, typeQName, nillable, parentModule);
	}

	public AttributeTest createAttributeTest(SchemaContextPath scp, QName elementQName, QName typeQName, XQueryModule parentModule) throws TypeException, XQueryException {
		return new AttributeTest(scp, elementQName, typeQName, parentModule);
	}

	public NodeTest createNodeTest(byte kind, XQueryExpression arg, XQueryModule parentModule) throws TypeException, XQueryException {
		return new NodeTest(kind, arg, parentModule);
	}

	public QName createQName(String namespace, String prefixname, String typeName, XQueryModule parentModule, NamespaceContext namespaceContext) throws TypeException, XQueryException {
		return new QName(namespace, prefixname, typeName, parentModule, namespaceContext);
	}

	public QuantifiedExpression createQuantifiedExpression(int kind, ArrayList variables, XQueryExpression expressionSat, XQueryModule parentModule) throws TypeException, XQueryException {
		return new QuantifiedExpression(kind, variables, expressionSat, parentModule);
	}

	public RangeExpression createRangeExpression(XQueryExpression expression1, XQueryExpression expression2, XQueryModule parentModule) throws TypeException, XQueryException {
		return new RangeExpression(expression1, expression2, parentModule);
	}

	public SchemaContextPath createSchemaContextPath(ArrayList steps, XQueryModule parentModule) throws TypeException, XQueryException {
		return new SchemaContextPath(steps, parentModule);
	}

	public SchemaContextStep createSchemaContextStep(byte axis, XQueryExpression expression, XQueryModule parentModule) throws TypeException, XQueryException {
		return new SchemaContextStep(axis, expression, parentModule);
	}

	public SequenceType createSequenceType(ItemType itemType, int occurrence, XQueryModule parentModule) throws TypeException, XQueryException {
		return new SequenceType(itemType, occurrence, parentModule);
	}

	public SortedExpression createSortedExpression(XQueryExpression expression, ArrayList tmpVect, boolean stable, XQueryModule parentModule) throws TypeException, XQueryException {
		return new SortedExpression(expression, tmpVect, stable, parentModule);
	}

	public Step createStep(boolean separator, byte axis, XQueryExpression expression, ArrayList predicates, XQueryModule parentModule) throws TypeException, XQueryException {
		return new Step(separator, axis, expression, predicates, parentModule);
	}

	public TypeSwitchExpression createTypeSwitchExpression(XQueryExpression switchExpression, HashMap cases, Variable defaultVariable, XQueryExpression defaultExpression, XQueryModule parentModule) throws TypeException, XQueryException {
		return new TypeSwitchExpression(switchExpression, cases, defaultVariable, defaultExpression, parentModule);
	}

	public UnOpMinusExpression createUnOpMinusExpression(XQueryExpression expression, boolean minus, XQueryModule parentModule) throws TypeException, XQueryException {
		return new UnOpMinusExpression(expression, minus, parentModule);
	}

	public ValidateExpression createValidateExpression(int scopeType, int validationType, SchemaContextPath schemaContext, XQueryExpression validateExpression, XQueryModule parentModule) throws TypeException, XQueryException {
		return new ValidateExpression(scopeType, validationType,schemaContext, validateExpression, parentModule);
	}

	public ValueBoolean createValueBoolean(String str, XQueryModule parentModule) throws TypeException, XQueryException {
		return new ValueBoolean(str, parentModule);
	}

	public ValueDecimal createValueDecimal(String str, XQueryModule parentModule) throws TypeException, XQueryException {
		return new ValueDecimal(str, parentModule);
	}

	public ValueDouble createValueDouble(String str, XQueryModule parentModule) throws TypeException, XQueryException {
		return new ValueDouble(str, parentModule);
	}

	public ValueInteger createValueInteger(String str, XQueryModule parentModule) throws TypeException, XQueryException {
		return new ValueInteger(str, parentModule);
	}

	public ValueString createValueString(String str, XQueryModule parentModule) throws TypeException, XQueryException {
		return new ValueString(str, parentModule);
	}

	public ValueText createValueText(String str, XQueryModule parentModule) throws TypeException, XQueryException {
		return new ValueText(str, parentModule);
	}

	public ExternalVariable createExternalVariable(Variable var, XQueryModule parentModule) throws TypeException, XQueryException {
		return new ExternalVariable(var, parentModule);
	}

	public Variable createVariable(QName var, XQueryModule parentModule) throws TypeException, XQueryException {
		return new Variable(var, parentModule);
	}

	public XMLComment createXMLComment(XQueryExpression comment, XQueryModule parentModule) throws TypeException, XQueryException {
		return new XMLComment(comment, parentModule);
	}

	public ComputedNamespace createComputedNamespace(String ncname, XQueryExpression value, XQueryModule parentModule) throws TypeException, XQueryException {
		return new ComputedNamespace(ncname,value,parentModule);
	}

	public ComputedText createComputedText(XQueryExpression value, XQueryModule parentModule) throws TypeException, XQueryException {
		return new ComputedText(value,parentModule);
	}

	public XMLProcessingInstruction createXMLProcessingInstruction(XQueryExpression PITarget, XQueryExpression data, XQueryModule parentModule) throws TypeException, XQueryException {
		return new XMLProcessingInstruction(PITarget, data, parentModule);
	}

	public XQueryExpressionSequence createXQueryExpressionSequence(ArrayList subExpressions, XQueryModule parentModule) throws TypeException, XQueryException {
		return new XQueryExpressionSequence(subExpressions, parentModule);
	}

	public XQueryModule createXQueryModule() {
		return new XQueryModule();
	}

	public ImportSchema createImportSchema(boolean hasPrefix, String prefix, String name, String location) {
		return new ImportSchema(hasPrefix, prefix, name, location);
	}

	public ImportModule createImportModule(String prefix, String name, String location, XQueryModule module) {
		return new ImportModule(prefix, name, location, module);
	}

	public XQueryVoid createXQueryVoid(XQueryModule parentModule) throws TypeException, XQueryException {
		return new XQueryVoid(parentModule);
	}
}
