/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2004 XQuark Group.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.parser;

import java.util.ArrayList;

import org.xquark.xpath.Axis;
import org.xquark.xquery.typing.TypeException;

public class SchemaContextPath extends XQueryExpression implements Cloneable {

	private static final String RCSRevision = "$Revision: 1.8 $";
	private static final String RCSName = "$Name:  $";

	// cannot be null or empty
	protected ArrayList steps = null;

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(ParserVisitor visitor) throws XQueryException {
		visitor.visit(this);
	}

	// #############################################################################
	// CONSTRUCTOR STUFF
	// #############################################################################

	public SchemaContextPath(ArrayList steps, XQueryModule parentModule) throws TypeException, XQueryException {
		this.steps = steps;
		setParentModule(parentModule);
//		if (parentModule != null && parentModule.getStaticContext().getTypeVisitor() != null)
//			accept(parentModule.getStaticContext().getTypeVisitor());
	}


	// #############################################################################
	// ATTRIBUTE ACCESS STUFF
	// #############################################################################

	public SchemaContextStep getStep(int num) {
		if (num >= steps.size())
			return null;
		return (SchemaContextStep) steps.get(num);
	}
//	public SchemaContextStep getGlobalContext() {
//		return ((SchemaContextStep)steps.get(0)).getExpression();
//	}
	public ArrayList getSteps() {
		return steps;
	}
	public void setSteps(ArrayList steps) throws XQueryException {
		// cannot be null or empty
		if (steps == null || steps.isEmpty())
			throw new XQueryException("steps of SchemaContextPath cannot be null or empty");
		this.steps = steps;
		for (int i = 0; i < this.steps.size(); i++) {
			SchemaContextStep step = (SchemaContextStep) this.steps.get(i);
			step.setParentExpression(this);
			step.setParentModule(parentModule);
		}
	}

	public void addStep(SchemaContextStep step) throws XQueryException {
		if (step == null)
			return;
		if (steps == null)
			steps = new ArrayList(1);
		steps.add(step);
		setSteps(steps);
	}

	public void addSteps(ArrayList steps) throws XQueryException {
		if (steps == null || steps.isEmpty())
			return;
		if (this.steps == null || this.steps.isEmpty()) {
			setSteps(steps);
			return;
		}
		this.steps.addAll(steps);
		setSteps(this.steps);
	}

	public void addStepsAt(int index, ArrayList steps) throws XQueryException {
		if (steps == null || steps.isEmpty())
			return;
		if (this.steps == null || this.steps.isEmpty()) {
			setSteps(steps);
			return;
		}
		this.steps.addAll(index, steps);
		setSteps(this.steps);
	}

	public boolean isRelativeExpression() {
		Step step0 = (Step)steps.get(0);
		if (step0.getAxis() == Axis.NONE && !step0.hasSeparator())
			return false;
		return true;
	}

	// #############################################################################
	// CLONE STUFF
	// #############################################################################
	public void addParentExpression(XQueryExpression parentExpression) {
		addParentExpression(parentExpression);
		for (int i = 0; i < steps.size(); i++)
			 ((SchemaContextStep) steps.get(i)).addParentExpression(parentExpression);
	}

}
