/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2004 XQuark Group.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.parser;

import org.xquark.xquery.parser.util.Constants;
import org.xquark.xquery.typing.TypeException;


public class SchemaContextStep extends XQueryUnaryOperatorExpression implements Cloneable {
	private static final String RCSRevision = "$Revision: 1.8 $";
	private static final String RCSName = "$Name:  $";

	private byte axis = Constants.NOAXIS;

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(ParserVisitor visitor) throws XQueryException {
		visitor.visit(this);
	}

	// #############################################################################
	// CONSTRUCTOR STUFF
	// #############################################################################

	public SchemaContextStep(byte axis, XQueryExpression expression, XQueryModule parentModule) throws TypeException, XQueryException {
		super(expression);
		setAxis(axis);
		setParentModule(parentModule);
	}

	// #############################################################################
	// ATTRIBUTE ACCESS STUFF
	// #############################################################################

	public byte getAxis() {
		return axis;
	}
	public void setAxis(byte axis) {
		this.axis = axis;
	}

	// #############################################################################
	// CLONE STUFF
	// #############################################################################
	public void addParentExpression(XQueryExpression parentExpression) {
		addParentExpression(parentExpression);
		expression.addParentExpression(parentExpression);
	}

}
