/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.parser;

import java.util.ArrayList;

import org.xquark.xquery.typing.TypeException;

public class SortedExpression extends XQueryListUnaryOperatorExpression implements Cloneable {
	private static final String RCSRevision = "$Revision: 1.9 $";
	private static final String RCSName = "$Name:  $";

	// cannot be null or empty
	protected ArrayList sortClauses = null;

	// W3C WD 16 August 2002
	protected boolean stable = false; // by default, implementation-defined

	// #############################################################################
	// CONSTRUCTOR STUFF
	// #############################################################################
	public SortedExpression(XQueryExpression expression, ArrayList sortClauses, boolean stable, XQueryModule parentModule) throws TypeException, XQueryException {
		super(expression);

		// W3C WD 16 August 2002
		setStable(stable);
		setSortClauses(sortClauses);

		setParentModule(parentModule);
		if (parentModule != null && parentModule.getStaticContext().getTypeVisitor() != null)
			accept(parentModule.getStaticContext().getTypeVisitor());
	}

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################
	public void accept(ParserVisitor visitor) throws XQueryException {
		visitor.visit(this);
	}

	// #############################################################################
	// ATTRIBUTE ACCESS STUFF
	// #############################################################################
	public void setParentModule(XQueryModule parentUnit) {
		if (parentUnit == null) {
			return;
		}

		this.parentModule = parentUnit;
		super.setParentModule(parentUnit);

		for (int i = 0; i < sortClauses.size(); i++)
			 ((XQueryExpression) sortClauses.get(i)).setParentModule(parentUnit);
	}

	public ArrayList getSortClauses() {
		return sortClauses;
	}

	public void setSortClauses(ArrayList sortClauses) throws XQueryException {
		// cannot be null or empty
		if ((sortClauses == null) || sortClauses.isEmpty()) {
			throw new XQueryException("sortClauses of SortedExpression cannot be null or empty");
		}

		this.sortClauses = sortClauses;

		for (int i = 0; i < this.sortClauses.size(); i++) {
			XQueryExpression expr = (XQueryExpression) this.sortClauses.get(i);
			expr.setParentModule(parentModule);
			expr.setParentExpression(this);
		}
	}

	// W3C WD 16 August 2002
	public void setStable(boolean stable) {
		this.stable = stable;
	}

	public boolean getStable() {
		return stable;
	}

	// #############################################################################
	// CLONE STUFF
	// #############################################################################
	public void addParentExpression(XQueryExpression parentExpression) {
		addParentExpression(parentExpression);
		expression.addParentExpression(parentExpression);

		for (int i = 0; i < sortClauses.size(); i++)
			 ((XQueryExpression) sortClauses.get(i)).addParentExpression(parentExpression);
	}

}
