/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

/*
 * ValidateExpression.java
 *
 * Created on 26 septembre 2002, 17:22
 */

package org.xquark.xquery.parser;

import org.xquark.xquery.parser.util.Constants;
import org.xquark.xquery.typing.TypeException;


public class ValidateExpression extends XQueryUnaryOperatorExpression implements Cloneable {
	
	private static final String RCSRevision = "$Revision: 1.9 $";
	private static final String RCSName     = "$Name:  $";
	
	protected int scopeType = Constants.NOTHING;
	protected int validationType = Constants.NOTHING;
	protected SchemaContextPath schemaContext = null;
	
	// #############################################################################
	// VISITOR STUFF
	// #############################################################################
	
	public void accept(ParserVisitor visitor) throws XQueryException {
		visitor.visit(this);
	}
	
	// #############################################################################
	// CONSTRUCTOR STUFF
	// #############################################################################
	
	public ValidateExpression(int scopeType, int validationType, SchemaContextPath schemaContext, XQueryExpression validateExpression, XQueryModule parentModule) throws TypeException, XQueryException {
		super(validateExpression);
		setScopeType(scopeType);
		setValidationType(validationType);
		setSchemaContext(schemaContext);
		setParentModule(parentModule);
		if (parentModule != null && parentModule.getStaticContext().getTypeVisitor() != null)
			accept(parentModule.getStaticContext().getTypeVisitor());
	}
	
	// #############################################################################
	// ATTRIBUTE ACCESS STUFF
	// #############################################################################
	
	public void setParentModule(XQueryModule parentUnit) {
		if (parentUnit == null) return;
		this.parentModule = parentUnit;
		super.setParentModule(parentUnit);
		if (schemaContext != null) schemaContext.setParentModule(parentUnit);
		expression.setParentModule(parentUnit);
	}
	
	public void setSchemaContext(SchemaContextPath schemaContext) {
		this.schemaContext = schemaContext;
		if (schemaContext != null) {
			this.schemaContext.setParentModule(parentModule);
			this.schemaContext.setParentExpression(this);
		}
	}
	
	public SchemaContextPath getSchemaContext() { return schemaContext; }
			
	public void setScopeType(int scopeType) {
		this.scopeType = scopeType;
	}
	
	public int getScopeType() {
		return scopeType;
	}

	public void setValidationType(int validationType) {
		this.validationType = validationType;
	}
	
	public int getValidationType() {
		return validationType;
	}
	
	
}
