/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * Copyright (C) 2003 XQuark Group.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */


package  org.xquark.xquery.parser;

import org.xquark.xquery.typing.TypeException;


public abstract class Value extends XQueryExpression implements Cloneable {
    private static final String RCSRevision = "$Revision: 1.8 $";
    private static final String RCSName     = "$Name:  $";
    
    // cannot be null or empty string
    protected String value = null;
    
    // #############################################################################
    // VISITOR STUFF
    // #############################################################################
    
    public void accept(ParserVisitor visitor) throws XQueryException {
        visitor.visit(this);
    }
    
    // #############################################################################
    // CONSTRUCTOR STUFF
    // #############################################################################
    
    public Value(String value) throws TypeException, XQueryException {
        setValue(value);
    }
    
    // #############################################################################
    // ATTRIBUTE SETTING STUFF
    // #############################################################################
    
    public String getValue() { return value; }
    abstract public Object getObjectValue();
    public void setValue(String value) throws XQueryException {
        // cannot be null or empty string
        if (value == null)
        	throw new XQueryException("value of Value cannot be null");
        // if (value == null || value.equals("")) throw new Exception("value of Value cannot be null or empty string");
        this.value = value;
    }
        
}



