/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */


package  org.xquark.xquery.parser;

import org.xquark.xquery.typing.TypeException;


public class ValueInteger extends Value implements Cloneable {
    private static final String RCSRevision = "$Revision: 1.7 $";
    private static final String RCSName     = "$Name:  $";
    
    
    // #############################################################################
    // VISITOR STUFF
    // #############################################################################
    
    public void accept(ParserVisitor visitor) throws XQueryException {
        visitor.visit(this);
    }
    
    // #############################################################################
    // CONSTRUCTOR STUFF
    // #############################################################################
    
    public ValueInteger(String value, XQueryModule parentModule) throws TypeException, XQueryException {
		super(value);
		setValue(value);
		setParentModule(parentModule);
		if (parentModule != null && parentModule.getStaticContext().getTypeVisitor() != null)
			accept(parentModule.getStaticContext().getTypeVisitor());
    }
    
    // #############################################################################
    // ATTRIBUTE ACCESS STUFF
    // #############################################################################
    
    public Integer getIntValue() { return new Integer(value); }
	public Object getObjectValue() { return getIntValue(); } 
    public void setValue(String value) throws XQueryException {
        // must be an integer
        try {
            Integer.parseInt(value);
        }
        catch (NumberFormatException e) {
            throw new XQueryException("value of valueInteger must be a valid Integer");
        }
        this.value = value;
    }
    public void setValue(int value) throws XQueryException {
        setValue(String.valueOf(value));
    }
    
}
