/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * Copyright (C) 2003 XQuark Group.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.parser;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Set;

import org.xquark.schema.SchemaManager;
import org.xquark.util.NamespaceContextStack;
import org.xquark.xquery.normalize.PutSkolemIDsVisitor;
import org.xquark.xquery.normalize.SubstituteVisitor;
import org.xquark.xquery.parser.util.Constants;
import org.xquark.xquery.typing.QType;
import org.xquark.xquery.typing.TypeException;
import org.xquark.xquery.typing.TypeVisitor;

public abstract class XQueryExpression implements Cloneable {
	private static final String RCSRevision = "$Revision: 1.51 $";
	private static final String RCSName = "$Name:  $";

	// for printing only
	protected boolean parenthesis = false;
	protected boolean context = false;
	// for ordering
	protected int[] order = null;
	// for typing
	protected ArrayList parentExpression = null;
	//	protected XQueryExpression parentExpression = null;
	protected QType qtype = null; // org.xquark.xml.typing.QType
	protected ArrayList xtrees = null;

	// for reconstruction
	protected ArrayList skolemIDs = null;
	protected ArrayList dependIDs = null;
	protected boolean root = false;
	protected boolean loop = false;

	//protected Set loopIDs = null;
	protected String loopIDs = null;

	// for referencing
	protected XQueryModule parentModule = null;

	// for decomposer : vector of sources matching the path
	protected Set sourceNames = null;
	protected Set urls = null;

	// to avoid building multiple string representations
	protected String stringValue = null;
	//boolean  modified = false;
	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(ParserVisitor visitor) throws XQueryException {
		visitor.visit(this);
	}

	// ******************************************
	// FOR ACCESS TO PRIVATE/PROTECTED ATTRIBUTES
	// ******************************************

	// parentExpression
	public ArrayList getParentExpression() {
		return parentExpression;
	}
	public void setParentExpression(XQueryExpression expression) {
		if (expression == null)
			return;
		if (parentExpression == null)
			parentExpression = new ArrayList(1);
		else
			parentExpression.clear();
		parentExpression.add(expression);
	}
	public void setParentExpressions(ArrayList parentExpression) {
		this.parentExpression = parentExpression;
	}

	// qtype
	public QType getQType() {
		return qtype;
	}
	public void setQType(QType qtype) {
		this.qtype = qtype;
	}

	// qtype
	public ArrayList getXTrees() {
		return xtrees;
	}
	public void setXTrees(ArrayList xtrees) {
		this.xtrees = xtrees;
	}

	//parentUnit
	public XQueryModule getParentModule() {
		return parentModule;
	}
	public void setParentModule(XQueryModule parentModule) {
		if (parentModule == null)
			return;
		this.parentModule = parentModule;
	}

	// reconstruction
	public ArrayList getSkolemIDs() {
		return skolemIDs;
	}
	public void setSkolemIDs(ArrayList skolemIDs) {
		if (this.skolemIDs == null)
			this.skolemIDs = skolemIDs;
		else if (skolemIDs != null) {
			for (int i = 0; i < skolemIDs.size(); i++) {
				Variable tmpVar = (Variable) skolemIDs.get(i);
				if (!(this.skolemIDs.contains(tmpVar)))
					this.skolemIDs.add(0, tmpVar);
			}
		}
	}

	public ArrayList getDependIDs() {
		return dependIDs;
	}
	public void setDependIDs(ArrayList dependIDs) {
		this.dependIDs = dependIDs;
	}

	public boolean getRoot() {
		return root;
	}

	//    public void setRoot(boolean root,boolean force) {
	//        if (force) this.root = root;
	//        else setRoot(root);
	//    }

	public void setRoot(boolean root) {
		if ((skolemIDs == null || skolemIDs.isEmpty()) && (dependIDs == null || dependIDs.isEmpty()))
			this.root = root;
		if (root && dependIDs != null && !dependIDs.isEmpty()) {
			this.loop = true;
			//dependIDs = null;
		}
	}

	public boolean getLoop() {
		return loop;
	}
	//	public void setRoot(boolean loop) { this.loop = loop; }

	//public Set getLoopIDs() { return loopIDs; }
	//public void setLoopIDs(Set loopIDs) { this.loopIDs = loopIDs; }
	public String getLoopIDs() {
		return loopIDs;
	}
	public void setLoopIDs(String loopIDs) {
		this.loopIDs = loopIDs;
	}

	// parenthesis
	public boolean getParenthesis() {
		return parenthesis;
	}
	public void setParenthesis(boolean parenthesis) {
		this.parenthesis = parenthesis;
	}

	// order
	public int[] getOrder() {
		return order;
	}
	public int getOrder(int index) {
		if (order == null)
			return Constants.NOTHING;
		return order[index];
	}
	public void setOrder(int[] order) {
		this.order = order;
	}
	public void setOrder(int order) {
		if (this.order == null) {
			this.order = new int[2];
			this.order[1] = Constants.NOTHING;
		}
		this.order[0] = order;
	}
	// context
	public boolean getContext() {
		return context;
	}
	public void setContext(boolean context) {
		this.context = context;
	}

	// name if exist
	public String getName() {
		return null;
	}

	// sequenceType if exist
	public void setTypeDeclaration(SequenceType typeDeclaration) {
		return;
	}

	// typeName if exist
	public void setTypeName(QName atomicType) {
		return;
	}

	// expression if exist
//	public void setExpression(XQueryExpression expression) throws TypeException, XQueryException {
//		return;
//	}

	// bindingType if exist
	public void setBindingType(int bindingtype) throws XQueryException {
		return;
	}

	// add   27/03/2003
	public XQueryExpression erasePart(XQueryExpression expr) throws XQueryException {
		if (this == expr)
			return null;
		return this;
	}

	// schemaManager
	public SchemaManager getSchemaManager() {
		if (parentModule == null)
			return null;
		return parentModule.getSchemaManager();
	}

	public boolean equals(XQueryExpression expr) {
		/** @todo : It is ugly, but we need equals()!*/
		if (this == expr) {
			return true;
		}
		String thisString = this.toString();
		String exprString = expr.toString();
		if (thisString.equals(exprString)) {
			return true;
		}
		return false;
	}

	public ArrayList getSteps() {
		return null;
	}

	//	public void isModified() {
	//		modified = true;
	//	}

	public String toString(boolean indent, boolean doNodeAccessor, boolean doPrefix) {
//		PrintVisitor pv = new PrintVisitor((parentModule == null) ? null : parentModule.getDeclarations());
//		pv.reset(indent, false, doNodeAccessor);
		PrintVisitor pv = new PrintVisitor();
		pv.reset(indent, false, doNodeAccessor, doPrefix);
		try {
			this.accept(pv);
		} catch (XQueryException qe) {
		}
		return pv.getString();
	}
	public String toString() {
		return toString(false, false, true);
	}

	public String getStringValue() {
		return getStringValue(false, true);
	}
	public String getStringValue(boolean doNodeAccessor, boolean doPrefix) {
		if (/*modified ||*/
			doNodeAccessor ||
			stringValue == null) {
			stringValue = toString(false, doNodeAccessor, doPrefix);
			//modified = false;
		}
		return stringValue;
	}
	
	// TODO make this better
	// special to wrote query for SQLWrapper
	public String getStringValue(NamespaceContextStack context) {
		PrintVisitor pv = new PrintVisitor();
		pv.reset(false, false, true, true);
		pv.setNamespaceContextStack(context);
		try {
			this.accept(pv);
		} catch (XQueryException qe) {
		}
		return pv.getString();
	}

	public void resetStringValue() {
		stringValue = null;
	}

	// #############################################################################
	// CLONE STUFF
	// #############################################################################

	public void addParentExpression(XQueryExpression expression) {
		if (expression == null)
			return;
		if (parentExpression == null)
			parentExpression = new ArrayList(1);
		parentExpression.add(expression);
	}

	public XQueryExpression shallowClone() {
		try {
			return (XQueryExpression) super.clone();
		} catch (CloneNotSupportedException e) {
		}
		return null;
	}

	public Object clone(boolean renameVars) throws CloneNotSupportedException {
		try {
			CloneVisitor cloneVisitor = new CloneVisitor(parentModule.getStaticContext(), renameVars);
			this.accept(cloneVisitor);
			return cloneVisitor.getClone();
		} catch (XQueryException xqe) {
			throw new CloneNotSupportedException("Could not clone : " + this);
		}
	}

	public Object clone() throws CloneNotSupportedException {
		try {
			CloneVisitor cloneVisitor = new CloneVisitor(parentModule.getStaticContext(), true);
			this.accept(cloneVisitor);
			return cloneVisitor.getClone();
		} catch (XQueryException xqe) {
			throw new CloneNotSupportedException("Could not clone : " + this);
		}
	}

	public void setSourceNames(Set sources) {
		this.sourceNames = sources;
	}

	public Set getSourceNames() {
		return sourceNames;
	}

	public void setUrls(Set urls) {
		this.urls = urls;
	}

	public Set getUrls() {
		return urls;
	}

	public boolean startsWithVariable() {
		return false;
	}

	// #############################################################################
	// RESTRUCTURE STUFF
	// #############################################################################

	public void substitute(HashMap map, TypeVisitor typeVisitor) throws XQueryException {
		SubstituteVisitor substituteVisitor = new SubstituteVisitor(map, typeVisitor);
		accept(substituteVisitor);
	}

	public void putSkolemIDs(ArrayList ids) throws XQueryException {
		PutSkolemIDsVisitor putSkolemIDsVisitor = new PutSkolemIDsVisitor(ids);
		accept(putSkolemIDsVisitor);
	}

}
