/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.parser;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;

import org.xquark.xquery.typing.TypeException;

public class XQueryExpressionSequence extends XQueryExpression {
	private static final String RCSRevision = "$Revision: 1.10 $";
	private static final String RCSName = "$Name:  $";

	ArrayList subExpressions = null; // ArrayList of XQueryExpression

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(ParserVisitor visitor) throws XQueryException {
		visitor.visit(this);
	}

	// #############################################################################
	// CONSTRUCTOR STUFF
	// #############################################################################

	public XQueryExpressionSequence(ArrayList subExpressions, XQueryModule parentModule) throws TypeException, XQueryException {
		setSubExpressions(subExpressions);
		setParentModule(parentModule);
		if (parentModule != null && parentModule.getStaticContext().getTypeVisitor() != null)
			accept(parentModule.getStaticContext().getTypeVisitor());
	}

	// #############################################################################
	// ATTRIBUTE ACCESS STUFF
	// #############################################################################

	public ArrayList getSubExpressions() {
		return subExpressions;
	}
	public void setSubExpressions(ArrayList subExpressions) throws XQueryException {
		if (subExpressions == null || subExpressions.isEmpty())
			throw new XQueryException("subExpressions of XQueryExpressionSequence cannot be null or empty");
		this.subExpressions = subExpressions;
		for (int j = 0; j < this.subExpressions.size(); j++) {
			XQueryExpression expression = (XQueryExpression) this.subExpressions.get(j);
			expression.setParentExpression(this);
			expression.setParentModule(parentModule);
		}
	}

	public void addSubExpressions(XQueryExpression subExpression) throws XQueryException {
		subExpressions.add(subExpression);
		subExpression.setParentExpression(this);
		subExpression.setParentModule(parentModule);
	}

	public void addSubExpressions(int index, XQueryExpression subExpression) throws XQueryException {
		subExpressions.add(index, subExpression);
		subExpression.setParentExpression(this);
		subExpression.setParentModule(parentModule);
	}

	public Set getSourceNames() {
		if (subExpressions == null)
			return null;
		Set tmplist = null;
		for (int i = 0; i < subExpressions.size(); i++) {
			XQueryExpression expressioni = (XQueryExpression) subExpressions.get(i);
			if (expressioni.getSourceNames() != null) {
				if (tmplist == null)
					tmplist = new HashSet(expressioni.getSourceNames());
				else
					tmplist.addAll(expressioni.getSourceNames());
			}
		}
		return tmplist;
	}

	public Set getUrls() {
		if (subExpressions == null)
			return null;
		Set tmplist = null;
		for (int i = 0; i < subExpressions.size(); i++) {
			XQueryExpression expressioni = (XQueryExpression) subExpressions.get(i);
			if (expressioni.getUrls() != null) {
				if (tmplist == null)
					tmplist = new HashSet(expressioni.getUrls());
				else
					tmplist.addAll(expressioni.getUrls());
			}
		}
		return tmplist;
	}

	// #############################################################################
	// CLONE STUFF
	// #############################################################################
	public void addParentExpression(XQueryExpression parentExpression) {
		addParentExpression(parentExpression);
		for (int j = 0; j < subExpressions.size(); j++)
			 ((XQueryExpression) subExpressions.get(j)).addParentExpression(parentExpression);
	}

}
