/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

/*
 * CData.java
 *
 * Created on 25 juin 2002, 15:46
 */

package org.xquark.xquery.parser.hinttree;

import java.util.ArrayList;

import org.xquark.xquery.parser.Variable;

public class HintTree extends Object {
    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName     = "$Name:  $";
    
    public static final byte NO_SIDE = 0;
    public static final byte LEFT_SIDE = 1;
    public static final byte RIGHT_SIDE = 2;
    public static final byte NO_TYPE = 0;
    public static final byte NESTED_TYPE = 1;
    public static final byte JOIN_TYPE = 2;
	public static final byte MERGE_TYPE = 3;
	public static final byte CARTESIAN_PRODUCT_TYPE = 4;
	public static final byte NESTED_LOOP_TYPE = 5;
    
    // can be null
    protected Object left = null;
    protected Object right = null;
    protected byte type = NO_TYPE;
    protected byte side = NO_SIDE;
    protected byte outertype = NO_TYPE;
    protected ArrayList varList  = null;

    // #############################################################################
    // VISITOR STUFF
    // #############################################################################
    
    // #############################################################################
    // CONSTRUCTOR STUFF
    // #############################################################################
    
    public HintTree(byte type, byte side, Object left, Object right) {
        setType(type);
        setSide(side);
        setLeft(left);
        setRight(right);
    }
    
    // #############################################################################
    // ATTRIBUTE ACCESS STUFF
    // #############################################################################
    
    public byte getType() {
        return type;
    }
    
    public byte getOuterType() {
        return outertype;
    }

    public byte getSide() {
        return side;
    }
    
    public Object getLeftHint() {
        return left;
    }

    public Object getRightHint() {
        return right;
    }

    public void setType(byte type) {
        this.type = type;
    }
    
    public void setOuterType(byte outertype) {
        this.outertype = outertype;
    }

    public void setSide(byte side) {
        this.side = side;
    }
    
    public void setLeft(Object left) {
        this.left = left;
        if (left == null) return;
        if (left instanceof Variable)
        	this.addVarList((Variable)left);
        else
        	this.addVarsList(((HintTree)left).getVarList());
    }

    public void setRight(Object right) {
        this.right = right;
        if (right == null) return;
        if (right instanceof Variable)
        	this.addVarList((Variable)right);
        else
        	this.addVarsList(((HintTree)right).getVarList());
    }
    
    public void setVarList(ArrayList varList) {
    	this.varList = varList;
    }
    
    public ArrayList getVarList() {
    	return this.varList;
    }
    
    public boolean hasMerge() {    	
    	if (type == MERGE_TYPE)
    		return true;
    	boolean has = false;
    	if (left instanceof HintTree) 
    		has = ((HintTree)left).hasMerge();
    	if (has)
    		return true;
    	if (right instanceof HintTree) 
    		has = ((HintTree)right).hasMerge();
    	return has;
    }

    private void addVarList(Variable var) {
    	if (this.varList == null) this.varList = new ArrayList(1);
    	this.varList.add(var);
    }

    private void addVarsList(ArrayList vars) {
    	if (this.varList == null) this.varList = (ArrayList)vars.clone();
    	else this.varList.addAll(vars);
    }
    
    // #############################################################################
    // PRINT STUFF
    // #############################################################################
    
    public String toString() {
        StringBuffer buf = new StringBuffer();
        switch (type) {
        	case NESTED_TYPE : buf.append("nested"); break;
        	case JOIN_TYPE : buf.append("join"); break;
        	case MERGE_TYPE : buf.append("merge"); break;
        	case CARTESIAN_PRODUCT_TYPE : buf.append("cartesian-product"); break;
        	case NESTED_LOOP_TYPE : buf.append("nested-loop"); break;
        	default : buf.append("ERROR");
        }
        buf.append("(");
        buf.append(left);
        buf.append(" , ");
        buf.append(right);
        buf.append(")");
        return buf.toString();
    }
    
    // #############################################################################
    // CLONE STUFF
    // #############################################################################
    
    
}
