/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * Copyright (C) 2003 XQuark Group.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.typing;

import java.util.List;

import org.xquark.schema.SimpleType;
import org.xquark.schema.Type;
import org.xquark.schema.datatypes.PrimitiveType;
import org.xquark.xquery.parser.XQueryExpression;

public abstract class QType implements QTypeVisitable, Cloneable {
	private static final String RCSRevision = "$Revision: 1.10 $";
	private static final String RCSName = "$Name:  $";

	// occurence types
	public static final byte OCC_0_0 = 0;
	public static final byte OCC_0_1 = 1;
	public static final byte OCC_1_1 = 2;
	public static final byte OCC_0_N = 3;
	public static final byte OCC_1_N = 4;

	// Qtype types
	public static final byte ELEMENT = 1;
	public static final byte ATOM = 2;
	public static final byte ATTRIBUTE = 3;
	public static final byte UNION = 4;
	public static final byte SEQUENCE = 5;
	public static final byte COMMENT = 6;
	public static final byte PROCESSING = 7;
	public static final byte CDATA = 8;
	public static final byte DOCUMENT = 9;
	public static final byte TEXT = 10;

	// primitive XQuery Types

	public static final byte PRIMITIVE_anySimpleType = 0;
	public static final byte PRIMITIVE_string = 1;
	public static final byte PRIMITIVE_float = 2;
	public static final byte PRIMITIVE_double = 3;
	public static final byte PRIMITIVE_decimal = 4;
	public static final byte PRIMITIVE_duration = 5;
	public static final byte PRIMITIVE_yearMonthDuration = 6;
	public static final byte PRIMITIVE_dayTimeDuration = 7;
	public static final byte PRIMITIVE_dateTime = 8;
	public static final byte PRIMITIVE_time = 9;
	public static final byte PRIMITIVE_date = 10;
	public static final byte PRIMITIVE_gYearMonth = 11;
	public static final byte PRIMITIVE_gYear = 12;
	public static final byte PRIMITIVE_gMonthDay = 13;
	public static final byte PRIMITIVE_gDay = 14;
	public static final byte PRIMITIVE_gMonth = 15;
	public static final byte PRIMITIVE_boolean = 16;
	public static final byte PRIMITIVE_base64Binary = 17;
	public static final byte PRIMITIVE_hexBinary = 18;
	public static final byte PRIMITIVE_anyURI = 19;
	public static final byte PRIMITIVE_Qname = 20;
	public static final byte PRIMITIVE_NOTATION = 21;

	/*
	
	aST = anySimpleType
	aURI = anyURI
	b64 = base64Binary
	bool = boolean
	dat = date
	Day = gDay
	dbl = double
	dec = decimal
	dT = dateTime
	dTD = dayTimeDuration
	dur = duration
	flt = float
	hxB = hexBinary
	MD = gMonthDay
	Mon = gMonth
	NOT = NOTATION
	QN = Qname
	str = string
	tim = time
	YM = gYearMonth
	yMD = yearMonthDuration
	Yr = gYear
	
	S\T  aST str flt dbl dec dur yMD dTD dT tim dat YM Yr MD Day Mon bool b64 hxB aURI QN NOT 
	aST  Y   Y   M   M   M   M   M   M   M  M   M   M  M  M  M   M   M    M   M   M    N  N 
	str  Y   Y   M   M   M   M   M   M   M  M   M   M  M  M  M   M   M    M   M   M    N  N 
	flt  Y   Y   Y   Y   M   N   N   N   N  N   N   N  N  N  N   N   Y    N   N   N    N  N 
	dbl  Y   Y   Y   Y   M   N   N   N   N  N   N   N  N  N  N   N   Y    N   N   N    N  N 
	dec  Y   Y   Y   Y   Y   N   N   N   N  N   N   N  N  N  N   N   Y    N   N   N    N  N 
	dur  Y   Y   N   N   N   Y   Y   Y   N  N   N   N  N  N  N   N   N    N   N   N    N  N 
	yMD  Y   Y   N   N   N   Y   Y   N   N  N   N   N  N  N  N   N   N    N   N   N    N  N 
	dTD  Y   Y   N   N   N   Y   N   Y   N  N   N   N  N  N  N   N   N    N   N   N    N  N 
	dT   Y   Y   N   N   N   N   N   N   Y  Y   Y   Y  Y  Y  Y   Y   N    N   N   N    N  N 
	tim  Y   Y   N   N   N   N   N   N   Y  Y   N   N  N  N  N   N   N    N   N   N    N  N 
	dat  Y   Y   N   N   N   N   N   N   Y  N   Y   Y  Y  Y  Y   Y   N    N   N   N    N  N 
	YM   Y   Y   N   N   N   N   N   N   N  N   N   Y  N  N  N   N   N    N   N   N    N  N 
	Yr   Y   Y   N   N   N   N   N   N   N  N   N   N  Y  N  N   N   N    N   N   N    N  N 
	MD   Y   Y   N   N   N   N   N   N   N  N   N   N  N  Y  N   N   N    N   N   N    N  N 
	Day  Y   Y   N   N   N   N   N   N   N  N   N   N  N  N  Y   N   N    N   N   N    N  N 
	Mon  Y   Y   N   N   N   N   N   N   N  N   N   N  N  N  N   Y   N    N   N   N    N  N 
	bool Y   Y   Y   Y   Y   N   N   N   N  N   N   N  N  N  N   N   Y    N   N   N    N  N 
	b64  Y   Y   N   N   N   N   N   N   N  N   N   N  N  N  N   N   M    Y   N   N    N  N 
	hxB  Y   Y   M   M   M   N   N   N   N  N   N   N  N  N  N   N   M    N   Y   N    N  N 
	aURI Y   Y   N   N   N   N   N   N   N  N   N   N  N  N  N   N   N    N   N   Y    N  N 
	QN   N   N   N   N   N   N   N   N   N  N   N   N  N  N  N   N   N    N   N   N    Y  N 
	NOT  N   N   N   N   N   N   N   N   N  N   N   N  N  N  N   N   N    N   N   N    N  Y 
	*/
	// NOTE : Maybe is Yes with restrictions!
	public static final boolean castTab[][] = { {
			//aST    str     flt     dbl    dec    dur     yMD     dTD     dT     tim     dat     YM     Yr      MD      Day    Mon    bool    b64     hxB    aURI   QN     NOT 
			/*aST*/
			true, true, true, true, true, true, true, true, true, true, true, true, true, true, true, true, true, true, true, true, false, false }, {
			/*str*/
			true, true, true, true, true, true, true, true, true, true, true, true, true, true, true, true, true, true, true, true, false, false }, {
			/*flt*/
			true, true, true, true, true, false, false, false, false, false, false, false, false, false, false, false, true, false, false, false, false, false }, {
			/*dbl*/
			true, true, true, true, true, false, false, false, false, false, false, false, false, false, false, false, true, false, false, false, false, false }, {
			/*dec*/
			true, true, true, true, true, false, false, false, false, false, false, false, false, false, false, false, true, false, false, false, false, false }, {
			/*dur*/
			true, true, false, false, false, true, true, true, false, false, false, false, false, false, false, false, false, false, false, false, false, false }, {
			/*yMD*/
			true, true, false, false, false, true, true, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false }, {
			/*dTD*/
			true, true, false, false, false, true, false, true, false, false, false, false, false, false, false, false, false, false, false, false, false, false }, {
			/*dT*/
			true, true, false, false, false, false, false, false, true, true, true, true, true, true, true, true, false, false, false, false, false, false }, {
			/*tim*/
			true, true, false, false, false, false, false, false, true, true, false, false, false, false, false, false, false, false, false, false, false, false }, {
			/*dat*/
			true, true, false, false, false, false, false, false, true, false, true, true, true, true, true, true, false, false, false, false, false, false }, {
			/*YM*/
			true, true, false, false, false, false, false, false, false, false, false, true, false, false, false, false, false, false, false, false, false, false }, {
			/*Yr*/
			true, true, false, false, false, false, false, false, false, false, false, false, true, false, false, false, false, false, false, false, false, false }, {
			/*MD*/
			true, true, false, false, false, false, false, false, false, false, false, false, false, true, false, false, false, false, false, false, false, false }, {
			/*Day*/
			true, true, false, false, false, false, false, false, false, false, false, false, false, false, true, false, false, false, false, false, false, false }, {
			/*Mon*/
			true, true, false, false, false, false, false, false, false, false, false, false, false, false, false, true, false, false, false, false, false, false }, {
			/*bool*/
			true, true, true, true, true, false, false, false, false, false, false, false, false, false, false, false, true, false, false, false, false, false }, {
			/*b64*/
			true, true, false, false, false, false, false, false, false, false, false, false, false, false, false, false, true, true, false, false, false, false }, {
			/*hxB*/
			true, true, true, true, true, false, false, false, false, false, false, false, false, false, false, false, true, false, true, false, false, false }, {
			/*aURI*/
			true, true, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, true, false, false }, {
			/*QN*/
			false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, true, false }, {
			/*NOT*/
			false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, true }
	};

	protected byte subclass = 0;
	protected byte occurence = OCC_1_1;
	//	protected int min = 1;
	//	protected int max = 1;

	public static byte computeOccurence(byte occ1, byte occ2) {
		switch (occ1) {
			case OCC_1_1 :
				return occ2;
			case OCC_0_1 :
				switch (occ2) {
					case OCC_1_1 :
					case OCC_0_1 :
						return OCC_0_1;
					case OCC_1_N :
					case OCC_0_N :
						return OCC_0_N;
				}
			case OCC_1_N :
				switch (occ2) {
					case OCC_1_1 :
					case OCC_1_N :
						return OCC_1_N;
					case OCC_0_1 :
					case OCC_0_N :
						return OCC_0_N;
				}
			default:
				return occ1;
		}
	}
	
	public static byte getOccurence(int min, int max) {
		if (min > 0) {
			if (max > 1)
				return QType.OCC_1_N;
			else
				return QType.OCC_1_1;
		} else {
			if (max > 1)
				return QType.OCC_0_N;
			else
				return QType.OCC_0_1;
		}
	}

	// methods on occurence
	public byte getOccurence() {
		return occurence;
	}
	public QType changeOccurence(byte occurence) {
		QType qtype = null;
		try {
			qtype = (QType) clone();
		} catch (CloneNotSupportedException cnse) {}
		qtype.occurence = occurence;
		return qtype;
	}
	public int getMaxOccurence() {
		switch (occurence) {
			case OCC_0_0 :
				return 0;
			case OCC_1_1 :
			case OCC_0_1 :
				return 1;
			case OCC_0_N :
			case OCC_1_N :
				return Integer.MAX_VALUE;
		}
		return -1;
	}
	public int getMinOccurence() {
		switch (occurence) {
			case OCC_0_N :
			case OCC_0_1 :
			case OCC_0_0 :
				return 0;
			case OCC_1_1 :
			case OCC_1_N :
				return 1;
		}
		return -1;
	}

	public byte getSubClass() {
		return subclass;
	}

	//	abstract public String toSimpleString();
	//	abstract public String toString();
	//	abstract public boolean equals(QType qtype);
	//
	public SimpleType getSimpleType() {
		return null;
	}
	public Type getType() {
		return null;
	}
	//	public QType getQType() {
	//		return null;
	//	}
	public List getList() {
		return null;
	}
	public XQueryExpression getName() {
		return null;
	}
	//	public QType getCollectionType() {
	//		return null;
	//	}

	abstract public Object clone() throws CloneNotSupportedException;

	// tells whether qtype is numeric ( DECIMAL, DOUBLE, FLOAT )
	// QTypeAtom with type numeric
	// QTypeAttribute with type numeric
	// QTypeElement with type numeric
	// QTypeCollection with type verifying isNumeric
	// QTypeSequence with all types verifying isNumeric
	// QTypeUnion with at leat one type verifying isNumeric
	public boolean isNumeric() {
		return false;
	}

	// tells whether qtype is boolean
	// QTypeAtom with type boolean
	// QTypeAttribute with type boolean
	// QTypeElement with type boolean
	// QTypeCollection with type verifying isBoolean
	// QTypeSequence with all types verifying isBoolean
	// QTypeUnion with at leat one type verifying isBoolean
	public boolean isBoolean() {
		return false;
	}

	// tells whether qtype is integer
	// QTypeAtom with type integer
	// QTypeAttribute with type integer
	// QTypeElement with type integer
	// QTypeCollection with type verifying isInteger
	// QTypeSequence with all types verifying isInteger
	// QTypeUnion with at leat one type verifying isInteger
	public boolean isInteger() {
		return false;
	}

	// tells whether qtype is string
	// QTypeAtom with type string
	// QTypeAttribute with type string
	// QTypeElement with type string
	// QTypeCollection with type verifying isString
	// QTypeSequence with all types verifying isString
	// QTypeUnion with at leat one type verifying isString
	public boolean isString() {
		return false;
	}

	// tells whether qtype is an atom
	// QTypeAtom
	// QTypeCollection with type verifying isAtom
	// QTypeSequence with all types verifying isAtom
	// QTypeUnion with at leat one type verifying isAtom
	public boolean isAtom() {
		return false;
	}

	// tells whether qtype can react to text()
	// QTypeAttribute
	// QTypeElement
	// QTypeCollection with type verifying isText
	// QTypeSequence with all types verifying isText
	// QTypeUnion with at leat one type verifying isText
	public boolean isText() {
		return false;
	}

	// tells whether qtype is a date
	// QTypeAtom with type date
	// QTypeAttribute with type date
	// QTypeElement with type date
	// QTypeCollection with type verifying isDate
	// QTypeSequence with all types verifying isDate
	// QTypeUnion with at leat one type verifying isDate
	public boolean isDate() {
		return false;
	}

	// not implemented yet
	public boolean isDuration() {
		return false;
	}

	// tells whether qtype is an attribte
	// QTypeAttribute
	// QTypeCollection with type verifying isAttribute
	// QTypeSequence with all types verifying isAttribute
	// QTypeUnion with at leat one type verifying isAttribute
	public boolean isAttribute() {
		return false;
	}
	// tells whether qtype is only an attribte
	// QTypeAttribute
	// QTypeCollection with type verifying isAttribute
	// QTypeSequence with all types verifying isAttribute
	// QTypeUnion with all type verifying isAttribute
	public boolean isOnlyAttribute() {
		return isAttribute();
	}

	// tells whether qtype is a ID, IDREF or IDREFS type
	// QTypeAttribute with special case
	// QTypeCollection with type verifying isIDAttribute
	// QTypeSequence with all types verifying isIDAttribute
	// QTypeUnion with at leat one type verifying isIDAttribute
	public boolean isIDAttribute() {
		return false;
	}

	// tells whether qtype is a node
	// QTypeAttribute
	// QTypeCData
	// QTypeComment
	// QTypeElement
	// QTypeProcessingInstruction
	// QTypeCollection with type verifying isNode
	// QTypeSequence with all types verifying isNode
	// QTypeUnion with at leat one type verifying isNode
	public boolean isNode() {
		return false;
	}

	// tells whether qtype is a node with simple type
	// QTypeAttribute
	// QTypeElement with simple type (typed value)
	// QTypeCollection with type verifying isSimpleTypeNode
	// QTypeSequence with all types verifying isSimpleTypeNode
	// QTypeUnion with at leat one type verifying isSimpleTypeNode
	public boolean isSimpleTypeNode() {
		return false;
	}

	//	// tells whether qtype is a collection
	//	// QTypeCollection
	//	// QTypeUnion with at leat one type verifying isCollection
	//	public boolean isCollection() {
	//		return false;
	//	}

	// tells whether qtype is a collection
	// QTypeCollection
	// QTypeSequence
	// QTypeUnion with at leat one type verifying isMultiple
	public boolean isMultiple() {
		switch (getOccurence()) {
			case OCC_0_0 :
			case OCC_1_1 :
			case OCC_0_1 :
				return false;
			case OCC_0_N :
			case OCC_1_N :
				return true;
		}
		return false;
	}

	// tells whether qtype is atomic
	// QTypeAtom
	// QTypeAttribute
	// QTypeElement with TypedValue not null
	// QTypeUnion with at leat one type verifying isAtomic        
	public boolean isAtomic() {
		return false;
	}

	// tells whether qtype is mixed
	// QTypeElement with mixed type (complex type mixed=
	// QTypeCollection with type verifying isMixed
	// QTypeSequence with all types verifying isMixed
	// QTypeUnion with at leat one type verifying isMixed
	public boolean isMixed() {
		return false;
	}

	// tells whether qtype is element
	// QTypeElement
	// QTypeCollection with type verifying isElement
	// QTypeSequence with all types verifying isElement
	// QTypeUnion with at leat one type verifying isElement
	public boolean isElement() {
		return false;
	}

	// tells whether qtype is document
	// QTypeDocument
	// QTypeUnion with at leat one type verifying QTypeDocument
	public boolean isDocument() {
		return false;
	}

	// tells whether qtype is qname
	// QTypeAtom with type qname
	public boolean isQName() {
		return false;
	}

	public QType applyDATAFunction() throws TypeException {
		return null;
	}

	public boolean canBeComparedTo(QType compType) {
		return false;
	}

	protected boolean compare(Type type1, Type type2) {
		if (type1 == null || type2 == null)
			return true;
		if (type1 == null && type2 != null)
			return false;
		if (type2 == null && type1 != null)
			return false;
		if (type1 == null || type2 == null || (type1.getName() == null && type2.getName() != null) || (type1.getName() != null && type2.getName() == null))
			return false;
		return ((type1.getName() == null && type2.getName() == null) || (type1.getName() != null && type1.getName().equals(type2.getName()))) && ((type1.getNamespace() == null && type2.getNamespace() == null) || (type1.getNamespace() != null && type1.getNamespace().equals(type2.getNamespace())));
	}


	protected boolean isNumeric(Type type) {
		SimpleType stype = type.getValueType();
		if (stype == null) {
			if (type.getName() != null && type.getName().equals(TypeVisitor.SC_ANYTYPE))
				return true;
			return false;
		}
		switch (stype.getPrimitive().getType()) {
			/*
			 case PrimitiveType.STRING :
			case PrimitiveType.BOOLEAN :
			 */
			case PrimitiveType.DECIMAL :
			case PrimitiveType.DOUBLE :
			case PrimitiveType.FLOAT :
				return true;
				/*
				 case PrimitiveType.DURATION :
				case PrimitiveType.DATE_TIME :
				case PrimitiveType.TIME :
				case PrimitiveType.DATE :
				case PrimitiveType.GYEAR_MONTH :
				case PrimitiveType.GYEAR :
				case PrimitiveType.GMONTH_DAY :
				case PrimitiveType.GDAY :
				case PrimitiveType.GMONTH :
				case PrimitiveType.HEX_BINARY :
				case PrimitiveType.BASE64_BINARY :
				case PrimitiveType.ANY_URI :
				case PrimitiveType.QNAME :
				case PrimitiveType.NOTATION :
				 
				case PrimitiveType.ID :
				case PrimitiveType.IDREF :
				 
				case PrimitiveType.LIST :
				case PrimitiveType.UNION :
				case PrimitiveType.ANYSIMPLETYPE :
				 */
			default :
				break;
		}
		return false;
	}

	protected boolean isInteger(Type type) {
		SimpleType stype = type.getValueType();
		if (stype == null)
			return false;
		switch (stype.getPrimitive().getType()) {
			/*
			 case PrimitiveType.STRING :
			case PrimitiveType.BOOLEAN :
			 */
			case PrimitiveType.DECIMAL :
				if (stype.getPrimitive().getFractionDigits() == 0)
					return true;
				/*
				 case PrimitiveType.DOUBLE :
				case PrimitiveType.FLOAT : return true;
				case PrimitiveType.DURATION :
				case PrimitiveType.DATE_TIME :
				case PrimitiveType.TIME :
				case PrimitiveType.DATE :
				case PrimitiveType.GYEAR_MONTH :
				case PrimitiveType.GYEAR :
				case PrimitiveType.GMONTH_DAY :
				case PrimitiveType.GDAY :
				case PrimitiveType.GMONTH :
				case PrimitiveType.HEX_BINARY :
				case PrimitiveType.BASE64_BINARY :
				case PrimitiveType.ANY_URI :
				case PrimitiveType.QNAME :
				case PrimitiveType.NOTATION :
				 
				case PrimitiveType.ID :
				case PrimitiveType.IDREF :
				 
				case PrimitiveType.LIST :
				case PrimitiveType.UNION :
				case PrimitiveType.ANYSIMPLETYPE :
				 */
			default :
				break;
		}
		return false;
	}

	protected boolean isString(Type type) {
		SimpleType stype = type.getValueType();
		if (stype == null)
			return false;
		switch (stype.getPrimitive().getType()) {
			case PrimitiveType.STRING :
			case PrimitiveType.ANYSIMPLETYPE :
				return true;
				/*
				 case PrimitiveType.BOOLEAN :
				case PrimitiveType.DECIMAL :
				case PrimitiveType.DOUBLE :
				case PrimitiveType.FLOAT :
				case PrimitiveType.DURATION :
				case PrimitiveType.DATE_TIME :
				case PrimitiveType.TIME :
				case PrimitiveType.DATE :
				case PrimitiveType.GYEAR_MONTH :
				case PrimitiveType.GYEAR :
				case PrimitiveType.GMONTH_DAY :
				case PrimitiveType.GDAY :
				case PrimitiveType.GMONTH :
				case PrimitiveType.HEX_BINARY :
				case PrimitiveType.BASE64_BINARY :
				case PrimitiveType.ANY_URI :
				case PrimitiveType.QNAME :
				case PrimitiveType.NOTATION :
				 
				case PrimitiveType.ID :
				case PrimitiveType.IDREF :
				 
				case PrimitiveType.LIST :
				case PrimitiveType.UNION :
				case PrimitiveType.ANYSIMPLETYPE :
				 **/
			default :
				break;
		}
		return false;
	}

	protected boolean isDate(Type type) {
		SimpleType stype = type.getValueType();
		if (stype == null)
			return false;
		switch (stype.getPrimitive().getType()) {
			/*
			 case PrimitiveType.STRING :
			case PrimitiveType.BOOLEAN :
			case PrimitiveType.DECIMAL :
			case PrimitiveType.DOUBLE :
			case PrimitiveType.FLOAT :
			case PrimitiveType.DURATION :
			 */
			case PrimitiveType.DATE_TIME :
				// case PrimitiveType.TIME :
			case PrimitiveType.DATE :
				return true;
				/*
				 case PrimitiveType.GYEAR_MONTH :
				case PrimitiveType.GYEAR :
				case PrimitiveType.GMONTH_DAY :
				case PrimitiveType.GDAY :
				case PrimitiveType.GMONTH :
				case PrimitiveType.HEX_BINARY :
				case PrimitiveType.BASE64_BINARY :
				case PrimitiveType.ANY_URI :
				case PrimitiveType.QNAME :
				case PrimitiveType.NOTATION :
				 
				case PrimitiveType.ID :
				case PrimitiveType.IDREF :
				 
				case PrimitiveType.LIST :
				case PrimitiveType.UNION :
				case PrimitiveType.ANYSIMPLETYPE :
				 */
			default :
				break;
		}
		return false;
	}

	protected boolean isBoolean(Type type) {
		SimpleType stype = type.getValueType();
		if (stype == null)
			return false;
		switch (stype.getPrimitive().getType()) {
			// case PrimitiveType.STRING :
			case PrimitiveType.BOOLEAN :
				return true;
				/*
				 case PrimitiveType.DECIMAL :
				case PrimitiveType.DOUBLE :
				case PrimitiveType.FLOAT :
				case PrimitiveType.DURATION :
				case PrimitiveType.DATE_TIME :
				case PrimitiveType.TIME :
				case PrimitiveType.DATE :
				case PrimitiveType.GYEAR_MONTH :
				case PrimitiveType.GYEAR :
				case PrimitiveType.GMONTH_DAY :
				case PrimitiveType.GDAY :
				case PrimitiveType.GMONTH :
				case PrimitiveType.HEX_BINARY :
				case PrimitiveType.BASE64_BINARY :
				case PrimitiveType.ANY_URI :
				case PrimitiveType.QNAME :
				case PrimitiveType.NOTATION :
				 
				case PrimitiveType.ID :
				case PrimitiveType.IDREF :
				 
				case PrimitiveType.LIST :
				case PrimitiveType.UNION :
				case PrimitiveType.ANYSIMPLETYPE :
				 */
			default :
				break;
		}
		return false;
	}

	protected boolean isQName(Type type) {
		SimpleType stype = type.getValueType();
		if (stype == null)
			return false;
		switch (stype.getPrimitive().getType()) {
			case PrimitiveType.STRING :
			/*
			case PrimitiveType.BOOLEAN :
			case PrimitiveType.DECIMAL :
			case PrimitiveType.DOUBLE :
			case PrimitiveType.FLOAT :
			case PrimitiveType.DURATION :
			case PrimitiveType.DATE_TIME :
			case PrimitiveType.TIME :
			case PrimitiveType.DATE :
			case PrimitiveType.GYEAR_MONTH :
			case PrimitiveType.GYEAR :
			case PrimitiveType.GMONTH_DAY :
			case PrimitiveType.GDAY :
			case PrimitiveType.GMONTH :
			case PrimitiveType.HEX_BINARY :
			case PrimitiveType.BASE64_BINARY :
			case PrimitiveType.ANY_URI :
			*/
			case PrimitiveType.QNAME :
				return true;
			/*
			case PrimitiveType.NOTATION :
			 
			case PrimitiveType.ID :
			case PrimitiveType.IDREF :
			 
			case PrimitiveType.LIST :
			case PrimitiveType.UNION :
			case PrimitiveType.ANYSIMPLETYPE :
			*/
			default :
				break;
		}
		return false;
	}
	
	protected boolean canBeCompared(int primitiveType1, int primitiveType2) {
		if (primitiveType1 == PrimitiveType.ANYSIMPLETYPE || primitiveType2 == PrimitiveType.ANYSIMPLETYPE)
			return true;
		if (primitiveType1 == primitiveType2)
			return true;
		switch (primitiveType1) {
			// case PrimitiveType.STRING :
			// case PrimitiveType.BOOLEAN :
			case PrimitiveType.DECIMAL :
			case PrimitiveType.DOUBLE :
			case PrimitiveType.FLOAT :
				if (primitiveType2 == PrimitiveType.DECIMAL)
					return true;
				if (primitiveType2 == PrimitiveType.DOUBLE)
					return true;
				if (primitiveType2 == PrimitiveType.FLOAT)
					return true;
				break;
				// case PrimitiveType.DURATION :
			case PrimitiveType.DATE_TIME :
			case PrimitiveType.TIME :
			case PrimitiveType.DATE :
			case PrimitiveType.GYEAR_MONTH :
			case PrimitiveType.GYEAR :
			case PrimitiveType.GMONTH_DAY :
			case PrimitiveType.GDAY :
			case PrimitiveType.GMONTH :
				if (primitiveType2 == PrimitiveType.DATE_TIME)
					return true;
				if (primitiveType2 == PrimitiveType.TIME)
					return true;
				if (primitiveType2 == PrimitiveType.DATE)
					return true;
				if (primitiveType2 == PrimitiveType.GYEAR_MONTH)
					return true;
				if (primitiveType2 == PrimitiveType.GYEAR)
					return true;
				if (primitiveType2 == PrimitiveType.GMONTH_DAY)
					return true;
				if (primitiveType2 == PrimitiveType.GDAY)
					return true;
				if (primitiveType2 == PrimitiveType.GMONTH)
					return true;
				break;
				// case PrimitiveType.HEX_BINARY :
				// case PrimitiveType.BASE64_BINARY :
				// case PrimitiveType.ANY_URI :
				// case PrimitiveType.QNAME :
				// case PrimitiveType.NOTATION :
				// case PrimitiveType.ID :
				// case PrimitiveType.IDREF :
				// case PrimitiveType.LIST :
				// case PrimitiveType.UNION :
				// case PrimitiveType.ANYSIMPLETYPE :
			default :
				break;
		}
		return false;
	}

	public boolean canBeCastedInto(int castType) {
		return false;
	}

	public int getCastType() {
		return -1;
	}

}
