/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.xdbc;

import java.io.StringReader;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xquark.xml.xdbc.XMLDBCException;

public class StringDocument extends AbstractXMLDocument {
    private static final String RCSRevision = "$Revision: 1.1 $";
    private static final String RCSName     = "$Name:  $";

  private static SAXParserFactory spf;
  static {
    spf = SAXParserFactory.newInstance();
    spf.setNamespaceAware(true);
  }
            
  private String xml;
  
  public StringDocument(String xml) {
    super();
    this.xml = xml;
  }
  public StringDocument(String xml, boolean readOnly) {
    super(readOnly);
    this.xml = xml;
  }

  public String getAsString() throws XMLDBCException {
    return xml;
  }
  
  public void getAsSAX() throws XMLDBCException, SAXException {
  	if (xml == null)
	{
		throw new XMLDBCException ("StringDocument.getAsSAX : Could not create SAX handler, xml string is null.") ;
	}
    StringReader sr = new StringReader(xml);
    try {
      InputSource source = new InputSource(sr);
      XMLReader reader = spf.newSAXParser().getXMLReader();
      plugHandlers(reader);
      reader.parse(source);
    } catch (java.io.IOException ioex) {
      throw new XMLDBCException(ioex.getMessage(), ioex);
    } catch (ParserConfigurationException pce) {
      throw new XMLDBCException("Could not create SAX parser using JAXP. Check a JAXP DOM implementation is in your classpath.", pce) ;
    } finally {
      sr.close();
    }
  }
}
