/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.xdbc;
import org.xquark.schema.validation.PSVInfoSetProvider;
import org.xquark.xml.xdbc.XMLDBCException;

/**
 * Internal interface between the common XDBC result set implementation (
 * {@link org.xquark.xquery.xdbc.XResultSetImpl} using the reconstruction
 * visitor from the XQuery front-end) and the XQuery back-end.
 */
public interface XDBCResultSetInterface
{
		
    /**
     * Return the value of the element specified by the path.
     * @param path
     * @param nodeAccessor
     * @param loopID the ID of the loop concerned if any
     * @return value of the element
     * @throws XDBCException if a database access error occurs.
     */
	public String fetch(String path,int nodeAccessor,String loopID, PSVInfoSetProvider psvisp) throws XMLDBCException;
    public String fetch(String path,int nodeAccessor,String loopID) throws XMLDBCException;
	public String fetch(String path,int nodeAccessor) throws XMLDBCException;

    /**
     * Generate the value of the result set specified by the path in the handler.
     * @param path
     * @param nodeAccessor the type of node to be generated
     * @param loopID the ID of the loop concerned if any
     * @throws XDBCException if a database access error occurs.
     */
	public void generate(String path,int nodeAccessor,String loopID, PSVInfoSetProvider psvisp) throws XMLDBCException;
    public void generate(String path,int nodeAccessor,String loopID) throws XMLDBCException;
	public void generate(String path,int nodeAccessor) throws XMLDBCException;

    /**
     * Recover the identifiers of the current result set and stores them in the table <I>identifiers</I>.
     * @param identifiers the table of identifiers
     * @throws XDBCException if a database access error occurs.
     */
    public void getIdentifiers(Object[] identifiers) throws XMLDBCException;

    /**
     * Recover the identifiers of the next result set and stores them in the table <I>identifiers</I>.
     * @param identifiers the table of identifiers
     * @throws XDBCException if a database access error occurs.
     */
    public void getNextIdentifiers(Object[] identifiers) throws XMLDBCException;

    /**
     * Give the new current result set (the next one).
     * @return true if the new current result set is valid, false otherwise
     * @throws XDBCException if a database access error occurs.
     */
    public boolean next() throws XMLDBCException;

    /**
     * Returns true if the result set has more elements.
     * @return true if the result set has more elements, false otherwise
     */
    public boolean hasNext() throws XMLDBCException;

    /**
     * To set a content handler (SAX 2) to intercept events produce when retrieves results by SAX 2 way.
     * @param handler the content handler implementation (see org.xml.sax.ContentHandler)
     * @throws XDBCException if a database access error occurs.
     */
    public void setContentHandler(org.xml.sax.ContentHandler handler);

    /**
     * To set a lexical handler (SAX 2) to intercept events produce when retrieves results by SAX 2 way.
     * @param handler the lexical handler implementation (see org.xml.sax.ext.LexicalHandler)
     * @throws XDBCException if a database access error occurs.
     */
    public void setLexicalHandler(org.xml.sax.ext.LexicalHandler handler);

    /**
     * To set a content handler (SAX 2) to intercept events produce when retrieves results by SAX 2 way.
     * @param handler the error handler implementation (see org.xml.sax.ErrorHandler)
     * @throws XDBCException if a database access error occurs.
     */
    public void setErrorHandler(org.xml.sax.ErrorHandler handler);

    /**
     * Releases this result set object's resources immediately instead of waiting
     * for this to happen when it is automatically closed.<BR>
     * <B>Note:</B> A XResultSet object is automatically closed by the XStatement
     * object that generated it when that XStatement object is closed,
     * re-executed, or is used to retrieve the next result from a sequence of
     * multiple results.
     * @throws XMLDBCException if a data source access error occurs.
     */
    public void close() throws XMLDBCException;
}
